package com.chuang.tauceti.acme.v2;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.annotation.JSONField;
import com.chuang.tauceti.acme.v2.connection.Connection;
import com.chuang.tauceti.acme.v2.exception.AcmeException;
import com.chuang.tauceti.acme.v2.model.Directory;
import com.chuang.tauceti.acme.v2.model.Identifier;
import com.chuang.tauceti.acme.v2.utils.AcmeUtils;
import com.chuang.tauceti.acme.v2.utils.JoseUtils;
import com.chuang.tauceti.tools.basic.StringKit;
import lombok.Data;
import org.apache.http.HttpHeaders;

import javax.annotation.Nullable;
import java.security.KeyPair;
import java.security.PublicKey;
import java.time.Instant;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Collectors;

import static com.chuang.tauceti.acme.v2.utils.AcmeUtils.base64UrlEncode;

@Data
public class Account {

    /** 状态 必填 */
    private Status status;
    private String initialIp;
    private List<String> contact;
    private String createdAt;

    /** 头信息中的Location，用作其他后面其他接口的kid, 这部分说明在 RFC 8555 44页 */
    private final String location;
    private final String keyRef;

    @JSONField(serialize = false, deserialize = false)
    private final Connection connection;
    @JSONField(serialize = false, deserialize = false)
    private final KeyPair keyPair;

    public Account(String location, Connection connection, String keyRef, KeyPair keyPair) {
        this.location = location;
        this.connection = connection;
        this.keyRef = keyRef;
        this.keyPair = keyPair;
    }

    public static Account create(String keyRef, KeyPair keyPair, Connection connection, String location, JSONObject json) {
        Account account = new Account(location, connection, keyRef, keyPair);

        account.status = Status.valueOf(json.getString("status"));
        account.initialIp = json.getString("initialIp");
        account.contact = json.getJSONArray("contact").toJavaList(String.class);
        account.createdAt = json.getString("createdAt");
        return account;
    }

    /**
     * 创建或查询订单
     * newOrder每次都会在头信息中返回一个Location，当调用者像再次刷新order的状态时，只需要直接通过Get方式访问Location就可以了。
     * 这个Location被封装在了返回的Order对象中。Order对象的update 功能会用到它。
     * @param identifiers 需要申请证书的域名
     * @param notBefore  证书提供的有效时间，协议中允许CA服务器忽略这个字段，lets encrypt好像就忽略了 （可选）
     * @param notAfter 证书提供的有效时间，协议中允许CA服务器忽略这个字段，lets encrypt好像就忽略了 （可选）
     */
    public CompletableFuture<Order> newOrder(String orderRef, List<Identifier> identifiers, @Nullable Instant notBefore, @Nullable Instant notAfter) {
        Map<String, Object> param = new HashMap<>();
        param.put("identifiers", identifiers);
        if (null != notBefore) {
            param.put("notBefore", DateTimeFormatter.ISO_INSTANT.format(notBefore));
        }
        if (null != notAfter) {
            param.put("notAfter", DateTimeFormatter.ISO_INSTANT.format(notAfter));
        }
        return newOrder(orderRef, param);
    }

    public CompletableFuture<Order> newDnsOrder(String orderRef, String domain, @Nullable Instant notBefore, @Nullable Instant notAfter, String... secDomain) {
        return newDnsOrder(orderRef, domain, notBefore, notAfter, Arrays.asList(secDomain));
    }

    public CompletableFuture<Order> newDnsOrder(String orderRef, String domain, @Nullable Instant notBefore, @Nullable Instant notAfter, Collection<String> secDomain) {
        List<Identifier> identifiers = secDomain.stream()
                .map(s -> Identifier.dns(s + "." + domain))
                .collect(Collectors.toList());
        identifiers.add(0, Identifier.dns(domain));
        return newOrder(orderRef, identifiers, notBefore, notAfter);
    }

    private CompletableFuture<Order> newOrder(String orderRef, Map<String, Object> param) {
        Session session = connection.getSession();
        Directory directory = session.getDirectory();
        return AcmeUtils.acmeRequest(this, param, directory.getNewOrder())
                .thenApply(response -> {
                    String location = response.getHeader(HttpHeaders.LOCATION).orElseThrow(() -> new AcmeException("cant get order location"));

                    Order order = new Order(orderRef, this, location);
                    order.fillValue(JSONObject.parseObject(response.asString()));
                    session.getStore().saveOrderLocation(orderRef, location);
                    return order;
                });
    }

    public CompletableFuture<Account> update() {
        return AcmeUtils.acmeRequest(this, null, getLocation())
                .thenApply(response -> {
                    JSONObject json = JSONObject.parseObject(response.asString());
                    String location = response.getHeader("Location")
                            .orElse(this.location);
                    return Account.create(this.keyRef, this.keyPair, this.getConnection(), location, json);
                })
                .thenApply(account -> {
                    getSession().getStore().save(account);
                    return account;
                });
    }

    public Session getSession() {
        return connection.getSession();
    }

    public String thumbprint() {
        PublicKey pk = getKeyPair().getPublic();
        return base64UrlEncode(JoseUtils.thumbprint(pk));
    }


    public enum Status {
        /** 有效 */
        valid,
        /** 停用，客户端行为（自己主动停用） */
        deactivated,
        /** 被撤销，服务端行为（被停用） */
        revoked
    }
}
