package com.chuang.tauceti.acme.v2;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.annotation.JSONField;
import com.chuang.tauceti.acme.v2.exception.AcmeProtocolException;
import com.chuang.tauceti.acme.v2.exception.AcmeStoreException;
import com.chuang.tauceti.acme.v2.model.Identifier;
import com.chuang.tauceti.acme.v2.model.enums.OrderStatus;
import com.chuang.tauceti.acme.v2.store.AcmeStore;
import com.chuang.tauceti.acme.v2.utils.AcmeUtils;
import com.chuang.tauceti.httpclient.Response;
import com.chuang.tauceti.tools.basic.FutureKit;
import com.chuang.tauceti.tools.basic.StringKit;
import lombok.Getter;

import java.io.IOException;
import java.security.KeyPair;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Collectors;

import static com.chuang.tauceti.acme.v2.utils.AcmeUtils.base64UrlEncode;

@Getter
public class Order extends OrderStatusUpdatable<Order> {
    /** 订单整体状态 */
    private OrderStatus status;
    /** 过期时间 */
    private String expires;
    /** 订单中需要申请的所有域名信息，每个都由各自的状态 */
    private List<Identifier> identifiers;

    /** 订单中每个域名的认证信息 */
    @JSONField(serialize = false, deserialize = false)
    private List<Authorization> authorizations;

    /** finalize 接口的地址 */
    private String finalize;

    /** 当申请成功时，刷新order能获取到这个证书地址 */
    private String certificate;

    private final String location;

    private final Account account;
    /**
     * 订单唯一编号
     */
    private final String orderRef;

    public Order(String orderRef, Account account, String orderLocation) {
        this.orderRef = orderRef;
        this.account = account;
        this.location = orderLocation;

        if(StringKit.isBlank(this.location)) {
            throw new AcmeProtocolException("找不到location头信息");
        }
    }

    public static CompletableFuture<Order> loadWithUpdate(Account account, String orderRef) {
        try {
            return load(account, orderRef).update();
        } catch (Exception e) {
            return FutureKit.error(e);
        }
    }

    public static Order load(Account account, String orderRef) throws IOException {
        Optional<String> location = account.getSession().getStore().loadOrderLocation(orderRef);
        if(location.isPresent()) {
            return new Order(orderRef, account, location.get());
        } else {
            throw new IOException(orderRef + "无法加载");
        }
    }

    public CompletableFuture<Certificate> doFinalize() throws IOException {
        byte[] csr = this.loadOrCreateCSR();

        JSONObject param = new JSONObject();
        String base64 = base64UrlEncode(csr);
        param.put("csr", base64);

        return AcmeUtils.acmeRequest(account, param, finalize)
                .thenApply(Response::asString)
                .thenApply(JSONObject::parseObject)
                .thenApply(this::fillValue)
                .thenApply(order -> new Certificate(this));

    }


    public Order fillValue(JSONObject json) {
        this.status = OrderStatus.valueOf(json.getString("status"));
        this.expires = json.getString("expires");
        this.identifiers = json.getJSONArray("identifiers").toJavaList(Identifier.class);
        this.finalize = json.getString("finalize");
        this.certificate = json.getString("certificate");

        if(null == this.authorizations) {
            this.authorizations = new ArrayList<>();
        }

        List<String> authStrList = json.getJSONArray("authorizations").stream().map(Object::toString).collect(Collectors.toList());

        for(Authorization auth: this.authorizations) {
            authStrList.remove(auth.getUrl());
        }
        authStrList.stream().map(s -> new Authorization(s, this)).forEach(this.authorizations::add);
        return this;
    }

    public CompletableFuture<Order> updateAuthOneByOne() {
        CompletableFuture<List<Authorization>> list = FutureKit.oneByOne(this.authorizations, Authorization::update, throwable -> null);
        return list.thenApply(auths -> this);
    }


    @Override
    public CompletableFuture<Order> update() {
        return AcmeUtils.acmeRequest(account, null, this.location)
                .thenApply(Response::asString)
                .thenApply(JSONObject::parseObject)
                .thenApply(this::fillValue);
    }

    private byte[] loadOrCreateCSR() throws IOException {

        AcmeStore store = account.getSession().getStore();

        Optional<byte[]> csrOpt = store.loadOrderCsr(orderRef);

        if(csrOpt.isPresent()) {
            return csrOpt.get();
        } else {
            List<String> domains = this.identifiers.stream().map(Identifier::getValue).collect(Collectors.toList());
            CSRBuilder csrb = new CSRBuilder();
            csrb.addDomains(domains);
            csrb.sign(loadOrGen());
            byte[] csr = csrb.getEncoded();
            store.saveOrderCsr(orderRef, csr); // 保存 csr 等下用
            return csr;
        }
    }

    private KeyPair loadOrGen() {
        Session session = getAccount().getSession();
        return session.getStore().loadOrderKeyPair(orderRef).orElseGet(() -> {
            KeyPair keyPair = session.getProvider().genKeyPair();
            session.getStore().saveOrderKeyPair(orderRef, keyPair);
            return keyPair;
        });
    }
}
