package com.chuang.tauceti.acme.v2;

import com.alibaba.fastjson.JSONObject;
import com.chuang.tauceti.acme.v2.exception.AcmeProtocolException;
import com.chuang.tauceti.acme.v2.model.Identifier;

import java.io.Serializable;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

import static java.util.stream.Collectors.toList;

/**
 * Represents a JSON Problem.
 *
 * @see <a href="https://tools.ietf.org/html/rfc7807">RFC 7807</a>
 */
public class Problem implements Serializable {
    private static final long serialVersionUID = -8418248862966754214L;

    private final URL baseUrl;
    private final JSONObject problemJson;

    /**
     * Creates a new {@link Problem} object.
     *
     * @param problem
     *            Problem as JSON structure
     * @param baseUrl
     *            Document's base {@link URL} to resolve relative URIs against
     */
    public Problem(JSONObject problem, URL baseUrl) {
        this.problemJson = problem;
        this.baseUrl = baseUrl;
    }

    /**
     * Returns the problem type. It is always an absolute URI.
     */
    public URI getType() {
        return Optional.ofNullable(problemJson.getString("type"))
                    .map(it -> {
                        try {
                            return baseUrl.toURI().resolve(it);
                        } catch (URISyntaxException ex) {
                            throw new IllegalArgumentException("Bad base URL", ex);
                        }
                    })
                    .orElseThrow(() -> new AcmeProtocolException("Problem without type"));
    }

    /**
     * Returns a short, human-readable summary of the problem. The text may be localized
     * if supported by the server. {@code null} if the server did not provide a title.
     *
     * @see #toString()
     */
    public Optional<String> getTitle() {
        return Optional.ofNullable(problemJson.getString("title"));
    }

    /**
     * Returns a detailed and specific human-readable explanation of the problem. The
     * text may be localized if supported by the server.
     *
     * @see #toString()
     */
    public Optional<String> getDetail() {
        return Optional.ofNullable(problemJson.getString("detail"));
    }

    /**
     * Returns an URI that identifies the specific occurence of the problem. It is always
     * an absolute URI.
     */
    public Optional<URI> getInstance() {
        return Optional.ofNullable(problemJson.getString("instance"))
                        .map(it ->  {
                            try {
                                return baseUrl.toURI().resolve(it);
                            } catch (URISyntaxException ex) {
                                throw new IllegalArgumentException("Bad base URL", ex);
                            }
                        });
    }

    /**
     * Returns the {@link Identifier} this problem relates to. May be {@code null}.
     *
     * @since 2.3
     */
    public Optional<Identifier> getIdentifier() {
        return Optional.of(problemJson.getObject("identifier", Identifier.class));
    }

    /**
     * Returns a list of sub-problems. May be empty, but is never {@code null}.
     */
    public List<Problem> getSubProblems() {
        return Collections.unmodifiableList(
                problemJson.getJSONArray("subproblems")
                        .stream()
                        .map(o -> new Problem((JSONObject) o, baseUrl))
                        .collect(toList())
        );
    }

    /**
     * Returns the problem as {@link JSONObject} object, to access other fields.
     *
     * @return Problem as {@link JSONObject} object
     */
    public JSONObject asJSON() {
        return problemJson;
    }

    /**
     * Returns a human-readable description of the problem, that is as specific as
     * possible. The description may be localized if supported by the server.
     * <p>
     * If {@link #getSubProblems()} exist, they will be appended.
     * <p>
     * Technically, it returns {@link #getDetail()}. If not set, {@link #getTitle()} is
     * returned instead. As a last resort, {@link #getType()} is returned.
     */
    @Override
    public String toString() {
        return problemJson.toString();
    }

}
