package com.chuang.tauceti.acme.v2;

import com.chuang.tauceti.acme.v2.connection.Connection;
import com.chuang.tauceti.acme.v2.model.Directory;
import com.chuang.tauceti.acme.v2.model.MetaData;
import com.chuang.tauceti.acme.v2.provider.AcmeProvider;
import com.chuang.tauceti.acme.v2.provider.GenericAcmeProvider;
import com.chuang.tauceti.acme.v2.store.AcmeStore;
import com.chuang.tauceti.httpclient.Https;
import com.chuang.tauceti.httpclient.async.AsyncHttpClientImpl;
import lombok.Getter;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Nullable;
import java.net.URI;
import java.time.ZonedDateTime;
import java.util.List;
import java.util.Locale;
import java.util.Optional;
import java.util.ServiceLoader;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

/**
 * 会话会保存申请证书过程中的上下文变量。
 */
@Slf4j
public class Session {

    /** 默认的通用实现 */
    private static final GenericAcmeProvider GENERIC_ACME_PROVIDER = new GenericAcmeProvider();

    @Getter
    private final URI serverUri;
    @Getter
    private final AcmeProvider provider;

    /** 防重复提交随机数,（acme协议接口请求时需要附带上一个接口返回的nonce，用来防重复提交） */
    @Nullable
    @Getter @Setter
    private String nonce;

    @Getter @Setter
    private Locale locale = Locale.getDefault();

    @Getter @Setter
    private Directory directory;

    @Getter
    private final AcmeStore store;

    /**
     * 调用接口时的httpclient实现，只支持异步httpclient.
     * 如：代理，超时时间等，需要说明的是 AsyncHttpClient只支持 http代理。原因可以查看{@link Https}
     */
    private static AsyncHttpClientImpl httpClient;

    @Nullable
    @Getter @Setter
    private ZonedDateTime directoryLastModified;

    @Nullable
    @Getter @Setter
    private ZonedDateTime directoryExpires;


    public Session(String serverUri, AcmeStore store) {
        this(URI.create(serverUri), load(URI.create(serverUri)), store);
    }

    public Session(URI serverUri, AcmeProvider provider, AcmeStore store) {
        if (!provider.support(serverUri)) {
            throw new IllegalArgumentException(provider.getClass().getSimpleName() + "不支持[" + serverUri + "]资源");
        }
        this.serverUri = serverUri;
        this.provider = provider;
        this.store = store;
        this.store.setProvider(provider);
    }



    public Connection connect() {
        return provider.connect(this);
    }

    private static AcmeProvider load(URI serverUri) {
        // 获取所有支持当前 uri 的Acme服务。
        List<AcmeProvider> providers = loadSupportProviders(serverUri);

        if (providers.isEmpty()) {
            log.warn("没有找到任何适配[" + serverUri + "]的provider, 将尝试使用通用实现（GenericAcmeProvider）");
            if(GENERIC_ACME_PROVIDER.support(serverUri)) {
                return GENERIC_ACME_PROVIDER;
            } else {
                throw new IllegalArgumentException("通用实现（GenericAcmeProvider）也无法适配[" + serverUri + "]");
            }
        } else {
            if(providers.size() > 1) {
                StringBuilder warn = new StringBuilder("发现多个符合 [" + serverUri + "] 的ACME providers \n");
                for(AcmeProvider provider: providers) {
                    warn.append(provider.getClass().getSimpleName()).append("\n");
                }
                warn.append("系统将选用第一个provider [").append(providers.get(0).getClass().getSimpleName()).append("]");
                log.warn(warn.toString());
            }
            return providers.get(0);
        }
    }

    private static List<AcmeProvider> loadSupportProviders(URI serverUri) {
        return StreamSupport.stream(ServiceLoader.load(AcmeProvider.class).spliterator(), false)
                .filter(provider -> provider.support(serverUri))
                .collect(Collectors.toList());
    }

    public Optional<MetaData> getMetaData() {
        return Optional.ofNullable(directory).map(Directory::getMeta);
    }

    /*public synchronized AsyncHttpClient getHttpClient() {
        if(null == httpClient) {
            httpClient = this.provider.httpBuilder()
                    .requestHold(this::requestHold)
                    .responseHold(this::responseHold)
                    .build()
                    .init();
        }
        return httpClient;
    }*/

//    public synchronized void setHttpClient(AsyncBuilder httpClientBuilder) {
//        if(null != this.httpClient) {
//            this.httpClient.shutdown();
//        }
//        this.httpClient = httpClientBuilder
//                .requestHold(this::requestHold)
//                .responseHold(this::responseHold)
//                .build()
//                .init();
//    }


//    protected void requestHold(HttpRequestBase request) {
//        if(null == request.getFirstHeader(HttpHeaders.CONTENT_TYPE)) {
//            request.setHeader(HttpHeaders.CONTENT_TYPE, "application/jose+json");
//        }
//    }
//
//    protected void responseHold(Response response) {
//        AcmeResponseUtils.getNonce(response).ifPresent(s -> this.nonce = s);
//
//        int rc = response.getStatusCode();
//        boolean ifModifiedSince = response.getRequest().containsHeader(HttpHeaders.IF_MODIFIED_SINCE);
//
//        if (rc != HttpURLConnection.HTTP_OK && rc != HttpURLConnection.HTTP_CREATED
//                && (rc != HttpURLConnection.HTTP_NOT_MODIFIED || !ifModifiedSince)) {
//            AcmeResponseUtils.throwAcmeException(response);
//        }
//    }


}
