package com.chuang.tauceti.acme.v2.connection;

import com.alibaba.fastjson.JSONObject;
import com.chuang.tauceti.acme.v2.Account;
import com.chuang.tauceti.acme.v2.Session;
import com.chuang.tauceti.acme.v2.exception.AcmeException;
import com.chuang.tauceti.acme.v2.exception.AcmeProtocolException;
import com.chuang.tauceti.acme.v2.model.AccountParam;
import com.chuang.tauceti.acme.v2.model.Directory;
import com.chuang.tauceti.acme.v2.utils.AcmeResponseUtils;
import com.chuang.tauceti.acme.v2.utils.AcmeUtils;
import com.chuang.tauceti.acme.v2.utils.JoseUtils;
import com.chuang.tauceti.httpclient.Request;
import com.chuang.tauceti.tools.basic.FutureKit;
import org.apache.http.HttpHeaders;

import javax.annotation.Nullable;
import javax.crypto.SecretKey;
import java.net.HttpURLConnection;
import java.net.URI;
import java.net.URL;
import java.security.Key;
import java.security.KeyPair;
import java.time.ZonedDateTime;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;
import java.util.function.Function;

import static java.time.format.DateTimeFormatter.RFC_1123_DATE_TIME;

/**
 * ACME 通讯接口
 */
public interface Connection {

    default CompletableFuture<Void> directory() {
        Session session = getSession();
        ZonedDateTime expires = session.getDirectoryExpires();
        if (expires != null && expires.isAfter(ZonedDateTime.now())) {
            return CompletableFuture.completedFuture(null);
        }


        ZonedDateTime lastModified = session.getDirectoryLastModified();

        Request.Builder builder = Request.Get(uriResolve().apply(session.getServerUri()) + "/directory")
                .header(HttpHeaders.CONTENT_TYPE, "application/jose+json");

        if(null != lastModified) {
            builder.header(HttpHeaders.IF_MODIFIED_SINCE, lastModified.format(RFC_1123_DATE_TIME));
        }

        return builder.build().asyncExecute()
                .thenApply(response -> AcmeUtils.handResponse(session, response))
                .thenApply(response -> {
                    if (lastModified != null && response.getStatusCode() == HttpURLConnection.HTTP_NOT_MODIFIED) {
                        return null;
                    }
                    // evaluate caching headers
                    session.setDirectoryLastModified(AcmeResponseUtils.getLastModified(response).orElse(null));
                    session.setDirectoryExpires(AcmeResponseUtils.getCacheControlOrExpiration(response).orElse(null));

                    AcmeResponseUtils.getNonce(response).ifPresent(session::setNonce);

                    Directory directory = JSONObject.parseObject(response.asString(), Directory.class);
                    session.setDirectory(directory);

                    return null;
                });
    }

    default CompletableFuture<String> nonce() {
        return Request.Head(getSession().getDirectory().getNewNonce())
                .header(HttpHeaders.CONTENT_TYPE, "application/jose+json")
                .build()
                .asyncExecute()
                .thenApply(response -> AcmeUtils.handResponse(getSession(), response))
                .thenApply(response -> AcmeResponseUtils.getNonce(response).orElse(null));
    }

    /**
     * 新建账号
     * @param userKeyRef 密钥的唯一编号（逻辑上的，由调用人自己保证）
     * @param onlyReturnExisting 是否仅仅只返回已经注册的。当设定为false时表示当用户不存在则自动申请。
     * @param contacts 联系方式，一般为email。格式以 mailto: account@mail.com
     * @return 账号
     */
    default CompletableFuture<Account> newAccount(String userKeyRef, boolean onlyReturnExisting, String... contacts) {
        AccountParam param = new AccountParam()
                .setTermsOfServiceAgreed(true)
                .setOnlyReturnExisting(onlyReturnExisting)
                .setContact(contacts);

        return newAccount(userKeyRef, param, null);
    }

    default Optional<Account> loadAccount(String userKeyRef) {
        return this.getSession().getStore().loadAccount(userKeyRef, this);
    }

    default CompletableFuture<Account> loadOrNewAccount(String userKeyRef, boolean onlyReturnExisting, String... contacts) {
        return loadAccount(userKeyRef)
                .map(account -> nonce().thenApply(s -> account)) // 先nonce一下，免得直接加载没有nonce可用
                .orElse(newAccount(userKeyRef, onlyReturnExisting, contacts));
    }

    default CompletableFuture<Account> newAccountWithExternalBinding(
            String userKeyRef, String kid, SecretKey macKey, Boolean onlyReturnExisting, String... contacts) {
        Directory directory = getSession().getDirectory();


        KeyPair userKeyPair = getSession().getProvider().genKeyPair();

        JSONObject binding =
                JoseUtils.createExternalAccountBinding(kid, userKeyPair.getPublic(), macKey, directory.getNewAccount());

        AccountParam param = new AccountParam()
                .setTermsOfServiceAgreed(true)
                .setOnlyReturnExisting(onlyReturnExisting)
                .setExternalAccountBinding(binding.toJSONString())
                .setContact(contacts);

        return newAccount(userKeyRef, param, userKeyPair);
    }

    default CompletableFuture<Account> newAccount(String userKeyRef, AccountParam param, @Nullable KeyPair _userKeyPair) {
        if(loadAccount(userKeyRef).isPresent()) {
            return FutureKit.error(new AcmeException(userKeyRef + "账号已经存在！！"));
        }
        KeyPair userKeyPair = null == _userKeyPair ? getSession().getProvider().genKeyPair() : _userKeyPair;

        return nonce().thenCompose(nonce -> {
            Directory directory = getSession().getDirectory();

            return AcmeUtils.acmeRequest(getSession(), userKeyPair, param, nonce, directory.getNewAccount())
                    .thenApply(response -> {
                        JSONObject json = JSONObject.parseObject(response.asString());
                        String location = response.getHeader("Location")
                                .orElseThrow(() -> new AcmeProtocolException("Server did not provide an account location"));
                        return Account.create(userKeyRef, userKeyPair, this, location, json);
                    })
                    .thenApply(account -> {
                        getSession().getStore().save(account);
                        return account;
                    });
        });
    }




    Session getSession();

    Function<URI, URL> uriResolve();
}
