package com.chuang.tauceti.acme.v2.store;

import com.alibaba.fastjson.JSONObject;
import com.chuang.tauceti.acme.v2.Account;
import com.chuang.tauceti.acme.v2.connection.Connection;
import com.chuang.tauceti.acme.v2.exception.AcmeStoreException;
import com.chuang.tauceti.acme.v2.provider.AcmeProvider;
import com.chuang.tauceti.acme.v2.utils.AcmeUtils;
import com.chuang.tauceti.acme.v2.utils.KeyPairUtils;
import com.chuang.tauceti.tools.basic.FileKit;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;

import java.io.*;
import java.security.KeyPair;
import java.security.cert.X509Certificate;
import java.util.Collection;
import java.util.Optional;


@Slf4j
public class FileStore implements AcmeStore {

    @Getter
    private final String rootPath;

    private AcmeProvider provider;

    private static final String KEY_FILE_NAME = "privkey.pem";
    private static final String CSR_FILE_NAME = "csr.key";
    private static final String CRT_FILE_NAME = "fullchain.pem";

    public FileStore(String rootPath) {
        this.rootPath = rootPath;
    }

    @Override
    public void setProvider(AcmeProvider provider) {
        this.provider = provider;
    }

    @Override
    public void save(Account account) throws AcmeStoreException {
        String path = basePath() + "account/";

        String keyFile = path + account.getKeyRef() + "_key.pem";
        String infoFile = path + account.getKeyRef() + "_info.json";

        try {
            FileKit.touch(keyFile);
            FileKit.touch(infoFile);
            try (FileWriter fw = new FileWriter(keyFile)) {
                KeyPairUtils.writeKeyPair(account.getKeyPair(), fw);
            }
            FileKit.writeString(JSONObject.toJSONString(account), infoFile, "utf-8");
        } catch (IOException e) {
            throw new AcmeStoreException("保存账号[{}]失败", account.getKeyRef(), e);
        }
    }

    @Override
    public void delete(String keyRef) {
        String path = basePath() + "account/";

        String keyFile = path + keyRef + "_key.pem";
        String infoFile = path + keyRef + "_info.json";

        FileKit.del(keyFile);
        FileKit.del(infoFile);
    }

    @Override
    public Optional<Account> loadAccount(String keyRef, Connection connection) {
        String path = basePath() + "account/";
        String keyFile = path + keyRef + "_key.pem";
        String infoFile = path + keyRef + "_info.json";

        KeyPair keyPair;
        try {
            if (FileKit.exist(keyFile) && FileKit.exist(infoFile)) {
                try (FileReader fr = new FileReader(keyFile)) {
                    keyPair = KeyPairUtils.readKeyPair(fr);
                }
                JSONObject json = JSONObject.parseObject(FileKit.readString(infoFile, "utf-8"));
                return Optional.of(Account.create(keyRef, keyPair, connection, json.getString("location"), json));
            } else {
                FileKit.del(keyFile);
                FileKit.del(infoFile);
                return Optional.empty();
            }
        } catch (Exception e) {
            log.warn("加载账号[{}]失败", keyRef, e);
            return Optional.empty();
        }

    }

    @Override
    public Optional<KeyPair> loadOrderKeyPair(String orderRef) {
        try {
            String file = basePath() + orderRef + "/" + KEY_FILE_NAME;
            if (FileKit.exist(file)) {
                try (FileReader fr = new FileReader(file)) {
                    return Optional.ofNullable(KeyPairUtils.readKeyPair(fr));
                }
            } else {
                return Optional.empty();
            }
        } catch (Exception e) {
            log.warn("加载 [{}] Order KeyPair 失败", orderRef, e);
            return Optional.empty();
        }
    }

    @Override
    public void saveOrderKeyPair(String orderRef, KeyPair keyPair) throws AcmeStoreException {
        String file = basePath() + orderRef + "/" + KEY_FILE_NAME;

        try {
            FileKit.touch(file);
            try (FileWriter fw = new FileWriter(file)) {
                KeyPairUtils.writeKeyPair(keyPair, fw);
            }
        } catch (IOException e) {
            throw new AcmeStoreException("保存[" + orderRef + "] Order KeyPair失败", orderRef, e);
        }
    }

    @Override
    public void deleteOrder(String orderRef) {
        String file = basePath() + orderRef + "/" + KEY_FILE_NAME;
        FileKit.del(file);
        file = basePath() + orderRef + "/" + CSR_FILE_NAME;
        FileKit.del(file);
        file = basePath() + orderRef + "/" + CRT_FILE_NAME;
        FileKit.del(file);
    }

    @Override
    public void saveOrderCsr(String orderRef, byte[] csr) throws AcmeStoreException {
        String file = basePath() + orderRef + "/" + CSR_FILE_NAME;
        try {
            FileKit.touch(file);
            try (FileOutputStream writer = new FileOutputStream(file)) {
                writer.write(csr);
                writer.flush();
            }
        } catch (Exception e) {
            throw new AcmeStoreException("保存[" + orderRef + "] Order CSR 失败", orderRef, e);
        }
    }

    @Override
    public Optional<byte[]> loadOrderCsr(String orderRef) {
        String file = basePath() + orderRef + "/" + CSR_FILE_NAME;
        try {
            if (FileKit.exist(file)) {
                byte[] csr = FileKit.readBytes(new File(file));
                return Optional.of(csr);
            } else {
                return Optional.empty();
            }
        } catch (Exception e) {
            log.warn("加载[{}] Order CSR 失败", orderRef, e);
            return Optional.empty();
        }
    }

    public void saveOrderCertificate(String orderRef, Collection<X509Certificate> x509Certificates) throws AcmeStoreException {
        String file = basePath() + orderRef + "/" + CRT_FILE_NAME;
        try {
            FileKit.touch(file);
            try (FileWriter writer = new FileWriter(file)) {
                for (X509Certificate x509Certificate : x509Certificates) {
                    AcmeUtils.writeToPem(x509Certificate.getEncoded(), AcmeUtils.PemLabel.CERTIFICATE, writer);
                }
            }
        } catch (Exception e) {
            throw new AcmeStoreException("保存[" + orderRef + "] Order X509_Chain 失败", orderRef, e);
        }
    }

    @Override
    public Optional<String> loadOrderLocation(String orderRef) {
        String file = basePath() + orderRef + "/location.txt";
        try {
            if (FileKit.exist(file)) {
                return Optional.of(FileKit.readString(file));
            }
        } catch (Exception e) {
            log.warn("加载[{}] Order CSR 失败", orderRef, e);
        }
        return Optional.empty();
    }

    @Override
    public void saveOrderLocation(String orderRef, String orderLocation) {
        String file = basePath() + orderRef + "/location.txt";
        try {
            FileKit.touch(file);
            FileKit.writeString(orderLocation, file, "utf-8");
        } catch (Exception e) {
            throw new AcmeStoreException("保存[" + orderRef + "] Order Location [" + orderLocation + "] 失败", orderRef, e);
        }
    }


    public String basePath() {
        return rootPath + "/" + provider.name() + "/";
    }
}
