package com.chuang.tauceti.acme.v2.utils;

import com.alibaba.fastjson.JSONObject;
import com.chuang.tauceti.acme.v2.Problem;
import com.chuang.tauceti.acme.v2.connection.TrimmingInputStream;
import com.chuang.tauceti.acme.v2.exception.*;
import com.chuang.tauceti.httpclient.Response;
import com.chuang.tauceti.tools.basic.collection.CollectionKit;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpHeaders;
import org.apache.http.NameValuePair;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.time.Instant;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeParseException;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import static java.time.format.DateTimeFormatter.RFC_1123_DATE_TIME;
import static java.util.stream.Collectors.toList;

@Slf4j
public class AcmeResponseUtils {

    private static final String LINK_HEADER = "Link";
    private static final String REPLAY_NONCE_HEADER = "Replay-Nonce";
    private static final String MIME_JSON_PROBLEM = "application/problem+json";
    private static final String ACME_ERROR_PREFIX = "urn:ietf:params:acme:error:";
    public static final String MIME_CERTIFICATE_CHAIN = "application/pem-certificate-chain";
    private static final Pattern NO_CACHE_PATTERN = Pattern.compile("(?:^|.*?,)\\s*no-(?:cache|store)\\s*(?:,.*|$)", Pattern.CASE_INSENSITIVE);
    private static final Pattern MAX_AGE_PATTERN = Pattern.compile("(?:^|.*?,)\\s*max-age=(\\d+)\\s*(?:,.*|$)", Pattern.CASE_INSENSITIVE);


    private AcmeResponseUtils() {}

    public static void throwAcmeException(Response response) {
        try {
            URL url = response.getRequest().getURI().toURL();

            response.getHeader(HttpHeaders.CONTENT_TYPE).ifPresent(contentType -> {
                String body = response.asString();
                if (!MIME_JSON_PROBLEM.equals(contentType)) {
                    throw new AcmeException("HTTP " + response.getStatusCode() + ": " + body);
                }

                Problem problem = new Problem(JSONObject.parseObject(body), url);

                String error = AcmeUtils.stripErrorPrefix(problem.getType().toString());

                if ("unauthorized".equals(error)) {
                    throw new AcmeUnauthorizedException(problem);
                }

                if ("userActionRequired".equals(error)) {
                    URI tos = collectLinks(response, "terms-of-service").stream()
                            .findFirst()
                            .map(s -> {
                                URI uri = response.getRequest().getURI();
                                return URI.create(uri.toString() + s);
                            })
                            .orElse(null);
                    throw new AcmeUserActionRequiredException(problem, tos);
                }

                if ("rateLimited".equals(error)) {
                    Optional<Instant> retryAfter = getRetryAfterHeader(response);
                    Collection<URL> rateLimits = getLinks(response, "help");
                    throw new AcmeRateLimitedException(problem, retryAfter.orElse(null), rateLimits);
                }

                throw new AcmeServerException(problem);
            });
        } catch (IOException ex) {
            throw new AcmeNetworkException(ex);
        }
    }

    public static Collection<URL> getLinks(Response response, String relation) {
        return collectLinks(response, relation).stream()
                .map(s -> {
                    try {
                        return URI.create(response.getRequest().getURI().toString() + s).toURL();
                    } catch (MalformedURLException e) {
                        return null;
                    }
                })
                .collect(toList());
    }

    public static Collection<String> collectLinks(Response response, String relation) {

        List<String> result = new ArrayList<>();

        List<String> links = Arrays.stream(response.getHeaders(LINK_HEADER))
                .map(NameValuePair::getName)
                .collect(toList());
        if (CollectionKit.isNotEmpty(links)) {
            Pattern p = Pattern.compile("<(.*?)>\\s*;\\s*rel=\"?"+ Pattern.quote(relation) + "\"?");
            for (String link : links) {
                Matcher m = p.matcher(link);
                if (m.matches()) {
                    String location = m.group(1);
                    log.debug("Link: {} -> {}", relation, location);
                    result.add(location);
                }
            }
        }
        return result;
    }

    public static List<X509Certificate> readCertificates(Response response) throws AcmeException {

        boolean certChain = response.getHeader(HttpHeaders.CONTENT_TYPE)
                .map(header -> header.equalsIgnoreCase(AcmeResponseUtils.MIME_CERTIFICATE_CHAIN))
                .orElse(false);

        if (!certChain) {
            throw new AcmeProtocolException("Unexpected content type: " + response.getHeader(HttpHeaders.CONTENT_TYPE).orElse(""));
        }

        try (InputStream in = new TrimmingInputStream(response.asStream())) {
            CertificateFactory cf = CertificateFactory.getInstance("X.509");
            return cf.generateCertificates(in).stream()
                    .map(c -> (X509Certificate) c)
                    .collect(toList());
        } catch (IOException ex) {
            throw new AcmeNetworkException(ex);
        } catch (CertificateException ex) {
            throw new AcmeProtocolException("Failed to read certificate", ex);
        }
    }

    public static Optional<Instant> getRetryAfterHeader(Response response) {
        // See RFC 2616 section 14.37
        return response.getHeader(HttpHeaders.RETRY_AFTER).flatMap(header -> {
            try {
                // delta-seconds
                if (header.matches("^\\d+$")) {
                    int delta = Integer.parseInt(header);
                    long date = response.getHeader(HttpHeaders.DATE)
                            .flatMap(AcmeResponseUtils::timeParseLong)
                            .orElse(System.currentTimeMillis());
                    return Optional.of(Instant.ofEpochMilli(date).plusSeconds(delta));
                }

                // HTTP-date
                return response.getHeader(HttpHeaders.RETRY_AFTER)
                        .flatMap(AcmeResponseUtils::timeParseLong)
                        .map(Instant::ofEpochMilli);
            } catch (Exception ex) {
                throw new AcmeProtocolException("Bad retry-after header value: " + header, ex);
            }
        });
    }

    public static Optional<Long> timeParseLong(String dateString) {
        if (!dateString.contains("GMT")) {
            dateString = dateString + " GMT";
        }
        try {
            return Optional.of(Date.parse(dateString));
        } catch (Exception e) {
            return Optional.empty();
        }
    }

    public static Optional<ZonedDateTime> getLastModified(Response response) {
        return response.getHeader(HttpHeaders.LAST_MODIFIED).flatMap(header -> {
            try {
                return Optional.of(ZonedDateTime.parse(header, RFC_1123_DATE_TIME));
            } catch (DateTimeParseException ex) {
                log.debug("Ignored invalid Last-Modified date: {}", header, ex);
                return Optional.empty();
            }
        });
    }

    public static Optional<ZonedDateTime> getCacheControl(Response response) {
        return response.getHeader(HttpHeaders.CACHE_CONTROL).flatMap(cacheHeader -> {
            if (NO_CACHE_PATTERN.matcher(cacheHeader).matches()) {
                return Optional.empty();
            }

            Matcher m = MAX_AGE_PATTERN.matcher(cacheHeader);
            if (m.matches()) {
                int maxAge = Integer.parseInt(m.group(1));
                if (maxAge == 0) {
                    return Optional.empty();
                }

                return Optional.of(ZonedDateTime.now(ZoneId.of("UTC")).plusSeconds(maxAge));
            } else {
                return Optional.empty();
            }
        });
    }

    public static Optional<ZonedDateTime> getExpiration(Response response) {

        return response.getHeader(HttpHeaders.EXPIRES).flatMap(expiresHeader -> {
            try {
                return Optional.of(ZonedDateTime.parse(expiresHeader, RFC_1123_DATE_TIME));
            } catch (DateTimeParseException ex) {
                log.debug("Ignored invalid Expires date: {}", expiresHeader, ex);
                return Optional.empty();
            }
        });


    }

    public static Optional<ZonedDateTime> getCacheControlOrExpiration(Response response) {
        Optional<ZonedDateTime> opt = getCacheControl(response);
        if(opt.isPresent()) {
            return opt;
        } else {
            return getExpiration(response);
        }
    }

    public static Optional<String> getNonce(Response response) {
        return response.getHeader(REPLAY_NONCE_HEADER).map(header -> {
            if(!AcmeUtils.isValidBase64Url(header)) {
                throw new AcmeProtocolException("Invalid replay nonce: " + header);
            }
            log.debug("Replay Nonce: {}", header);

            return header;
        });
    }


}
