package com.chuang.tauceti.blockchain.payment.client;

import com.chuang.tauceti.blockchain.payment.model.Coin;
import com.chuang.tauceti.blockchain.payment.model.IBlock;
import com.chuang.tauceti.blockchain.payment.model.ITransaction;
import com.chuang.tauceti.blockchain.payment.model.Wallet;
import com.chuang.tauceti.blockchain.payment.watch.BlockRange;
import com.chuang.tauceti.blockchain.payment.watch.BlockWatcher;
import com.chuang.tauceti.blockchain.payment.watch.strategy.WatchRetryStrategy;

import javax.annotation.Nullable;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.concurrent.CompletableFuture;

public interface IBlockChainClient<B extends IBlock<T>, T extends ITransaction> {

    /**
     * 链的名字，逻辑名字，只要能区分，随便取都行，不同的链用不同的名字
     */
    String getChainName();

    /**
     * 创建钱包
     * @param id 唯一标识
     * @param password 密码
     * @return 钱包
     */
    Wallet createWallet(@Nullable String password);

    /**
     * 转账
     * @param coin 转账的币种
     * @param from 打款地址
     * @param to   收款地址
     * @param fromPrivateKey 打款地址的私钥
     * @param value 打款额度
     * @return 交易哈希
     */
    CompletableFuture<String> transfer(Coin coin, String from, String to, String fromPrivateKey, BigInteger value);

    /**
     * 转账
     * @param coin 转账的币种
     * @param from 打款地址
     * @param to   收款地址
     * @param fromPrivateKey 打款地址的私钥
     * @param value 打款额度
     * @return 交易哈希
     */
    default CompletableFuture<String> transfer(Coin coin, String from, String to, String fromPrivateKey, BigDecimal value) {
        return this.transfer(coin, from, to, fromPrivateKey, coin.decimalToInt(value));
    }

    /**
     * 查询地址中coin币种的额度
     * @param coin    币种
     * @param address 地址
     * @return 额度
     */
    CompletableFuture<BigInteger> getBalance(Coin coin, String address);

    /**
     * 查询地址中coin币种的额度
     * @param coin    币种
     * @param address 地址
     * @return 额度
     */
    default CompletableFuture<BigDecimal> getBalanceAsDecimal(Coin coin, String address) {
        return getBalance(coin, address).thenApply(coin::intToDecimal);
    }

    /**
     * 获取最后一个区块的高度
     * @return 最后一个区块的高度
     */
    CompletableFuture<BigInteger> getLastBlockNum();

    /**
     * 获取最后一个区块的哈希
     * @return 最后一个区块的哈希
     */
    CompletableFuture<String> getLastBlockHash();

    /**
     * 根据区块哈希获取高度
     * @param blockHash 区块哈希
     * @return 区块高度
     */
    CompletableFuture<BigInteger> asBlockNum(String blockHash);

    /**
     * 根据区块高度获取区块哈希
     * @param num 区块高度
     * @return 区块哈希
     */
    CompletableFuture<String> asBlockHash(BigInteger num);

    /**
     * 根据区块哈希，查询区块信息
     * @param blockHash 区块哈希
     * @return 区块信息
     */
    CompletableFuture<B> getBlockInfoByHash(String blockHash);


    /**
     * 根据区块高度，查询区块信息
     * @param num 区块高度
     * @return 区块信息
     */
    default CompletableFuture<B> getBlockInfoByNum(BigInteger num) {
        return asBlockHash(num).thenCompose(this::getBlockInfoByHash);
    }

    CompletableFuture<T> queryTransfer(String txHash);


    /**
     * 根据区块哈希，查询下一个区块信息
     * @param blockHash 区块哈希
     * @return 区块信息
     */
    CompletableFuture<B> getNextBlockInfo(String blockHash);

    /**
     * 根据区块哈希，查询上一个区块信息
     * @param blockHash 区块哈希
     * @return 区块信息
     */
    CompletableFuture<B> getPreviousBlockInfo(String blockHash);

    /**
     * 从某个 begin 区块开始，一直抓到 end 区块结束。
     * 如果 begin < end 则抓取方式为升序
     * 如果 end < begin 则抓取方式为降序
     *
     * 如果 end 的值为null，则表示无限大
     *
     * 注意，注意，注意。
     * 注意，内部调用 Watcher.create 每次调用都会创建一个全新的 Watcher，这样可能导致严重的性能问题。
     * 如果确实需要对抓取的数据进行不同形式的监听，只需要调用一次 watch，并在返回的 Watcher对象上多次调用 watcher.watch 方法进行监听。
     * @param retryStrategy 当抓取区块失败后的重试策略
     * @param range 范围
     * @return 区块信息
     */
    default BlockWatcher<B, T> watch(BlockRange range, @Nullable WatchRetryStrategy<B> retryStrategy, @Nullable Integer offset) {
        return new BlockWatcher<>(this, range, retryStrategy, offset);
    }

    /**
     * {@link #watch(BlockRange, WatchRetryStrategy, Integer)}
     * @param range 范围
     * @return 区块信息
     */
    default BlockWatcher<B, T> watch(BlockRange range) {
        return watch(range, null, null);
    }

}

