package com.chuang.tauceti.blockchain.payment.client.eth;

import com.chuang.tauceti.blockchain.payment.BlockChainConst;
import com.chuang.tauceti.blockchain.payment.BlockChainUtils;
import com.chuang.tauceti.blockchain.payment.client.IBlockChainClient;
import com.chuang.tauceti.blockchain.payment.client.IContractClient;
import com.chuang.tauceti.blockchain.payment.model.Coin;
import com.chuang.tauceti.blockchain.payment.model.Wallet;
import com.chuang.tauceti.blockchain.payment.model.eth.EthBlock;
import com.chuang.tauceti.blockchain.payment.model.eth.EthTransaction;
import com.chuang.tauceti.blockchain.payment.model.eth.Log;
import com.chuang.tauceti.blockchain.payment.model.eth.Token;
import com.chuang.tauceti.blockchain.payment.watch.BlockRange;
import com.chuang.tauceti.blockchain.payment.watch.eth.BufferedLogWatcher;
import com.chuang.tauceti.blockchain.payment.watch.eth.ITopic;
import com.chuang.tauceti.blockchain.payment.watch.eth.LogWatcher;
import com.chuang.tauceti.blockchain.payment.watch.strategy.WatchRetryStrategy;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.support.exception.SystemException;
import com.chuang.tauceti.tools.basic.FutureKit;
import com.chuang.tauceti.tools.basic.ScheduleKit;
import com.chuang.tauceti.tools.basic.StringKit;
import lombok.extern.slf4j.Slf4j;
import org.web3j.abi.FunctionEncoder;
import org.web3j.abi.TypeReference;
import org.web3j.abi.datatypes.Address;
import org.web3j.abi.datatypes.Function;
import org.web3j.abi.datatypes.Type;
import org.web3j.abi.datatypes.generated.Uint256;
import org.web3j.crypto.*;
import org.web3j.protocol.Web3j;
import org.web3j.protocol.Web3jService;
import org.web3j.protocol.core.DefaultBlockParameter;
import org.web3j.protocol.core.DefaultBlockParameterName;
import org.web3j.protocol.core.Response;
import org.web3j.protocol.core.methods.request.EthFilter;
import org.web3j.protocol.core.methods.request.Transaction;
import org.web3j.protocol.core.methods.response.*;
import org.web3j.protocol.http.HttpService;
import org.web3j.protocol.websocket.WebSocketService;
import org.web3j.utils.Numeric;

import javax.annotation.Nullable;
import java.math.BigInteger;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import static com.chuang.tauceti.blockchain.payment.BlockChainConst.HEX_PRE;


/**
 * 以太坊客户端， 所有以太坊侧链都可以使用它
 */
@Slf4j
public abstract class EthClient implements IBlockChainClient<EthBlock, EthTransaction>, IContractClient {

    private final Web3j web3j;
    protected final GasProvider gasProvider;
    protected final NonceProvider nonceProvider;

    public EthClient(Web3j web3j, GasProvider gasProvider, @Nullable NonceProvider nonceProvider) {
        this.web3j = web3j;
        this.gasProvider = gasProvider;
        if(null == nonceProvider) {
            this.nonceProvider = new NonceProvider() {};
        } else {
            this.nonceProvider = nonceProvider;
        }
    }

    public EthClient(Web3jService service, GasProvider gasProvider, @Nullable NonceProvider nonceProvider) {
        this(Web3j.build(service), gasProvider, nonceProvider);
    }

    public EthClient(String chainSeed, GasProvider gasProvider, @Nullable NonceProvider nonceProvider) {
        this(create(chainSeed), gasProvider, nonceProvider);
    }

    private static Web3j create(String chainSeed) {
        if (chainSeed.toLowerCase().startsWith(BlockChainConst.HTTP.toLowerCase())) {
            return Web3j.build(new HttpService(chainSeed));
        } else {
            return Web3j.build(new WebSocketService(chainSeed, true));
        }
    }


    @Override
    public Wallet createWallet(@Nullable String password) {
        password = StringKit.nullToEmpty(password);
        try {
            ECKeyPair ecKeyPair = Keys.createEcKeyPair();
            String privateKey = ecKeyPair.getPrivateKey().toString(16);
            String publicKey =  ecKeyPair.getPublicKey().toString(16);

            WalletFile wallet = org.web3j.crypto.Wallet.createLight(password, ecKeyPair);
            String address = wallet.getAddress().toLowerCase();
            if(!address.toLowerCase().startsWith(HEX_PRE)) {
                address = HEX_PRE + address;
            }

            return new Wallet().setAddress(address)
                    .setEcKeyPair(ecKeyPair)
                    .setPrivateKey(privateKey)
                    .setPublicKey(publicKey)
                    .setPassword(password);
        } catch (Exception e) {
            throw new BusinessException("创建钱包失败", e);
        }


    }

    @Override
    public CompletableFuture<String> functionCall(String contractAddress, String funcName, String from, String fromPrivateKey, Type<?>... args) {
        return functionCallWithInfo(funcName, from, fromPrivateKey, contractAddress, args).thenApply(this::getTxHash);
    }

    @Override
    public CompletableFuture<String> transfer(Coin coin, String from, String to, String fromPrivateKey, BigInteger value) {
        if(coin.isToken20()) {
            return transferRC20(from, to, value, fromPrivateKey, ((Token)coin).getContractAddress())
                    .thenApply(this::getTxHash);
        }

        // 检查是否为这个链的本币
        return asyncCheckValueCoin(coin)
                .thenCompose(o -> transferValue(from, to, value, fromPrivateKey, ""))
                .thenApply(this::getTxHash);

    }

    @Override
    public CompletableFuture<BigInteger> getBalance(Coin coin, String address) {
        if(coin.isToken20()) {
           return getBalanceRC20(((Token)coin).getContractAddress(), address);
        }

        // 检查是否为这个链的本币
        return asyncCheckValueCoin(coin)
                .thenCompose(o -> getBalanceValue(address));
    }

    @Override
    public CompletableFuture<BigInteger> getLastBlockNum() {
        return web3j.ethBlockNumber().sendAsync()
                .thenApply(EthBlockNumber::getBlockNumber);
    }

    @Override
    public CompletableFuture<String> getLastBlockHash() {
        return getLastBlockNum()
                .thenCompose(this::asBlockHash);
    }

    @Override
    public CompletableFuture<BigInteger> asBlockNum(String blockHash) {
        return getBlockInfoByHash(blockHash, false).thenApply(EthBlock::getNumber);
    }

    @Override
    public CompletableFuture<String> asBlockHash(BigInteger num) {
        return getBlockInfoByNum(num, false).thenApply(EthBlock::getHash);
    }

    @Override
    public CompletableFuture<EthBlock> getBlockInfoByHash(String blockHash) {
        return getBlockInfoByHash(blockHash, true);
    }

    @Override
    public CompletableFuture<EthBlock> getBlockInfoByNum(BigInteger num) {
        return getBlockInfoByNum(num, true);
    }

    @Override
    public CompletableFuture<EthBlock> getNextBlockInfo(String blockHash) {
        return asBlockNum(blockHash)
                .thenApply(number -> number.add(BigInteger.ONE))
                .thenCompose(this::getBlockInfoByNum);
    }

    @Override
    public CompletableFuture<EthBlock> getPreviousBlockInfo(String blockHash) {
        // 实际上可以通过parentHash来查找上一个区块，但这里为了跟 getNextBlockInfo 对应，就用这种方式把。
        return asBlockNum(blockHash)
                .thenApply(number -> number.subtract(BigInteger.ONE))
                .thenCompose(this::getBlockInfoByNum);
    }

    protected CompletableFuture<EthBlock> getBlockInfoByNum(BigInteger num, boolean full) {
        return web3j.ethGetBlockByNumber(DefaultBlockParameter.valueOf(num), full).sendAsync()
                .thenApply(EthBlock::parseOf);
    }

    protected CompletableFuture<EthBlock> getBlockInfoByHash(String blockHash, boolean full) {
        return web3j.ethGetBlockByHash(blockHash, full).sendAsync()
                .thenApply(EthBlock::parseOf);
    }

    /**
     * 交易本币（gas）
     * @param from  打款地址
     * @param to    收款地址
     * @param value 额度
     * @param privateKey 私钥
     * @param data 备注
     * @return 交易记录
     */
    public CompletableFuture<EthSendTransaction> transferValue(String from, String to, BigInteger value, String privateKey, String data) {
        //获取nonce，交易笔数
        return getNonce(from).thenCompose(nonce -> {
            //加载转账所需的凭证，用私钥
            Credentials credentials = Credentials.create(privateKey);

            //创建RawTransaction交易对象
            RawTransaction rawTransaction = RawTransaction.createTransaction(nonce, gasProvider.getGasPrice(""), gasProvider.getGasLimit(""), to, value, data);
            //签名Transaction，这里要对交易做签名
            byte[] signMessage = TransactionEncoder.signMessage(rawTransaction, credentials);
            String hexValue = Numeric.toHexString(signMessage);
            //发送交易
            return web3j.ethSendRawTransaction(hexValue).sendAsync();
        });
    }

    /**
     * 交易代币（token）
     * @param from   打款地址
     * @param to     收款地址
     * @param value  额度
     * @param privateKey 私钥
     * @param contractAddress 合约地址
     * @return 交易哈希
     */
    public CompletableFuture<EthSendTransaction> transferRC20(String from, String to, BigInteger value, String privateKey, String contractAddress) {
        return functionCallWithInfo("transfer", from, privateKey, contractAddress, new Address(to), new Uint256(value));
    }

    /**
     * 获取nonce，交易笔数
     */
    public CompletableFuture<BigInteger> getNonce(String from) {
        return this.nonceProvider.getNonce(this, from);
    }

    /**
     * 调用方法
     * @param funcName  方法名
     * @param from      调用地址
     * @param privateKey 私钥
     * @param contractAddress 合约地址
     * @param args  参数
     * @return 交易记录
     */
    public CompletableFuture<EthSendTransaction> functionCallWithInfo(String funcName, String from, String privateKey, String contractAddress, Type<?>... args) {
        //获取nonce，交易笔数
        return getNonce(from).thenCompose(nonce -> {
            //加载转账所需的凭证，用私钥
            Credentials credentials = Credentials.create(privateKey);

            //创建RawTransaction交易对象
            Function function = new Function(
                funcName,
                Arrays.asList(args),
                Collections.singletonList(new TypeReference<Type<?>>() {})
            );

            String encodedFunction = FunctionEncoder.encode(function);

            RawTransaction rawTransaction = RawTransaction.createTransaction(nonce,
                    gasProvider.getGasPrice(funcName),
                    gasProvider.getGasLimit(funcName),
                    contractAddress, encodedFunction);


            //签名Transaction，这里要对交易做签名
            byte[] signMessage = TransactionEncoder.signMessage(rawTransaction, credentials);
            String hexValue = Numeric.toHexString(signMessage);
            //发送交易
            return web3j.ethSendRawTransaction(hexValue).sendAsync();
        });

    }

    /**
     * 获取本币额度
     * @param address 地址
     * @return 本币额度
     */
    public CompletableFuture<BigInteger> getBalanceValue(String address) {
        return web3j.ethGetBalance(address, DefaultBlockParameterName.LATEST).sendAsync()
                .thenApply(EthGetBalance::getBalance);
    }

    public Web3j getImpl() {
        return web3j;
    }

    /**
     * 获取本币额度, 实现方式是直接通过调用 balanceOf 方法获取额度
     * @param address 地址
     * @return 本币额度
     */
    public CompletableFuture<BigInteger> getBalanceRC20(String contract, String address) {
        Function function = new Function("balanceOf",
                Collections.singletonList(new Address(address)),
                Collections.singletonList(new TypeReference<Address>() {
                }));

        String encode = FunctionEncoder.encode(function);

        Transaction ethCallTransaction = Transaction.createEthCallTransaction(address, contract, encode);

        return web3j.ethCall(ethCallTransaction, DefaultBlockParameterName.LATEST).sendAsync().thenApply(call -> {
            if(null == call.getResult()) {
                throw new BusinessException(call.getError().getMessage());
            }
            return BlockChainUtils.hexToInt(call.getResult());
        });
    }

    @Override
    public CompletableFuture<EthTransaction> queryTransfer(String txHash) {
        return getImpl().ethGetTransactionByHash(txHash).sendAsync()
                .thenApply(Response::getResult)
                .thenApply(EthTransaction::parseOf);
    }


    /**
     * 查询回执
     * @param txHash 交易哈希
     */
    public CompletableFuture<TransactionReceipt> queryTransferReceipt(String txHash) {
        return getImpl().ethGetTransactionReceipt(txHash).sendAsync()
                .thenApply(Response::getResult);
    }

    protected void checkValueCoin(Coin coin) {
        // 如果Coin被当作本币处理，但是又和链的本币标识不一样，则抛出异常
        if(!coin.getSymbol().equalsIgnoreCase(getValueSymbol())) {
            throw new BusinessException("{0}的本币为{1}而不是{2},我们不知道如何处理这种情况。", getChainName(), getValueSymbol(), coin.getSymbol());
        }
    }

    protected <T> CompletableFuture<T> asyncCheckValueCoin(Coin coin) {
        // 如果Coin被当作本币处理，但是又和链的本币标识不一样，则抛出异常
        if(!coin.getSymbol().equalsIgnoreCase(getValueSymbol())) {
            return FutureKit.error(new BusinessException("{0}的本币为{1}而不是{2},我们不知道如何处理这种情况。", getChainName(), getValueSymbol(), coin.getSymbol()));
        }
        return CompletableFuture.completedFuture(null);
    }

    protected String getTxHash(EthSendTransaction transaction) {
        if(null == transaction.getResult()) {
            throw new BusinessException(transaction.getError().getMessage());
        } else {
            return transaction.getResult();
        }
    }

    public LogWatcher watchLog(BlockRange range, ITopic topic, String... contracts) {
        return watchLog(range, Collections.singletonList(topic), Arrays.asList(contracts), null);
    }

    public LogWatcher watchLog(BlockRange range, ITopic topic, Integer offset, String... contracts) {
        return watchLog(range, Collections.singletonList(topic), Arrays.asList(contracts), offset);
    }

    public LogWatcher watchLog(BlockRange range, List<ITopic> topics, String... contracts) {
        return watchLog(range, topics, Arrays.asList(contracts), null);
    }

    public LogWatcher watchLog(BlockRange range, List<ITopic> topics, List<String> contracts, @Nullable Integer offset) {
        return watchLog(range, topics, contracts, null, offset);
    }

    public LogWatcher watchLog(BlockRange range, List<ITopic> topics, List<String> contracts,
                               @Nullable WatchRetryStrategy<List<Log>> retry, @Nullable Integer offset) {
        return new LogWatcher(this, range, topics, contracts, retry, offset);
    }

    public LogWatcher watchBufferedLog(BlockRange range, List<ITopic> topics, List<String> contracts, Integer buffer,
                                       @Nullable WatchRetryStrategy<List<Log>> retry,
                                       @Nullable Integer offset) {
        return new BufferedLogWatcher(this, range, topics, contracts, retry, offset, buffer);
    }

    public LogWatcher watchBufferedLog(BlockRange range, ITopic topic, Integer buffer, Integer offset, String... contracts) {
        return watchBufferedLog(range, topic, buffer, offset, Arrays.asList(contracts));
    }

    public LogWatcher watchBufferedLog(BlockRange range, ITopic topic, Integer buffer, Integer offset, List<String> contracts) {
        return watchBufferedLog(range, Collections.singletonList(topic), contracts, buffer, null, offset);
    }

    public CompletableFuture<List<Log>> queryLogs(EthFilter filter) {

        return web3j.ethGetLogs(filter).sendAsync()
                .thenCompose(ethLog -> {
                    if (ethLog.hasError()) {
                        log.warn("[Log-Fetch-Error]{} 抓取失败 -> {}, from->{}, to -> {}", getChainName(), ethLog.getError().getMessage(), filter.getFromBlock().getValue(), filter.getToBlock().getValue());
                        return ScheduleKit.scheduleFuture(() -> queryLogs(filter), 10, TimeUnit.SECONDS);
                    } else {
                        List<EthLog.LogResult> logResults = ethLog.getLogs();
                        return CompletableFuture.completedFuture(
                                logResults.stream()
                                        .map(Log::parseOf)
                                        .collect(Collectors.toList())
                        );

                    }
                });
    }

}
