package com.chuang.tauceti.blockchain.payment.model.eth;

import com.chuang.tauceti.blockchain.payment.BlockChainUtils;
import com.chuang.tauceti.blockchain.payment.model.IBlock;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.basic.reflect.ConvertKit;
import lombok.Data;

import java.math.BigInteger;
import java.util.List;
import java.util.stream.Collectors;

@Data
public class EthBlock implements IBlock<EthTransaction> {

    /**
     * 区块哈希
     * [Block.hash -> common.Hash]
     */
    private String hash;

    /**
     * 区块高度
     * [Block.header.Number -> *big.Int]
     */
    public BigInteger number;

    /**
     * 父区块哈希
     * [Block.header.ParentHash -> common.Hash]
     */
    public String parentHash;

    /**
     *  Nonce值
     *  [Block.header.Nonce -> BlockNonce]
     */
    public BigInteger nonce;

    /**
     * 交易 Merkle 树根
     * [Block.header.TxHash -> common.Hash]
     */
    public String transactionsRoot;

    /**
     * 区块产出的时间戳
     * [Block.header.Time -> *big.Int]
     */
    public BigInteger timestamp;

    /** 交易记录
     * [Block.transactions -> *Transaction]
     */
    public List<EthTransaction> transactions;

    /**
     *  叔叔区块的 sha3 哈希 [Block.header.UncleHash -> common.Hash]
     */
    public String sha3Uncles;
    /**
     *  [Block.header.Bloom -> Bloom]
     */
    public String logsBloom;
    /**
     *  状态 Merkle 树根 [Block.header.Root -> common.Hash]
     */
    public String stateRoot;
    /**
     *  收据 Merkle 树根 [Block.header.ReceiptHash -> common.Hash]
     */
    public String receiptsRoot;
    /**
     *
     */
    public String author;
    /**
     *
     *  [Block.header.Coinbase -> common.Address]
     */
    public String miner;
    /**
     *
     *  [Block.header.MixDigest -> common.Hash]
     */
    public String mixHash;
    /**
     *  [Block.header.Difficulty -> *big.Int]
     */
    public BigInteger difficulty;
    /**
     *  [Block.td -> * bigInt]?
     */
    public BigInteger totalDifficulty;
    /**
     *  拓展信息
     *  [Block.header.Extra -> []byte]
     */
    public String extraData;
    /**
     *
     */
    public BigInteger size;
    /**
     *  消耗gas的上限
     *  [Block.header.GasLimit -> unit64]
     */
    public BigInteger gasLimit;
    /**
     *  使用的gas费
     *  [Block.header.GasUsed -> unit64]
     */
    public BigInteger gasUsed;
    /**
     *  叔父区块的哈希
     */
    public List<String> uncles;
    /**
     *
     */
    public List<String> sealFields;
    /**
     *
     */
    public BigInteger baseFeePerGas;


    public static EthBlock parseOf(org.web3j.protocol.core.methods.response.EthBlock ethBlock) {
        org.web3j.protocol.core.methods.response.EthBlock.Block block = unpack(ethBlock);

        EthBlock b = ConvertKit.toBean(block, EthBlock::new,
                "number",
                "nonce",
                "timestamp",
                "difficulty",
                "totalDifficulty",
                "size",
                "gasLimit",
                "gasUsed",
                "baseFeePerGas");

        b.setNumber(BlockChainUtils.rawToInteger(block.getNumberRaw()));
        b.setNonce(BlockChainUtils.rawToInteger(block.getNonceRaw()));
        b.setTimestamp(BlockChainUtils.rawToInteger(block.getTimestampRaw()));
        b.setDifficulty(BlockChainUtils.rawToInteger(block.getDifficultyRaw()));
        b.setTotalDifficulty(BlockChainUtils.rawToInteger(block.getTotalDifficultyRaw()));
        b.setSize(BlockChainUtils.rawToInteger(block.getSizeRaw()));
        b.setGasLimit(BlockChainUtils.rawToInteger(block.getGasLimitRaw()));
        b.setGasUsed(BlockChainUtils.rawToInteger(block.getGasUsedRaw()));
        b.setBaseFeePerGas(BlockChainUtils.rawToInteger(block.getBaseFeePerGasRaw()));


        List<EthTransaction> transactions = block.getTransactions().stream()
                .map(EthTransaction::parseOf)
                .collect(Collectors.toList());

        b.setTransactions(transactions);
        return b;
    }

    protected static org.web3j.protocol.core.methods.response.EthBlock.Block unpack(org.web3j.protocol.core.methods.response.EthBlock ethBlock) {
        if(ethBlock.hasError()) {
            throw new BusinessException(ethBlock.getError().getCode(), ethBlock.getError().getMessage());
        }

        return ethBlock.getBlock();
    }

    @Override
    public String getStorageKey() {
        return getNumber().toString(16);
    }
}
