package com.chuang.tauceti.blockchain.payment.watch;

import com.chuang.tauceti.support.exception.SystemException;
import io.reactivex.internal.subscribers.LambdaSubscriber;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Sinks;

import javax.annotation.Nullable;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Consumer;
import java.util.function.Function;

/**
 * 数据流监听器接口
 * 监听器总是数据流的形式提供对数据的监听。通过 {@link #start()} 或 {@link #watch()} 获取一个热的数据流进行监听。
 * IWatcher 会不断监听数据和产生数据时的异常。直到结束。 需要注意的是 IWatcher 与 Flux不同，IWatcher遇到错误后不一定会停止数据流。
 * 是否停止数据流取决于子类的实现。比如：{@link BlockchainWatcher} 的实现是，根据重试策略来确定数据流是否在遇到错误后停止。
 * @param <T>
 */
public interface IWatcher<T> {

    /**
     * 启动 并获取响应流
     * 需要注意的是返回的 Flux 通过调用subscribe(); 获得Disposable对象，当使用Disposable关闭监听后，并不会影响响应流的工作。
     * 关闭Flux后仅仅只影响Flux自身的监听。响应流会继续工作，其他调用watch()获取的Flux仍然会监听到数据。
     */
    void start();

    /**
     * 获取响应流，如果没有启动，则响应流不会有任何数据发射。
     * 需要注意的是，实现程序必须返回一个 热 的数据流。
     * 需要注意的是返回的 Flux 通过调用subscribe(); 获得Disposable对象，当使用Disposable关闭监听后，并不会影响响应流的工作。
     * 关闭Flux后仅仅只影响Flux自身的监听。响应流会继续工作，其他调用watch()获取的Flux仍然会监听到数据。
     * @deprecated 此方法已经过期，请采用 {@link #watch(Consumer, Consumer, Runnable)} 进行监听。原因是 Flux 发射一个 error 后流会停止，但是IWatcher的接口设计要求发射 数据和异常 都不停止数据流的运行。
     * 原本返回Flux后，开发者可以自行考虑是否要将 error map成正常对象，但考虑到开发者可能因为理解错误，或粗心忘记了做相关操作，而理解上
     */
    @Deprecated
    default Flux<T> watch() {
        throw new SystemException("not support");
    }
    /**
     * 监听数据，
     * 由于监听方式不是 Flux的形式，开发者就无法使用数据流的操作。为此，可以使用 Fluxes.formWatcher(watcher) 来创建一个 Flux 流。
     * @param next 当发射下一个结果时
     * @param error 当发射下一个错误时
     * @param completed 当完成时
     */
    void watch(Consumer<T> next, @Nullable Consumer<Throwable> error, @Nullable Runnable completed);

    default void watch(Consumer<T> next, @Nullable Consumer<Throwable> error) {
        watch(next, error, null);
    }

    default void watch(Consumer<T> next) {
        watch(next, null, null);
    }


    /**
     * 当前区块信息再抓一次。
     */
    void restart();

    /**
     * 停止响应流
     * 调用此方法后所有调用 watch() 获得的Flux都将无法再获取新的数据。
     */
    void shutdown();

    /**
     * 将一种类型的监听器转成另一种类型
     * @param map 转换器
     */
    default <R> IWatcher<R> map(Function<T, R> map) {
        return new WatcherWrapper<>(this, map);
    }

    /**
     * 同时监听多个监听器的内容
     * @param watchers 监听器
     */
    @Deprecated
    static <T> Flux<T> watchAll(IWatcher<T>... watchers) {
        Sinks.Many<T> hotSource = Sinks.many().unicast().onBackpressureBuffer();

        AtomicInteger count = new AtomicInteger(watchers.length);
        for (IWatcher<T> watcher : watchers) {
            watcher.watch().subscribe(new LambdaSubscriber<>(hotSource::tryEmitNext, hotSource::tryEmitError, () -> {
                count.decrementAndGet();
                if(count.get() <= 0) {
                    hotSource.tryEmitComplete();
                }
            }, subscription -> subscription.request(Long.MAX_VALUE)));
        }
        return hotSource.asFlux();
    }

//    static <T> Flux<T> watch(IWatcher<T>... watchers) {
//        AtomicInteger count = new AtomicInteger(watchers.length);
//        return Flux.create(sink -> {
//            for (IWatcher<T> watcher : watchers) {
//                watcher.watch().subscribeWith(new LambdaSubscriber<T>(sink::next, sink::error, () -> {
//                    count.decrementAndGet();
//                    if(count.get() == 0) {
//                        sink.complete();
//                    }
//                }, subscription -> {
//                    subscription.request();
//                }));
//            }
//        });
//    }
}
