package com.chuang.tauceti.blockchain.client.contract;

import com.chuang.tauceti.blockchain.BlockChainUtils;
import com.chuang.tauceti.blockchain.client.contract.ether.EthClient;
import com.chuang.tauceti.blockchain.model.eth.Token;
import org.web3j.abi.TypeReference;
import org.web3j.abi.datatypes.Address;
import org.web3j.abi.datatypes.Bool;
import org.web3j.abi.datatypes.generated.Uint256;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.concurrent.CompletableFuture;

public class Erc20Client {

    private final IContractClient client;

    public Erc20Client(EthClient client) {
        this.client = client;
    }

    /**
     * 获取授权额度
     * @param owner 钱包所有者
     * @param spender 接受授权的合约
     * @return 允许划账的金额
     */
    public CompletableFuture<BigInteger> allowance(String contract, String owner, String spender) {
        Function<BigInteger, Uint256> func = new Function<BigInteger, Uint256>()
                .caller(owner)
                .contract(contract)
                .function("allowance")
                .params(new Address(owner), new Address(spender))
                .reader(TypeReference.create(Uint256.class), BlockChainUtils::hexToInt);
        return client.callReader(func);
    }

    public CompletableFuture<BigDecimal> allowance(Token token, String owner, String spender) {
        return allowance(token.getContractAddress(), owner, spender).thenApply(token::intToDecimal);
    }

    /**
     * 获取代币额度, 实现方式是直接通过调用 balanceOf 方法获取额度
     * @param address 地址
     * @return 本币额度
     */
    public CompletableFuture<BigInteger> balanceOf(String contract, String address) {
        Function<BigInteger, Uint256> func = new Function<BigInteger, Uint256>()
                .caller(address)
                .contract(contract)
                .function("balanceOf")
                .params(new Address(address))
                .reader(TypeReference.create(Uint256.class), BlockChainUtils::hexToInt);
        return client.callReader(func);
    }

    public CompletableFuture<BigDecimal> balanceOf(Token token, String address) {
        return balanceOf(token.getContractAddress(), address).thenApply(token::intToDecimal);
    }


    /**
     * 交易代币（token）
     * @param caller   打款地址
     * @param to     收款地址
     * @param value  额度
     * @param callerPriKey 私钥
     * @param contract 合约地址
     * @return 交易哈希
     */
    public CompletableFuture<String> transfer(String contract, String caller, String to, BigInteger value, String callerPriKey) {
        Function<Boolean, Bool> function = new Function<Boolean, Bool>()
                .contract(contract)
                .caller(caller)
                .function("transfer")
                .params(new Address(to), new Uint256(value))
                .writer(callerPriKey);
        return client.callWriter(function);
    }

    /**
     * 交易代币（token）
     * @param caller   打款地址
     * @param to     收款地址
     * @param value  额度
     * @param callerPriKey 私钥
     * @param token 代币信息
     * @return 交易哈希
     */
    public CompletableFuture<String> transfer(Token token, String caller, String to, BigDecimal value, String callerPriKey) {
        return transfer(token.getContractAddress(), caller, to, token.decimalToInt(value), callerPriKey);
    }

    /**
     * 授权合约可划走的金额
     * @param tokenContract   代币合约
     * @param spenderContract 被授权合约
     * @param caller          调用者
     * @param value           授权金额
     * @param callerPriKey    调用者私钥
     * @return 交易哈希
     */
    public CompletableFuture<String> approve(String tokenContract, String spenderContract, String caller, BigInteger value, String callerPriKey) {
        Function<Boolean, Bool> function = new Function<Boolean, Bool>()
                .contract(tokenContract)
                .caller(caller)
                .function("approve")
                .params(new Address(spenderContract), new Uint256(value))
                .writer(callerPriKey);
        return client.callWriter(function);
    }

}
