package com.chuang.tauceti.blockchain.client.contract.ether;

import com.chuang.tauceti.blockchain.BlockChainConst;
import com.chuang.tauceti.blockchain.client.IBlockChainClient;
import com.chuang.tauceti.blockchain.client.contract.Function;
import com.chuang.tauceti.blockchain.client.contract.IContractClient;
import com.chuang.tauceti.blockchain.model.Wallet;
import com.chuang.tauceti.blockchain.model.eth.EthBlock;
import com.chuang.tauceti.blockchain.model.eth.EthTransaction;
import com.chuang.tauceti.blockchain.model.eth.Log;
import com.chuang.tauceti.blockchain.watch.BlockRange;
import com.chuang.tauceti.blockchain.watch.eth.BufferedLogWatcher;
import com.chuang.tauceti.blockchain.watch.eth.ITopic;
import com.chuang.tauceti.blockchain.watch.eth.LogWatcher;
import com.chuang.tauceti.blockchain.watch.strategy.WatchRetryStrategy;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.basic.ScheduleKit;
import com.chuang.tauceti.tools.basic.StringKit;
import lombok.extern.slf4j.Slf4j;
import org.web3j.abi.FunctionEncoder;
import org.web3j.abi.TypeReference;
import org.web3j.abi.datatypes.Type;
import org.web3j.crypto.*;
import org.web3j.protocol.Web3j;
import org.web3j.protocol.Web3jService;
import org.web3j.protocol.core.DefaultBlockParameter;
import org.web3j.protocol.core.DefaultBlockParameterName;
import org.web3j.protocol.core.Response;
import org.web3j.protocol.core.methods.request.EthFilter;
import org.web3j.protocol.core.methods.request.Transaction;
import org.web3j.protocol.core.methods.response.*;
import org.web3j.protocol.http.HttpService;
import org.web3j.protocol.websocket.WebSocketService;
import org.web3j.utils.Numeric;

import javax.annotation.Nullable;
import java.math.BigInteger;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import static com.chuang.tauceti.blockchain.BlockChainConst.HEX_PRE;


/**
 * 以太坊客户端， 所有以太坊侧链都可以使用它
 */
@Slf4j
public abstract class EthClient implements IBlockChainClient<EthBlock, EthTransaction>, IContractClient {

    private final Web3j web3j;
    protected final GasProvider gasProvider;
    protected final NonceProvider nonceProvider;

    public EthClient(Web3j web3j, GasProvider gasProvider, @Nullable NonceProvider nonceProvider) {
        this.web3j = web3j;
        this.gasProvider = gasProvider;
        if(null == nonceProvider) {
            this.nonceProvider = new NonceProvider() {};
        } else {
            this.nonceProvider = nonceProvider;
        }
    }

    public EthClient(Web3jService service, GasProvider gasProvider, @Nullable NonceProvider nonceProvider) {
        this(Web3j.build(service), gasProvider, nonceProvider);
    }

    public EthClient(String chainSeed, GasProvider gasProvider, @Nullable NonceProvider nonceProvider) {
        this(create(chainSeed), gasProvider, nonceProvider);
    }

    private static Web3j create(String chainSeed) {
        if (chainSeed.toLowerCase().startsWith(BlockChainConst.HTTP.toLowerCase())) {
            return Web3j.build(new HttpService(chainSeed));
        } else {
            return Web3j.build(new WebSocketService(chainSeed, true));
        }
    }


    @Override
    public Wallet createWallet(@Nullable String password) {
        password = StringKit.nullToEmpty(password);
        try {
            ECKeyPair ecKeyPair = Keys.createEcKeyPair();
            String privateKey = ecKeyPair.getPrivateKey().toString(16);
            String publicKey =  ecKeyPair.getPublicKey().toString(16);

            WalletFile wallet = org.web3j.crypto.Wallet.createLight(password, ecKeyPair);
            String address = wallet.getAddress().toLowerCase();
            if(!address.toLowerCase().startsWith(HEX_PRE)) {
                address = HEX_PRE + address;
            }

            return new Wallet().setAddress(address)
                    .setEcKeyPair(ecKeyPair)
                    .setPrivateKey(privateKey)
                    .setPublicKey(publicKey)
                    .setPassword(password);
        } catch (Exception e) {
            throw new BusinessException("创建钱包失败", e);
        }


    }


    @Override
    public <T, R extends Type<T>> CompletableFuture<T> callReader(Function<T, R> function) {
        org.web3j.abi.datatypes.Function f = new org.web3j.abi.datatypes.Function(
                function.getFunction(),
                function.getParams(),
                null == function.getResult() ? Collections.singletonList(function.getResult()) : Collections.emptyList()
        );
        String encode = FunctionEncoder.encode(f);
        Transaction ethCallTransaction = Transaction.createEthCallTransaction(function.getCaller(), function.getContract(), encode);

        return web3j.ethCall(ethCallTransaction, DefaultBlockParameterName.LATEST).sendAsync().thenApply(call -> {
            if(null == call.getResult()) {
                throw new BusinessException(call.getError().getMessage());
            }

            return function.getResultMap().apply(call.getResult());
        });
    }

    @Override
    public <T, R extends Type<T>> CompletableFuture<String> callWriter(Function<T, R> function) {
        return getNonce(function.getCaller()).thenCompose(nonce -> {

            //加载转账所需的凭证，用私钥
            Credentials credentials = Credentials.create(function.getPrivateKey());

            // 构建function
            org.web3j.abi.datatypes.Function f = new org.web3j.abi.datatypes.Function(
                    function.getFunction(),
                    function.getParams(),
                    Collections.singletonList(new TypeReference<Type<?>>() {})
            );
            String encode = FunctionEncoder.encode(f);

            // 签名的transaction
            RawTransaction rawTransaction = RawTransaction.createTransaction(nonce,
                    gasProvider.getGasPrice(function.getFunction()),
                    gasProvider.getGasLimit(function.getFunction()),
                    function.getContract(), encode);

            //签名Transaction，这里要对交易做签名
            byte[] signMessage = TransactionEncoder.signMessage(rawTransaction, credentials);
            String hexValue = Numeric.toHexString(signMessage);
            //发送交易
            return web3j.ethSendRawTransaction(hexValue).sendAsync().thenApply(this::getTxHash);
        });

    }


    @Override
    public CompletableFuture<String> transfer(String from, String to, String fromPrivateKey, BigInteger value, String data) {
        return getNonce(from).thenCompose(nonce -> {
                    //加载转账所需的凭证，用私钥
                    Credentials credentials = Credentials.create(fromPrivateKey);

                    //创建RawTransaction交易对象
                    RawTransaction rawTransaction = RawTransaction.createTransaction(nonce, gasProvider.getGasPrice(""), gasProvider.getGasLimit(""), to, value, data);
                    //签名Transaction，这里要对交易做签名
                    byte[] signMessage = TransactionEncoder.signMessage(rawTransaction, credentials);
                    String hexValue = Numeric.toHexString(signMessage);
                    //发送交易
                    return web3j.ethSendRawTransaction(hexValue).sendAsync();
                })
                .thenApply(this::getTxHash);

    }

    @Override
    public CompletableFuture<BigInteger> getBalance(String address) {
        return web3j.ethGetBalance(address, DefaultBlockParameterName.LATEST).sendAsync()
                .thenApply(EthGetBalance::getBalance);
    }

    @Override
    public CompletableFuture<BigInteger> getLastBlockNum() {
        return web3j.ethBlockNumber().sendAsync()
                .thenApply(EthBlockNumber::getBlockNumber);
    }

    @Override
    public CompletableFuture<String> getLastBlockHash() {
        return getLastBlockNum()
                .thenCompose(this::asBlockHash);
    }

    @Override
    public CompletableFuture<BigInteger> asBlockNum(String blockHash) {
        return getBlockInfoByHash(blockHash, false).thenApply(EthBlock::getNumber);
    }

    @Override
    public CompletableFuture<String> asBlockHash(BigInteger num) {
        return getBlockInfoByNum(num, false).thenApply(EthBlock::getHash);
    }

    @Override
    public CompletableFuture<EthBlock> getBlockInfoByHash(String blockHash) {
        return getBlockInfoByHash(blockHash, true);
    }

    @Override
    public CompletableFuture<EthBlock> getBlockInfoByNum(BigInteger num) {
        return getBlockInfoByNum(num, true);
    }

    @Override
    public CompletableFuture<EthBlock> getNextBlockInfo(String blockHash) {
        return asBlockNum(blockHash)
                .thenApply(number -> number.add(BigInteger.ONE))
                .thenCompose(this::getBlockInfoByNum);
    }

    @Override
    public CompletableFuture<EthBlock> getPreviousBlockInfo(String blockHash) {
        // 实际上可以通过parentHash来查找上一个区块，但这里为了跟 getNextBlockInfo 对应，就用这种方式把。
        return asBlockNum(blockHash)
                .thenApply(number -> number.subtract(BigInteger.ONE))
                .thenCompose(this::getBlockInfoByNum);
    }

    protected CompletableFuture<EthBlock> getBlockInfoByNum(BigInteger num, boolean full) {
        return web3j.ethGetBlockByNumber(DefaultBlockParameter.valueOf(num), full).sendAsync()
                .thenApply(EthBlock::parseOf);
    }

    protected CompletableFuture<EthBlock> getBlockInfoByHash(String blockHash, boolean full) {
        return web3j.ethGetBlockByHash(blockHash, full).sendAsync()
                .thenApply(EthBlock::parseOf);
    }


    /**
     * 获取nonce，交易笔数
     */
    public CompletableFuture<BigInteger> getNonce(String from) {
        return this.nonceProvider.getNonce(this, from);
    }


    public Web3j getImpl() {
        return web3j;
    }


    @Override
    public CompletableFuture<EthTransaction> queryTransfer(String txHash) {
        return getImpl().ethGetTransactionByHash(txHash).sendAsync()
                .thenApply(Response::getResult)
                .thenApply(EthTransaction::parseOf);
    }

    /**
     * 查询回执
     * @param txHash 交易哈希
     */
    public CompletableFuture<TransactionReceipt> queryTransferReceipt(String txHash) {
        return getImpl().ethGetTransactionReceipt(txHash).sendAsync()
                .thenApply(Response::getResult);
    }

    protected String getTxHash(EthSendTransaction transaction) {
        if(null == transaction.getResult()) {
            throw new BusinessException(transaction.getError().getMessage());
        } else {
            return transaction.getResult();
        }
    }

    public LogWatcher watchLog(BlockRange range, ITopic topic, String... contracts) {
        return watchLog(range, Collections.singletonList(topic), Arrays.asList(contracts), null);
    }

    public LogWatcher watchLog(BlockRange range, ITopic topic, Integer offset, String... contracts) {
        return watchLog(range, Collections.singletonList(topic), Arrays.asList(contracts), offset);
    }

    public LogWatcher watchLog(BlockRange range, List<ITopic> topics, String... contracts) {
        return watchLog(range, topics, Arrays.asList(contracts), null);
    }

    public LogWatcher watchLog(BlockRange range, List<ITopic> topics, List<String> contracts, @Nullable Integer offset) {
        return watchLog(range, topics, contracts, null, offset);
    }

    public LogWatcher watchLog(BlockRange range, List<ITopic> topics, List<String> contracts,
                               @Nullable WatchRetryStrategy<List<Log>> retry, @Nullable Integer offset) {
        return new LogWatcher(this, range, topics, contracts, retry, offset);
    }

    public LogWatcher watchBufferedLog(BlockRange range, List<ITopic> topics, List<String> contracts, Integer buffer,
                                       @Nullable WatchRetryStrategy<List<Log>> retry,
                                       @Nullable Integer offset) {
        return new BufferedLogWatcher(this, range, topics, contracts, retry, offset, buffer);
    }

    public LogWatcher watchBufferedLog(BlockRange range, ITopic topic, Integer buffer, Integer offset, String... contracts) {
        return watchBufferedLog(range, topic, buffer, offset, Arrays.asList(contracts));
    }

    public LogWatcher watchBufferedLog(BlockRange range, ITopic topic, Integer buffer, Integer offset, List<String> contracts) {
        return watchBufferedLog(range, Collections.singletonList(topic), contracts, buffer, null, offset);
    }

    public CompletableFuture<List<Log>> queryLogs(EthFilter filter) {

        return web3j.ethGetLogs(filter).sendAsync()
                .thenCompose(ethLog -> {
                    if (ethLog.hasError()) {
                        log.warn("[Log-Fetch-Error]{} 抓取失败 -> {}, from->{}, to -> {}", getChainName(), ethLog.getError().getMessage(), filter.getFromBlock().getValue(), filter.getToBlock().getValue());
                        return ScheduleKit.scheduleFuture(() -> queryLogs(filter), 10, TimeUnit.SECONDS);
                    } else {
                        List<EthLog.LogResult> logResults = ethLog.getLogs();
                        return CompletableFuture.completedFuture(
                                logResults.stream()
                                        .map(Log::parseOf)
                                        .collect(Collectors.toList())
                        );

                    }
                });
    }

}
