package com.chuang.tauceti.blockchain.watch;

import com.chuang.tauceti.blockchain.BlockChainUtils;
import com.chuang.tauceti.blockchain.client.IBlockChainClient;
import com.chuang.tauceti.support.exception.SystemException;
import com.chuang.tauceti.tools.basic.FutureKit;
import com.chuang.tauceti.tools.basic.RetryStrategy;
import lombok.extern.slf4j.Slf4j;
import org.web3j.protocol.core.DefaultBlockParameter;
import org.web3j.protocol.core.DefaultBlockParameterName;
import reactor.core.publisher.Flux;

import javax.annotation.Nullable;
import java.math.BigInteger;
import java.util.Iterator;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicLong;
import java.util.function.Consumer;
import java.util.function.Predicate;
import java.util.function.Supplier;

/**
 * 区块范围
 * 用来表示区块高度 from - to 的范围，包含 from 和 to.
 * BlockRange 实现了 Iterable<BigInteger> 接口，可以被遍历.
 * 如果 from <= to, 则遍历时产生的 BigInteger 是由小到大，每次 +1.
 * 如果 from > to, 则遍历时产生的 BigInteger 是由大到小，每次 -1
 *
 * BlockRange 可以是一个由小到大的无界区间，to是无穷大。
 *
 * 需要注意的是 BlockRange 除了要表示范围外，其遍历的过程中，产生的 BigInteger 不能超过链上最高高度。
 * 假设区间是 0 - 10000， 而链上高度是 8000. 那么遍历时 0-8000的产生速度取决于CPU的计算速度，是非常快的，但是到8000以后，需要跟链上同步，因此8000以后的数字产生会很慢。
 * 这么做的目的是为监听做基础的服务，避免遍历到 8001 - 10000时，链上区块还没产生，抓取不到区块。
 * 由于以上这种问题的存在，不能把BlockRange看作一个单纯的迭代器。只有范围小于链上最大高度时才与一般迭代器一样。
 *
 * 关于 BlockRange 的异步迭代器内部由 AscIteratorImpl 维护。需要注意的是，该迭代器可以如果直接迭代，会将异步转同步，会阻塞线程。
 * 如果希望使用异步策略，可以调用 {@link AscIteratorImpl#nextFlux()} 获取一个Flux流，其内部是异步生成数据流。且内部数据流的数据不会主动产生，必须由背压确定request数量才会根据数据进行数据生产。
 * 这样做的原因是 BlockRange 可能会是一个无限大的区间。这可能导致在生产的数字远远小于 链上最新区块高度时， 其生产速度远大于消费速度，消费端可能会主动采取背压策略。而因为生产端的速度和规模可能太大，如果主动生产数据，
 * 则数据会进入缓冲区等待消费端的 request。最终可能会导致生产端的缓存被撑爆。因此这里采取的生产端策略是惰性生产，消费端需要多少就生产多少。
 * 目前对于惰性策略的实现有一点问题，但来不急改了。原因描述如下：
 * 当BlockRange是一个较大，且范围在最大区块内。则内部的一部迭代器就变成了一个单纯的数字累加，其产生数字的速度极快，如果消费端的消费速度也很快的话。可能导致下面的异常。这个异常来源于虚拟机。无法被捕获。开发者会发现 dump 线程没有死锁，但莫名其妙的整个迭代任务就停止了。
 * *** java.lang.instrument ASSERTION FAILED ***: "!errorOutstanding" with message transform method call failed at JPLISAgent.c line: 844
 * 导致这个问题的原因是内部异步产生数据的方式是通过递归迭代的，如果消费端消费很快，那么生产端理论上也是会不断迭代的。当递归层次太多就会报这个错。现在之所以没解决，一来是没时间，二来是因为消费端的消费速度没那么快，迭代会因为消费端阻塞而退出。等待消费端下一次请求数据时再进行新一轮迭代。
 *
 * 接上面的，我有点怕，还是决定把这个问题解决一下。现在，迭代太深的问题已经解决了。解决方式是：每次异步递归都走 thenApplySync 让其在新新线程中执行，这样栈就不会被撑爆了。
 *
 */
@Slf4j
public class BlockRange implements Iterable<BigInteger> {

    private final DefaultBlockParameter fromBlock, toBlock;
    private final IBlockChainClient<?, ?> client;
    private final RetryStrategy<BigInteger> retryStrategy;

    public static final RetryStrategy<BigInteger> DEFAULT_RETRY_STRATEGY = new RetryStrategy<>(Integer.MAX_VALUE, 1, TimeUnit.MINUTES, (o, throwable) -> {
        if(null == throwable) {
            return false;
        } else {
            log.error("get the last block number exception", throwable);
            return true;
        }
    });

    private BlockRange(IBlockChainClient<?, ?> client, DefaultBlockParameter fromBlock, DefaultBlockParameter toBlock,
                       @Nullable RetryStrategy<BigInteger> retryStrategy) {
        if(fromBlock == DefaultBlockParameterName.LATEST) {
            throw new SystemException("fromBlock arg can not set LATEST");
        }
        this.client = client;
        this.fromBlock = fromBlock;
        this.toBlock = toBlock;
        this.retryStrategy = null == retryStrategy ? DEFAULT_RETRY_STRATEGY : retryStrategy;

    }

    public static void each(BigInteger from, BigInteger to, Consumer<BigInteger> each) {
        if(from.compareTo(to) <= 0) {
            while (from.compareTo(to) <= 0) {
                each.accept(from);
                from = from.add(BigInteger.ONE);
            }
        } else {
            while (from.compareTo(to) >= 0) {
                each.accept(from);
                from = from.subtract(BigInteger.ONE);
            }
        }
    }

    public static boolean between(BigInteger from, BigInteger to, BigInteger value) {
        if(from.compareTo(to) <= 0) {
            return from.compareTo(value) <= 0 && value.compareTo(to) <= 0;
        } else {
            return from.compareTo(value) >= 0 && value.compareTo(to) >= 0;
        }
    }


    @Override
    public Iterator<BigInteger> iterator() {
        BigInteger from = BlockChainUtils.hexToInt(fromBlock.getValue());
        BigInteger to = DefaultBlockParameterName.LATEST == toBlock ? null : BlockChainUtils.hexToInt(toBlock.getValue());

        if(isAsc()) {
            return new AscIteratorImpl(from, client, retryStrategy, to);
        } else {
            return new DescIteratorImpl(from, to);
        }
    }

    public boolean isAsc() {
        if (DefaultBlockParameterName.LATEST == toBlock) {
            return true;
        } else {
            BigInteger from = BlockChainUtils.hexToInt(fromBlock.getValue());
            BigInteger to = BlockChainUtils.hexToInt(toBlock.getValue());

            return from.compareTo(to) <= 0;
        }
    }

    public DefaultBlockParameter getFromBlock() {
        return fromBlock;
    }

    public DefaultBlockParameter getToBlock() {
        return toBlock;
    }

    public BlockRange offset(int offset) {
        DefaultBlockParameter from = BlockChainUtils.add(this.fromBlock, offset);
        DefaultBlockParameter to = BlockChainUtils.add(this.toBlock, offset);

        return new BlockRange(client, from, to, this.retryStrategy);
    }

    public static BlockRange create(IBlockChainClient<?, ?> client, BigInteger fromBlock,
                                    @Nullable BigInteger toBlock, @Nullable RetryStrategy<BigInteger> retry) {
        DefaultBlockParameter to = null == toBlock ? DefaultBlockParameterName.LATEST : DefaultBlockParameter.valueOf(toBlock);
        return new BlockRange(client, DefaultBlockParameter.valueOf(fromBlock), to, retry);
    }

    public static BlockRange create(IBlockChainClient<?, ?> client, BigInteger fromBlock,
                                    @Nullable BigInteger toBlock) {
        return create(client, fromBlock, toBlock, null);
    }

    public static BlockRange create(IBlockChainClient<?, ?> client, BigInteger fromBlock) {
        return create(client, fromBlock, null, null);
    }



    public static class AscIteratorImpl implements Iterator<BigInteger> {

        private final IBlockChainClient<?, ?> client;
        private final RetryStrategy<BigInteger> retryStrategy;
        private final BigInteger to;

        private BigInteger current;

        private BigInteger last;

        public AscIteratorImpl(BigInteger fromBlock, IBlockChainClient<?, ?> client, RetryStrategy<BigInteger> retryStrategy, @Nullable BigInteger to) {
            this.client = client;
            this.retryStrategy = retryStrategy;
            this.current = fromBlock.subtract(BigInteger.ONE);
            this.to = to;
        }

        @Override
        public boolean hasNext() {
            if(null == to) {
                return true;
            } else {
                return current.compareTo(to) < 0;
            }
        }

        @Override
        public BigInteger next() {
            return asyncNext().join();
        }

        private CompletableFuture<BigInteger> asyncNext() {
            return asyncLast().thenApply(last -> {
                this.current = current.add(BigInteger.ONE);
                return current;
            });
        }

        public Flux<BigInteger> nextFlux() {
            return Flux.create(emitter -> {
                AtomicLong request = new AtomicLong(0);
                AtomicBoolean completed = new AtomicBoolean(false);
                Supplier<Boolean> check = () -> hasNext() && !emitter.isCancelled() && !completed.get();

                Predicate<BigInteger> emitAndCheck = n -> {
                    System.out.println("1 ======================> next " + n + " current " + current);
                    request.decrementAndGet();
                    emitter.next(n);
                    if(check.get()) {
                        return true;
                    } else {
                        emitter.complete();
                        completed.set(true);
                        return false;
                    }
                };

                emitter.onRequest(value -> {
                    // 检查是否还继续执行
                    if(check.get()) {
                        request.addAndGet(value);
                        getNext(emitAndCheck);
                    } else {
                        emitter.complete();
                        completed.set(true);
                    }
                });

            });
        }

        public Flux<BigInteger> oneByOneBackPressureFlux() {
            return Flux.create(emitter -> {
                AtomicBoolean completed = new AtomicBoolean(false);
                emitter.onRequest(value -> {
                    // 检查是否还继续执行
                    if(hasNext() && !emitter.isCancelled() && !completed.get()) {
                        asyncNext().thenAccept(emitter::next);
                    } else {
                        emitter.complete();
                        completed.set(true);
                    }
                });

            });
        }

        private void getNext(Predicate<BigInteger> emitAndCheck) {
            asyncNext().thenAcceptAsync(number -> {
                if(emitAndCheck.test(number)) {
                    getNext(emitAndCheck);
                } else {

                }
            });
        }

        private CompletableFuture<BigInteger> asyncLast() {
            return asyncLast0().thenCompose(last -> {
                this.last = last;
                if(current.compareTo(last) >= 0) {
                    return FutureKit.delay(this::asyncLast, 10, TimeUnit.SECONDS);
                } else {
                    return CompletableFuture.completedFuture(last);
                }
            });
        }

        private CompletableFuture<BigInteger> asyncLast0() {
            return FutureKit.retry(() -> {
                if(null == last || current.compareTo(last) >= 0) {
                    return client.getLastBlockNum();
                } else {
                    return CompletableFuture.completedFuture(this.last);
                }
            }, retryStrategy.getTimes(), retryStrategy.getDelay(), retryStrategy.getTimeUnit(), retryStrategy.getTest());
        }

        public BigInteger cacheLast() {
            return last;
        }


    }

    public static class DescIteratorImpl implements Iterator<BigInteger> {

        private final BigInteger to;

        private BigInteger current;

        public DescIteratorImpl(BigInteger fromBlock, @Nullable BigInteger to) {

            this.current = fromBlock.add(BigInteger.ONE);
            this.to = to;
        }

        @Override
        public boolean hasNext() {
            return current.compareTo(to) > 0;
        }

        @Override
        public BigInteger next() {
            current = current.subtract(BigInteger.ONE);
            return current;
        }
    }
}



