package com.chuang.tauceti.blockchain.watch;

import com.chuang.tauceti.blockchain.Fluxes;
import com.chuang.tauceti.blockchain.watch.strategy.WatchRetryStrategy;
import com.chuang.tauceti.support.exception.SystemException;
import com.chuang.tauceti.tools.basic.FutureKit;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.reactivestreams.Subscription;
import reactor.core.Disposable;
import reactor.core.publisher.BaseSubscriber;
import reactor.core.publisher.Flux;

import javax.annotation.Nullable;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.function.Consumer;

/**
 * 区块链监听器
 * BlockchainWatcher 实现 {@link IWatcher}, 因此它也是基于对数据流的订阅实现的监听。
 * BlockchainWatcher 不负责数据抓取的工作，这部分工作讲由实现者完成。通过实现 {@link #fetch(BigInteger)} 方法实现数据从 BigInteger 到 T 的转换。可以把 fetch 方法看作是数据流的 mapping 谓词。只是这个 mapping 谓词是异步的。
 * 而BlockchainWatcher的主要工作就是将异步的 mapping 谓词添加到数据流中。为了实现并理解这一点，开发者需要注意一下几点：
 * 1. 区块链监听器的数据流基于 BlockRange(区块高度) 数据流, 在此之前需要先了解 {@link BlockRange}
 * 2. 由于BlockRange可能是一个无限的 BigInteger 数据流。导致 BlockchainWatcher 基于的 BigInteger数据流可能发射的很快，也可能发射的很慢。一般来说一个有限，且范围不超过最高区块高度的数据流发射数据很快。
 * 3. 虽然 BlockRange 数据流产生的 BigInteger(区块高度）流是有序的，但考虑到 {@link #fetch(BigInteger)} 是一个异步的谓词，先发射的 BigInteger 处理后的数据可能比 后发射的先到。导致最终顺序是混乱的。
 * 4、 为了解决异步谓词导致的顺序混乱问题，BlockchainWatcher 内部采取了异步背压的方式来保障谓词数据后的结果是有序的，异步背压使得处理异步谓词时是一个接一个的执行。关于异步背压更详细的说明可以查看{@link #fetchRange()}方法。
 *
 * 什么时候会停止数据流监听：
 * 1. range 流结束
 * 2. range 流报错, range流会有自己的重试机制，重试期间的错误不停止range数据流。
 * 子类实现的fetch接口返回的  Flux 流报错不会中断监听。
 * @param <T>
 */
@Slf4j
public abstract class BlockchainWatcher<T, C> implements IWatcher<T> {

    protected final BlockRange range;


    protected final WatchRetryStrategy<C> retryStrategy;
    private final Listeners<T> listeners = new Listeners<>();
    private final List<Consumer<BigInteger>> blockNumberChange = new ArrayList<>();
    private final BigInteger offset;
    private final Iterator<BigInteger> rangeIterator;
    private Disposable disposable;

    private volatile boolean logNumber;
    public BlockchainWatcher(BlockRange range) {
        this(range, null, 0);
    }

    public BlockchainWatcher(BlockRange range, int offset) {
        this(range, null, offset);
    }

    public BlockchainWatcher(BlockRange range, WatchRetryStrategy<C> retryStrategy, Integer offset) {
        if(null == offset) {
            offset = 0;
        }
        this.offset = BigInteger.valueOf(offset);
        this.range = range.offset(offset);
        this.rangeIterator = this.range.iterator();
        if(null == retryStrategy) {
            retryStrategy =  WatchRetryStrategy.errRetryFailRestart(0, 1, TimeUnit.MINUTES);
        }
        this.retryStrategy = retryStrategy;
    }

    public void logNumber(boolean logNumber) {
        this.logNumber = logNumber;
    }

    @Override
    public void start() {
        stopFlow();
        fetchRange();
    }

    protected BigInteger iteratorLast() {
        if(this.rangeIterator instanceof BlockRange.AscIteratorImpl) {
            return ((BlockRange.AscIteratorImpl) this.rangeIterator).cacheLast().subtract(offset);
        } else {
            throw new SystemException("desc range not support iterator last");
        }
    }

    @Override
    public void watch(Consumer<T> next, @Nullable Consumer<Throwable> error, @Nullable Runnable completed) {
        if(null == error) {
            error = throwable -> {};
        }
        if(null == completed) {
            completed = () -> {};
        }
        this.listeners.add(new Listener<>(next, error, completed));
    }

    public void onBlockNumberChange(Consumer<BigInteger> listener) {
        blockNumberChange.add(listener);
    }

    protected void fireBlockNumberChange(BigInteger number) {
        blockNumberChange.forEach(l -> {
            try {
                l.accept(number);
            } catch (Exception e) {
                log.warn("block number listen error", e);
            }
        });
    }

    @Override
    public void restart() {
        start();
    }

    @Override
    public void shutdown() {
        done(true);
    }

    /**
     * 把 BlockRange 的数据流映射成 T 数据流。
     * 由于转换数据流的 谓词 是异步操作，同时为了保障异步操作后数据仍然是有序的，方法内部采取了异步背压的方式来处理谓词。
     * 背压使得数据从 BigInteger -> 异步<T> 时，是一个接一个执行的. 换句话说，只有一个BigInteger得到异步结果后，才会拉取数据流中下一个 BigInteger进行谓词处理，保证了异步结果的有序。
     * 这样处理的后果是:
     * 所有BigInteger处理前都必须等待上一个 BigInteger 的结果返回才能执行。区块链的数据抓取是通过 http json rpc 方式进行. 因此数据流的处理是一个请求接一个请求的串行执行来保证有序。
     * 这样请求速度可能比较慢。如果开发者希望多个BigInteger请求能够并发处理，且对顺序无感，可以重写这个方法。重写代码如下：
     *
     * return Fluxes.range(range).flatMap(this::fetch);
     * 这个不会去考虑和控制同时并发的量，这里推荐实现者在 {@link #fetch(BigInteger)} 做并发限制，通常这些限制是交给 httpclient 来限制的。没必要在 {@link  #fetchRange()}方法中做过于复杂的处理。
     *
     * 实际上通过对异步结果进行缓存 -> 重排 -> 重新发射 的方式可以实现数据流的 并发 + 有序。但考虑到实现的复杂度较高，且区块链的监听串行已经足够满足需要也就没有去实现。这里将描述实现 并发 + 有序的难点：
     * 1. 并发将遇到 `return Fluxes.range(range).flatMap(this::fetch);` 这种实现一样的问题。当然，推荐的解决方式也和它一样。
     * 2. 缓存 进入 重排 时，如果中间有一个数据迟迟未到时需要考虑应对策略。
     *    策略1. 超时报错，并停止发射数据，缓存数据直接丢掉。调用者需要从错误的区块开始重启监听。
     *    策略2. 超时报错，发射错误数据后，继续发射下一个数据。调用者需要额外重新查询错误区块的数据。
     *    策略3. 不断重试，直到获取数据。调用者不需要做其他事情。但可能因为迟迟无法成功获取数据，从而导致缓冲区溢出。
     * 3. 缓冲 到 重排 ，重排 到 发射的触发时机需要把握好，否则可能出现用户监听不到的情况。
     * 4. 数据流调用 complete 的时机，不再和 BlackRange complete的时机一致，需要实现者自己把握好时机。
     * @deprecated 不符合 IWatcher 对发射错误后不中断流的要求。实现改为 {@link #fetchRange()}
     */
    @Deprecated
    protected Flux<T> fetchRangeFlux() {
        /*
         这种方式太过粗暴，range虽然按顺序产生了大量的number，但因为fetch是异步的，后发起的查询可能先到，最终会导致 Flux 的内容是乱序的
         且这种方式可能会因为短时间内产生了大量number导致瞬间发起多次请求，对网络造成负担。
         */
        // return Fluxes.range(range).flatMap(this::fetch);

        return Flux.create(sink -> {
            Fluxes.range(range).subscribe(new BaseSubscriber<BigInteger>() {
                volatile boolean complete = false;
                @Override
                protected void hookOnSubscribe(Subscription subscription) {
                    subscription.request(1);
                }

                @Override
                protected void hookOnNext(BigInteger number) {
                    BigInteger value = number.subtract(offset);
                    fireBlockNumberChange(value);
                    BlockchainWatcher.this.query(value).subscribe(sink::next, this::onError, () -> {
                        if(complete) {
                            sink.complete();
                        } else {
                            upstream().request(1);
                        }
                    });
                }

                @Override
                protected void hookOnComplete() {
                    complete = true;
                }

                @Override
                protected void hookOnError(Throwable throwable) {
                    sink.error(throwable);
                }

            });
        });
    }

    protected void fetchRange() {
        BlockchainWatcher<T, C> watcher = BlockchainWatcher.this;
        Fluxes.backPressureIterator(rangeIterator).subscribe(new BaseSubscriber<BigInteger>() {
            final AtomicBoolean complete = new AtomicBoolean(false);
            @Override
            protected void hookOnSubscribe(Subscription subscription) {
                subscription.request(1);
            }

            @Override
            protected void hookOnNext(BigInteger number) {
                BigInteger value = number.subtract(offset);
                fireBlockNumberChange(value);
                if(logNumber) {
                    log.info("[{}] -> number -> {}, offset -> {}, fetch -> {}", watchName(), number, offset, value);
                }

                watcher.query(value).onErrorContinue((throwable, o) -> {
                    // 这里是为了确保子类实现返回的 Flux 不会中断流的监听。
                    watcher.whenError(throwable);
                }).subscribe(watcher::whenNext, watcher::whenError, () -> {
                    if(!complete.get()) {
                        upstream().request(1);
                    }
                });
            }

            @Override
            protected void hookOnComplete() {
                watcher.done(false);
                complete.set(true);
            }

            @Override
            protected void hookOnError(Throwable throwable) {
                watcher.whenError(throwable);
            }

        });
    }

    private Flux<T> query(BigInteger number) {
        return asFlux(fetchWithRetry(number));
    }

    protected abstract Flux<T> asFlux(CompletableFuture<C> future);

    private CompletableFuture<C> fetchWithRetry(BigInteger number) {
        return FutureKit.retry(() -> fetch(number), this.retryStrategy)
                .exceptionally(throwable -> this.retryStrategy.getRetryFail().apply(this, throwable));
//        return fetch(number).exceptionally(throwable -> this.retryStrategy.getRetryFail().apply(this, throwable));
    }


    protected abstract CompletableFuture<C> fetch(BigInteger number);

    public abstract String watchName();


    protected void whenError(Throwable throwable) {
        log.warn(throwable.getMessage());
        listeners.emitError(throwable);
    }

    protected void whenNext(T t) {
        listeners.emitNext(t);
    }


    /**
     * 停止抓取
     * @param manual 是否为手动停止
     */
    protected void done(boolean manual) {
        if(manual) {
            log.warn("{} Watcher is shutdown manual", this.watchName());
            listeners.emitError(new WatchShutdownException("Watcher is shutdown manual"));
        }
        this.stopFlow();
        listeners.emitCompleted();
        this.blockNumberChange.clear();
        this.listeners.clear();
    }

    private void stopFlow() {
        try {
            if (null != disposable && !disposable.isDisposed()) {
                disposable.dispose();
                disposable = null;
            }
        } catch (Exception ignore) {}
    }

    @Getter
    @AllArgsConstructor
    private static class Listener<T> {
        private Consumer<T> next;
        private Consumer<Throwable> error;
        private Runnable completed;
    }

    private static class Listeners<T> {
        private final List<Listener<T>> listeners = new ArrayList<>();

        public void add(Listener<T> listener) {
            listeners.add(listener);
        }

        public void remove(Listener<T> listener) {
            listeners.remove(listener);
        }

        public void emitNext(T next) {
            listeners.forEach(l -> {
                try {
                    l.getNext().accept(next);
                } catch (Exception e) {
                    log.warn("emit next error", e);
                }
            });
        }

        public void emitError(Throwable throwable) {
            listeners.forEach(l -> {
                try {
                    l.getError().accept(throwable);
                } catch (Exception e) {
                    log.warn("emit throwable error", e);
                }
            });
        }

        public void emitCompleted() {

            listeners.forEach(l -> {
                try {
                    l.getCompleted().run();
                } catch (Exception e) {
                    log.warn("emit completed error", e);
                }

            });
        }

        public void clear() {
            listeners.clear();
        }
    }
}


//    @Override
//    public void onSubscribe(Subscription subscription) {
//        this.subscription = subscription;
//        this.subscription.request(1);
//    }
//
//    @Override
//    public void onNext(BigInteger bigInteger) {
//        AbstractWatcher.this.fetch(bigInteger).subscribe(t -> {
//            sink.next(t);
//            subscription.request(1);
//        }, this::onError);
//    }
//
//    @Override
//    public void onError(Throwable throwable) {
//        sink.error(throwable);
//    }
//
//    @Override
//    public void onComplete() {
//        sink.complete();
//    }
