package com.chuang.tauceti.httpclient.async;

import com.chuang.tauceti.httpclient.Request;
import com.chuang.tauceti.httpclient.Response;
import com.chuang.tauceti.httpclient.Tools;
import com.chuang.tauceti.support.CompletableFutureWrapper;
import org.apache.http.HttpResponse;
import org.apache.http.annotation.Contract;
import org.apache.http.annotation.ThreadingBehavior;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.HttpRequestBase;
import org.apache.http.concurrent.FutureCallback;
import org.apache.http.impl.nio.client.CloseableHttpAsyncClient;
import org.apache.http.protocol.HttpContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.concurrent.ThreadSafe;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Future;
import java.util.function.Consumer;


/**
 * Created by ath on 2017/1/6.
 */
@ThreadSafe
@Contract(threading = ThreadingBehavior.IMMUTABLE_CONDITIONAL)
public class AsyncHttpClientImpl implements IAsyncHttpClient {
    private final String defaultCharset;
    private final CloseableHttpAsyncClient asyncHttpClient;
    private final RequestConfig defaultConfig;
    private final List<Consumer<HttpRequestBase>> requestHold;
    private final List<Consumer<Response>> responseHold;
    private static final Logger logger = LoggerFactory.getLogger(AsyncHttpClientImpl.class);


    public AsyncHttpClientImpl(final String defaultCharset,
                               final CloseableHttpAsyncClient asyncHttpClient,
                               final RequestConfig defaultConfig,
                               final List<Consumer<HttpRequestBase>> requestHold,
                               final List<Consumer<Response>> responseHold) {
        this.defaultCharset = defaultCharset;
        this.asyncHttpClient = asyncHttpClient;
        this.defaultConfig = defaultConfig;
        this.requestHold = requestHold;
        this.responseHold = responseHold;
    }

    @Override
    public AsyncHttpClientImpl init() {
        try {
            start();
            Runtime.getRuntime().addShutdownHook(new Thread(() -> {
                logger.info("开始关闭异步Http客户端");
                AsyncHttpClientImpl.this.shutdown();
                logger.info("异步Http客户端 成功 关闭");
            }));
        } catch (Throwable e) {
            logger.error("无法开启异步http客户端", e);
        }
        return this;
    }

    @Override
    public void start() {
        asyncHttpClient.start();
    }

    @Override
    public void shutdown() {
        try {
            asyncHttpClient.close();
        } catch (Exception e) {
            logger.error("无法关闭异步http客户端，请检查相关问题，避免内存泄露", e);
        }
    }


    @Override
    public CompletableFuture<String> doGet(String url, Map<String, String> params) {
        return doGet(url, params, defaultCharset);
    }

    @Override
    public CompletableFuture<String> doPost(String url, Map<String, String> params) {
        return doPost(url, params, defaultCharset);
    }


    @Override
    public CompletableFuture<Response> exec(Request request, HttpContext context) {
        CompletableFutureWrapper<Response> future = new CompletableFutureWrapper<>();

        RequestConfig config = request.getConfig().cover(defaultConfig);


        HttpRequestBase requestBase = request.get();

        requestBase.setConfig(config);

        Map<String, String> headers = request.getHeaders();
        for(String key : headers.keySet()) {
            requestBase.addHeader(key, headers.get(key));
        }


        String charset = request.getCharset();
        if(Tools.isBlank(charset)) {
            charset = this.defaultCharset;
        }



        try {
           Tools.handEntity(request, charset);
        } catch (IOException e) {
            future.completeExceptionally(e);
        }

        final String finalCharset = charset;

        FutureCallback<HttpResponse> fc = new FutureCallback<HttpResponse>() {
            @Override
            public void completed(HttpResponse httpResponse) {
                synchronized (future) {
                    Response response = new Response(requestBase, httpResponse, finalCharset);
                    future.complete(response);
                }
            }
            @Override
            public void failed(Exception e) {
                logger.debug("tauceti request 失败", e);
                future.completeExceptionally(new IOException(requestBase.toString() + "失败", e));
            }

            @Override
            public void cancelled() {
                logger.debug("tauceti request 取消");
                future.cancel(true);
            }
        };

        this.requestHold.forEach(hold -> hold.accept(requestBase));

        Future<HttpResponse> f;
        if(null == context) {
            f = asyncHttpClient.execute(requestBase, fc);
        } else {
            f = asyncHttpClient.execute(requestBase, context, fc);
        }
        future.setCancelHandler(f::cancel);
        return future.thenApply(response -> {
            responseHold.forEach(hold -> hold.accept(response));
            return response;
        });
    }

}
