package com.chuang.tauceti.httpclient.async;

import com.chuang.tauceti.httpclient.HttpMethod;
import com.chuang.tauceti.httpclient.Request;
import com.chuang.tauceti.httpclient.Response;
import org.apache.http.HttpHost;
import org.apache.http.protocol.HttpContext;

import java.util.Collections;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;

public interface IAsyncHttpClient {

    IAsyncHttpClient init();

    void start();
    void shutdown();

    default CompletableFuture<String> doGet(String url) {
        return doGet(url, Collections.emptyMap());
    }

    default CompletableFuture<String> doPost(String url) {
        return doPost(url, Collections.emptyMap());
    }

    CompletableFuture<String> doGet(String url, Map<String, String> params);

    CompletableFuture<String> doPost(String url, Map<String, String> params);

    default CompletableFuture<String> doPost(String url, Map<String, String> params, String charset) {
        return doPost(url, params, null, null, charset, null).thenApply(Response::asString);
    }

    default CompletableFuture<String> doGet(String url, Map<String, String> params, String charset) {
        return doGet(url, params, null, null,charset, null, -1, -1).thenApply(Response::asString);
    }

    default Optional<Request> asRequest(String url,
                                        HttpMethod method,
                                        Map<String, String> params,
                                        String body,
                                        Map<String, String> heads,
                                        String charset,
                                        HttpHost proxy,
                                        Integer connTimeout,
                                        Integer readTimeout) {
        if(url == null || method == null) {
            return Optional.empty();
        }

        Request.Builder request = Request.newBuilder().url(url).method(method);
        Request.MyCovertConfigBuilder config = request.config();

        if (null != params && !params.isEmpty()) {
            request.parameter(params);
        }
        if (null != body && !body.isEmpty()) {
            request.body(body);
        }
        if (null != heads && !heads.isEmpty()) {
            request.header(heads);
        }
        if (null != charset && !charset.isEmpty()) {
            request.charset(charset);
        }

        if (null != proxy) {
            config.setProxy(proxy);
        }
        if (null != connTimeout) {
            config.setConnectTimeout(connTimeout)
                    .setConnectionRequestTimeout(connTimeout);
        }
        if (null != readTimeout) {
            config.setSocketTimeout(readTimeout);
        }


        return Optional.of(request.build());

    }

    /**
     * HTTP Get 获取内容
     *
     * @param url     请求的url地址 ?之前的地址 不能为空
     * @param params  请求的参数 允许为空
     * @param charset 编码格式  允许为空，若为空取httpclient配置中的默认字符编码
     * @param proxy 代理地址，允许为空，若为空，则不适用代理
     * @return 页面内容
     */
    default CompletableFuture<Response> doGet(String url, Map<String, String> params, Map<String, String> heads, HttpContext context, String charset, HttpHost proxy, int connTimeout, int readTimeout) {
        return asRequest(url, HttpMethod.GET, params, null, heads, charset, proxy, connTimeout, readTimeout)
                .map(request -> exec(request, context))
                .orElse(CompletableFuture.completedFuture(null));
    }


    /**
     * HTTP post 获取内容
     *
     * @param url     请求的url地址 ?之前的地址 不能为空
     * @param params  请求的参数 允许为空
     * @param charset 编码格式  允许为空，若为空取httpclient配置中的默认字符编码
     * @param proxy 代理地址，允许为空，若为空，则不适用代理
     * @return 页面内容
     */
    default CompletableFuture<Response> doPost(String url, Map<String, String> params, HttpContext context, Map<String, String> heads, String charset, HttpHost proxy) {

        return asRequest(url, HttpMethod.POST, params, null, heads, charset, proxy, null, null)
                .map(request -> exec(request, context))
                .orElse(CompletableFuture.completedFuture(null));
    }



    /**
     * HTTP Post 获取内容
     *
     * @param url     请求的url地址 ?之前的地址
     * @param requestData  请求体字符串
     * @param charset 编码格式
     * @return 页面内容
     */
    default CompletableFuture<Response> doPost(String url, String requestData, HttpContext context, Map<String, String> heads, String charset, HttpHost proxy) {

        return asRequest(url, HttpMethod.POST, null, requestData, heads, charset, proxy, null, null)
                .map(request -> exec(request, context))
                .orElse(CompletableFuture.completedFuture(null));
    }


    /**
     * HTTP put 获取内容
     *
     * @param url     请求的url地址 ?之前的地址
     * @param requestData  请求体字符串
     * @param charset 编码格式
     * @return 页面内容
     */
    default CompletableFuture<Response> doPut(String url, String requestData, HttpContext context, Map<String,String> heads, String charset, HttpHost proxy) {
        return asRequest(url, HttpMethod.POST, null, requestData, heads, charset, proxy, null, null)
                .map(request -> exec(request, context))
                .orElse(CompletableFuture.completedFuture(null));
    }

    /**
     * HTTP put 获取内容
     *
     * @param url     请求的url地址 ?之前的地址
     * @param params  请求体
     * @param charset 编码格式
     * @return 页面内容
     */
    default CompletableFuture<Response> doPut(String url, Map<String, String> params, HttpContext context, Map<String,String> heads, String charset, HttpHost proxy) {

        return asRequest(url, HttpMethod.PUT, params, null, heads, charset, proxy, null, null)
                .map(request -> exec(request, context))
                .orElse(CompletableFuture.completedFuture(null));
    }


    CompletableFuture<Response> exec(Request request, HttpContext context);
}
