package com.chuang.tauceti.httpclient.sync;

import com.chuang.tauceti.httpclient.Request;
import com.chuang.tauceti.httpclient.Response;
import com.chuang.tauceti.httpclient.Tools;
import org.apache.http.annotation.Contract;
import org.apache.http.annotation.ThreadingBehavior;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpRequestBase;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.protocol.HttpContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.concurrent.ThreadSafe;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.function.Consumer;


/**
 * 一个简易,线程安全的httpclient。 只要职责是提供http客户端配置。如 http 和 ssl 构建，并提供一些简单的通用方法。具体实现一来apache httpclient。
 *
 * 该类是线程安全的，允许多线程访问单例。
 * 示例：
 * <code>
 *      SimpleHttpClient client = SimpleHttpClient.http().build();
 *      Map<String, String> headers = new HashMap();
 *      headers.put(key1, val1);
 *      headers.put(key2, val2);
 *      Map<String, String> params = new HashMap();
 *      params.put(key1, val1);
 *      params.put(key2, val2);
 *
 *      String response = client.doGet(url, params, headers, "UTF-8", null);
 *
 * </code>
 * 以上代码可能过于繁琐。参数含义不够清晰，而且重载方法太多，不容易选择。可以考虑使用HttpRequest
 * <code>
 *      HttpClient client = HttpClient.http().build();
 *      String response = HttpRequest.newBuilder().url(url)
 *          .header(headkey1, headval1)
 *          .header(headkey2, headval2)
 *          .parameter(pk1, pv1)
 *          .parameter(pk2, pv2)
 *          .method("GET")
 *          .charsetUTF()
 *          .build()
 *          .execute(client);
 *
 * </code>
 *
 * Created by ath on 2016/8/18.
 */
@ThreadSafe
@Contract(threading = ThreadingBehavior.IMMUTABLE_CONDITIONAL)
public class HttpClientImpl implements IHttpClient {

    public HttpClientImpl(String defaultCharset,
                          CloseableHttpClient httpClient,
                          RequestConfig defaultConfig,
                          List<Consumer<HttpRequestBase>> requestHold,
                          List<Consumer<Response>> responseHold) {
        this.defaultCharset = defaultCharset;
        this.httpClient = httpClient;
        this.defaultConfig = defaultConfig;
        this.requestHold = requestHold;
        this.responseHold = responseHold;
    }

    private final String defaultCharset;
    private final CloseableHttpClient httpClient;
    private final RequestConfig defaultConfig;
    private final List<Consumer<HttpRequestBase>> requestHold;
    private final List<Consumer<Response>> responseHold;

    private static final Logger logger = LoggerFactory.getLogger(HttpClientImpl.class);


    @Override
    public String doGet(String url, Map<String, String> params) throws IOException {
        return doGet(url, params, defaultCharset);
    }

    @Override
    public String doPost(String url, Map<String, String> params) throws IOException {
        return doPost(url, params, defaultCharset);
    }


    /**
     * 执行请求
     */
    @Override
    public Response exec(Request request, HttpContext context) throws IOException {

        HttpRequestBase requestBase = request.get();
        requestBase.setConfig(request.getConfig().cover(defaultConfig));

        String charset = request.getCharset();
        if(Tools.isBlank(charset)) {
            charset = this.defaultCharset;
        }


        Map<String, String> headers = request.getHeaders();
        for(String key : headers.keySet()) {
            requestBase.addHeader(key, headers.get(key));
        }


        this.requestHold.forEach(hold -> hold.accept(requestBase));

        Tools.handEntity(request, charset);

        try {
            CloseableHttpResponse response;
            if(null == context) {
                response = httpClient.execute(requestBase);
            } else {
                response = httpClient.execute(requestBase, context);
            }
            Response res = new Response(requestBase, response, charset);
            this.responseHold.forEach(hold -> hold.accept(res));
            return res;
        } catch (IOException e) {
            logger.error("http 请求失败", e);
            throw e;
        }
    }
}
