package com.chuang.tauceti.httpclient.sync;

import com.chuang.tauceti.httpclient.Request;
import com.chuang.tauceti.httpclient.Response;
import com.chuang.tauceti.httpclient.Tools;
import org.apache.http.HttpHost;
import org.apache.http.protocol.HttpContext;

import java.io.IOException;
import java.util.Map;

public interface IHttpClient {

    default String doGet(String url) throws IOException {
        return doGet(url, null);
    }

    default String doPost(String url) throws IOException {
        return doPost(url, null);
    }

    String doGet(String url, Map<String, String> params) throws IOException;

    String doPost(String url, Map<String, String> params) throws IOException;

    default String doPost(String url, Map<String, String> params, String charset)  throws IOException {
        return doPost(url, params, null, charset, null).asString();
    }

    default String doGet(String url, Map<String, String> params, String charset) throws IOException {
        return doGet(url, params, null,charset, null, -1, -1).asString() ;
    }

    /**
     * HTTP Get 获取内容
     *
     * @param url     请求的url地址 ?之前的地址 不能为空
     * @param params  请求的参数 允许为空
     * @param charset 编码格式  允许为空，若为空取httpclient配置中的默认字符编码
     * @param proxy 代理地址，允许为空，若为空，则不适用代理
     * @return 页面内容
     */
    default Response doGet(String url, Map<String, String> params, Map<String, String> heads, String charset, HttpHost proxy, int connTimeout, int readTimeout) throws IOException {

        if (Tools.isBlank(url)) {
            return null;
        }

        return exec(Request.Get(url).parameter(params).header(heads).charset(charset).config().setProxy(proxy).setConnectTimeout(connTimeout).setSocketTimeout(readTimeout).done().build(), null);

    }


    /**
     * HTTP post 获取内容
     *
     * @param url     请求的url地址 ?之前的地址 不能为空
     * @param params  请求的参数 允许为空
     * @param charset 编码格式  允许为空，若为空取httpclient配置中的默认字符编码
     * @param proxy 代理地址，允许为空，若为空，则不适用代理
     * @return 页面内容
     */
    default Response doPost(String url, Map<String, String> params, Map<String, String> heads, String charset, HttpHost proxy)  throws IOException {

        if (Tools.isBlank(url)) {
            return null;
        }

        return exec(Request.Post(url).parameter(params).header(heads).charset(charset).config().setProxy(proxy).done().build(), null);
    }



    /**
     * HTTP Post 获取内容
     *
     * @param url     请求的url地址 ?之前的地址
     * @param requestData  请求体字符串
     * @param charset 编码格式
     * @return 页面内容
     */
    default Response doPost(String url, String requestData, Map<String, String> heads, String charset, HttpHost proxy)  throws IOException {

        if (Tools.isBlank(url)) {
            return null;
        }
        return exec(Request.Post(url).body(requestData).header(heads).charset(charset).config().setProxy(proxy).done().build(), null);
    }


    /**
     * HTTP put 获取内容
     *
     * @param url     请求的url地址 ?之前的地址
     * @param requestData  请求体字符串
     * @param charset 编码格式
     * @return 页面内容
     */
    default Response doPut(String url, String requestData, Map<String,String> heads, String charset, HttpHost proxy)  throws IOException {

        if (Tools.isBlank(url)) {
            return null;
        }
        return exec(Request.Put(url).body(requestData).header(heads).charset(charset).config().setProxy(proxy).done().build(), null);
    }

    /**
     * HTTP put 获取内容
     *
     * @param url     请求的url地址 ?之前的地址
     * @param params  请求体
     * @param charset 编码格式
     * @return 页面内容
     */
    default Response doPut(String url, Map<String, String> params, Map<String,String> heads, String charset, HttpHost proxy)  throws IOException {

        if (Tools.isBlank(url)) {
            return null;
        }
        return exec(Request.Put(url).parameter(params).header(heads).charset(charset).config().setProxy(proxy).done().build(), null);
    }


    /**
     * 执行请求
     */
    Response exec(Request request, HttpContext context) throws IOException;
}
