package com.chuang.tauceti.rbac2.crud.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.toolkit.SqlHelper;
import com.chuang.tauceti.rbac2.crud.entity.api.TreeModel;
import com.chuang.tauceti.rowquery.IRowQueryService;
import com.chuang.tauceti.support.BiValue;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.basic.tree.IRelation;
import com.chuang.tauceti.tools.basic.tree.Node;
import com.chuang.tauceti.tools.basic.tree.NodeBuilder;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * 树结构服务
 * @author ath
 */
public interface ITreeService<T extends TreeModel> extends IRowQueryService<T> {
    /** 默认排序 */
    int DEFAULT_RANK = 100;

    int ROOT_ID = 0;

    /**
     * 删除节点及其所有子节点
     * @param id 节点id
     */
    @Transactional(rollbackFor = Exception.class)
    default boolean deleteTree(Integer id) {
        T del = findById(id).orElseThrow(() -> new BusinessException("节点{}不存在", id));
        String parents = del.getParents() + del.getId() + "/";

        removeById(id);
        return remove(
                Wrappers.lambdaUpdate(getEntityClass()).likeRight(T::getParents, parents)
        );
    }

    /**
     * 将 fromId 节点 移动到 toId 节点的后面
     * @param fromId 被移动的节点
     * @param toId 目标节点
     * @return 是否移动成功
     */
    @Transactional(rollbackFor = Exception.class)
    default boolean moveAfter(int fromId, int toId) {
        T to = findById(toId).orElseThrow(() -> new BusinessException("节点{}不存在", toId));
        T from = findById(fromId).orElseThrow(() -> new BusinessException("节点{}不存在", fromId));

        from.setParentId(to.getParentId());
        from.setParents(to.getParents());
        from.setSortRank(to.getSortRank() + 1);

        List<T> greatList = list(
                Wrappers.lambdaQuery(getEntityClass())
                        .eq(T::getParentId, to.getParentId())
                        .gt(T::getSortRank, to.getSortRank())
        );

        if(!greatList.isEmpty()) {
            greatList.forEach(t -> t.setSortRank(t.getSortRank() + 1));
            updateBatchById(greatList);
        }

        List<T> all = refreshChildrenParents(from);
        return updateBatchById(all);
    }

    /**
     * 将 fromId 节点 移动到 toId 节点的前面
     * @param fromId 被移动的节点
     * @param toId 目标节点
     * @return 是否移动成功
     */
    @Transactional(rollbackFor = Exception.class)
    default boolean moveBefore(int fromId, int toId) {
        T to = findById(toId).orElseThrow(() -> new BusinessException("节点{}不存在", toId));
        T from = findById(fromId).orElseThrow(() -> new BusinessException("节点{}不存在", fromId));

        from.setParentId(to.getParentId());
        from.setParents(to.getParents());
        from.setSortRank(to.getSortRank() - 1);


        List<T> lessList = list(
                Wrappers.lambdaQuery(getEntityClass())
                        .eq(T::getParentId, to.getParentId())
                        .lt(T::getSortRank, to.getSortRank())
        );

        if(!lessList.isEmpty()) {
            lessList.forEach(t -> t.setSortRank(t.getSortRank() - 1));
            updateBatchById(lessList);
        }


        List<T> all = refreshChildrenParents(from);
        return updateBatchById(all);
    }

    /**
     * 将 fromId 节点 移动到 toId 节点的里面
     * @param fromId 被移动的节点
     * @param toId 目标节点
     * @return 是否移动成功
     */
    default boolean moveInner(int fromId, int toId) {
        T to = findById(toId).orElseThrow(() -> new BusinessException("节点{}不存在", toId));
        T from = findById(fromId).orElseThrow(() -> new BusinessException("节点{}不存在", fromId));

        from.setParentId(to.getId());
        from.setParents(to.getParents() + to.getId() + "/");

        // 获取 to 内部最后一个菜单的rank值，并对其加1，如果没有，则直接去默认rank
        // 放在最后

        int rank = this.limitOne(Wrappers.lambdaQuery(getEntityClass())
                .select(T::getSortRank)
                .eq(T::getParentId, to.getId())
                .orderByDesc(T::getSortRank))
                .map(T::getSortRank)
                .map(i -> ++i)
                .orElse(DEFAULT_RANK);
        from.setSortRank(rank);

//        updateById(from);

        List<T> children = refreshChildrenParents(from);

        return updateBatchById(children);
    }

    default List<T> refreshChildrenParents(int parentId) {
        T parent = findById(parentId).orElseThrow(() -> new BusinessException("节点{}不存在", parentId));
        return refreshChildrenParents(parent);
    }

    default List<T> refreshChildrenParents(T parent) {
        List<T> updateAll = new ArrayList<>();
        updateAll.add(parent);

        List<T> children = list(
                Wrappers.lambdaQuery(getEntityClass()).eq(T::getParentId, parent.getId())
        );

        children.forEach(t -> {
            t.setParents(parent.getParents() + parent.getId() + "/");
            updateAll.addAll(refreshChildrenParents(t));
        });

        return updateAll;
    }

    /**
     * 将 fromId 节点 向 toId 节点移动
     * @param fromId 被移动的节点
     * @param toId 目标节点
     * @param pos 移动方式， 大于0表示移动到后面，小于0表示移动到前面，等于1表示移动到里面
     * @return 是否移动成功
     */
    default boolean move(int fromId, int toId, int pos) {
        if (pos < 0) {
            // 移动到前面
            return moveBefore(fromId, toId);
        } else if (pos > 0) {
            // 移动到后面
            return moveAfter(fromId, toId);
        } else {
            // 移动到里面
            return moveInner(fromId, toId);
        }
    }

    /**
     * 保存一个树对象
     * @param tree 树对象
     * @return 是否存储成功
     */
    @Override
    default boolean save(T tree) {

        String parentPath;
        if(tree.getParentId() == 0) {
            // 表示是根目录
            parentPath = ROOT_ID + "/";
        } else {
            T parent = findById(tree.getParentId())
                    .orElseThrow(() -> new BusinessException("父级节点无法找到"));
            parentPath = parent.getParents() + parent.getId() + "/";
        }

        tree.setParents(parentPath);

        if(Optional.ofNullable(tree.getId()).map(id -> 0 == id).orElse(true)) {
            //是否不存在id
            return SqlHelper.retBool(getBaseMapper().insert(tree));
        } else {
            return updateById(tree);
        }
    }

    /**
     * 将一条记录和其子节点转成 json 格式返回。
     * @param id 节点id
     */
    default String asJson(Integer id) {
        T root = findById(id).orElseThrow(() -> new BusinessException("节点{}不存在", id));

        String parentPath = root.getParents() + id + "/";



//        LambdaMeta meta = LambdaUtils.extract(T::getParents);
//        String fieldName = PropertyNamer.methodToProperty(meta.getImplMethodName());
//
//        List<T> children = new QueryChainWrapper<>(getBaseMapper())
//                .likeLeft(fieldName, parentPath)
//                .list();



        List<T> children = list(
                Wrappers.lambdaQuery(getEntityClass())
                        .likeRight(T::getParents, parentPath)
        );

        children.add(root);

        List<Node<T>> tree = new NodeBuilder<Integer, T>().relation(relation()).toNode(children);

        return asJson(tree);
    }

    /**
     * 将根节点转换成json返回
     */
    default String asJson() {
        List<Node<T>> tree = new NodeBuilder<Integer, T>().relation(relation()).toNode(list());
        return asJson(tree);
    }

    /**
     * 将节点集，转成json返回
     */
    default String asJson(List<Node<T>> tree) {
        if(tree.size() == 1) {
            return toJson(tree.get(0)).toJSONString();
        } else {
            return toJSONArray(tree).toJSONString();
        }
    }

    /**
     * 将json格式的菜单加入到 指定id 的节点下
     * @param id 父节点id
     * @param json 菜单json
     */
    default void fromJson(Integer id, String json) {
        T root = findById(id).orElseThrow(() -> new BusinessException("节点{}不存在", id));
        Object obj = JSONObject.parse(json);
        saveJsonMenu(obj, root.getId(), root.getParents() + id + "/");
    }

    /**
     * 将json格式的菜单加入到 根节点下
     * @param json 菜单json
     */
    default void fromJson(String json) {
        Object obj = JSON.parse(json);
        saveJsonMenu(obj, ROOT_ID, ROOT_ID + "/");
    }


    /**
     * 将json菜单保存到新的节点下
     * @param json  菜单json
     * @param newParentId 新父节点
     * @param newParentPath 新父节点路径
     */
    @Transactional(rollbackFor = Exception.class)
    default void saveJsonMenu(Object json, Integer newParentId, String newParentPath) {

        if(null == json) {
            return;
        }

        if(json instanceof JSONArray) {
            JSONArray array = (JSONArray) json;
            for (int i = 0; i < array.size(); i++) {
                saveJsonMenu(array.getJSONObject(i), newParentId, newParentPath);
            }

        } else {
            JSONObject obj = (JSONObject) json;
            JSONArray children = obj.getJSONArray("children");
            obj.remove("children");

            T t = JSONObject.toJavaObject(obj, getEntityClass());
            t.setId(null);
            t.setParentId(newParentId);
            t.setParents(newParentPath);
            save(t);
            saveJsonMenu(children, t.getId(), t.getParents() + t.getId() + "/");
        }

    }


    static <T extends Serializable> JSONObject toJson(Node<T> node) {
        T source = node.getSource();
        List<Node<T>> children = node.getChildren();
        JSONObject json = JSONObject.parseObject(JSONObject.toJSONString(source));
        json.put("children", toJSONArray(children));
        return json;

    }

    static <T extends Serializable> JSONArray toJSONArray(List<Node<T>> list) {
        JSONArray ary = new JSONArray();
        for(Node<T> node: list) {
            ary.add(toJson(node));
        }

        return ary;
    }

    default IRelation<Integer, T> relation() {
        return t -> {
            Integer pid = t.getParentId();
            if(null == pid || pid == 0 || pid.equals(t.getId())) {
                pid = null;
            }
            return new BiValue<>(t.getId(), pid);
        };
    }



}
