package com.chuang.tauceti.rowquery;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.support.exception.SystemException;
import com.chuang.tauceti.support.rowquery.RowQuery;
import com.chuang.tauceti.support.rowquery.filters.DateFilter;
import com.chuang.tauceti.support.rowquery.filters.NumberFilter;
import com.chuang.tauceti.support.rowquery.filters.SetFilter;
import com.chuang.tauceti.support.rowquery.filters.TextFilter;
import com.chuang.tauceti.tools.basic.reflect.ConvertKit;
import lombok.extern.slf4j.Slf4j;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @author ATH
 */
@Slf4j
public class FilterWrappers {
    private static final String EQ = "equals";
    private static final String NOT_EQ = "notequal";
    private static final String STARS_WITH = "startswith";
    private static final String ENDS_WITH = "endswith";
    private static final String CONTAINS = "contains";
    private static final String NOT_CONTAINS = "notcontains";

    private static final String IN = "in";
    private static final String NOT_IN = "notin";

    private static final String LESS_THEN = "lessthan";
    private static final String LESS_THEN_OR_EQ = "lessthanorequal";

    private static final String GREATER_THAN = "greaterthan";
    private static final String GREATER_THAN_OR_EQ = "greaterthanorequal";
    private static final String IN_RANGE = "inrange";


    public static void handWrapper(QueryWrapper<?> criteria, RowQuery.Filter filter, Class<?> clazz) {
        if (filter instanceof DateFilter) {
            buildDateFilter(criteria, (DateFilter) filter);
        } else if (filter instanceof TextFilter) {
            buildTextFilter(criteria, (TextFilter) filter);
        } else if (filter instanceof NumberFilter) {
            buildNumberFilter(criteria, (NumberFilter) filter);
        } else if (filter instanceof SetFilter) {
            buildSetFilter(criteria, (SetFilter) filter, clazz);
        } else {
            throw new SystemException("无法解析 [{}] filter", filter.getClass().getName());
        }
    }

    public static void buildTextFilter(QueryWrapper<?> criteria, TextFilter filter) {
        String field = StringUtils.camelToUnderline(filter.getField());
        String value = filter.getValue();
        String option = filter.getOption().toLowerCase();
        switch (option) {
            case EQ:
                criteria.eq(field, value);
                break;
            case NOT_EQ:
                criteria.ne(field, value);
                break;
            case STARS_WITH:
                criteria.likeRight(field, value);
                break;
            case ENDS_WITH:
                criteria.likeLeft(field, value);
                break;
            case CONTAINS:
                criteria.like(field, value);
                break;
            case NOT_CONTAINS:
                criteria.notLike(field, value);
                break;
            default:
                break;
        }
    }

    public static void buildSetFilter(QueryWrapper<?> criteria, SetFilter filter, Class<?> entityClass) {
        String field = StringUtils.camelToUnderline(filter.getField());
        String[] value = filter.getValue();
        String option = filter.getOption().toLowerCase();

        if(value.length == 0) {
            throw new BusinessException("{}至少要选一项进行查询。否则查询无意义。", field);
        }

        List<Object> list = new ArrayList<>();
        try {
            Class<?> fieldType = entityClass.getDeclaredField(filter.getField()).getType();
            if(fieldType == String.class) {
                list =  Arrays.asList(value);
            } else if(Enum.class.isAssignableFrom(fieldType)) {
                for (String v: value) {
                    Enum<?> e =  Enum.valueOf((Class<Enum>)fieldType, v);
                    list.add(e);
                }
            } else {
                for(String v: value) {
                    list.add(ConvertKit.parseBasic(fieldType, v));
                }
            }

        } catch (NoSuchFieldException e) {
            log.error("set filter error, 这里字符串方式", e);
            list =  Arrays.asList(value);
        }

        if(IN.equals(option)) {
            criteria.in(field, list);
        } else if(NOT_IN.equals(option)) {
            criteria.notIn(field, list);
        }
    }

    public static void buildNumberFilter(QueryWrapper<?> criteria, NumberFilter filter) {
        String field = StringUtils.camelToUnderline(filter.getField());
        BigDecimal value = filter.getValue();
        BigDecimal valueTo = filter.getValueTo();
        String option = filter.getOption().toLowerCase();
        switch (option) {
            case EQ:
                criteria.eq(field, value);
                break;
            case NOT_EQ:
                criteria.ne(field, value);
                break;
            case LESS_THEN:
                criteria.lt(field, value);
                break;
            case LESS_THEN_OR_EQ:
                criteria.le(field, value);
                break;
            case GREATER_THAN:
                criteria.gt(field, value);
                break;
            case GREATER_THAN_OR_EQ:
                criteria.ge(field, value);
                break;
            case IN_RANGE:
                criteria.between(field, value, valueTo);
                break;
            default:
                break;
        }
    }
    public static void buildDateFilter(QueryWrapper<?> criteria, DateFilter filter) {
        String field = StringUtils.camelToUnderline(filter.getField());
        LocalDateTime value = LocalDateTime.parse(filter.getValue(), DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        LocalDateTime valueTo = LocalDateTime.parse(filter.getValueTo(), DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        switch (filter.getOption().toLowerCase()) {
            case EQ:
                criteria.eq(field, value);
                break;
            case GREATER_THAN:
                criteria.gt(field, value);
                break;
            case GREATER_THAN_OR_EQ:
                criteria.ge(field, value);
                break;
            case LESS_THEN:
                criteria.lt(field, value);
                break;
            case LESS_THEN_OR_EQ:
                criteria.le(field, value);
                break;
            case NOT_EQ:
                criteria.ne(field, value);
                break;
            case IN_RANGE:
                criteria.between(field, value, valueTo);
                break;
            default:
                break;
        }
    }
}
