package com.chuang.tauceti.rowquery;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.TableInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfoHelper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.support.SFunction;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.chuang.tauceti.rowquery.exception.OptimisticLockerException;
import com.chuang.tauceti.support.rowquery.RowQuery;
import com.chuang.tauceti.tools.basic.collection.CollectionKit;
import com.chuang.tauceti.tools.basic.reflect.BeanKit;

import java.io.Serializable;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

public interface IRowQueryService<T> extends IService<T> {

    /**
     * @see #findById(Serializable)
     */
    @Override
    @Deprecated
    default T getById(Serializable id) {
        return getBaseMapper().selectById(id);
    }

    /**
     * @see #findOne(Wrapper)
     */
    @Override
    @Deprecated
    default T getOne(Wrapper<T> queryWrapper) {
        return getOne(queryWrapper, true);
    }

    /**
     * @see #findOne(Wrapper, boolean)
     */
    @Deprecated
    T getOne(Wrapper<T> queryWrapper, boolean throwEx);

    /**
     * @see #findObj(Wrapper, Function)
     */
    @Deprecated
    <V> V getObj(Wrapper<T> queryWrapper, Function<? super Object, V> mapper);

    default List<T> limit(LambdaQueryWrapper<T> wrapper, int limit) {
        return this.getBaseMapper()
                .selectPage(new Page<>(1, limit, -1, false), wrapper)
                .getRecords();
    }

    default Optional<T> limitOne(LambdaQueryWrapper<T> wrapper) {
        List<T> list = limit(wrapper, 1);

        if(list.isEmpty()) {
            return Optional.empty();
        } else {
            return Optional.ofNullable(list.get(0));
        }
    }


    default List<T> limit(LambdaQueryChainWrapper<T> wrapper, int limit) {
        return wrapper.page(new Page<>(1, limit, -1, false))
                .getRecords();
    }

    default Optional<T> limitOne(LambdaQueryChainWrapper<T> wrapper) {
        List<T> list = limit(wrapper, 1);

        if(list.isEmpty()) {
            return Optional.empty();
        } else {
            return Optional.ofNullable(list.get(0));
        }
    }

    /**
     * 根据 ID 查询
     *
     * @param id 主键ID
     */
    default Optional<T> findById(Serializable id) {
        return Optional.ofNullable(getBaseMapper().selectById(id));
    }

    /**
     * 根据 Wrapper，查询一条记录
     *
     * @param queryWrapper 实体对象封装操作类 {@link com.baomidou.mybatisplus.core.conditions.query.QueryWrapper}
     */
    default Optional<T> findOne(Wrapper<T> queryWrapper) {
        return findOne(queryWrapper, true);
    }

    /**
     * 根据 Wrapper，查询一条记录
     *
     * @param queryWrapper 实体对象封装操作类 {@link com.baomidou.mybatisplus.core.conditions.query.QueryWrapper}
     * @param throwEx      有多个 result 是否抛出异常
     */
    default Optional<T> findOne(Wrapper<T> queryWrapper, boolean throwEx) {
        return Optional.ofNullable(getOne(queryWrapper, throwEx));
    }

    /**
     * 根据 Wrapper，查询一条记录
     *
     * @param queryWrapper 实体对象封装操作类 {@link com.baomidou.mybatisplus.core.conditions.query.QueryWrapper}
     * @param mapper       转换函数
     */
    default <V> Optional<V> findObj(Wrapper<T> queryWrapper, Function<? super Object, V> mapper) {

        return Optional.ofNullable(getObj(queryWrapper, mapper));
    }


    /**
     * 查找出所有已存在的数据
     * @param map 字段
     * @param values 需要检测的值
     */
    default <R> Set<R> contains(SFunction<T, R> map, R... values) {
        return contains(map, Arrays.stream(values).collect(Collectors.toSet()));
    }

    default <R> Set<R> contains(SFunction<T, R> map, Set<R> values) {
        if(values.isEmpty()) {
            return new HashSet<>();
        }
        return lambdaQuery()
                .select(map)
                .in(map, values)
                .list()
                .stream()
                .map(map)
                .collect(Collectors.toSet());
    }

    /**
     * 查找出所有不存在的数据
     * @param map 字段
     * @param values 需要检测的值
     */
    default <R> Set<R> notContains(SFunction<T, R> map, R... values) {
        return notContains(map, Arrays.stream(values).collect(Collectors.toSet()));
    }

    /**
     * 查找出所有不存在的数据
     * @param map 字段
     * @param values 需要检测的值
     */
    default <R> Set<R> notContains(SFunction<T, R> map, Set<R> values) {
        Set<R> contains = contains(map, values);
        return CollectionKit.subtract(values, contains, HashSet::new);
    }

    default <C> Collection<C> saveNotContains(SFunction<T, C> map, Collection<T> list) {
        Set<C> contains = contains(map, list.stream().map(map).collect(Collectors.toSet()));
        List<T> noContains = list.stream().filter(t -> !contains.contains(map.apply(t))).collect(Collectors.toList());
        this.saveBatch(noContains);
        return contains;
    }

    /**
     * 根据condition参数的字段作为条件进行删除，为null的字段不参与条件
     * @param condition 条件对象
     * @return 是否删除
     */
    default boolean remove(T condition) {
        return remove(new QueryWrapper<>(condition));
    }

    /**
     * 根据condition参数的字段作为条件进行查询，为null的字段不参与条件
     * @param condition 条件对象
     * @return 查询结果
     */
    default List<T> list(T condition) {
        return list(new QueryWrapper<>(condition));
    }


    default IPage<T> pageByRowQuery(RowQuery<T> rowQuery) {
        QueryWrapper<T> query = toQueryWrapper(rowQuery);
        IPage<T> page = getBaseMapper().selectPage(new Page<>(rowQuery.getPageNum(), rowQuery.getPageSize()), query);

        if (CollectionKit.isNotEmpty(rowQuery.getStatisticRows())) {
            return new StatisticsPage<>(page, statistics(rowQuery));
        } else {
            return page;
        }
    }

    default List<StatisticsPage.StatisticRow<T>> statistics(RowQuery<T> rowQuery) {
        RowQuery.StatisticRow[] rows = rowQuery.getStatisticRows();

        return Arrays.stream(rows).map(statisticRow -> {
            QueryWrapper<T> query = new QueryWrapper<>();
            // 将所有filter 转化为条件
            CollectionKit.foreach(rowQuery.getFilters(), filter -> FilterWrappers.handWrapper(query, filter, getEntityClass()));

            String[] selectField = new String[statisticRow.getFields().size()];

            for (int i = 0; i < statisticRow.getFields().size(); i++) {
                String field = statisticRow.getFields().get(i);
                selectField[i] = statisticRow.getFunc() + "(" + StringUtils.camelToUnderline(field) + ") AS " + StringUtils.camelToUnderline(field);
            }

            T t = getBaseMapper().selectOne(query.select(selectField));

            StatisticsPage.StatisticRow<T> s = new StatisticsPage.StatisticRow<>();
            s.setFunc(statisticRow.getFunc().name());
            s.setRow(t);
            return s;
        }).collect(Collectors.toList());
    }

    default QueryWrapper<T> toQueryWrapper(RowQuery<T> rowQuery) {
        QueryWrapper<T> query = new QueryWrapper<>();
        // 查询字段
        if(CollectionKit.isNotEmpty(rowQuery.getFields())) {
            Set<String> fields = Arrays.stream(rowQuery.getFields()).collect(Collectors.toSet());
            query.select(info -> fields.remove(info.getField().getName()));
        }

        //order by
        CollectionKit.foreach(rowQuery.getSorts(), sort -> {
            if("asc".equalsIgnoreCase(sort.getSort())) {
                query.orderByAsc(StringUtils.camelToUnderline(sort.getField()));
            } else {
                query.orderByDesc(StringUtils.camelToUnderline(sort.getField()));
            }
        });

        // 将所有filter 转化为条件
        CollectionKit.foreach(rowQuery.getFilters(), filter -> FilterWrappers.handWrapper(query, filter, getEntityClass()));

        return query;
    }

    @Override
    default boolean updateById(T entity) {
        boolean success = IService.super.updateById(entity);

        TableInfo info = TableInfoHelper.getTableInfo(getEntityClass());
        // 如果有版本字段，且没有成功，则直接抛出乐观锁异常
        if(info.isWithVersion() && !success) {
            String id = BeanKit.getFieldValue(entity, info.getKeyProperty()).orElse("").toString();
            String version = BeanKit.getFieldValue(entity, info.getVersionFieldInfo().getField()).orElse("").toString();
            throw new OptimisticLockerException("table: " + info.getTableName() + " 触发乐观锁, 数据id为: " + id + " 版本为: " + version);
        }

        return success;
    }

    @Override
    default boolean update(T entity, Wrapper<T> updateWrapper) {
        boolean success = IService.super.update(entity, updateWrapper);

        TableInfo info = TableInfoHelper.getTableInfo(getEntityClass());
        // 如果有版本字段，且没有成功，则直接抛出乐观锁异常
        if(info.isWithVersion() && !success) {
            String id = BeanKit.getFieldValue(entity, info.getKeyProperty()).orElse("").toString();
            String version = BeanKit.getFieldValue(entity, info.getVersionFieldInfo().getField()).orElse("").toString();
            throw new OptimisticLockerException("table: " + info.getTableName() + " 触发乐观锁, 数据id为: " + id + " 版本为: " + version);
        }

        return success;
    }
}
