package com.chuang.tauceti.limit;

import com.chuang.tauceti.limit.annotations.Limit;
import com.chuang.tauceti.limit.counter.LimitCountRunner;
import com.chuang.tauceti.limit.group.IGroupMapping;
import com.chuang.tauceti.limit.quote.IQuote;
import com.chuang.tauceti.support.exception.SystemException;
import com.chuang.tauceti.tools.basic.StringKit;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.lang.reflect.Method;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

/**
 * @author ATH
 */
@Slf4j
@Component
public class Limiter {

    @Resource private LimitProperties properties;

    private final Map<String, QuarantineParser> cache = new ConcurrentHashMap<>();

    private final List<IQuote> quotes;
    private final Map<String, IGroupMapping> mappingMap;
    private final LimitCountRunner countRunner;

    @Autowired
    public Limiter(List<IQuote> limitQuotes, List<IGroupMapping> limitGroupMappings, LimitCountRunner countRunner) {
        this.quotes = limitQuotes;
        this.mappingMap = limitGroupMappings.stream().collect(Collectors.toMap(IGroupMapping::groupName, s -> s));
        this.countRunner = countRunner;
    }

    public Object doLimit(ProceedingJoinPoint pjp) throws Throwable {
        Method method = ((MethodSignature)pjp.getSignature()).getMethod();

        Limit limit = method.getAnnotation(Limit.class);
        LimitInfo info = properties.asInfo(limit);

        if (!info.isLimit()) {
            return pjp.proceed();
        }

        String quarantine = quarantineAsStr(pjp, info.getQuarantine());

        String group = null;
        if(StringKit.isNotBlank(info.getGroupName())) {
            IGroupMapping mapping = mappingMap.get(info.getGroupName());
            if (null == mapping) {
                throw new SystemException(info.getGroupName() + "无法找到对象");
            }
            group = mapping.mapping(method);
        }

        if(StringKit.isNotBlank(info.getGroup())) {
            group = info.getGroup();
        }
        if (StringKit.isBlank(group)) {
            if (log.isDebugEnabled()) {
                log.debug("[严重警告] {}无法获取group，这可能导致严重的性能问题", method);
                log.debug("[严重警告] {}无法获取group，这可能导致严重的性能问题", method);
                log.debug("[严重警告] {}无法获取group，这可能导致严重的性能问题", method);
            }
            group = "__EMPTY_GROUP__";
        }

        return countRunner.countRun(group, quarantine, info.getMaxMulti(), pjp::proceed);
    }

    public String quarantineAsStr(ProceedingJoinPoint point, String quarantine) {
        QuarantineParser parser = cache.computeIfAbsent(quarantine, s -> new QuarantineParser(s, this.quotes));
        return parser.parser(point).trim();
    }

}
