package com.chuang.tauceti.rbac2.starter.aspect;

import com.chuang.tauceti.rbac2.starter.exception.PageException;
import com.chuang.tauceti.shiro.spring.web.jwt.ShiroUtils;
import com.chuang.tauceti.support.MapResult;
import com.chuang.tauceti.support.Result;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.basic.StringKit;
import com.chuang.tauceti.tools.third.servlet.HttpKit;
import com.chuang.tauceti.tools.third.spring.SpringKit;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.ShiroException;
import org.apache.shiro.authc.IncorrectCredentialsException;
import org.springframework.beans.propertyeditors.StringTrimmerEditor;
import org.springframework.context.MessageSource;
import org.springframework.context.NoSuchMessageException;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.http.HttpStatus;
import org.springframework.validation.BindException;
import org.springframework.validation.BindingResult;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.i18n.SessionLocaleResolver;

import javax.annotation.Resource;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.Arrays;
import java.util.Locale;
import java.util.Objects;
import java.util.UUID;
import java.util.concurrent.CompletionException;
import java.util.stream.Collectors;

/**
 * @author ath
 */
@Slf4j
@ControllerAdvice
public class AnrControllerAdvice {

    @Resource private MessageSource messageSource;

    @InitBinder
    public void initBinder(WebDataBinder binder) {
        // 创建 String trim 编辑器
        // 构造方法中 boolean 参数含义为如果是空白字符串,是否转换为null
        // 即如果为true,那么 " " 会被转换为 null,否者为 ""
        StringTrimmerEditor propertyEditor = new StringTrimmerEditor(false);
        // 为 String 类对象注册编辑器
        binder.registerCustomEditor(String.class, propertyEditor);
    }

    @ExceptionHandler(value = PageException.class)
    public Object exception(PageException exception) {
        return pageException((Exception) exception.getCause());
    }

    @ExceptionHandler(value = Exception.class)
    @ResponseBody
    public Object exception(Exception exception) throws PageException {
        boolean isPage = HttpKit.getRequest()
                .map(request -> {
                    Object b = request.getAttribute(PageOrRestInterceptor.PAGE_VIEW_ATTR);
                    return null != b && (Boolean) b;
                })
                //如果没有记录，默认不是页面
                .orElse(false);
        if(isPage) {
            throw new PageException(exception);
        }
        return jsonException(exception);
    }


    private Result<?> jsonException(Throwable exception) {
        return Result.fail(errorMsg(exception));
    }

    private ModelAndView pageException(Exception exception) {
        ModelAndView view = new ModelAndView();
        view.setStatus(HttpStatus.BAD_REQUEST);
        view.setViewName("/500");
        view.addObject("error", errorMsg(exception));
        return view;
    }


    private String errorMsg(Throwable exception) {
        if(exception instanceof CompletionException) {
            if (exception != exception.getCause()) {
                return errorMsg(exception.getCause());
            }
        }

        if(exception instanceof IncorrectCredentialsException) {
            return getMessage("msg.error.credentials");
        } else if(isShiroException(exception)) {
            return "RBAC Err：" + exception.getMessage();
        } if (isConstraintViolationException(exception)) {
            ConstraintViolationException ex = (ConstraintViolationException) exception;
            return ex.getConstraintViolations().stream()
                    .map(ConstraintViolation::getMessageTemplate)
                    .map(String::trim)
                    .map(msg0 -> {
                        try {
                            return getMessage(msg0);
                        } catch (Exception e) {
                            return msg0;
                        }
                    })
                    .collect(Collectors.joining(","));
        } else if (isBindException(exception)) {
            BindingResult bindingResult = ((BindException) exception).getBindingResult();
            if (bindingResult.getFieldError() == null) {
                return "param valid error";
            } else {
                return StringKit.nullToEmpty(bindingResult.getFieldError().getDefaultMessage());
            }
        } else if(isBusinessException(exception) || isHttpRequestMethodNotSupportedException(exception)) {
            return exception.getMessage();
        } else if(exception instanceof MethodArgumentNotValidException){
            return Objects.requireNonNull(((MethodArgumentNotValidException) exception).getBindingResult().getFieldError()).getDefaultMessage();
        } else if(isDuplicateKeyException(exception)) {
            return "出现重复数据:" + Objects.requireNonNull(((DuplicateKeyException) exception).getRootCause()).getMessage();
        }

        String err = "Ex:" + UUID.randomUUID();
        log.error("controller异常, 错误号:" + err, exception);
        return getMessage("msg.system-error", err);

    }

    private boolean isHttpRequestMethodNotSupportedException(Throwable exception) {
        return exception instanceof HttpRequestMethodNotSupportedException;
    }

    private boolean isBusinessException(Throwable exception) {
        return exception instanceof BusinessException;
    }

    private boolean isShiroException(Throwable e) {
        return "org.apache.shiro.ShiroException".equals(e.getClass().getName());
    }
    private boolean isShiroAuthenticationException(Throwable e) {
        return "org.apache.shiro.authc.AuthenticationException".equals(e.getClass().getName());
    }
    private boolean isDuplicateKeyException(Throwable e) {
        return "org.springframework.dao.DuplicateKeyException".equals(e.getClass().getName());
    }

    private boolean isBindException(Throwable e) {
        return "org.springframework.validation.BindException".equals(e.getClass().getName());
    }

    private boolean isConstraintViolationException(Throwable e) {
        return "javax.validation.ConstraintViolationException".equals(e.getClass().getName());
    }

    private String getMessage(String i18n, Object... args) {
        MessageSource messageSource = SpringKit.getBean(MessageSource.class);

        if(null == messageSource) {
            return i18n;
        }
        try {
            return messageSource.getMessage(i18n, args, LocaleContextHolder.getLocale());
        } catch (NoSuchMessageException e) {
            return i18n + Arrays.toString(args);
        }
    }
}
