package com.chuang.tauceti.shiro.spring.web.jwt;

import com.alibaba.fastjson.JSONObject;
import com.chuang.tauceti.shiro.spring.web.jwt.properties.JwtProperties;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.basic.StringKit;
import com.chuang.tauceti.tools.third.servlet.HttpKit;
import com.nimbusds.jose.JOSEException;
import lombok.extern.slf4j.Slf4j;

import javax.servlet.http.HttpServletRequest;
import javax.swing.text.html.Option;
import java.text.ParseException;
import java.util.Optional;
import java.util.UUID;

import static com.chuang.tauceti.shiro.spring.web.jwt.filter.JwtAuthFilter.JWT_PAYLOAD_REQUEST_ATTR;
import static com.chuang.tauceti.shiro.spring.web.jwt.filter.JwtAuthFilter.JWT_STR_REQUEST_ATTR;

@Slf4j
public class JwtManager {

    private final JwtProperties properties;

    public JwtManager(JwtProperties properties) {
        this.properties = properties;
    }


    public JwtProperties properties() {
        return properties;
    }

    public Optional<String> requestToken() {
        HttpServletRequest request = HttpKit.getRequest().orElseThrow(() -> new IllegalArgumentException("无法从request中获取Jwt，原因：request为空"));
        // 从请求头中获取token
        String token = request.getHeader(properties.getTokenHeader());
        if (StringKit.isBlank(token)) {
            // 从请求参数中获取token
            token = request.getParameter(properties.getTokenHeader());
        }
        return Optional.ofNullable(token);
    }

    public Optional<JwtPayload> requestJwtPayload() {
        return requestToken().flatMap(s -> {
            try {
                JwtPayload payload = this.parse(s);
                return Optional.ofNullable(payload);
            } catch (ClassNotFoundException e) {
                log.error("jwt转换失败", e);
                return Optional.empty();
            }
        });
    }

    public Optional<String> refreshCurrentToken(boolean checkValid) {
        return requestJwtPayload().map(jwtPayload -> {
            if(checkValid && jwtPayload.expired()) {
                throw new BusinessException("旧令牌已经过期，不能刷新令牌");
            }
            try {
                String token =  refreshAngGet(jwtPayload);
                token2Attr(token, jwtPayload);
                return token;
            } catch (JOSEException e) {
                throw new BusinessException("生成token失败", e);
            }
        });
    }

    public JwtPayload makePayload(String aud, Object body) {
        long now = System.currentTimeMillis();
        return JwtPayload.builder()
                .aud(aud)
                .iss(properties.getIssuer())
                .exp(now + properties.getExpire())
                .sub(properties.getSubject())
                .nbf(now)
                .iat(now)
                .jti(UUID.randomUUID().toString())
                .body(body)
                .bodyClass(body.getClass().getName())
                .build();
    }

    public void refresh(JwtPayload payload) {
        payload.setExp(System.currentTimeMillis() + properties.getExpire());
    }

    public String refreshAngGet(JwtPayload payload) throws JOSEException {
        refresh(payload);
        return makeToken(payload);
    }

    public String makeToken(String aud, Object body) throws JOSEException {
        return makeToken(makePayload(aud, body));
    }

    public String makeToken(JwtPayload payload) throws JOSEException {
        return JwtUtil.generateToken(JSONObject.toJSONString(payload), properties.getSecret());
    }

    public JwtPayload parse(String token) throws ClassNotFoundException {
        String payloadStr;
        try {
            if(token.startsWith("Bearer ")) {
                token = token.substring("Bearer ".length());
            }
            payloadStr = JwtUtil.verifySignature(token, properties.getSecret());
        } catch (JOSEException | JwtUtil.JwtSignatureVerifyException | ParseException e) {
            throw new BusinessException("jwt 解码错误", e);
        }
        return parse(JSONObject.parseObject(payloadStr));
    }

    public JwtPayload parseNoErr(String token) {
        String payloadStr;
        try {
            if(token.startsWith("Bearer ")) {
                token = token.substring("Bearer ".length());
            }
            payloadStr = JwtUtil.verifySignature(token, properties.getSecret());
        } catch (JOSEException | JwtUtil.JwtSignatureVerifyException | ParseException e) {
            throw new BusinessException("jwt 解码错误", e);
        }
        try {
            return parse(JSONObject.parseObject(payloadStr));
        } catch (ClassNotFoundException e) {
            throw new BusinessException("JWT 解码失败," + e.getMessage());
        }
    }

    private JwtPayload parse(JSONObject json) throws ClassNotFoundException {
        JSONObject bodyJson = json.getJSONObject("body");
        String bodyClass = json.getString("bodyClass");

        Object body = bodyJson.toJavaObject(Class.forName(bodyClass));
        return JwtPayload.builder()
                .aud(json.getString("aud"))
                .iss(json.getString("iss"))
                .exp(json.getLong("exp"))
                .sub(json.getString("sub"))
                .nbf(json.getLong("nbf"))
                .iat(json.getLong("iat"))
                .jti(json.getString("jti"))
                .bodyClass(bodyClass)
                .body(body)
                .build();
    }

    public void token2Attr(String token, JwtPayload payload) {
        HttpKit.getRequest().ifPresent(httpServletRequest -> {
            httpServletRequest.setAttribute(JWT_PAYLOAD_REQUEST_ATTR, payload);
            httpServletRequest.setAttribute(JWT_STR_REQUEST_ATTR, token);
        });
    }
}
