package com.chuang.tauceti.shiro.spring.web.jwt.configuration;

import com.chuang.tauceti.shiro.spring.web.jwt.*;
import com.chuang.tauceti.shiro.spring.web.jwt.filter.JwtAuthFilter;
import com.chuang.tauceti.shiro.spring.web.jwt.filter.PasswordFilter;
import com.chuang.tauceti.shiro.spring.web.jwt.properties.ShiroProperties;
import com.chuang.tauceti.shiro.spring.web.jwt.realm.IShiroService;
import com.chuang.tauceti.shiro.spring.web.jwt.realm.JwtRealm;
import com.chuang.tauceti.shiro.spring.web.jwt.realm.LoginRealm;
import com.chuang.tauceti.support.exception.SystemException;
import com.chuang.tauceti.tools.basic.collection.CollectionKit;
import com.chuang.tauceti.tools.third.spring.SpringKit;
import org.apache.commons.collections.CollectionUtils;
import org.apache.shiro.authc.Authenticator;
import org.apache.shiro.authc.credential.HashedCredentialsMatcher;
import org.apache.shiro.authc.pam.ModularRealmAuthenticator;
import org.apache.shiro.authz.Authorizer;
import org.apache.shiro.mgt.SessionStorageEvaluator;
import org.apache.shiro.mgt.SessionsSecurityManager;
import org.apache.shiro.mgt.SubjectFactory;
import org.apache.shiro.spring.config.web.autoconfigure.ShiroWebFilterConfiguration;
import org.apache.shiro.spring.web.ShiroFilterFactoryBean;
import org.apache.shiro.spring.web.config.AbstractShiroWebConfiguration;
import org.apache.shiro.spring.web.config.DefaultShiroFilterChainDefinition;
import org.apache.shiro.spring.web.config.ShiroFilterChainDefinition;
import org.apache.shiro.web.mgt.DefaultWebSessionStorageEvaluator;
import org.apache.shiro.web.servlet.AbstractShiroFilter;
import org.springframework.aop.framework.autoproxy.DefaultAdvisorAutoProxyCreator;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.boot.web.servlet.FilterRegistrationBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.DependsOn;
import org.springframework.core.annotation.Order;

import javax.annotation.Resource;
import javax.servlet.DispatcherType;
import javax.servlet.Filter;
import java.util.*;

/**
 * shiro 配置
 * 需要注意的是，我们这里使用了 shiro-starter, Application
 *
 * @author ATH
 */
@ConditionalOnProperty(name = "shiro.web.enabled", matchIfMissing = true)
@Configuration
@EnableConfigurationProperties({
		ShiroProperties.class
})
public class ShiroJwtConfiguration extends AbstractShiroWebConfiguration {

	public static final String FILTER_NAME = "filterMap";

	@Resource
	private ShiroProperties shiroProperties;

	@Bean
	public JwtManager manager() {
		return new JwtManager(shiroProperties.getJwt());
	}

	@Bean(name = FILTER_NAME)
	@ConditionalOnMissingBean(name = FILTER_NAME)
	@Order
	public Map<String, Filter> filterMap() {
		// 其他默认的filter会在 ShiroFilterFactoryBean.getObject 时初始化进去
		Map<String, Filter> filterMap = new LinkedHashMap<>();
		filterMap.put("auth", new PasswordFilter(manager(), shiroProperties, authService(), authErrorMsg()));
		filterMap.put("jwt", new JwtAuthFilter(manager(), shiroProperties, authService(), authErrorMsg()));
		return filterMap;
	}

	/**
	 * ShiroWebFilterConfiguration 已经有进行配置，但DispatcherType不包含Async，导致异步代码中访问 SecurityUtils会报错。
	 * 由于导入的配置会优先加载，ShiroWebFilterConfiguration 会先加载，所以这里加 @ConditionalOnMissingBean 就不会再执行了。
	 * 通常情况下用户已经不需要来覆盖这个实现。但万一需要覆盖，可以通过设置 filterRegistrationBean.setOrder() 来进行覆盖。
	 */
	@Bean(name = ShiroWebFilterConfiguration.REGISTRATION_BEAN_NAME)
	protected FilterRegistrationBean<AbstractShiroFilter> filterShiroFilterRegistrationBean() throws Exception {

		FilterRegistrationBean<AbstractShiroFilter> filterRegistrationBean = new FilterRegistrationBean<>();
		filterRegistrationBean.setDispatcherTypes(DispatcherType.REQUEST, DispatcherType.ASYNC, DispatcherType.FORWARD
				, DispatcherType.INCLUDE, DispatcherType.ERROR);
		filterRegistrationBean.setFilter((AbstractShiroFilter) shiroFilterFactoryBean().getObject());
		filterRegistrationBean.setName(ShiroWebFilterConfiguration.FILTER_NAME);
		filterRegistrationBean.setOrder(1);

		return filterRegistrationBean;
	}

//    /**
//     * @see org.apache.shiro.spring.web.config.AbstractShiroWebFilterConfiguration
//     */
//

	@Bean
	public ShiroFilterFactoryBean shiroFilterFactoryBean() {
		ShiroFilterFactoryBean filterFactoryBean = new ShiroFilterFactoryBean();

		filterFactoryBean.setLoginUrl(shiroProperties.getLoginUrl());
		filterFactoryBean.setSuccessUrl(shiroProperties.getSuccessUrl());
		filterFactoryBean.setUnauthorizedUrl(shiroProperties.getUnauthorizedUrl());

		filterFactoryBean.setSecurityManager(createSecurityManager());
//        filterFactoryBean.setGlobalFilters(Arrays.asList(
//                DefaultFilter.anon.name(),
//                DefaultFilter.invalidRequest.name()
//        ));
		filterFactoryBean.setFilters(filterMap());
		filterFactoryBean.setFilterChainDefinitionMap(shiroFilterChainDefinition().getFilterChainMap());

		return filterFactoryBean;
	}

	@Bean
	@Override
	protected SessionsSecurityManager createSecurityManager() {
		SessionsSecurityManager securityManager = super.createSecurityManager();
		securityManager.setAuthenticator(authenticator());
		securityManager.setAuthorizer(authorizer());
		securityManager.setRealms(Arrays.asList(loginRealm(), jwtRealm()));
		securityManager.setSessionManager(sessionManager());
		securityManager.setEventBus(eventBus);
		if (cacheManager != null) {
			securityManager.setCacheManager(cacheManager);
		}

		return securityManager;
	}

	@Override
	@Bean
	public Authenticator authenticator() {
		ModularRealmAuthenticator authenticator = new MyModularRealmAuthenticator();
		authenticator.setAuthenticationStrategy(this.authenticationStrategy());
		return authenticator;
	}

	@Bean
	@ConditionalOnMissingBean
	public HashedCredentialsMatcher hashedCredentialsMatcher() {
		HashedCredentialsMatcher credentialsMatcher = new HashedCredentialsMatcher();
		credentialsMatcher.setHashIterations(shiroProperties.getHashedCredential().getIterations());
		credentialsMatcher.setHashAlgorithmName(shiroProperties.getHashedCredential().getAlgorithm());
		credentialsMatcher.setStoredCredentialsHexEncoded(true);
		return credentialsMatcher;
	}

	@Bean
	@ConditionalOnMissingBean
	public IShiroService authService() {
		throw new SystemException("IShiroService bean can not found~~");
	}

	@Bean
	@ConditionalOnMissingBean
	public JwtCredentialsMatcher jwtCredentialsMatcher() {
		return new JwtCredentialsMatcher();
	}

	@Bean
	@ConditionalOnMissingBean
	public JwtRealm jwtRealm() {
		JwtRealm realm = new JwtRealm(authService());
		realm.setCredentialsMatcher(jwtCredentialsMatcher());
		return realm;
	}

	@Bean
	@ConditionalOnMissingBean
	public LoginRealm loginRealm() {
		LoginRealm realm = new LoginRealm(authService());
		realm.setCredentialsMatcher(hashedCredentialsMatcher());
		return realm;
	}


	/**
	 * Shiro starter 填坑说明
	 * 这里必须自定义授权器，虽然 ShiroWebAutoConfiguration 会定义，但由于我们设定自己的配置早于 ShiroWebAutoConfiguration 执行。
	 * 因此spring加载 authorizer时判断该类型对象是否被加载时，spring容器中已经存在realm对象，又因为realm对象本身实现了 Authorizer 接口。
	 * 因此被spring认为 Authorizer 对象已经被加载。这样导致 ShiroWebAutoConfiguration 在创建 securityManager 时需要通过调用 authorizer()来获取
	 * Authorizer 对象时导致获取失败。
	 * 进而导致启动失败，并提示 authorizer bean不存在。
	 * <p>
	 * 该问题具体的debug和分析步骤可以查阅下面这篇帖子：
	 * https://www.cnblogs.com/insaneXs/p/11028286.html
	 */
	@Bean
	@Override
	protected Authorizer authorizer() {
		return super.authorizer();
	}


	@Override
	@Bean
	protected SessionStorageEvaluator sessionStorageEvaluator() {
		DefaultWebSessionStorageEvaluator evaluator = new DefaultWebSessionStorageEvaluator();
		evaluator.setSessionStorageEnabled(false);
		return evaluator;
	}

	@Override
	@Bean
	protected SubjectFactory subjectFactory() {
		return new StatelessWebSubjectFactory();
	}

	@Override
	@Bean
	protected ShiroFilterChainDefinition shiroFilterChainDefinition() {
		Map<String, String> filterChainMap = getFilterChainDefinitionMap();

		DefaultShiroFilterChainDefinition chainDefinition = new DefaultShiroFilterChainDefinition();
		chainDefinition.addPathDefinitions(filterChainMap);
		return chainDefinition;
	}

	private Map<String, String> getFilterChainDefinitionMap() {


		Map<String, String> filterChainDefinitionMap = new LinkedHashMap<>();


		filterChainDefinitionMap.put(shiroProperties.getLoginUrl(), "auth");

		if (shiroProperties.getFilterChainDefinitions() != null) {
			filterChainDefinitionMap.putAll(shiroProperties.getFilterChainDefinitions());
		}

		List<String[]> authList = shiroProperties.getAuth();
		if (CollectionUtils.isNotEmpty(authList)) {
			authList.forEach(authArray -> {
				if (CollectionKit.isNotEmpty(authArray)) {
					for (String authPath : authArray) {
						filterChainDefinitionMap.put(authPath, "auth");
					}
				}
			});
		}

		List<String> annos = new ArrayList<>();
		// 获取排除的路径
		List<String[]> anonList = shiroProperties.getAnon();
		if (CollectionUtils.isNotEmpty(anonList)) {
			anonList.forEach(anonArray -> {
				if (CollectionKit.isNotEmpty(anonArray)) {
					annos.addAll(Arrays.asList(anonArray));
				}
			});
		}

		anonList = shiroProperties.getAnonExt();
		if (CollectionUtils.isNotEmpty(anonList)) {
			anonList.forEach(anonArray -> {
				if (CollectionKit.isNotEmpty(anonArray)) {
					for(String n: anonArray) {
						if(!annos.contains(n)) {
							annos.add(n);
						}
					}
				}
			});
		}


		annos.forEach(s -> filterChainDefinitionMap.put(s, "anon"));
		filterChainDefinitionMap.put("/**", "jwt, user");
		return filterChainDefinitionMap;
	}

	@Bean
	@ConditionalOnMissingBean
	@DependsOn("lifecycleBeanPostProcessor")
	public DefaultAdvisorAutoProxyCreator defaultAdvisorAutoProxyCreator() {
		DefaultAdvisorAutoProxyCreator daap = new DefaultAdvisorAutoProxyCreator();
		// 不加这个的话，当遇到@RequireRole 之类的注解会404
		daap.setProxyTargetClass(true);
		return daap;
	}

	@Bean
	public SpringKit springKit() {
		return new SpringKit();
	}

	@Bean
	@ConditionalOnMissingBean
	public AuthErrorMsg authErrorMsg() {
		return Throwable::getMessage;
	}

}
