package com.chuang.tauceti.shiro.spring.web.jwt.realm;

import com.chuang.tauceti.shiro.spring.web.jwt.JwtPayload;
import com.chuang.tauceti.shiro.spring.web.jwt.properties.HashedCredentialProperties;
import com.chuang.tauceti.shiro.spring.web.jwt.properties.JwtProperties;
import com.chuang.tauceti.shiro.spring.web.jwt.properties.ShiroProperties;
import com.chuang.tauceti.support.BiValue;
import com.chuang.tauceti.support.Result;
import com.chuang.tauceti.support.ThreeValue;
import com.chuang.tauceti.tools.basic.HashKit;
import com.chuang.tauceti.tools.basic.HexKit;
import com.chuang.tauceti.tools.third.servlet.HttpKit;
import com.chuang.tauceti.tools.third.spring.SpringKit;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.shiro.authc.AuthenticationException;
import org.apache.shiro.authc.AuthenticationInfo;
import org.apache.shiro.authz.AuthorizationInfo;
import org.apache.shiro.codec.Hex;
import org.apache.shiro.subject.PrincipalCollection;
import org.apache.shiro.web.util.WebUtils;
import org.springframework.lang.Nullable;

import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletResponse;

/**
 * @author ATH
 */
public interface IShiroService {

    /**
     * 根据用户名查找对象
     * @param token 登录
     */
    AuthenticationInfo getAuthenticationInfoByLoginToken(LoginToken token, String realmName);

    AuthorizationInfo getAuthorizationInfo(PrincipalCollection principals);

    /**
     * 当登录成功后， 只有 loginHosting = true 时登录成功后才会触发。
     * @param token 登录的token
     */
    default void loginSuccess(LoginToken token) {}
    /**
     * 当登录成功后， 只有 loginHosting = true 时登录失败后才会触发。
     * @param token 登录的token
     */
    default void loginFailure(LoginToken token, Throwable e) {}

    /**
     * 当登录成功后， 只有 loginHosting = true 时登录成功后才会触发。
     * @param newToken 登录成功后生成的token
     */
    default void onRefreshToken(@Nullable String oldToken, String newToken, JwtPayload payload, ShiroProperties properties, ServletResponse response) {
        HttpServletResponse httpServletResponse = WebUtils.toHttp(response);
        httpServletResponse.setStatus(properties.getJwt().getRefreshStatus());
        httpServletResponse.setHeader(properties.getJwt().getTokenHeader(), newToken);
//        HttpKit.printJson(Result.success(jwtToken));
    }

    /**
     * 随机密码给用户，加密后的密码和盐入库。
     * @return 1. 随机密码, 2.加密后的密码, 3. 盐
     */
    default ThreeValue<String, String, String> randomSaltAndPassword() {
        String password = RandomStringUtils.randomAlphanumeric(12);
        BiValue<String, String> bi = randomSaltWithPassword(password);
        return new ThreeValue<>(password, bi.getOne(), bi.getTwo());
    }

    /**
     * 比对是否匹配
     * @param password 密码
     * @param salt 盐
     * @param passwordHash 密码hash
     * @return 是否匹配成功
     */
    default boolean match(String password, String salt, String passwordHash) {
        HashedCredentialProperties properties = SpringKit.getBean(ShiroProperties.class).getHashedCredential();
        String hash = HashKit.encodeHex(properties.getAlgorithm(), password.getBytes(), Hex.decode(salt), properties.getIterations());
        return hash.equalsIgnoreCase(passwordHash);
    }

    /**
     * 根据密码原文，随机一个盐，并返回盐和加密后的密码
     * @param password 密码原文
     * @return  1.加密后的密码， 2，盐
     */
    default BiValue<String, String> randomSaltWithPassword(String password) {
        HashedCredentialProperties properties = SpringKit.getBean(ShiroProperties.class).getHashedCredential();
        byte [] salt = HashKit.genSalt(properties.getSaltLen());
        String newPwdHash = HashKit.encodeHex(properties.getAlgorithm(), password.getBytes(), salt, properties.getIterations());
        return new BiValue<>(newPwdHash, HexKit.encodeHexStr(salt));
    }
}
