package com.chuang.tauceti.support;

import lombok.AccessLevel;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.time.*;
import java.time.format.DateTimeFormatter;
import java.time.temporal.*;
import java.util.Calendar;

@Getter
@AllArgsConstructor(access = AccessLevel.PRIVATE)
public class LocalDateRange {
    
    private final LocalDateTime begin;
    private final LocalDateTime end;

    /**
     * 通过添加时间获得一个新的区间
     * plus(Period.of(year, monty, days));
     * plus(Period.ofYear(years));
     * plus(Duration.ofDays(days));
     * plus(Duration.ofSeconds(seconds));
     * @param amountToAdd 添加时间 {@link Period#parse(CharSequence)}} 或者 {@link Duration#parse(CharSequence)}
     */
    public LocalDateRange plus(TemporalAmount amountToAdd) {
        return between(begin.plus(amountToAdd), end.plus(amountToAdd));
    }

    /**
     * 通过给开始时间添加时间获得一个新的区间
     * plusBegin(Period.of(year, monty, days));
     * plusBegin(Period.ofYear(years));
     * plusBegin(Duration.ofDays(days));
     * plusBegin(Duration.ofSeconds(seconds));
     * @param amountToAdd 添加时间 {@link Period#parse(CharSequence)}} 或者 {@link Duration#parse(CharSequence)}
     */
    public LocalDateRange plusBegin(TemporalAmount amountToAdd) {
        return between(begin.plus(amountToAdd), end);
    }

    /**
     * 通过给结束时间添加时间获得一个新的区间
     * plusEnd(Period.of(year, monty, days));
     * plusEnd(Period.ofYear(years));
     * plusEnd(Duration.ofDays(days));
     * plusEnd(Duration.ofSeconds(seconds));
     * @param amountToAdd 添加时间 {@link Period#parse(CharSequence)}} 或者 {@link Duration#parse(CharSequence)}
     */
    public LocalDateRange plusEnd(TemporalAmount amountToAdd) {
        return between(begin, end.plus(amountToAdd));
    }

    /**
     * 通过添加时间获得一个新的区间
     * plus(1, ChronoUnit.MINUTES);
     * @param amountToAdd 添加时间
     * @param unit 单位
     */
    public LocalDateRange plus(long amountToAdd, TemporalUnit unit) {
        return between(begin.plus(amountToAdd, unit), end.plus(amountToAdd, unit));
    }

    /**
     * 通过给开始时间添加时间获得一个新的区间
     * plus(1, ChronoUnit.MINUTES);
     * @param amountToAdd 添加时间
     * @param unit 单位
     */
    public LocalDateRange plusBegin(long amountToAdd, TemporalUnit unit) {
        return between(begin.plus(amountToAdd, unit), end);
    }

    /**
     * 通过给结束时间添加时间获得一个新的区间
     * plus(1, ChronoUnit.MINUTES);
     * @param amountToAdd 添加时间
     * @param unit
     */
    public LocalDateRange plusEnd(long amountToAdd, TemporalUnit unit) {
        return between(begin, end.plus(amountToAdd, unit));
    }

    //===========================

    /**
     * 通过减少时间获得一个新的区间
     * minus(Period.of(year, monty, days));
     * minus(Period.ofYear(years));
     * minus(Duration.ofDays(days));
     * minus(Duration.ofSeconds(seconds));
     * @param amountToMinus 减少时间 {@link Period#parse(CharSequence)}} 或者 {@link Duration#parse(CharSequence)}
     */
    public LocalDateRange minus(TemporalAmount amountToMinus) {
        return between(begin.minus(amountToMinus), end.minus(amountToMinus));
    }

    /**
     * 通过给开始时间减少时间获得一个新的区间
     * minusBegin(Period.of(year, monty, days));
     * minusBegin(Period.ofYear(years));
     * minusBegin(Duration.ofDays(days));
     * minusBegin(Duration.ofSeconds(seconds));
     * @param amountToMinus 减少时间 {@link Period#parse(CharSequence)}} 或者 {@link Duration#parse(CharSequence)}
     */
    public LocalDateRange minusBegin(TemporalAmount amountToMinus) {
        return between(begin.minus(amountToMinus), end);
    }

    /**
     * 通过给结束时间减少时间获得一个新的区间
     * minusEnd(Period.of(year, monty, days));
     * minusEnd(Period.ofYear(years));
     * minusEnd(Duration.ofDays(days));
     * minusEnd(Duration.ofSeconds(seconds));
     * @param amountToMinus 减少时间 {@link Period#parse(CharSequence)}} 或者 {@link Duration#parse(CharSequence)}
     */
    public LocalDateRange minusEnd(TemporalAmount amountToMinus) {
        return between(begin, end.minus(amountToMinus));
    }

    /**
     * 通过减少时间获得一个新的区间
     * minus(1, ChronoUnit.MINUTES);
     * @param amountToMinus 减少时间
     * @param unit 单位
     */
    public LocalDateRange minus(long amountToMinus, TemporalUnit unit) {
        return between(begin.minus(amountToMinus, unit), end.minus(amountToMinus, unit));
    }

    /**
     * 通过给开始时间减少时间获得一个新的区间
     * minus(1, ChronoUnit.MINUTES);
     * @param amountToMinus 减少时间
     * @param unit 单位
     */
    public LocalDateRange minusBegin(long amountToMinus, TemporalUnit unit) {
        return between(begin.minus(amountToMinus, unit), end);
    }

    /**
     * 通过给结束时间减少时间获得一个新的区间
     * minus(1, ChronoUnit.MINUTES);
     * @param amountToMinus 减少时间
     * @param unit
     */
    public LocalDateRange minusEnd(long amountToMinus, TemporalUnit unit) {
        return between(begin, end.minus(amountToMinus, unit));
    }

    @Override
    public String toString() {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss.SSSSSSSSS");
        return "[" + begin.format(formatter) + " - " + end.format(formatter) + "]";
    }

    /**
     * 开始时间和结束时间是否为反的
     */
    public boolean isReverse() {
        return begin.isAfter(end);
    }

    /**
     * 反转开始和结束时间
     */
    public LocalDateRange reverse() {
        return between(end, begin);
    }

    public LocalDateRange setBegin(LocalDateTime begin) {
        return between(begin, end);
    }

    public LocalDateRange setEnd(LocalDateTime end) {
        return between(begin, end);
    }

    //======================================

    public static LocalDateRange between(LocalDateTime begin, LocalDateTime end) {
        return new LocalDateRange(begin, end);
    }

    public static LocalDateRange between(LocalDate begin, LocalDate end) {
        return between(asTime(begin), asTime(end));
    }

    public static LocalDateRange hour() {
        return hour(LocalDateTime.now());
    }

    public static LocalDateRange hour(LocalDateTime time) {
        return hour(0, time);
    }

    public static LocalDateRange hour(int offsetOfHour, LocalDateTime time) {
        LocalTime max = LocalTime.MAX;
        return between(
            time.toLocalDate().atTime(time.getHour(), 0, 0, 0),
            time.toLocalDate().atTime(time.getHour(), max.getMinute(), max.getSecond(), max.getNano())
        ).minus(offsetOfHour, ChronoUnit.HOURS);
    }

    public static LocalDateRange minute() {
        return minute(LocalDateTime.now());
    }

    public static LocalDateRange minute(LocalDateTime time) {
        return minute(0, time);
    }

    public static LocalDateRange minute(int offsetOfMinute, LocalDateTime time) {
        LocalTime max = LocalTime.MAX;
        return between(
                time.toLocalDate().atTime(time.getHour(), time.getMinute(), 0, 0),
                time.toLocalDate().atTime(time.getHour(), time.getMinute(), max.getSecond(), max.getNano())
        ).minus(offsetOfMinute, ChronoUnit.MINUTES);
    }

    public static LocalDateRange second() {
        return second(LocalDateTime.now());
    }

    public static LocalDateRange second(LocalDateTime time) {
        return second(0, time);
    }

    public static LocalDateRange second(int offsetOfSecond, LocalDateTime time) {
        LocalTime max = LocalTime.MAX;
        return between(
                time.toLocalDate().atTime(time.getHour(), time.getMinute(), time.getSecond(), 0),
                time.toLocalDate().atTime(time.getHour(), time.getMinute(), time.getSecond(), max.getNano())
        ).minus(offsetOfSecond, ChronoUnit.SECONDS);
    }

    /**
     * 以 当前 所在时间，获取这一天的时间区间
     */
    public static LocalDateRange day() {
        return day(LocalDateTime.now());
    }

    /**
     * 以 当前 所在时间，获取这一天的时间区间
     * @param offsetOfDay 偏移天数
     */
    public static LocalDateRange day(int offsetOfDay) {
        return day(LocalDateTime.now(), offsetOfDay);
    }

    /**
     * 以 time 所在时间，获取这一天的时间区间
     * @param time 时间
     */
    public static LocalDateRange day(LocalDateTime time) {
        return day(time.toLocalDate());
    }

    /**
     * 以 date 所在时间，获取这一天的时间区间
     * @param date 时间
     */
    public static LocalDateRange day(LocalDate date) {
        return day(date, 0);
    }

    /**
     * 以 date 所在时间，获取这一天的时间区间
     * @param date 时间
     * @param offsetOfDay 偏移天数
     */
    public static LocalDateRange day(LocalDateTime date, int offsetOfDay) {
        return day(date.toLocalDate(), offsetOfDay);
    }

    /**
     * 以 date 所在时间，获取这一天的时间区间
     * @param date 时间
     * @param offsetOfDay 偏移天数
     */
    public static LocalDateRange day(LocalDate date, int offsetOfDay) {
        date = date.plusDays(offsetOfDay);
        return between(dayStart(date), dayEnd(date));
    }


    /**
     * 以 当前 所在时间，获取这一周的时间区间
     */
    public static LocalDateRange week() {
        return week(LocalDateTime.now());
    }

    /**
     * 以 当前 所在时间，获取这一周的时间区间
     * @param offsetOfWeek 偏移周数
     */
    public static LocalDateRange week(int offsetOfWeek) {
        return week(LocalDateTime.now(), offsetOfWeek);
    }

    /**
     * 以 time 所在时间，获取这一周的时间区间
     * @param date 时间
     */
    public static LocalDateRange week(LocalDateTime date) {
        return week(date, 0);
    }

    /**
     * 以 date 所在时间，获取这一周的时间区间
     * @param date 时间
     */
    public static LocalDateRange week(LocalDate date) {
        return week(date, 0);
    }

    /**
     * 以 time 所在时间，获取这一周的时间区间
     * @param date 时间
     * @param offsetOfWeek 偏移多少周
     */
    public static LocalDateRange week(LocalDateTime date, int offsetOfWeek) {
        return week(date.toLocalDate(), offsetOfWeek);
    }

    /**
     * 以 date 所在时间，获取这一周的时间区间
     * @param date 时间
     * @param offsetOfWeek 偏移多少周
     */
    public static LocalDateRange week(LocalDate date, int offsetOfWeek) {
        date = date.plusWeeks(offsetOfWeek);
        TemporalField fieldIso = WeekFields.of(DayOfWeek.MONDAY, 1).dayOfWeek();
        LocalDate b = date.with(fieldIso, 1);
        LocalDate e = date.with(fieldIso, 7);
        return between(dayStart(b), dayEnd(e));
    }


    /**
     * 以 当前 所在时间，获取这个月第一周的时间区间
     */
    public static LocalDateRange firstWeek() {
        return firstWeek(LocalDate.now(), 0);
    }

    /**
     * 以 当前 所在时间，获取这个月第一周的时间区间
     * @param offsetOfMonth 偏移月数
     */
    public static LocalDateRange firstWeek(int offsetOfMonth) {
        return firstWeek(LocalDate.now(), offsetOfMonth);
    }

    /**
     * 以 date 所在时间，获取这个月第一周的时间区间
     * @param date 时间
     */
    public static LocalDateRange firstWeek(LocalDate date) {
        return firstWeek(date, 0);
    }

    /**
     * 以 time 所在时间，获取这个月第一周的时间区间
     * @param time 时间
     */
    public static LocalDateRange firstWeek(LocalDateTime time) {
        return firstWeek(time.toLocalDate(), 0);
    }

    /**
     * 以 time 所在时间，获取这个月第一周的时间区间
     * @param time 时间
     * @param offsetOfMonth 偏移多少月
     */
    public static LocalDateRange firstWeek(LocalDateTime time, int offsetOfMonth) {
        return firstWeek(time.toLocalDate(), offsetOfMonth);
    }

    /**
     * 以 date 所在时间，获取这个月第一周的时间区间
     * @param date 时间
     * @param offsetOfMonth 偏移多少月
     */
    public static LocalDateRange firstWeek(LocalDate date, int offsetOfMonth) {
        date = date.plusMonths(offsetOfMonth).with(TemporalAdjusters.firstDayOfMonth());
        return week(date);
    }


    /**
     * 以 当前 所在时间，获取这个月第一周的时间区间
     */
    public static LocalDateRange lastWeek() {
        return lastWeek(LocalDate.now(), 0);
    }

    /**
     * 以 当前 所在时间，获取这个月第一周的时间区间
     * @param offsetOfMonth 偏移月数
     */
    public static LocalDateRange lastWeek(int offsetOfMonth) {
        return lastWeek(LocalDate.now(), offsetOfMonth);
    }

    /**
     * 以 time 所在时间，获取这个月第一周的时间区间
     * @param time 时间
     */
    public static LocalDateRange lastWeek(LocalDateTime time) {
        return lastWeek(time.toLocalDate(), 0);
    }

    /**
     * 以 date 所在时间，获取这个月第一周的时间区间
     * @param date 时间
     */
    public static LocalDateRange lastWeek(LocalDate date) {
        return lastWeek(date, 0);
    }

    /**
     * 以 time 所在时间，获取这个月第一周的时间区间
     * @param time 时间
     * @param offsetOfMonth 偏移多少月
     */
    public static LocalDateRange lastWeek(LocalDateTime time, int offsetOfMonth) {
        return lastWeek(time.toLocalDate(), offsetOfMonth);
    }

    /**
     * 以 date 所在时间，获取这个月第一周的时间区间
     * @param date 时间
     * @param offsetOfMonth 偏移多少月
     */
    public static LocalDateRange lastWeek(LocalDate date, int offsetOfMonth) {
        date = date.plusMonths(offsetOfMonth).with(TemporalAdjusters.lastDayOfMonth());
        return week(date);
    }


    /**
     * 以 当前 所在时间，获取这一月的时间区间
     */
    public static LocalDateRange month() {
        return month(LocalDate.now(), 0);
    }

    /**
     * 以 当前 所在时间，获取这一月的时间区间
     * @param offsetOfMonth 偏移月数
     */
    public static LocalDateRange month(int offsetOfMonth) {
        return month(LocalDate.now(), offsetOfMonth);
    }

    /**
     * 以 time 所在时间，获取这一月的时间区间
     * @param date 时间
     */
    public static LocalDateRange month(LocalDateTime date) {
        return month(date.toLocalDate(), 0);
    }

    /**
     * 以 date 所在时间，获取这一月的时间区间
     * @param date 时间
     */
    public static LocalDateRange month(LocalDate date) {
        return month(date, 0);
    }

    /**
     * 以 time 所在时间，获取这一月的时间区间
     * @param date 时间
     * @param offsetOfMonth 偏移月数
     */
    public static LocalDateRange month(LocalDateTime date, int offsetOfMonth) {
        return month(date.toLocalDate(), offsetOfMonth);
    }

    /**
     * 以 date 所在时间，获取这一月的时间区间
     * @param date 时间
     * @param offsetOfMonth 偏移月数
     */
    public static LocalDateRange month(LocalDate date, int offsetOfMonth) {
        date = date.plusMonths(offsetOfMonth);
        LocalDate b = date.with(TemporalAdjusters.firstDayOfMonth());
        LocalDate e = date.with(TemporalAdjusters.lastDayOfMonth());
        return between(dayStart(b), dayEnd(e));
    }


    /**
     * 以 当前 所在时间，获取这一年的时间区间
     */
    public static LocalDateRange year() {
        return year(LocalDate.now(), 0);
    }

    /**
     * 以 当前 所在时间，获取这一年的时间区间
     * @param offsetOfYear 偏移年数
     */
    public static LocalDateRange year(int offsetOfYear) {
        return year(LocalDate.now(), offsetOfYear);
    }

    /**
     * 以 time 所在时间，获取这一年的时间区间
     * @param date 时间
     */
    public static LocalDateRange year(LocalDateTime date) {
        return year(date.toLocalDate(), 0);
    }

    /**
     * 以 date 所在时间，获取这一年的时间区间
     * @param date 时间
     */
    public static LocalDateRange year(LocalDate date) {
        return year(date, 0);
    }

    /**
     * 以 time 所在时间，获取这一年的时间区间
     * @param date 时间
     * @param offsetOfYear 偏移年数
     */
    public static LocalDateRange year(LocalDateTime date, int offsetOfYear) {
        return year(date.toLocalDate(), offsetOfYear);
    }

    /**
     * 以 date 所在时间，获取这一年的时间区间
     * @param date 时间
     * @param offsetOfYear 偏移年数
     */
    public static LocalDateRange year(LocalDate date, int offsetOfYear) {
        date = date.plusYears(offsetOfYear);
        LocalDate b = date.with(TemporalAdjusters.firstDayOfYear());
        LocalDate e = date.with(TemporalAdjusters.lastDayOfYear());
        return between(dayStart(b), dayEnd(e));
    }



    //======================================


    /**
     * 获取一天的开始时间
     */
    private static LocalDateTime dayStart(LocalDate date) {
        return date.atStartOfDay();
    }

    /**
     * 获取一天的结束时间
     */
    private static LocalDateTime dayEnd(LocalDate date) {
        return date.atTime(LocalTime.MAX);
    }

    /**
     * date 转  datetime
     */
    private static LocalDateTime asTime(LocalDate date) {
        return date.atTime(0, 0, 0);
    }

}
