package com.chuang.telegram.bot;

import com.chuang.tauceti.tools.basic.StringKit;
import com.chuang.telegram.bot.configs.TelegramBotConfig;
import com.chuang.telegram.bot.interfaces.Sender;
import com.chuang.telegram.bot.interfaces.WebHookBots;
import com.chuang.telegram.bot.methods.updates.DeleteWebhook;
import com.chuang.telegram.bot.methods.updates.GetWebhookInfo;
import com.chuang.telegram.bot.methods.updates.SetWebhook;
import com.chuang.telegram.bot.model.Update;
import com.chuang.telegram.bot.model.hook.HookUpdate;
import com.chuang.telegram.bot.model.hook.WebHookBot;
import com.chuang.telegram.bot.model.hook.WebhookInfo;
import com.google.inject.Inject;
import com.google.inject.Singleton;
import lombok.extern.slf4j.Slf4j;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Sinks;

import javax.annotation.Nullable;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@Slf4j
@Singleton
public class DefaultWebHookBots implements WebHookBots {

    private final Map<String, WebHookBot> keyHook = new ConcurrentHashMap<>();
    private final Map<String, WebHookBot> pathHook = new ConcurrentHashMap<>();
    private final Sender sender;
    private final TelegramBotConfig config;

    private final Sinks.Many<HookUpdate> sink;
    private final Flux<HookUpdate> flux;

    @Inject
    public DefaultWebHookBots(Sender sender, TelegramBotConfig config) {
        this.sender = sender;
        this.config = config;
        sink = Sinks.many().unicast().onBackpressureBuffer();
        flux = sink.asFlux();
    }


    @Override
    public void firePathUpdate(String listenPath, Update update, @Nullable Object payload) {
        WebHookBot hook = pathHook.get(listenPath);
        if(null == hook) {
            log.warn("一个update被挂在了{}, 无法找到相关的监听实体，请仔细检查", listenPath);
            return;
        }
        sink.emitNext(new HookUpdate(hook, update, payload), (signalType, emitResult) -> {
            log.warn("一个Update推送失败,类型为:{}, 错误信息：{}, 系统将放弃本次Update推送", signalType, emitResult);
            return false;
        });
    }

    @Override
    public void fireKeyUpdate(String botKey, Update update, @Nullable Object payload) {
        WebHookBot hook = keyHook.get(botKey);
        if(null == hook) {
            log.warn("一个update被发送到了{}, 无法找到相关的监听实体，请仔细检查", botKey);
            return;
        }
        sink.emitNext(new HookUpdate(hook, update, payload), (signalType, emitResult) -> {
            log.warn("一个Update推送失败,类型为:{}, 错误信息：{}, 系统将放弃本次Update推送", signalType, emitResult);
            return false;
        });
    }

    @Override
    public void listen(String botKey, String token, String listenPath, boolean forceRestart, boolean dropHistory) {
        keyHook.put(botKey, new WebHookBot(botKey, token, listenPath));
        pathHook.put(listenPath, new WebHookBot(botKey, token, listenPath));
        WebhookInfo info = getWebhookInfo(botKey);
        if(StringKit.isNotBlank(info.getUrl())) {
            if(forceRestart) {
                new DeleteWebhook().send(sender,botKey).join();
                new SetWebhook()
                        .setDropPendingUpdates(dropHistory)
                        .setUrl(config.getListenServer() + listenPath)
                        .setSecretToken(token)
                        .send(sender, botKey)
                        .join();
            }
        } else {
            new SetWebhook()
                    .setUrl(config.getListenServer() + listenPath)
                    .setDropPendingUpdates(dropHistory)
                    .setSecretToken(token)
                    .send(sender, botKey)
                    .join();
        }
    }

    @Override
    public void stopListen(String botKey, boolean stopHook) {
        WebHookBot bot = keyHook.remove(botKey);
        pathHook.remove(bot.getListenPath());

        if(stopHook) {
            new DeleteWebhook().send(sender, botKey);
        }
    }

    @Override
    public void stopListen(boolean stopHook) {
        keyHook.keySet().forEach(s -> stopListen(s, stopHook));
    }


    @Override
    public WebhookInfo getWebhookInfo(String botKey) {
        return new GetWebhookInfo().send(sender, botKey).join();
    }

    @Override
    public WebHookBot getBot(String botKey) {
        return keyHook.get(botKey);
    }

    @Override
    public WebHookBot getBotByPath(String path) {
        return pathHook.get(path);
    }


    @Override
    public Flux<HookUpdate> asFlux() {
        return flux;
    }

}
