package com.chuang.telegram.bot.interfaces;

import com.chuang.tauceti.httpclient.HttpMethod;
import com.chuang.tauceti.httpclient.Request;
import com.chuang.tauceti.httpclient.Response;
import com.chuang.tauceti.httpclient.async.IAsyncHttpClient;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.basic.FutureKit;
import com.chuang.telegram.bot.exception.TelegramCallApiException;
import com.chuang.telegram.bot.model.BotApiResponse;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.http.Consts;
import org.apache.http.HttpHeaders;
import org.apache.http.entity.ContentType;

import java.io.Serializable;
import java.util.concurrent.CompletableFuture;

@JsonIgnoreProperties(ignoreUnknown = true)
@JsonInclude(JsonInclude.Include.NON_NULL)
public interface BotApiMethod<T extends Serializable> extends Validable {
    ContentType TEXT_PLAIN_UTF8 = ContentType.create("text/plain", Consts.UTF_8);

    @JsonIgnore
    ObjectMapper OBJECT_MAPPER = new ObjectMapper();
    String METHOD_FIELD = "method";

    /**
     * Getter for method path (that is the same as method name)
     * @return Method path
     */
    @JsonProperty(METHOD_FIELD)
    String getMethod();

    default Request asRequest(String api, String botKey) throws Exception {
        String body = OBJECT_MAPPER.writeValueAsString(this);
        return Request.newBuilder()
                .header(HttpHeaders.CONTENT_TYPE, "application/json")
                .url(requestUrl(api, botKey))
                .method(HttpMethod.POST)
                .body(body)
                .build();
    }


    default CompletableFuture<T> send(String api, String botKey, IAsyncHttpClient client) {
        try {
            validate();
            return this.asRequest(api, botKey)
                    .execute(client)
                    .thenApply(Response::asString)
                    .thenApply(this::deserialize);
        } catch (Exception e) {
            return FutureKit.error(e);
        }
    }

    default CompletableFuture<T> send(Sender sender, String botKey) {
        return sender.asyncSend(botKey, this);
    }

    default String requestUrl(String api, String botKey) {
        return api + "/bot" + botKey + "/" + getMethod();
    }

    default T deserialize(String answer) {
        BotApiResponse<T> bar;
        try {
            bar = OBJECT_MAPPER.readValue(answer, typeReference());
        } catch (JsonProcessingException e) {
            throw new BusinessException(getMethod() + " response JSON 解析异常", e);
        }

        if (bar.getOk()) {
            return bar.getResult();
        } else {
            throw new TelegramCallApiException(
                    getMethod() + " request 错误:->" + bar.getErrorDescription(),
                    bar.getErrorCode(),
                    bar.getParameters());
        }
    }

    default TypeReference<BotApiResponse<T>> typeReference() {
        return new TypeReference<BotApiResponse<T>>() {};
    }
}
