package com.chuang.telegram.bot.methods;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.chuang.telegram.bot.model.inlinequery.result.InlineQueryResult;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import java.util.Arrays;
import java.util.List;
import java.util.regex.Pattern;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Use this method to send answers to an inline query. On success, True is returned.
 */
@Data
@Accessors(chain = true)
public class AnswerInlineQuery implements BotApiMethod<Boolean> {
    public static final String PATH = "answerInlineQuery";

    private static final String INLINE_QUERY_ID_FIELD = "inline_query_id";
    private static final String RESULTS_FIELD = "results";
    private static final String CACHE_TIME_FIELD = "cache_time";
    private static final String IS_PERSONAL_FIELD = "is_personal";
    private static final String NEXT_OFFSET_FIELD = "next_offset";
    private static final String SWITCH_PM_TEXT_FIELD = "switch_pm_text";
    private static final String SWITCH_PM_PARAMETER_FIELD = "switch_pm_parameter";

	/**  Unique identifier for answered query */
    @JsonProperty(INLINE_QUERY_ID_FIELD)
    private String inlineQueryId;
	/**  A JSON-serialized array of results for the inline query */
    @JsonProperty(RESULTS_FIELD)
    private List<InlineQueryResult> results;
	/**  Optional	The maximum amount of time the result of the inline query may be cached on the server */
    @JsonProperty(CACHE_TIME_FIELD)
    private Long cacheTime;
	/**  Pass True, if results may be cached on the server side only for the user that sent the query. By default, results may be returned to any user who sends the same query */
    @JsonProperty(IS_PERSONAL_FIELD)
    private Boolean isPersonal;
	/**  Optional. Pass the offset that a client should send in the next query with the same text to receive more results. Pass an empty string if there are no more results or if you don‘t support pagination. Offset length can’t exceed 64 bytes. */
    @JsonProperty(NEXT_OFFSET_FIELD)
    private String nextOffset;
	/**  Optional. If passed, clients will display a button with specified text that switches the user to a private chat with the bot and sends the bot a start message with the parameter switch_pm_parameter */
    @JsonProperty(SWITCH_PM_TEXT_FIELD)
    private String switchPmText;
	/**  Optional. Parameter for the start message sent to the bot when user presses the switch button */
    @JsonProperty(SWITCH_PM_PARAMETER_FIELD)
    private String switchPmParameter;


    public AnswerInlineQuery setResults(List<InlineQueryResult> results) {
        this.results = results;
        return this;
    }

    @JsonIgnore
    public AnswerInlineQuery setResults(InlineQueryResult... results) {
        this.results = Arrays.asList(results);
        return this;
    }

    @Override
    public void validate() throws TelegramApiValidationException {
        if (inlineQueryId == null || inlineQueryId.isEmpty()) {
            throw new TelegramApiValidationException("InlineQueryId can't be empty", this);
        }
        if (results == null) {
            throw new TelegramApiValidationException("Results array can't be null", this);
        }
        if (switchPmText != null) {
            if (switchPmText.isEmpty()) {
                throw new TelegramApiValidationException("SwitchPmText can't be empty", this);
            }
            if (switchPmParameter == null || switchPmParameter.isEmpty()) {
                throw new TelegramApiValidationException("SwitchPmParameter can't be empty if switchPmText is present", this);
            }
            if (switchPmParameter.length() > 64) {
                throw new TelegramApiValidationException("SwitchPmParameter can't be longer than 64 chars", this);
            }
            if (!Pattern.matches("[A-Za-z0-9_]+", switchPmParameter.trim() )) {
                throw new TelegramApiValidationException("SwitchPmParameter only allows A-Z, a-z, 0-9 and _ characters", this);
            }
        }
        for (InlineQueryResult result : results) {
            result.validate();
        }
    }

    @Override
    public String getMethod() {
        return PATH;
    }

}
