package com.chuang.telegram.bot.methods;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 1.0
 *
 * Once the user has confirmed their payment and shipping details, the Bot API sends the final confirmation
 * in the form of an Update with the field pre_checkout_query. Use this method to respond to such pre-checkout queries.
 *
 * On success, True is returned
 *
 * @apiNote The Bot API must receive an answer within 10 seconds after the pre-checkout query was sent.
 */
@Data
@Accessors(chain = true)
public class AnswerPreCheckoutQuery implements BotApiMethod<Boolean> {
    public static final String PATH = "answerPreCheckoutQuery";

    private static final String PRE_CHECKOUT_QUERY_ID_FIELD = "pre_checkout_query_id";
    private static final String OK_FIELD = "ok";
    private static final String ERROR_MESSAGE_FIELD = "error_message";

	/**  Unique identifier for the query to be answered */
    @JsonProperty(PRE_CHECKOUT_QUERY_ID_FIELD)
    private String preCheckoutQueryId;
	/**  Specify True if everything is alright (goods are available, etc.) and the bot is ready to proceed with the order. Use False if there are any problems. */
    @JsonProperty(OK_FIELD)
    private Boolean ok;
	/**  Optional. Required if ok is False. Error message in human readable form that explains the reason for failure to proceed with the checkout */
    @JsonProperty(ERROR_MESSAGE_FIELD)
    private String errorMessage;

    /**
     * Creates an empty answer pre-checkout query
     */
    public AnswerPreCheckoutQuery() {
        super();
    }

    /**
     * Creates an answer pre-checkout query with mandatory parameters
     * @param preCheckoutQueryId Unique identifier for the query to be answered
     * @param ok Specify True if delivery to the specified address is possible and False if there are any problems
     */
    public AnswerPreCheckoutQuery(String preCheckoutQueryId, Boolean ok) {
        this.preCheckoutQueryId = checkNotNull(preCheckoutQueryId);
        this.ok = checkNotNull(ok);
    }


    @Override
    public void validate() throws TelegramApiValidationException {
        if (preCheckoutQueryId == null || preCheckoutQueryId.isEmpty()) {
            throw new TelegramApiValidationException("PreCheckoutQueryId can't be empty", this);
        }
        if (ok == null) {
            throw new TelegramApiValidationException("Ok can't be null", this);
        }
        if (!ok) {
            if (errorMessage == null || errorMessage.isEmpty()) {
                throw new TelegramApiValidationException("ErrorMessage can't be empty if not ok", this);
            }
        }
    }

    @Override
    public String getMethod() {
        return PATH;
    }

}
