package com.chuang.telegram.bot.methods;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.AnyBotApiMethod;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import java.util.Objects;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Use this method to send text messages. On success, the sent Message is returned.
 */
@Data
@Accessors(chain = true)
public class ForwardMessage implements AnyBotApiMethod {
    public static final String PATH = "forwardmessage";

    private static final String CHAT_ID_FIELD = "chat_id";
    private static final String FROM_CHAT_ID_FIELD = "from_chat_id";
    private static final String MESSAGE_ID_FIELD = "message_id";
    private static final String DISABLE_NOTIFICATION_FIELD = "disable_notification";

	/**  Unique identifier for the chat to send the message to (or username for channels) */
    @JsonProperty(CHAT_ID_FIELD)
    private String chatId;
	/**  Unique identifier for the chat where the original message was sent — User or GroupChat id */
    @JsonProperty(FROM_CHAT_ID_FIELD)
    private String fromChatId;
	/**  Unique message identifier */
    @JsonProperty(MESSAGE_ID_FIELD)
    private Long messageId;
    /**
     * Optional. Sends the message silently.
     * iOS users will not receive a notification,
     * Android users will receive a notification with no sound.
     * Other apps coming soon
     */
    @JsonProperty(DISABLE_NOTIFICATION_FIELD)
    private Boolean disableNotification;

    public ForwardMessage() {
        super();
    }

    public ForwardMessage(String chatId, String fromChatId, Long messageId) {
        this();
        Objects.requireNonNull(chatId);
        Objects.requireNonNull(fromChatId);
        this.chatId = chatId;
        this.fromChatId = fromChatId;
        this.messageId = messageId;
    }

    public ForwardMessage(String chatId, Long fromChatId, Long messageId) {
        this();
        Objects.requireNonNull(chatId);
        Objects.requireNonNull(fromChatId);
        this.chatId = chatId;
        this.fromChatId = fromChatId.toString();
        this.messageId = messageId;
    }

    public ForwardMessage(Long chatId, String fromChatId, Long messageId) {
        this();
        Objects.requireNonNull(chatId);
        Objects.requireNonNull(fromChatId);
        this.chatId = chatId.toString();
        this.fromChatId = fromChatId;
        this.messageId = messageId;
    }

    public ForwardMessage(Long chatId, Long fromChatId, Long messageId) {
        this();
        Objects.requireNonNull(chatId);
        Objects.requireNonNull(fromChatId);
        this.chatId = chatId.toString();
        this.fromChatId = fromChatId.toString();
        this.messageId = messageId;
    }

    public ForwardMessage setChatId(String chatId) {
        this.chatId = chatId;
        return this;
    }

    public ForwardMessage setChatId(Long chatId) {
        Objects.requireNonNull(chatId);
        this.chatId = chatId.toString();
        return this;
    }

    public ForwardMessage setFromChatId(String fromChatId) {
        this.fromChatId = fromChatId;
        return this;
    }

    public ForwardMessage setFromChatId(Long fromChatId) {
        Objects.requireNonNull(fromChatId);
        this.fromChatId = fromChatId.toString();
        return this;
    }

    public ForwardMessage enableNotification() {
        this.disableNotification = false;
        return this;
    }

    public ForwardMessage disableNotification() {
        this.disableNotification = true;
        return this;
    }

    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null || chatId.isEmpty()) {
            throw new TelegramApiValidationException("ChatId can't be empty", this);
        }
        if (fromChatId == null || fromChatId.isEmpty()) {
            throw new TelegramApiValidationException("FromChatId can't be empty", this);
        }
        if (messageId == null) {
            throw new TelegramApiValidationException("MessageId can't be empty", this);
        }
    }

    @Override
    public String getMethod() {
        return PATH;
    }

}
