/*
 * This file is part of TelegramBots.
 *
 * TelegramBots is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TelegramBots is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TelegramBots.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.chuang.telegram.bot.methods.games;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.AnyBotApiMethod;
import com.chuang.telegram.bot.model.BotApiResponse;
import com.chuang.telegram.bot.model.games.GameHighScore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.type.TypeReference;
import lombok.Data;
import lombok.experimental.Accessors;

import java.util.ArrayList;

/**
 * @author Ruben Bermudez
 * @version 2.4
 * Use this method to set the score of the specified user in a game.
 * On success, if the message was sent by the bot, returns the edited Message,
 * otherwise returns True.
 *
 * Returns an error, if the new score is not greater than the user's current score in
 * the chat and force is False.
 */
@Data
@Accessors(chain = true)
public class SetGameScore implements AnyBotApiMethod {
    public static final String PATH = "setGameScore";

    private static final String USER_ID_FIELD = "user_id";
    private static final String SCORE_FIELD = "score";
    private static final String CHAT_ID_FIELD = "chat_id";
    private static final String MESSAGE_ID_FIELD = "message_id";
    private static final String INLINE_MESSAGE_ID_FIELD = "inline_message_id";
    private static final String DISABLE_EDIT_MESSAGE_FIELD = "disable_edit_message";
    private static final String FORCE_FIELD = "force";

	/**  Optional	Required if inline_message_id is not specified. Unique identifier for the target chat (or username of the target channel in the format @channelusername) */
    @JsonProperty(CHAT_ID_FIELD)
    private String chatId;
	/**  Optional	Required if inline_message_id is not specified. Unique identifier of the sent message */
    @JsonProperty(MESSAGE_ID_FIELD)
    private Long messageId;
	/**  Optional	Required if chat_id and message_id are not specified. Identifier of the inline message */
    @JsonProperty(INLINE_MESSAGE_ID_FIELD)
    private String inlineMessageId;
	/**  Optional	Pass True, if the game message should not be automatically edited to include the current scoreboard. Defaults to False */
    @JsonProperty(DISABLE_EDIT_MESSAGE_FIELD)
    private Boolean disableEditMessage;
	/**  User identifier */
    @JsonProperty(USER_ID_FIELD)
    private Long userId;
	/**  New score, must be positive */
    @JsonProperty(SCORE_FIELD)
    private Long score;
	/**  Opfional. Pass True, if the high score is allowed to decrease. This can be useful when fixing mistakes or banning cheaters */
    @JsonProperty(FORCE_FIELD)
    private Boolean force;


    @Override
    public String getMethod() {
        return PATH;
    }


    @Override
    public void validate() throws TelegramApiValidationException {
        if (userId == null) {
            throw new TelegramApiValidationException("UserId parameter can't be empty", this);
        }
        if (score == null) {
            throw new TelegramApiValidationException("Score parameter can't be empty", this);
        }
        if (inlineMessageId == null) {
            if (chatId == null) {
                throw new TelegramApiValidationException("ChatId parameter can't be empty if inlineMessageId is not present", this);
            }
            if (messageId == null) {
                throw new TelegramApiValidationException("MessageId parameter can't be empty if inlineMessageId is not present", this);
            }
        } else {
            if (chatId != null) {
                throw new TelegramApiValidationException("ChatId parameter must be empty if inlineMessageId is provided", this);
            }
            if (messageId != null) {
                throw new TelegramApiValidationException("MessageId parameter must be empty if inlineMessageId is provided", this);
            }
        }
    }

}
