package com.chuang.telegram.bot.methods.groupadmin;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import java.util.Objects;

/**
 * @author Ruben Bermudez
 * @version 3.1
 * Use this method to restrict a user in a supergroup.
 * The bot must be an administrator in the supergroup for this to work and must have the appropriate admin rights.
 * Pass True for all boolean parameters to lift restrictions from a user. Returns True on success.
 *
 */
@Data
@Accessors(chain = true)
public class RestrictChatMember implements BotApiMethod<Boolean> {
    public static final String PATH = "restrictchatmember";

    private static final String CHAT_ID_FIELD = "chat_id";
    private static final String USER_ID_FIELD = "user_id";
    private static final String UNTIL_DATE_FIELD = "until_date";
    private static final String CAN_SEND_MESSAGES_FIELD = "can_send_messages";
    private static final String CAN_SEND_MEDIA_MESSAGES_FIELD = "can_send_media_messages";
    private static final String CAN_SEND_OTHER_MESSAGES_FIELD = "can_send_other_messages";
    private static final String CAN_ADD_WEB_PAGE_PREVIEWS_FIELD = "can_add_web_page_previews";

	/**  Required. Unique identifier for the chat to send the message to (Or username for channels) */
    @JsonProperty(CHAT_ID_FIELD)
    private String chatId;
	/**  Required. Unique identifier of the target user */
    @JsonProperty(USER_ID_FIELD)
    private Long userId;
	/**  Date when restrictions will be lifted for the user, unix time. If user is restricted for more than 366 days or less than 30 seconds from the current time, they are considered to be banned forever */
    @JsonProperty(UNTIL_DATE_FIELD)
    private Long untilDate;
	/**  Pass True, if the user can send text messages, contacts, locations and venues */
    @JsonProperty(CAN_SEND_MESSAGES_FIELD)
    private Boolean canSendMessages;
	/**  Pass True, if the user can send audios, documents, photos, videos, video notes and voice notes, implies can_send_messages */
    @JsonProperty(CAN_SEND_MEDIA_MESSAGES_FIELD)
    private Boolean canSendMediaMessages;
	/**  Pass True, if the user can send animations, games, stickers and use inline bots, implies can_send_media_messages */
    @JsonProperty(CAN_SEND_OTHER_MESSAGES_FIELD)
    private Boolean canSendOtherMessages;
	/**  Pass True, if the user may add web page previews to their messages, implies can_send_messages */
    @JsonProperty(CAN_ADD_WEB_PAGE_PREVIEWS_FIELD)
    private Boolean canAddWebPagePreviews;


    public RestrictChatMember() {
        super();
    }

    public RestrictChatMember(String chatId, Long userId) {
        this.chatId = checkNotNull(chatId);
        this.userId = checkNotNull(userId);
    }

    public RestrictChatMember(Long chatId, Long userId) {
        this.chatId = checkNotNull(chatId).toString();
        this.userId = checkNotNull(userId);
    }

    public RestrictChatMember setChatId(String chatId) {
        this.chatId = chatId;
        return this;
    }

    public RestrictChatMember setChatId(Long chatId) {
        Objects.requireNonNull(chatId);
        this.chatId = chatId.toString();
        return this;
    }

    @Override
    public String getMethod() {
        return PATH;
    }

    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null || chatId.isEmpty()) {
            throw new TelegramApiValidationException("ChatId can't be empty", this);
        }
        if (userId == null) {
            throw new TelegramApiValidationException("UserId can't be empty", this);
        }
    }

}
