package com.chuang.telegram.bot.methods.groupadmin;

import com.chuang.tauceti.httpclient.Request;
import com.chuang.telegram.bot.HttpUtils;
import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.chuang.telegram.bot.methods.stickers.AddStickerToSet;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.MultipartEntityBuilder;

import java.io.File;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

/**
 * @author Ruben Bermudez
 * @version 3.1
 * Use this method to set a new profile photo for the chat. Photos can't be changed for private chats.
 * The bot must be an administrator in the chat for this to work and must have the appropriate admin rights.
 * Returns True on success.
 *
 * @apiNote In regular groups (non-supergroups), this method will only work if the ‘All Members Are Admins’ setting is off in the target group.
 */
@Data
@Accessors(chain = true)
public class SetChatPhoto implements BotApiMethod<Boolean> {
    public static final String PATH = "setChatPhoto";

    public static final String CHAT_ID_FIELD = "chat_id";
    public static final String PHOTO_FIELD = "photo";

    /** Unique identifier for the target chat or username of the target channel (in the format @channelusername) */
    @JsonProperty(CHAT_ID_FIELD)
    private String chatId;
    /** Name of new chat photo */
    private String photoName;
    /** New chat photo as InputStream, uploaded using multipart/form-data */
    private InputStream photoStream;
    /** New chat photo as File, uploaded using multipart/form-data */
    @JsonProperty(PHOTO_FIELD)
    private File photo;

    public SetChatPhoto() {}

    public SetChatPhoto(String chatId, File photo) {
        super();
        this.chatId = checkNotNull(chatId);
        this.photo = checkNotNull(photo);
    }

    public SetChatPhoto(String chatId, InputStream photoStream, String photoName) {
        super();
        this.chatId = checkNotNull(chatId);
        this.photoStream = checkNotNull(photoStream);
        this.photoName = checkNotNull(photoName);
    }


    public SetChatPhoto(Long chatId, File photo) {
        super();
        this.chatId = checkNotNull(chatId).toString();
        this.photo = checkNotNull(photo);
    }

    public SetChatPhoto(Long chatId, InputStream photoStream, String photoName) {
        super();
        this.chatId = checkNotNull(chatId).toString();
        this.photoStream = checkNotNull(photoStream);
        this.photoName = checkNotNull(photoName);
    }

    public SetChatPhoto setChatId(String chatId) {
        this.chatId = chatId;
        return this;
    }

    public SetChatPhoto setChatId(Long chatId) {
        Objects.requireNonNull(chatId);
        this.chatId = chatId.toString();
        return this;
    }


    public SetChatPhoto setNewPhoto(String photoName, InputStream inputStream) {
        Objects.requireNonNull(photoName, "photoName cannot be null!");
        Objects.requireNonNull(inputStream, "inputStream cannot be null!");
        this.photoName = photoName;
        this.photoStream = inputStream;
        return this;
    }


    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null || chatId.isEmpty()) {
            throw new TelegramApiValidationException("ChatId can't be empty", this);
        }
        if (photo == null) {
            if (photoStream == null) {
                throw new TelegramApiValidationException("Photo parameter is required", this);
            } else if (photoName == null || photoName.isEmpty()){
                throw new TelegramApiValidationException("Photo name can't be empty", this);
            }
        }
    }

    @Override
    public String getMethod() {
        return PATH;
    }

    @Override
    public Request asRequest(String api, String botKey) throws Exception {

        MultipartEntityBuilder builder = MultipartEntityBuilder.create()
                .setLaxMode()
                .setCharset(StandardCharsets.UTF_8)
                .addTextBody(CHAT_ID_FIELD, getChatId(), TEXT_PLAIN_UTF8);

        if (getPhoto() != null) {
            builder.addBinaryBody(PHOTO_FIELD, getPhoto());
        } else if (getPhotoStream() != null) {
            builder.addBinaryBody(PHOTO_FIELD, getPhotoStream(), ContentType.APPLICATION_OCTET_STREAM, getPhotoName());
        }

        return Request.Post(requestUrl(api, botKey))
                .entity(builder.build())
                .build();
    }
}
