package com.chuang.telegram.bot.methods.groupadmin;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import java.util.Objects;

/**
 * @author Ruben Bermudez
 * @version 3.1
 * Use this method to change the title of a chat. Titles can't be changed for private chats.
 * The bot must be an administrator in the chat for this to work and must have the appropriate admin rights.
 * Returns True on success.
 *
 * @apiNote In regular groups (non-supergroups), this method will only work if the ‘All Members Are Admins’ setting is off in the target group.
 */
@Data
@Accessors(chain = true)
public class SetChatTitle implements BotApiMethod<Boolean> {
    public static final String PATH = "setChatTitle";

    private static final String CHAT_ID_FIELD = "chat_id";
    private static final String TITLE_FIELD = "title";

    /**
     * Required. Unique identifier for the target chat or username of the target channel (in the format @channelusername)
     */
    @JsonProperty(CHAT_ID_FIELD)
    private String chatId;
    /**
     * Required. New chat title, 1-255 characters
     */
    @JsonProperty(TITLE_FIELD)
    private String title;

    public SetChatTitle() {
        super();
    }

    public SetChatTitle(String chatId, String title) {
        super();
        this.chatId = checkNotNull(chatId);
        this.title = checkNotNull(title);
    }

    public SetChatTitle(Long chatId, String title) {
        super();
        this.chatId = checkNotNull(chatId).toString();
        this.title = checkNotNull(title);
    }

    public String getChatId() {
        return chatId;
    }

    public SetChatTitle setChatId(String chatId) {
        this.chatId = chatId;
        return this;
    }

    public SetChatTitle setChatId(Long chatId) {
        Objects.requireNonNull(chatId);
        this.chatId = chatId.toString();
        return this;
    }

    public String getTitle() {
        return title;
    }

    public SetChatTitle setTitle(String title) {
        Objects.requireNonNull(title);
        this.title = title;
        return this;
    }

    @Override
    public String getMethod() {
        return PATH;
    }

    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null || chatId.isEmpty()) {
            throw new TelegramApiValidationException("ChatId can't be empty", this);
        }
        if (title == null || title.isEmpty()) {
            throw new TelegramApiValidationException("Title can't be empty", this);
        }
    }
}
