package com.chuang.telegram.bot.methods.send;

import com.chuang.tauceti.httpclient.Request;
import com.chuang.telegram.bot.HttpUtils;
import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.AnyBotApiMethod;
import com.chuang.telegram.bot.model.InputFile;
import com.chuang.telegram.bot.model.replaykeyboard.ReplyKeyboard;
import lombok.Data;
import lombok.experimental.Accessors;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.MultipartEntityBuilder;

import java.io.File;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

/**
 * @author Ruben Bermudez
 * @version 4.0.0
 * Use this method to send animation files (GIF or H.264/MPEG-4 AVC video without sound).
 * On success, the sent Message is returned.
 *
 * Bots can currently send animation files of up to 50 MB in size, this limit may be changed in the future.
 */
@Data
@Accessors(chain = true)
public class SendAnimation implements AnyBotApiMethod {
    public static final String PATH = "sendAnimation";

    public static final String CHAT_ID_FIELD = "chat_id";
    public static final String ANIMATION_FIELD = "animation";
    public static final String DURATION_FIELD = "duration";
    public static final String WIDTH_FIELD = "width";
    public static final String HEIGHT_FIELD = "height";
    public static final String CAPTION_FIELD = "caption";
    public static final String PARSE_MODE_FIELD = "parse_mode";
    public static final String DISABLE_NOTIFICATION_FIELD = "disable_notification";
    public static final String REPLY_TO_MESSAGE_ID_FIELD = "reply_to_message_id";
    public static final String REPLY_MARKUP_FIELD = "reply_markup";
    public static final String THUMB_FIELD = "thumb";


    /** Unique identifier for the chat to send the message to (Or username for channels) */
    private String chatId;
    /**
     * Animation to send. Pass a file_id as String to send an animation that exists on the
     * Telegram servers (recommended), pass an HTTP URL as a String for Telegram to get an animation
     * from the Internet, or upload a new animation using multipart/form-data.
     */
    private InputFile animation;
    /** Optional. Duration of sent animation in seconds */
    private Long duration;
    /** OptionaL. Animation caption (may also be used when resending videos by file_id). */
    private String caption;
    /** Optional. Animation width */
    private Integer width;
    /** OptionaL. Animation height */
    private Integer height;
    /** Optional. Sends the message silently. Users will receive a notification with no sound. */
    private Boolean disableNotification;
    /** Optional. If the message is a reply, ID of the original message */
    private Long replyToMessageId;
    /** Optional. JSON-serialized object for a custom reply keyboard */
    private ReplyKeyboard replyMarkup;
    /** Send Markdown or HTML, if you want Telegram apps to show bold, italic, fixed-width text or inline URLs in the media caption. */
    private String parseMode;
    /**
     * Thumbnail of the file sent. The thumbnail should be in JPEG format and less than 200 kB in size.
     * A thumbnail‘s width and height should not exceed 90.
     * Ignored if the file is not uploaded using multipart/form-data.
     * Thumbnails can’t be reused and can be only uploaded as a new file, so you can pass “attach://<file_attach_name>”
     * if the thumbnail was uploaded using multipart/form-data under <file_attach_name>.
     */
    private InputFile thumb;


    public SendAnimation setChatId(String chatId) {
        this.chatId = chatId;
        return this;
    }
    public SendAnimation setChatId(Long chatId) {
        Objects.requireNonNull(chatId);
        this.chatId = chatId.toString();
        return this;
    }


    public SendAnimation enableNotification() {
        this.disableNotification = false;
        return this;
    }

    public SendAnimation disableNotification() {
        this.disableNotification = true;
        return this;
    }

    public SendAnimation setAnimation(File file) {
        this.animation = new InputFile(file, file.getName());
        return this;
    }

    public SendAnimation setAnimation(String animationName, InputStream inputStream) {
    	Objects.requireNonNull(animationName, "animationName cannot be null!");
    	Objects.requireNonNull(inputStream, "inputStream cannot be null!");
        this.animation = new InputFile(inputStream, animationName);
        return this;
    }

    public SendAnimation setAnimation(InputFile animation) {
        Objects.requireNonNull(animation, "animation cannot be null!");
        this.animation = animation;
        return this;
    }


    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null) {
            throw new TelegramApiValidationException("ChatId parameter can't be empty", this);
        }
        if (animation == null) {
            throw new TelegramApiValidationException("Animation parameter can't be empty", this);
        }

        animation.validate();

        if (replyMarkup != null) {
            replyMarkup.validate();
        }
        if (thumb != null) {
            thumb.validate();
        }
    }

    @Override
    public String getMethod() {
        return PATH;
    }

    @Override
    public Request asRequest(String api, String botKey) throws Exception {
        MultipartEntityBuilder builder = MultipartEntityBuilder.create()
                .setLaxMode()
                .setCharset(StandardCharsets.UTF_8)
                .addTextBody(CHAT_ID_FIELD, chatId, TEXT_PLAIN_UTF8);

        HttpUtils.addInputFile(builder, animation, ANIMATION_FIELD, true);

        if (null != replyMarkup) {
            builder.addTextBody(REPLY_MARKUP_FIELD, OBJECT_MAPPER.writeValueAsString(replyMarkup), TEXT_PLAIN_UTF8);
        }
        if (null != duration) {
            builder.addTextBody(DURATION_FIELD, duration.toString(), TEXT_PLAIN_UTF8);
        }
        if (null != caption) {
            builder.addTextBody(CAPTION_FIELD, caption, TEXT_PLAIN_UTF8);
            if (null != parseMode) {
                builder.addTextBody(PARSE_MODE_FIELD, parseMode, TEXT_PLAIN_UTF8);
            }
        }
        if (null != width) {
            builder.addTextBody(WIDTH_FIELD, width.toString(), TEXT_PLAIN_UTF8);
        }
        if (null != height) {
            builder.addTextBody(HEIGHT_FIELD, height.toString(), TEXT_PLAIN_UTF8);
        }
        if (null != disableNotification) {
            builder.addTextBody(DISABLE_NOTIFICATION_FIELD, disableNotification.toString(), TEXT_PLAIN_UTF8);
        }
        if (null != replyToMessageId) {
            builder.addTextBody(REPLY_TO_MESSAGE_ID_FIELD, replyToMessageId.toString(), TEXT_PLAIN_UTF8);
        }
        if (null != thumb) {
            HttpUtils.addInputFile(builder, thumb, THUMB_FIELD, false);
            builder.addTextBody(THUMB_FIELD, thumb.getAttachName(), TEXT_PLAIN_UTF8);
        }

        return Request.Post(requestUrl(api, botKey))
                .entity(builder.build())
                .build();
    }

}
