
package com.chuang.telegram.bot.methods.send;

import com.chuang.tauceti.httpclient.Request;
import com.chuang.telegram.bot.HttpUtils;
import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.AnyBotApiMethod;
import com.chuang.telegram.bot.model.InputFile;
import com.chuang.telegram.bot.model.replaykeyboard.ReplyKeyboard;
import lombok.Data;
import lombok.experimental.Accessors;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.MultipartEntityBuilder;

import java.io.File;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Use this method to send audio files, Use this method to send audio files, if you want
 * Telegram clients to display them in the music player. Your audio must be in an .mp3 format. On
 * success, the sent Message is returned. Bots can currently send audio files of up to 50 MB in
 * size, this limit may be changed in the future.
 * @apiNote For sending voice notes, use sendVoice method instead.
 */
@Data
@Accessors(chain = true)
public class SendAudio implements AnyBotApiMethod {
    public static final String PATH = "sendaudio";

    public static final String DURATION_FIELD = "duration";
    public static final String CHAT_ID_FIELD = "chat_id";
    public static final String AUDIO_FIELD = "audio";
    public static final String REPLY_TO_MESSAGE_ID_FIELD = "reply_to_message_id";
    public static final String DISABLE_NOTIFICATION_FIELD = "disable_notification";
    public static final String REPLY_MARKUP_FIELD = "reply_markup";
    public static final String PERFORMER_FIELD = "performer";
    public static final String TITLE_FIELD = "title";
    public static final String CAPTION_FIELD = "caption";
    public static final String PARSE_MODE_FIELD = "parse_mode";
    public static final String THUMB_FIELD = "thumb";

    /** Integer	Duration of the audio in seconds as defined by sender */
    private Long duration;
    /** Unique identifier for the chat to send the message to (or Username fro channels) */
    private String chatId;
    /** Audio file to send. file_id as String to resend an audio that is already on the Telegram servers or Url to upload it */
    private InputFile audio;
    /** Optional. If the message is a reply, ID of the original message */
    private Long replyToMessageId;
    /** Optional. Sends the message silently. Users will receive a notification with no sound. */
    private Boolean disableNotification;
    /** Optional. JSON-serialized object for a custom reply keyboard */
    private ReplyKeyboard replyMarkup;
    /** Optional. Performer of sent audio */
    private String performer;
    /** Optional. Title of sent audio */
    private String title;
    /** Optional. Audio caption (may also be used when resending documents by file_id), 0-200 characters */
    private String caption;
    /** Send Markdown or HTML, if you want Telegram apps to show bold, italic, fixed-width text or inline URLs in the media caption. */
    private String parseMode;
    /**
     * Thumbnail of the file sent. The thumbnail should be in JPEG format and less than 200 kB in size.
     * A thumbnail‘s width and height should not exceed 90.
     * Ignored if the file is not uploaded using multipart/form-data.
     * Thumbnails can’t be reused and can be only uploaded as a new file, so you can pass
     * “attach://<file_attach_name>” if the thumbnail was uploaded using multipart/form-data under <file_attach_name>.
     */
    private InputFile thumb;

    public SendAudio setChatId(String chatId) {
        this.chatId = chatId;
        return this;
    }

    public SendAudio setChatId(Long chatId) {
        Objects.requireNonNull(chatId);
        this.chatId = chatId.toString();
        return this;
    }


    /**
     * Use this method to set the audio to an audio existing in Telegram system
     *
     * @param audio File_id of the audio to send
     * @apiNote The file_id must have already been received or sent by your bot
     */
    public SendAudio setAudio(String audio) {
        this.audio = new InputFile(audio);
        return this;
    }

    /**
     * Use this method to set the audio to a new file
     *
     * @param file New audio file
     */
    public SendAudio setAudio(File file) {
        Objects.requireNonNull(file, "file cannot be null!");
        this.audio = new InputFile(file, file.getName());
        return this;
    }

    public SendAudio setAudio(String audioName, InputStream inputStream) {
    	Objects.requireNonNull(audioName, "audioName cannot be null!");
    	Objects.requireNonNull(inputStream, "inputStream cannot be null!");
    	this.audio = new InputFile(inputStream, audioName);
        return this;
    }

    public SendAudio setAudio(InputFile audio) {
        Objects.requireNonNull(audio, "audio cannot be null!");
        this.audio = audio;
        return this;
    }



    public SendAudio enableNotification() {
        this.disableNotification = false;
        return this;
    }

    public SendAudio disableNotification() {
        this.disableNotification = true;
        return this;
    }



    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null) {
            throw new TelegramApiValidationException("ChatId parameter can't be empty", this);
        }

        if (audio == null) {
            throw new TelegramApiValidationException("Audio parameter can't be empty", this);
        }

        audio.validate();

        if (thumb != null) {
            thumb.validate();
        }

        if (replyMarkup != null) {
            replyMarkup.validate();
        }
    }

    @Override
    public String getMethod() {
        return PATH;
    }


    @Override
    public Request asRequest(String api, String botKey) throws Exception {
        MultipartEntityBuilder builder = MultipartEntityBuilder.create()
                .setLaxMode()
                .setCharset(StandardCharsets.UTF_8)
                .addTextBody(CHAT_ID_FIELD, chatId, TEXT_PLAIN_UTF8);

        HttpUtils.addInputFile(builder, audio, AUDIO_FIELD, true);

        if (null != replyMarkup) {
            builder.addTextBody(REPLY_MARKUP_FIELD, OBJECT_MAPPER.writeValueAsString(replyMarkup), TEXT_PLAIN_UTF8);
        }
        if (null != duration) {
            builder.addTextBody(DURATION_FIELD, duration.toString(), TEXT_PLAIN_UTF8);
        }
        if (null != caption) {
            builder.addTextBody(CAPTION_FIELD, caption, TEXT_PLAIN_UTF8);
            if (null != parseMode) {
                builder.addTextBody(PARSE_MODE_FIELD, parseMode, TEXT_PLAIN_UTF8);
            }
        }
        if (null != title) {
            builder.addTextBody(TITLE_FIELD, title, TEXT_PLAIN_UTF8);
        }
        if (null != performer) {
            builder.addTextBody(PERFORMER_FIELD, performer, TEXT_PLAIN_UTF8);
        }

        if(null != disableNotification) {
            builder.addTextBody(DISABLE_NOTIFICATION_FIELD, disableNotification.toString(), TEXT_PLAIN_UTF8);
        }
        if(null != replyToMessageId) {
            builder.addTextBody(REPLY_TO_MESSAGE_ID_FIELD, replyToMessageId.toString(), TEXT_PLAIN_UTF8);
        }
        if (null != thumb) {
            HttpUtils.addInputFile(builder, thumb, THUMB_FIELD, false);
            builder.addTextBody(THUMB_FIELD, thumb.getAttachName(), TEXT_PLAIN_UTF8);
        }

        return Request.Post(requestUrl(api, botKey))
                .entity(builder.build())
                .build();
    }

}
