package com.chuang.telegram.bot.methods.send;

import com.chuang.telegram.bot.enums.ActionType;
import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import java.util.Objects;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Use this method when you need to tell the user that something is happening on the bot's
 * side. The status is set for 5 seconds or less (when a message arrives from your bot, Telegram
 * clients clear its typing status).
 */
@Data
@Accessors(chain = true)
public class SendChatAction implements BotApiMethod<Boolean> {

    public static final String PATH = "sendChatAction";

    public static final String CHAT_ID_FIELD = "chat_id";
    public static final String ACTION_FIELD = "action";

	/**  Unique identifier for the chat to send the message to (Or username for channels) */
    @JsonProperty(CHAT_ID_FIELD)
    private String chatId;
    /**
     * Type of action to broadcast. Choose one, depending on what the user is about to receive: typing for text messages,
     * upload_photo for photos, record_video or upload_video for videos, record_audio or upload_audio for audio files,
     * upload_document for general files, find_location for location data,
     * record_video_note or upload_video_note for video notes.
     */
    @JsonProperty(ACTION_FIELD)
    private String action;

    public SendChatAction() {
        super();
    }

    public SendChatAction(String chatId, String action) {
        this.chatId = checkNotNull(chatId);
        this.action = checkNotNull(action);
    }

    public SendChatAction(Long chatId, String action) {
        this.chatId = checkNotNull(chatId).toString();
        this.action = checkNotNull(action);
    }

    public String getChatId() {
        return chatId;
    }

    @JsonIgnore
    public ActionType getAction() {
        return ActionType.of(action);
    }

    public SendChatAction setChatId(String chatId) {
        this.chatId = chatId;
        return this;
    }

    public SendChatAction setChatId(Long chatId) {
        Objects.requireNonNull(chatId);
        this.chatId = chatId.toString();
        return this;
    }

    @JsonIgnore
    public SendChatAction setAction(ActionType action) {
        this.action = action.toString();
        return this;
    }

    @Override
    public String getMethod() {
        return PATH;
    }



    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null) {
            throw new TelegramApiValidationException("ChatId parameter can't be empty", this);
        }
        if (action == null) {
            throw new TelegramApiValidationException("Action parameter can't be empty", this);
        }
    }
}
