package com.chuang.telegram.bot.methods.send;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.AnyBotApiMethod;
import com.chuang.telegram.bot.model.payment.LabeledPrice;
import com.chuang.telegram.bot.model.replaykeyboard.InlineKeyboardMarkup;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import java.util.List;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Use this method to send an invoice. On success, the sent Message is returned.
 */
@Data
@Accessors(chain = true)
public class SendInvoice implements AnyBotApiMethod {
    public static final String PATH = "sendinvoice";

    private static final String CHAT_ID_FIELD = "chat_id";
    private static final String TITLE_FIELD = "title";
    private static final String DESCRIPTION_FIELD = "description";
    private static final String PAYLOAD_FIELD = "payload";
    private static final String PROVIDER_TOKEN_FIELD = "provider_token";
    private static final String START_PARAMETER_FIELD = "start_parameter";
    private static final String CURRENCY_FIELD = "currency";
    private static final String PRICES_FIELD = "prices";
    private static final String PHOTO_URL_FIELD = "photo_url";
    private static final String PHOTO_SIZE_FIELD = "photo_size";
    private static final String PHOTO_WIDTH_FIELD = "photo_width";
    private static final String PHOTO_HEIGHT_FIELD = "photo_height";
    private static final String NEED_NAME_FIELD = "need_name";
    private static final String NEED_PHONE_NUMBER_FIELD = "need_phone_number";
    private static final String NEED_EMAIL_FIELD = "need_email";
    private static final String NEED_SHIPPING_ADDRESS_FIELD = "need_shipping_address";
    private static final String IS_FLEXIBLE_FIELD = "is_flexible";
    private static final String DISABLE_NOTIFICATION_FIELD = "disable_notification";
    private static final String REPLY_TO_MESSAGE_ID_FIELD = "reply_to_message_id";
    private static final String REPLY_MARKUP_FIELD = "reply_markup";
    private static final String PROVIDER_DATA_FIELD = "provider_data";

	/**  Unique identifier for the target private chat */
    @JsonProperty(CHAT_ID_FIELD)
    private Long chatId;
	/**  Product name */
    @JsonProperty(TITLE_FIELD)
    private String title;
	/**  Product description */
    @JsonProperty(DESCRIPTION_FIELD)
    private String description;
	/**  Bot-defined invoice payload, 1-128 bytes. This will not be displayed to the user, use for your internal processes. */
    @JsonProperty(PAYLOAD_FIELD)
    private String payload;
	/**  Payments provider token, obtained via Botfather */
    @JsonProperty(PROVIDER_TOKEN_FIELD)
    private String providerToken;
	/**  Unique deep-linking parameter that can be used to generate this invoice when used as a start parameter. */
    @JsonProperty(START_PARAMETER_FIELD)
    private String startParameter;
	/**  3-letter ISO 4217 currency code */
    @JsonProperty(CURRENCY_FIELD)
    private String currency;
	/**  Price breakdown, a list of components (e.g. product price, tax, discount, delivery cost, delivery tax, bonus, etc.) */
    @JsonProperty(PRICES_FIELD)
    private List<LabeledPrice> prices;
    @JsonProperty(PHOTO_URL_FIELD)
    /**
     * Optional. URL of the product photo for the invoice. Can be a photo of the goods or a marketing image for a service.
     * People like it better when they see what they are paying for
     */
    private String photoUrl;
	/**  Optional. Photo size */
    @JsonProperty(PHOTO_SIZE_FIELD)
    private Long photoSize;
	/**  Optional. Photo width */
    @JsonProperty(PHOTO_WIDTH_FIELD)
    private Integer photoWidth;
	/**  Optional. Photo height */
    @JsonProperty(PHOTO_HEIGHT_FIELD)
    private Integer photoHeight;
	/**  Optional. Pass True, if you require the user's full name to complete the order */
    @JsonProperty(NEED_NAME_FIELD)
    private Boolean needName;
	/**  Optional. Pass True, if you require the user's phone number to complete the order */
    @JsonProperty(NEED_PHONE_NUMBER_FIELD)
    private Boolean needPhoneNumber;
	/**  Optional. Pass True, if you require the user's email to complete the order */
    @JsonProperty(NEED_EMAIL_FIELD)
    private Boolean needEmail;
	/**  Optional. Pass True, if you require the user's shipping address to complete the order */
    @JsonProperty(NEED_SHIPPING_ADDRESS_FIELD)
    private Boolean needShippingAddress;
	/**  Optional. Pass True, if the final price depends on the shipping method */
    @JsonProperty(IS_FLEXIBLE_FIELD)
    private Boolean isFlexible;
	/**  Optional. Sends the message silently. Users will receive a notification with no sound. */
    @JsonProperty(DISABLE_NOTIFICATION_FIELD)
    private Boolean disableNotification;
	/**  Optional. If the message is a reply, ID of the original message */
    @JsonProperty(REPLY_TO_MESSAGE_ID_FIELD)
    private Long replyToMessageId;
    @JsonProperty(REPLY_MARKUP_FIELD)
    /**
     * Optional. A JSON-serialized object for an inline keyboard.
     *
     * @apiNote If empty, one 'Buy title' button will be shown. If not empty, the first button must be a Pay button.
     */
    private InlineKeyboardMarkup replyMarkup;
    @JsonProperty(PROVIDER_DATA_FIELD)
    /**
     * Optional JSON-encoded data about the invoice, which will be shared with the payment provider.
     *
     * @apiNote A detailed description of required fields should be provided by the payment provider.
     */
    private String providerData;


    /**
     * Build an empty SendInvoice object
     */
    public SendInvoice() {
        super();
    }

    /**
     * Build a SendInvoice object with empty parameters
     * @param chatId Unique identifier for the target private chat
     * @param title Product name
     * @param description Product description
     * @param payload Bot defined invoice payload, 1-128 bytes.
     * @param providerToken Payments provider token
     * @param startParameter Unique deep-linking parameter.
     * @param currency 3-letter ISO 4217 currency code
     * @param prices Price breakdown, a list of components
     */
    public SendInvoice(Long chatId, String title, String description, String payload, String providerToken,
                       String startParameter, String currency, List<LabeledPrice> prices) {
        this.chatId = checkNotNull(chatId);
        this.title = checkNotNull(title);
        this.description = checkNotNull(description);
        this.payload = checkNotNull(payload);
        this.providerToken = checkNotNull(providerToken);
        this.startParameter = checkNotNull(startParameter);
        this.currency = checkNotNull(currency);
        this.prices = checkNotNull(prices);
    }


    @Override
    public String getMethod() {
        return PATH;
    }


    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null) {
            throw new TelegramApiValidationException("ChatId parameter can't be empty", this);
        }
        if (title == null || title.isEmpty()) {
            throw new TelegramApiValidationException("Title parameter can't be empty", this);
        }
        if (description == null || description.isEmpty()) {
            throw new TelegramApiValidationException("Description parameter can't be empty", this);
        }
        if (payload == null || payload.isEmpty()) {
            throw new TelegramApiValidationException("Payload parameter can't be empty", this);
        }
        if (providerToken == null || providerToken.isEmpty()) {
            throw new TelegramApiValidationException("ProviderToken parameter can't be empty", this);
        }
        if (startParameter == null || startParameter.isEmpty()) {
            throw new TelegramApiValidationException("StartParameter parameter can't be empty", this);
        }
        if (currency == null || currency.isEmpty()) {
            throw new TelegramApiValidationException("Currency parameter can't be empty", this);
        }
        if (prices == null || prices.isEmpty()) {
            throw new TelegramApiValidationException("Prices parameter can't be empty", this);
        } else {
            for (LabeledPrice price : prices) {
                price.validate();
            }
        }
        if (replyMarkup != null) {
            replyMarkup.validate();
        }
    }

}
