package com.chuang.telegram.bot.methods.send;

import com.chuang.tauceti.httpclient.Request;
import com.chuang.telegram.bot.HttpUtils;
import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.chuang.telegram.bot.model.BotApiResponse;
import com.chuang.telegram.bot.model.Message;
import com.chuang.telegram.bot.model.media.InputMedia;
import com.chuang.telegram.bot.model.media.InputMediaPhoto;
import com.chuang.telegram.bot.model.media.InputMediaVideo;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.type.TypeReference;
import lombok.Data;
import lombok.experimental.Accessors;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.MultipartEntityBuilder;

import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Ruben Bermudez
 * @version 3.5
 *
 * Use this method to send a group of photos or videos as an album.
 * On success, an array of the sent Messages is returned.
 */
@Data
@Accessors(chain = true)
public class SendMediaGroup implements BotApiMethod<ArrayList<Message>> {
    public static final String PATH = "sendMediaGroup";

    public static final String CHAT_ID_FIELD = "chat_id";
    public static final String MEDIA_FIELD = "media";
    public static final String REPLY_TO_MESSAGE_ID_FIELD = "reply_to_message_id";
    public static final String DISABLE_NOTIFICATION_FIELD = "disable_notification";

	/**   	Unique identifier for the target chat or username of the target channel (in the format @channelusername) */
    @JsonProperty(CHAT_ID_FIELD)
    private String chatId;
	/**  A JSON-serialized array describing photos and videos to be sent, must include 2–10 items */
    @JsonProperty(MEDIA_FIELD)
    private List<InputMedia<?>> media;
	/**  Optional. If the messages are a reply, ID of the original message */
    @JsonProperty(REPLY_TO_MESSAGE_ID_FIELD)
    private Long replyToMessageId;
	/**  Optional. Sends the messages silently. Users will receive a notification with no sound. */
    @JsonProperty(DISABLE_NOTIFICATION_FIELD)
    private Boolean disableNotification;

    public SendMediaGroup() {
        super();
    }

    public SendMediaGroup(String chatId, List<InputMedia<?>> media) {
        super();
        this.chatId = checkNotNull(chatId);
        this.media = checkNotNull(media);
    }

    public SendMediaGroup(Long chatId, List<InputMedia<?>> media) {
        super();
        this.chatId = checkNotNull(chatId).toString();
        this.media = checkNotNull(media);
    }

    public SendMediaGroup setChatId(String chatId) {
        this.chatId = checkNotNull(chatId);
        return this;
    }

    public SendMediaGroup setChatId(Long chatId) {
        this.chatId = checkNotNull(chatId).toString();
        return this;
    }
    public SendMediaGroup enableNotification() {
        this.disableNotification = false;
        return this;
    }

    public SendMediaGroup disableNotification() {
        this.disableNotification = true;
        return this;
    }
    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null) {
            throw new TelegramApiValidationException("ChatId parameter can't be empty", this);
        }

        if (media == null || media.isEmpty()) {
            throw new TelegramApiValidationException("Media parameter can't be empty", this);
        }

        for (InputMedia<?> inputMedia : media) {
            if (inputMedia instanceof InputMediaPhoto || inputMedia instanceof InputMediaVideo) {
                inputMedia.validate();
            } else {
                throw new TelegramApiValidationException("Media parameter can only be Photo or Video", this);
            }
        }
    }

    @Override
    public String getMethod() {
        return PATH;
    }

    @Override
    public Request asRequest(String api, String botKey) throws Exception {
        MultipartEntityBuilder builder = MultipartEntityBuilder.create()
                .setLaxMode()
                .setCharset(StandardCharsets.UTF_8)
                .addTextBody(CHAT_ID_FIELD, chatId, TEXT_PLAIN_UTF8);

        HttpUtils.addInputData(builder, media, MEDIA_FIELD);


        if(null != replyToMessageId) {
            builder.addTextBody(REPLY_TO_MESSAGE_ID_FIELD, replyToMessageId.toString(), TEXT_PLAIN_UTF8);
        }

        if(null != disableNotification) {
            builder.addTextBody(DISABLE_NOTIFICATION_FIELD, disableNotification.toString(), TEXT_PLAIN_UTF8);
        }

        return Request.Post(requestUrl(api, botKey))
                .entity(builder.build())
                .build();
    }

    @Override
    public TypeReference<BotApiResponse<ArrayList<Message>>> typeReference() {
        return new TypeReference<BotApiResponse<ArrayList<Message>>>() {};
    }
}
