package com.chuang.telegram.bot.methods.send;

import com.chuang.tauceti.httpclient.NoLimitMultipartFormEntity;
import com.chuang.tauceti.httpclient.Request;
import com.chuang.telegram.bot.HttpUtils;
import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.AnyBotApiMethod;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.chuang.telegram.bot.model.BotApiResponse;
import com.chuang.telegram.bot.model.InputFile;
import com.chuang.telegram.bot.model.Message;
import com.chuang.telegram.bot.model.replaykeyboard.ReplyKeyboard;
import com.fasterxml.jackson.core.type.TypeReference;
import lombok.Data;
import lombok.experimental.Accessors;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.MultipartEntityBuilder;

import java.io.File;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Use this method to send photos. On success, the sent Message is returned.
 */
@Data
@Accessors(chain = true)
public class SendPhoto implements AnyBotApiMethod {
    public static final String PATH = "sendphoto";

    public static final String CHAT_ID_FIELD = "chat_id";
    public static final String PHOTO_FIELD = "photo";
    public static final String CAPTION_FIELD = "caption";
    public static final String DISABLE_NOTIFICATION_FIELD = "disable_notification";
    public static final String REPLY_TO_MESSAGE_ID_FIELD = "reply_to_message_id";
    public static final String REPLY_MARKUP_FIELD = "reply_markup";
    public static final String PARSE_MODE_FIELD = "parse_mode";

    /** Unique identifier for the chat to send the message to (Or username for channels) */
    private String chatId;
    /** Photo to send. file_id as String to resend a photo that is already on the Telegram servers or URL to upload it */
    private InputFile photo;
    /** Optional Photo caption (may also be used when resending photos by file_id). */
    private String caption;
    /** Optional. Sends the message silently. Users will receive a notification with no sound. */
    private Boolean disableNotification;
    /** Optional. If the message is a reply, ID of the original message */
    private Long replyToMessageId;
    /** Optional. JSON-serialized object for a custom reply keyboard */
    private ReplyKeyboard replyMarkup;
    /** Send Markdown or HTML, if you want Telegram apps to show bold, italic, fixed-width text or inline URLs in the media caption. */
    private String parseMode;

    public SendPhoto() {
        super();
    }

    public SendPhoto setChatId(String chatId) {
        this.chatId = chatId;
        return this;
    }

    public SendPhoto setChatId(Long chatId) {
        Objects.requireNonNull(chatId);
        this.chatId = chatId.toString();
        return this;
    }


    public SendPhoto setPhoto(String photo) {
        this.photo = new InputFile(photo);
        return this;
    }

    public SendPhoto setPhoto(File file) {
        Objects.requireNonNull(file, "file cannot be null!");
        this.photo = new InputFile(file, file.getName());
        return this;
    }

    public SendPhoto setPhoto(InputFile photo) {
        Objects.requireNonNull(photo, "photo cannot be null!");
        this.photo = photo;
        return this;
    }

    public SendPhoto setPhoto(String photoName, InputStream inputStream) {
    	Objects.requireNonNull(photoName, "photoName cannot be null!");
    	Objects.requireNonNull(inputStream, "inputStream cannot be null!");
    	this.photo = new InputFile(inputStream, photoName);
        return this;
    }

    public SendPhoto enableNotification() {
        this.disableNotification = false;
        return this;
    }

    public SendPhoto disableNotification() {
        this.disableNotification = true;
        return this;
    }


    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null) {
            throw new TelegramApiValidationException("ChatId parameter can't be empty", this);
        }

        if (photo == null) {
            throw new TelegramApiValidationException("Photo parameter can't be empty", this);
        }

        photo.validate();

        if (replyMarkup != null) {
            replyMarkup.validate();
        }
    }

    @Override
    public String getMethod() {
        return PATH;
    }

    @Override
    public Request asRequest(String api, String botKey) throws Exception {
        MultipartEntityBuilder builder = MultipartEntityBuilder.create()
                .setLaxMode()
                .setCharset(StandardCharsets.UTF_8)
                .addTextBody(CHAT_ID_FIELD, chatId, TEXT_PLAIN_UTF8);

        HttpUtils.addInputFile(builder, photo, PHOTO_FIELD, true);

        if (null != replyMarkup) {
            builder.addTextBody(REPLY_MARKUP_FIELD, OBJECT_MAPPER.writeValueAsString(replyMarkup), TEXT_PLAIN_UTF8);
        }

        if(null != replyToMessageId) {
            builder.addTextBody(REPLY_TO_MESSAGE_ID_FIELD, replyToMessageId.toString(), TEXT_PLAIN_UTF8);
        }

        if (null != caption) {
            builder.addTextBody(CAPTION_FIELD, caption, TEXT_PLAIN_UTF8);
            if (null != parseMode) {
                builder.addTextBody(PARSE_MODE_FIELD, parseMode, TEXT_PLAIN_UTF8);
            }
        }

        if(null != disableNotification) {
            builder.addTextBody(DISABLE_NOTIFICATION_FIELD, disableNotification.toString(), TEXT_PLAIN_UTF8);
        }


        return Request.Post(requestUrl(api, botKey))
                .entity(new NoLimitMultipartFormEntity(builder.build()))
                .build();
    }

}
