
package com.chuang.telegram.bot.methods.send;

import com.chuang.tauceti.httpclient.Request;
import com.chuang.telegram.bot.HttpUtils;
import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.AnyBotApiMethod;
import com.chuang.telegram.bot.model.InputFile;
import com.chuang.telegram.bot.model.replaykeyboard.ReplyKeyboard;
import lombok.Data;
import lombok.experimental.Accessors;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.MultipartEntityBuilder;

import java.io.File;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Use this method to send .webp stickers. On success, the sent Message is returned.
 */
@Data
@Accessors(chain = true)
public class SendSticker implements AnyBotApiMethod {
    public static final String PATH = "sendsticker";

    public static final String CHAT_ID_FIELD = "chat_id";
    public static final String STICKER_FIELD = "sticker";
    public static final String DISABLE_NOTIFICATION_FIELD = "disable_notification";
    public static final String REPLY_TO_MESSAGE_ID_FIELD = "reply_to_message_id";
    public static final String REPLY_MARKUP_FIELD = "reply_markup";

    /** Unique identifier for the chat to send the message to (Or username for channels) */
    private String chatId;
    /** Sticker file to send. file_id as String to resend a sticker that is already on the Telegram servers or URL to upload it */
    private InputFile sticker;
    /** Optional. Sends the message silently. Users will receive a notification with no sound. */
    private Boolean disableNotification;
    /** Optional. If the message is a reply, ID of the original message */
    private Long replyToMessageId;
    /** Optional. JSON-serialized object for a custom reply keyboard */
    private ReplyKeyboard replyMarkup;


    public SendSticker setChatId(String chatId) {
        this.chatId = chatId;
        return this;
    }

    public SendSticker setChatId(Long chatId) {
        Objects.requireNonNull(chatId);
        this.chatId = chatId.toString();
        return this;
    }

    public SendSticker setSticker(InputFile sticker) {
        Objects.requireNonNull(sticker, "sticker cannot be null!");
        this.sticker = sticker;
        return this;
    }

    public SendSticker setSticker(File file) {
        Objects.requireNonNull(file, "file cannot be null!");
        this.sticker = new InputFile(file, file.getName());
        return this;
    }

    public SendSticker setSticker(String stickerName, InputStream inputStream) {
    	Objects.requireNonNull(stickerName, "stickerName cannot be null!");
    	Objects.requireNonNull(inputStream, "inputStream cannot be null!");
    	this.sticker = new InputFile(inputStream, stickerName);
        return this;
    }

    public SendSticker enableNotification() {
        this.disableNotification = false;
        return this;
    }

    public SendSticker disableNotification() {
        this.disableNotification = true;
        return this;
    }


    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null) {
            throw new TelegramApiValidationException("ChatId parameter can't be empty", this);
        }

        if (sticker == null) {
            throw new TelegramApiValidationException("Sticker parameter can't be empty", this);
        }

        sticker.validate();

        if (replyMarkup != null) {
            replyMarkup.validate();
        }
    }

    @Override
    public String getMethod() {
        return PATH;
    }

    @Override
    public Request asRequest(String api, String botKey) throws Exception {
        MultipartEntityBuilder builder = MultipartEntityBuilder.create()
                .setLaxMode()
                .setCharset(StandardCharsets.UTF_8)
                .addTextBody(CHAT_ID_FIELD, chatId, TEXT_PLAIN_UTF8);

        HttpUtils.addInputFile(builder, sticker, STICKER_FIELD, true);

        if (null != replyMarkup) {
            builder.addTextBody(REPLY_MARKUP_FIELD, OBJECT_MAPPER.writeValueAsString(replyMarkup), TEXT_PLAIN_UTF8);
        }

        if(null != replyToMessageId) {
            builder.addTextBody(REPLY_TO_MESSAGE_ID_FIELD, replyToMessageId.toString(), TEXT_PLAIN_UTF8);
        }

        if(null != disableNotification) {
            builder.addTextBody(DISABLE_NOTIFICATION_FIELD, disableNotification.toString(), TEXT_PLAIN_UTF8);
        }

        return Request.Post(requestUrl(api, botKey))
                .entity(builder.build())
                .build();
    }

}
