
package com.chuang.telegram.bot.methods.send;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.AnyBotApiMethod;
import com.chuang.telegram.bot.model.replaykeyboard.ReplyKeyboard;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import java.util.Objects;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Use this method to send information about a venue. On success, the sent Message is
 * returned.
 */
@Data
@Accessors(chain = true)
public class SendVenue implements AnyBotApiMethod {
    public static final String PATH = "sendVenue";

    private static final String CHAT_ID_FIELD = "chat_id";
    private static final String LATITUDE_FIELD = "latitude";
    private static final String LONGITUDE_FIELD = "longitude";
    private static final String TITLE_FIELD = "title";
    private static final String DISABLE_NOTIFICATION_FIELD = "disable_notification";
    private static final String ADDRESS_FIELD = "address";
    private static final String FOURSQUARE_ID_FIELD = "foursquare_id";
    private static final String REPLY_TO_MESSAGE_ID_FIELD = "reply_to_message_id";
    private static final String REPLY_MARKUP_FIELD = "reply_markup";
    private static final String FOURSQUARE_TYPE_FIELD = "foursquare_type";

	/**  Unique identifier for the chat to send the message to (Or username for channels) */
    @JsonProperty(CHAT_ID_FIELD)
    private String chatId;
	/**  Latitude of venue location */
    @JsonProperty(LATITUDE_FIELD)
    private Float latitude;
	/**  Longitude of venue location */
    @JsonProperty(LONGITUDE_FIELD)
    private Float longitude;
	/**  Title of the venue */
    @JsonProperty(TITLE_FIELD)
    private String title;
	/**  Optional. Sends the message silently. Users will receive a notification with no sound. */
    @JsonProperty(DISABLE_NOTIFICATION_FIELD)
    private Boolean disableNotification;
	/**  Address of the venue */
    @JsonProperty(ADDRESS_FIELD)
    private String address;
	/**  Optional. Foursquare identifier of the venue */
    @JsonProperty(FOURSQUARE_ID_FIELD)
    private String foursquareId;
	/**  Optional. If the message is a reply, ID of the original message */
    @JsonProperty(REPLY_TO_MESSAGE_ID_FIELD)
    private Long replyToMessageId;
	/**  Optional. JSON-serialized object for a custom reply keyboard */
    @JsonProperty(REPLY_MARKUP_FIELD)
    private ReplyKeyboard replyMarkup;
	/**  Optional. Foursquare type of the venue, if known. */
    @JsonProperty(FOURSQUARE_TYPE_FIELD)
    private String foursquareType;


    public SendVenue setChatId(String chatId) {
        this.chatId = chatId;
        return this;
    }

    public SendVenue setChatId(Long chatId) {
        Objects.requireNonNull(chatId);
        this.chatId = chatId.toString();
        return this;
    }

    public SendVenue enableNotification() {
        this.disableNotification = false;
        return this;
    }

    public SendVenue disableNotification() {
        this.disableNotification = true;
        return this;
    }

    @Override
    public String getMethod() {
        return PATH;
    }



    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null) {
            throw new TelegramApiValidationException("ChatId parameter can't be empty", this);
        }
        if (longitude == null) {
            throw new TelegramApiValidationException("Longitude parameter can't be empty", this);
        }
        if (latitude == null) {
            throw new TelegramApiValidationException("Latitude parameter can't be empty", this);
        }
        if (title == null || title.isEmpty()) {
            throw new TelegramApiValidationException("Title parameter can't be empty", this);
        }
        if (address == null) {
            throw new TelegramApiValidationException("Address parameter can't be empty", this);
        }
        if (replyMarkup != null) {
            replyMarkup.validate();
        }
    }

}
