package com.chuang.telegram.bot.methods.send;

import com.chuang.tauceti.httpclient.Request;
import com.chuang.telegram.bot.HttpUtils;
import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.AnyBotApiMethod;
import com.chuang.telegram.bot.model.InputFile;
import com.chuang.telegram.bot.model.replaykeyboard.ReplyKeyboard;
import lombok.Data;
import lombok.experimental.Accessors;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.MultipartEntityBuilder;

import java.io.File;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Use this method to send video files, Telegram clients support mp4 videos (other formats
 * may be sent as Document). On success, the sent Message is returned.
 */
@Data
@Accessors(chain = true)
public class SendVideo implements AnyBotApiMethod {
    public static final String PATH = "sendvideo";

    public static final String CHAT_ID_FIELD = "chat_id";
    public static final String VIDEO_FIELD = "video";
    public static final String DURATION_FIELD = "duration";
    public static final String CAPTION_FIELD = "caption";
    public static final String WIDTH_FIELD = "width";
    public static final String HEIGHT_FIELD = "height";
    public static final String DISABLE_NOTIFICATION_FIELD = "disable_notification";
    public static final String SUPPORTS_STREAMING_FIELD = "supports_streaming";
    public static final String REPLY_TO_MESSAGE_ID_FIELD = "reply_to_message_id";
    public static final String REPLY_MARKUP_FIELD = "reply_markup";
    public static final String PARSE_MODE_FIELD = "parse_mode";
    public static final String THUMB_FIELD = "thumb";

    /** Unique identifier for the chat to send the message to (Or username for channels) */
    private String chatId;
    /** Video to send. file_id as String to resend a video that is already on the Telegram servers or URL to upload it */
    private InputFile video;
    /** Optional. Duration of sent video in seconds */
    private Long duration;
    /** OptionaL. Video caption (may also be used when resending videos by file_id). */
    private String caption;
    /** Optional. Video width */
    private Integer width;
    /** OptionaL. Video height */
    private Integer height;
    /** Optional. Pass True, if the uploaded video is suitable for streaming */
    private Boolean supportsStreaming;
    /** Optional. Sends the message silently. Users will receive a notification with no sound. */
    private Boolean disableNotification;
    /** Optional. If the message is a reply, ID of the original message */
    private Long replyToMessageId;
    /** Optional. JSON-serialized object for a custom reply keyboard */
    private ReplyKeyboard replyMarkup;
    /** Send Markdown or HTML, if you want Telegram apps to show bold, italic, fixed-width text or inline URLs in the media caption. */
    private String parseMode;
    /**
     * Thumbnail of the file sent. The thumbnail should be in JPEG format and less than 200 kB in size.
     * A thumbnail‘s width and height should not exceed 90.
     * Ignored if the file is not uploaded using multipart/form-data.
     * Thumbnails can’t be reused and can be only uploaded as a new file, so you can pass “attach://<file_attach_name>”
     * if the thumbnail was uploaded using multipart/form-data under <file_attach_name>.
     */
    private InputFile thumb;

    public SendVideo setChatId(String chatId) {
        this.chatId = chatId;
        return this;
    }


    public SendVideo setChatId(Long chatId) {
        Objects.requireNonNull(chatId);
        this.chatId = chatId.toString();
        return this;
    }

    public SendVideo enableNotification() {
        this.disableNotification = false;
        return this;
    }

    public SendVideo disableNotification() {
        this.disableNotification = true;
        return this;
    }

    public SendVideo setVideo(InputFile video) {
        Objects.requireNonNull(video, "video cannot be null!");
        this.video = video;
        return this;
    }

    public SendVideo setVideo(File file) {
        Objects.requireNonNull(file, "file cannot be null!");
        this.video = new InputFile(file, file.getName());
        return this;
    }

    public SendVideo setVideo(String videoName, InputStream inputStream) {
    	Objects.requireNonNull(videoName, "videoName cannot be null!");
    	Objects.requireNonNull(inputStream, "inputStream cannot be null!");
    	this.video = new InputFile(inputStream, videoName);
        return this;
    }

    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null) {
            throw new TelegramApiValidationException("ChatId parameter can't be empty", this);
        }

        if (video == null) {
            throw new TelegramApiValidationException("Video parameter can't be empty", this);
        }

        video.validate();

        if (thumb != null) {
            thumb.validate();
        }
        if (replyMarkup != null) {
            replyMarkup.validate();
        }
    }

    @Override
    public String getMethod() {
        return PATH;
    }


    @Override
    public Request asRequest(String api, String botKey) throws Exception {
        MultipartEntityBuilder builder = MultipartEntityBuilder.create()
                .setLaxMode()
                .setCharset(StandardCharsets.UTF_8)
                .addTextBody(CHAT_ID_FIELD, chatId, TEXT_PLAIN_UTF8);

        HttpUtils.addInputFile(builder, video, VIDEO_FIELD, true);

        if (null != replyMarkup) {
            builder.addTextBody(REPLY_MARKUP_FIELD, OBJECT_MAPPER.writeValueAsString(replyMarkup), TEXT_PLAIN_UTF8);
        }

        if(null != replyToMessageId) {
            builder.addTextBody(REPLY_TO_MESSAGE_ID_FIELD, replyToMessageId.toString(), TEXT_PLAIN_UTF8);
        }

        if (null != caption) {
            builder.addTextBody(CAPTION_FIELD, caption, TEXT_PLAIN_UTF8);
            if (null != parseMode) {
                builder.addTextBody(PARSE_MODE_FIELD, parseMode, TEXT_PLAIN_UTF8);
            }
        }

        if (null != supportsStreaming) {
            builder.addTextBody(SUPPORTS_STREAMING_FIELD, supportsStreaming.toString(), TEXT_PLAIN_UTF8);
        }

        if (null != duration) {
            builder.addTextBody(DURATION_FIELD, duration.toString(), TEXT_PLAIN_UTF8);
        }
        if (null != width) {
            builder.addTextBody(WIDTH_FIELD, width.toString(), TEXT_PLAIN_UTF8);
        }
        if (null != height) {
            builder.addTextBody(HEIGHT_FIELD, height.toString(), TEXT_PLAIN_UTF8);
        }

        if(null != disableNotification) {
            builder.addTextBody(DISABLE_NOTIFICATION_FIELD, disableNotification.toString(), TEXT_PLAIN_UTF8);
        }

        if (null != thumb) {
            HttpUtils.addInputFile(builder, thumb, THUMB_FIELD, false);
            builder.addTextBody(THUMB_FIELD, thumb.getAttachName(), TEXT_PLAIN_UTF8);
        }

        return Request.Post(requestUrl(api, botKey))
                .entity(builder.build())
                .build();
    }

}

