package com.chuang.telegram.bot.methods.send;

import com.chuang.tauceti.httpclient.Request;
import com.chuang.telegram.bot.HttpUtils;
import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.AnyBotApiMethod;
import com.chuang.telegram.bot.model.InputFile;
import com.chuang.telegram.bot.model.replaykeyboard.ReplyKeyboard;
import lombok.Data;
import lombok.experimental.Accessors;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.MultipartEntityBuilder;

import java.io.File;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * As of v.4.0, Telegram clients support rounded square mp4 videos of up to 1 minute long.
 * Use this method to send video messages. On success, the sent Message is returned.
 */
@SuppressWarnings({"UnusedReturnValue", "WeakerAccess"})
@Data
@Accessors(chain = true)
public class SendVideoNote implements AnyBotApiMethod {
    public static final String PATH = "sendvideonote";

    public static final String CHAT_ID_FIELD = "chat_id";
    public static final String VIDEO_NOTE_FIELD = "video_note";
    public static final String DURATION_FIELD = "duration";
    public static final String LENGTH_FIELD = "length";
    public static final String DISABLE_NOTIFICATION_FIELD = "disable_notification";
    public static final String REPLY_TO_MESSAGE_ID_FIELD = "reply_to_message_id";
    public static final String REPLY_MARKUP_FIELD = "reply_markup";
    public static final String THUMB_FIELD = "thumb";

    /** Unique identifier for the chat to send the message to (Or username for channels) */
    private String chatId;
    /** Videonote to send. file_id as String to resend a video that is already on the Telegram servers. */
    private InputFile videoNote;
    /** Optional. Duration of sent video in seconds */
    private Long duration;
    /** Optional. Video width and height */
    private Long length;
    /** Optional. Sends the message silently. Users will receive a notification with no sound. */
    private Boolean disableNotification;
    /** Optional. If the message is a reply, ID of the original message */
    private Long replyToMessageId;
    /** Optional. JSON-serialized object for a custom reply keyboard */
    private ReplyKeyboard replyMarkup;
    /**
     * Thumbnail of the file sent. The thumbnail should be in JPEG format and less than 200 kB in size.
     * A thumbnail‘s width and height should not exceed 90.
     * Ignored if the file is not uploaded using multipart/form-data.
     * Thumbnails can’t be reused and can be only uploaded as a new file, so you can pass “attach://<file_attach_name>”
     * if the thumbnail was uploaded using multipart/form-data under <file_attach_name>.
     */
    private InputFile thumb;

    /**
     * Creates a new video note with a video already present in telegram servers
     * @param chatId Chat Id to send the video note
     * @param videoNote Video note file_id to send
     */
    public SendVideoNote(String chatId, String videoNote) {
        this.chatId = checkNotNull(chatId);
        this.setVideoNote(checkNotNull(videoNote));
    }

    /**
     * Creates a new video note with a video already present in telegram servers
     * @param chatId Chat Id to send the video note
     * @param videoNote Video note file_id to send
     */
    public SendVideoNote(Long chatId, String videoNote) {

        this.chatId = checkNotNull(chatId).toString();
        this.setVideoNote(checkNotNull(videoNote));
    }

    /**
     * Creates a new video note with a new video note
     * @param chatId Chat Id to send the video note
     * @param videoNote Video note file to upload
     */
    public SendVideoNote(String chatId, File videoNote) {
        this.chatId = checkNotNull(chatId);
        this.setVideoNote(videoNote);
    }

    /**
     * Creates a new video note with a video already present in telegram servers
     * @param chatId Chat Id to send the video note
     * @param videoNote Video note file to upload
     */
    public SendVideoNote(Long chatId, File videoNote) {
        this.chatId = checkNotNull(chatId).toString();
        this.setVideoNote(videoNote);
    }

    /**
     * Creates a new video note with a new video note
     * @param chatId Chat Id to send the video note
     * @param videoNoteName Name of the video note file
     * @param videoNote Video note file to upload
     */
    public SendVideoNote(String chatId, String videoNoteName, InputStream videoNote) {
        this.chatId = checkNotNull(chatId);
        this.setVideoNote(videoNoteName, videoNote);
    }

    /**
     * Creates a new video note with a video already present in telegram servers
     * @param chatId Chat Id to send the video note
     * @param videoNoteName Name of the video note file
     * @param videoNote Video note file to upload
     */
    public SendVideoNote(Long chatId, String videoNoteName, InputStream videoNote) {
        this.chatId = checkNotNull(chatId).toString();
        this.setVideoNote(videoNoteName, videoNote);
    }

    public SendVideoNote setVideoNote(String videoNote) {
        this.videoNote = new InputFile(videoNote);
        return this;
    }


    public SendVideoNote setVideoNote(InputFile videoNote) {
        Objects.requireNonNull(videoNote, "videoNote cannot be null!");
        this.videoNote = videoNote;
        return this;
    }

    public SendVideoNote setVideoNote(File file) {
        Objects.requireNonNull(file, "file cannot be null!");
        this.videoNote = new InputFile(file, file.getName());
        return this;
    }

    public SendVideoNote setVideoNote(String videoName, InputStream inputStream) {
        Objects.requireNonNull(videoName, "videoName cannot be null!");
        Objects.requireNonNull(inputStream, "inputStream cannot be null!");
        this.videoNote = new InputFile(inputStream, videoName);
        return this;
    }

    public SendVideoNote setChatId(String chatId) {
        this.chatId = chatId;
        return this;
    }



    public SendVideoNote setChatId(Long chatId) {
        Objects.requireNonNull(chatId);
        this.chatId = chatId.toString();
        return this;
    }


    public SendVideoNote enableNotification() {
        this.disableNotification = false;
        return this;
    }

    public SendVideoNote disableNotification() {
        this.disableNotification = true;
        return this;
    }


    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null) {
            throw new TelegramApiValidationException("ChatId parameter can't be empty", this);
        }

        if (videoNote == null) {
            throw new TelegramApiValidationException("VideoNote parameter can't be empty", this);
        }

        videoNote.validate();

        if (thumb != null) {
            thumb.validate();
        }
        if (replyMarkup != null) {
            replyMarkup.validate();
        }
    }

    @Override
    public String getMethod() {
        return PATH;
    }

    @Override
    public Request asRequest(String api, String botKey) throws Exception {
        MultipartEntityBuilder builder = MultipartEntityBuilder.create()
                .setLaxMode()
                .setCharset(StandardCharsets.UTF_8)
                .addTextBody(CHAT_ID_FIELD, chatId, TEXT_PLAIN_UTF8);

        HttpUtils.addInputFile(builder, videoNote, VIDEO_NOTE_FIELD, true);

        if (null != replyMarkup) {
            builder.addTextBody(REPLY_MARKUP_FIELD, OBJECT_MAPPER.writeValueAsString(replyMarkup), TEXT_PLAIN_UTF8);
        }

        if(null != replyToMessageId) {
            builder.addTextBody(REPLY_TO_MESSAGE_ID_FIELD, replyToMessageId.toString(), TEXT_PLAIN_UTF8);
        }
        if (null != duration) {
            builder.addTextBody(DURATION_FIELD, duration.toString(), TEXT_PLAIN_UTF8);
        }

        if (null != length) {
            builder.addTextBody(LENGTH_FIELD, length.toString(), TEXT_PLAIN_UTF8);
        }

        if(null != disableNotification) {
            builder.addTextBody(DISABLE_NOTIFICATION_FIELD, disableNotification.toString(), TEXT_PLAIN_UTF8);
        }

        if (null != thumb) {
            HttpUtils.addInputFile(builder, thumb, THUMB_FIELD, false);
            builder.addTextBody(THUMB_FIELD, thumb.getAttachName(), TEXT_PLAIN_UTF8);
        }

        return Request.Post(requestUrl(api, botKey))
                .entity(builder.build())
                .build();
    }

}
