package com.chuang.telegram.bot.methods.send;

import com.chuang.tauceti.httpclient.Request;
import com.chuang.telegram.bot.HttpUtils;
import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.AnyBotApiMethod;
import com.chuang.telegram.bot.model.InputFile;
import com.chuang.telegram.bot.model.replaykeyboard.ReplyKeyboard;
import lombok.Data;
import lombok.experimental.Accessors;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.MultipartEntityBuilder;

import java.io.File;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Use this method to send voice notes, if you want Telegram clients to display the file as a
 * playable voice message. For this to work, your audio must be in an .ogg file encoded with OPUS
 * (other formats may be sent as Audio or Document).
 */
@Data
@Accessors(chain = true)
public class SendVoice implements AnyBotApiMethod {
    public static final String PATH = "sendvoice";

    public static final String CHAT_ID_FIELD = "chat_id";
    public static final String VOICE_FIELD = "voice";
    public static final String DISABLE_NOTIFICATION_FIELD = "disable_notification";
    public static final String REPLY_TO_MESSAGE_ID_FIELD = "reply_to_message_id";
    public static final String REPLY_MARKUP_FIELD = "reply_markup";
    public static final String DURATION_FIELD = "duration";
    public static final String CAPTION_FIELD = "caption";
    public static final String PARSE_MODE_FIELD = "parse_mode";

    /** Unique identifier for the chat sent message to (Or username for channels) */
    private String chatId;
    /** Audio file to send. You can either pass a file_id as String to resend an audio that is already on the Telegram servers, or upload a new audio file using multipart/form-data. */
    private InputFile voice;
    /** Optional. Sends the message silently. Users will receive a notification with no sound. */
    private Boolean disableNotification;
    /** Optional. If the message is a reply, ID of the original message */
    private Long replyToMessageId;
    /** Optional. JSON-serialized object for a custom reply keyboard */
    private ReplyKeyboard replyMarkup;
    /** Optional. Duration of sent audio in seconds */
    private Long duration;
    /** Optional. Voice caption (may also be used when resending videos by file_id). */
    private String caption;
    /** Send Markdown or HTML, if you want Telegram apps to show bold, italic, fixed-width text or inline URLs in the media caption. */
    private String parseMode;


    public SendVoice enableNotification() {
        this.disableNotification = false;
        return this;
    }

    public SendVoice disableNotification() {
        this.disableNotification = true;
        return this;
    }


    public SendVoice setChatId(String chatId) {
        this.chatId = chatId;
        return this;
    }

    public SendVoice setChatId(Long chatId) {
        Objects.requireNonNull(chatId);
        this.chatId = chatId.toString();
        return this;
    }


    public SendVoice setVoice(String voice) {
        Objects.requireNonNull(voice, "voice cannot be null!");
        this.voice = new InputFile(voice);
        return this;
    }

    public SendVoice setVoice(File voice) {
        Objects.requireNonNull(voice, "voice cannot be null!");
        this.voice = new InputFile(voice, voice.getName());
        return this;
    }

    public SendVoice setVoice(InputFile voice) {
        Objects.requireNonNull(voice, "voice cannot be null!");
        this.voice = voice;
        return this;
    }

    public SendVoice setVoice(String voiceName, InputStream inputStream) {
    	Objects.requireNonNull(voiceName, "voiceName cannot be null!");
    	Objects.requireNonNull(inputStream, "inputStream cannot be null!");
    	this.voice = new InputFile(inputStream, voiceName);
        return this;
    }

    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null) {
            throw new TelegramApiValidationException("ChatId parameter can't be empty", this);
        }

        if (voice == null) {
            throw new TelegramApiValidationException("Voice parameter can't be empty", this);
        }

        voice.validate();

        if (replyMarkup != null) {
            replyMarkup.validate();
        }
    }

    @Override
    public String getMethod() {
        return PATH;
    }

    @Override
    public Request asRequest(String api, String botKey) throws Exception {
        MultipartEntityBuilder builder = MultipartEntityBuilder.create()
                .setLaxMode()
                .setCharset(StandardCharsets.UTF_8)
                .addTextBody(CHAT_ID_FIELD, chatId, TEXT_PLAIN_UTF8);

        HttpUtils.addInputFile(builder, voice, VOICE_FIELD, true);

        if (null != replyMarkup) {
            builder.addTextBody(REPLY_MARKUP_FIELD, OBJECT_MAPPER.writeValueAsString(replyMarkup), TEXT_PLAIN_UTF8);
        }

        if(null != replyToMessageId) {
            builder.addTextBody(REPLY_TO_MESSAGE_ID_FIELD, replyToMessageId.toString(), TEXT_PLAIN_UTF8);
        }
        if (null != duration) {
            builder.addTextBody(DURATION_FIELD, duration.toString(), TEXT_PLAIN_UTF8);
        }
        if(null != disableNotification) {
            builder.addTextBody(DISABLE_NOTIFICATION_FIELD, disableNotification.toString(), TEXT_PLAIN_UTF8);
        }

        if (null != caption) {
            builder.addTextBody(CAPTION_FIELD, caption, TEXT_PLAIN_UTF8);
            if (null != parseMode) {
                builder.addTextBody(PARSE_MODE_FIELD, parseMode, TEXT_PLAIN_UTF8);
            }
        }

        return Request.Post(requestUrl(api, botKey))
                .entity(builder.build())
                .build();
    }

}
