package com.chuang.telegram.bot.methods.stickers;

import com.chuang.tauceti.httpclient.Request;
import com.chuang.telegram.bot.HttpUtils;
import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.chuang.telegram.bot.model.InputFile;
import com.chuang.telegram.bot.model.stickers.MaskPosition;
import lombok.Data;
import lombok.experimental.Accessors;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.MultipartEntityBuilder;

import java.io.File;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Use this method to add a new sticker to a set created by the bot. Returns True on success.
 */
@Data
@Accessors(chain = true)
public class AddStickerToSet implements BotApiMethod<Boolean> {
    public static final String PATH = "addStickerToSet";

    public static final String USER_ID_FIELD = "user_id";
    public static final String NAME_FIELD = "name";
    public static final String PNG_STICKER_FIELD = "png_sticker";
    public static final String EMOJIS_FIELD = "emojis";
    public static final String MASK_POSITION_FIELD = "mask_position";

    /** User identifier of sticker set owner */
    private Long userId;
    /** Sticker set name */
    private String name;
    /** One or more emoji corresponding to the sticker */
    private String emojis;
    /** Position where the mask should be placed on faces */
    private MaskPosition maskPosition;
    /**
     * Png image with the sticker, must be up to 512 kilobytes in size, dimensions must not exceed 512px,
     * and either width or height must be exactly 512px. Pass a file_id as a String to send a file
     * that already exists on the Telegram servers, pass an HTTP URL as a String for Telegram
     * to get a file from the Internet, or upload a new one using multipart/form-data.
     */
    private InputFile pngSticker;

    public AddStickerToSet() {
        super();
    }

    public AddStickerToSet(Long userId, String name, String emojis) {
        this.userId = checkNotNull(userId);
        this.name = checkNotNull(name);
        this.emojis = checkNotNull(emojis);
    }
    public AddStickerToSet setUserId(Long userId) {
        this.userId = userId;
        return this;
    }

    public AddStickerToSet setPngSticker(String pngSticker) {
        this.pngSticker = new InputFile(pngSticker);
        return this;
    }

    public AddStickerToSet setPngSticker(File pngSticker) {
        Objects.requireNonNull(pngSticker, "pngSticker cannot be null!");
        this.pngSticker = new InputFile(pngSticker, pngSticker.getName());
        return this;
    }

    public AddStickerToSet setPngSticker(String pngStickerName, InputStream pngSticker) {
        Objects.requireNonNull(pngStickerName, "pngStickerName cannot be null!");
        Objects.requireNonNull(pngSticker, "pngSticker cannot be null!");
        this.pngSticker = new InputFile(pngSticker, pngStickerName);
        return this;
    }


    @Override
    public void validate() throws TelegramApiValidationException {
        if (userId == null || userId <= 0) {
            throw new TelegramApiValidationException("userId can't be empty", this);
        }
        if (name == null || name.isEmpty()) {
            throw new TelegramApiValidationException("name can't be empty", this);
        }
        if (emojis == null || emojis.isEmpty()) {
            throw new TelegramApiValidationException("emojis can't be empty", this);
        }

        if (pngSticker == null) {
            throw new TelegramApiValidationException("PngSticker can't be empty", this);
        }

        pngSticker.validate();

        if (maskPosition != null) {
            maskPosition.validate();
        }
    }

    @Override
    public String getMethod() {
        return PATH;
    }

    @Override
    public Request asRequest(String api, String botKey) throws Exception {
        MultipartEntityBuilder builder = MultipartEntityBuilder.create()
                .setLaxMode()
                .setCharset(StandardCharsets.UTF_8)
                .addTextBody(USER_ID_FIELD, userId.toString(), TEXT_PLAIN_UTF8)
                .addTextBody(NAME_FIELD, name, TEXT_PLAIN_UTF8)
                .addTextBody(EMOJIS_FIELD, emojis, TEXT_PLAIN_UTF8);

        HttpUtils.addInputFile(builder, pngSticker, PNG_STICKER_FIELD, true);
        if (maskPosition != null) {
            builder.addTextBody(MASK_POSITION_FIELD, OBJECT_MAPPER.writeValueAsString(maskPosition), TEXT_PLAIN_UTF8);
        }
        return Request.Post(requestUrl(api, botKey))
                .entity(builder.build())
                .build();
    }



}
