package com.chuang.telegram.bot.methods.stickers;

import com.chuang.tauceti.httpclient.Request;
import com.chuang.telegram.bot.HttpUtils;
import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.chuang.telegram.bot.model.InputFile;
import com.chuang.telegram.bot.model.stickers.MaskPosition;
import lombok.Data;
import lombok.experimental.Accessors;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.MultipartEntityBuilder;

import java.io.File;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Use this method to create new sticker set owned by a user. The bot will be able to edit the created sticker set.
 * Returns True on success.
 */
@Data
@Accessors(chain = true)
public class CreateNewStickerSet implements BotApiMethod<Boolean> {
    public static final String PATH = "createNewStickerSet";

    public static final String USER_ID_FIELD = "user_id";
    public static final String NAME_FIELD = "name";
    public static final String TITLE_FIELD = "title";
    public static final String PNG_STICKER_FIELD = "png_sticker";
    public static final String EMOJIS_FIELD = "emojis";
    public static final String CONTAINS_MASKS_FIELD = "contains_masks";
    public static final String MASK_POSITION_FIELD = "mask_position";

    /** User identifier of created sticker set owner */
    private Long userId;
    /**
     * Sticker set title, 1-64 characters
     * Name of sticker set, to be used in t.me/addstickers/<name> URLs.
     * Can contain only english letters, digits and underscores.
     * Must begin with a letter, can't contain consecutive underscores and must end in “_by_<bot username>”.
     * <bot_username> is case insensitive. 7-64 characters.
     */
    private String name;
    /** User identifier of created sticker set owner */
    private String title;
    /** One or more emoji corresponding to the sticker */
    private String emojis;
    /** Pass True, if a set of mask stickers should be created */
    private Boolean containsMasks;
    /** Position where the mask should be placed on faces */
    private MaskPosition maskPosition;
    /**
     * Png image with the sticker, must be up to 512 kilobytes in size, dimensions must not exceed 512px,
     * and either width or height must be exactly 512px.
     * Pass a file_id as a String to send a file that already exists on the Telegram servers,
     * pass an HTTP URL as a String for Telegram to get a file from the Internet, or upload a new one
     * using multipart/form-data. More info on Sending Files »
     */
    private InputFile pngSticker;

    public CreateNewStickerSet() {
        super();
        containsMasks = true;
    }

    public CreateNewStickerSet(Long userId, String name, String title, String emojis) {
        this.userId = checkNotNull(userId);
        this.name = checkNotNull(name);
        this.title = checkNotNull(title);
        this.emojis = checkNotNull(emojis);
        this.containsMasks = true;
    }



    public CreateNewStickerSet setPngSticker(String pngSticker) {
        this.pngSticker = new InputFile(pngSticker);
        return this;
    }

    public CreateNewStickerSet setPngStickerFile(InputFile pngStickerFile) {
        Objects.requireNonNull(pngStickerFile, "pngStickerFile cannot be null!");
        this.pngSticker = pngStickerFile;
        return this;
    }

    public CreateNewStickerSet setPngStickerFile(File pngStickerFile) {
        Objects.requireNonNull(pngStickerFile, "pngStickerFile cannot be null!");
        this.pngSticker = new InputFile(pngStickerFile, pngStickerFile.getName());
        return this;
    }

    public CreateNewStickerSet setPngStickerStream(String pngStickerName, InputStream pngStickerStream) {
        Objects.requireNonNull(pngStickerName, "pngStickerName cannot be null!");
        Objects.requireNonNull(pngStickerStream, "pngStickerStream cannot be null!");
        this.pngSticker = new InputFile(pngStickerStream, pngStickerName);
        return this;
    }


    @Override
    public void validate() throws TelegramApiValidationException {
        if (userId == null || userId <= 0) {
            throw new TelegramApiValidationException("userId can't be empty", this);
        }
        if (name == null || name.isEmpty()) {
            throw new TelegramApiValidationException("name can't be empty", this);
        }
        if (title == null || title.isEmpty()) {
            throw new TelegramApiValidationException("userId can't be empty", this);
        }
        if (emojis == null || emojis.isEmpty()) {
            throw new TelegramApiValidationException("emojis can't be empty", this);
        }

        if (pngSticker == null) {
            throw new TelegramApiValidationException("PngSticker parameter can't be empty", this);
        }

        pngSticker.validate();

        if (maskPosition != null) {
            maskPosition.validate();
        }
    }

    @Override
    public Request asRequest(String api, String botKey) throws Exception {
        MultipartEntityBuilder builder = MultipartEntityBuilder.create()
                .setLaxMode()
                .setCharset(StandardCharsets.UTF_8)
                .addTextBody(USER_ID_FIELD, userId.toString(), TEXT_PLAIN_UTF8)
                .addTextBody(NAME_FIELD, name, TEXT_PLAIN_UTF8)
                .addTextBody(TITLE_FIELD, title, TEXT_PLAIN_UTF8)
                .addTextBody(EMOJIS_FIELD, emojis, TEXT_PLAIN_UTF8)
                .addTextBody(CONTAINS_MASKS_FIELD, containsMasks.toString(), TEXT_PLAIN_UTF8);

        HttpUtils.addInputFile(builder, pngSticker, PNG_STICKER_FIELD, true);
        if (maskPosition != null) {
            builder.addTextBody(MASK_POSITION_FIELD, OBJECT_MAPPER.writeValueAsString(maskPosition), TEXT_PLAIN_UTF8);
        }
        return Request.Post(requestUrl(api, botKey))
                .entity(builder.build())
                .build();
    }

    @Override
    public String getMethod() {
        return PATH;
    }

}
