package com.chuang.telegram.bot.methods.stickers;

import com.chuang.tauceti.httpclient.Request;
import com.chuang.telegram.bot.HttpUtils;
import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.chuang.telegram.bot.model.BotApiResponse;
import com.chuang.telegram.bot.model.InputFile;
import com.fasterxml.jackson.core.type.TypeReference;
import lombok.Data;
import lombok.experimental.Accessors;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.MultipartEntityBuilder;

import java.io.File;
import java.nio.charset.StandardCharsets;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Use this method to upload a .png file with a sticker for later use in createNewStickerSet and addStickerToSet
 * methods (can be used multiple times). Returns the uploaded File on success.
 */
@Data
@Accessors(chain = true)
public class UploadStickerFile implements BotApiMethod<File> {
    public static final String PATH = "uploadStickerFile";

    public static final String USER_ID_FIELD = "user_id";
    public static final String PNG_STICKER_FIELD = "png_sticker";

    /** User identifier of sticker file owner */
    private Long userId;
    /**
     * Png image with the sticker, must be up to 512 kilobytes in size, dimensions must not exceed 512px,
     * and either width or height must be exactly 512px. More info on Sending Files »
     */
    private InputFile pngSticker;



    public UploadStickerFile() {
        super();
    }

    public UploadStickerFile(Long userId) {
        super();
        this.userId = checkNotNull(userId);
    }


    @Override
    public void validate() throws TelegramApiValidationException {
        if (userId == null || userId <= 0) {
            throw new TelegramApiValidationException("userId can't be empty", this);
        }

        if (pngSticker == null) {
            throw new TelegramApiValidationException("PngSticker parameter can't be empty", this);
        }

        pngSticker.validate();
    }

    @Override
    public String getMethod() {
        return PATH;
    }

    @Override
    public Request asRequest(String api, String botKey) throws Exception {
        MultipartEntityBuilder builder = MultipartEntityBuilder.create()
                .setLaxMode()
                .setCharset(StandardCharsets.UTF_8)
                .addTextBody(USER_ID_FIELD, userId.toString(), TEXT_PLAIN_UTF8);

        HttpUtils.addInputFile(builder, pngSticker, PNG_STICKER_FIELD, true);

        return Request.Post(requestUrl(api, botKey))
                .entity(builder.build())
                .build();
    }

    @Override
    public TypeReference<BotApiResponse<File>> typeReference() {
        return new TypeReference<BotApiResponse<File>>() {};
    }
}
