package com.chuang.telegram.bot.methods.updates;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.AnyBotApiMethod;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import java.util.List;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * @brief Use this method to specify a url and receive incoming updates via an outgoing webhook.
 * Whenever there is an update for the bot, we will send an HTTPS POST request to the specified url,
 * containing a JSON-serialized Update. In case of an unsuccessful request, we will give up after a
 * reasonable amount of attempts.
 * @date 20 of June of 2015
 */
@Data
@Accessors(chain = true)
public class SetWebhook implements AnyBotApiMethod {
    public static final String PATH = "setwebhook";

    public static final String URL_FIELD = "url";
    public static final String CERTIFICATE_FIELD = "certificate";
    public static final String MAX_CONNECTIONS_FIELD = "max_connections";
    public static final String ALLOWED_UPDATES_FIELD = "allowed_updates";
    public static final String SECRET_TOKEN_FIELD = "secret_token";
    public static final String DROP_PENDING_UPDATES_FIELD = "drop_pending_updates";

    /** Optional. HTTPS url to send updates to. Use an empty string to remove webhook integration */
    @JsonProperty(URL_FIELD)
    private String url;
    /** Optional. Upload your public key certificate so that the root certificate in use can be checked */
    @JsonProperty(CERTIFICATE_FIELD)
    private String certificateFile;
    /**
     * Maximum allowed number of simultaneous HTTPS connections to the webhook for update delivery, 1-100.
     * Defaults to 40. Use lower values to limit the load on your bot‘s server,
     * and higher values to increase your bot’s throughput.
     */
    @JsonProperty(MAX_CONNECTIONS_FIELD)
    private Integer maxConnections;
    /**
     * List the types of updates you want your bot to receive.
     * For example, specify [“message”, “edited_channel_post”, “callback_query”] to only receive
     * updates of these types. Specify an empty list to receive all updates regardless of type (default).
     * If not specified, the previous setting will be used.
     *
     * Please note that this parameter doesn't affect updates created before the call to the setWebhook,
     * so unwanted updates may be received for a short period of time.
     */
    @JsonProperty(ALLOWED_UPDATES_FIELD)
    private List<String> allowedUpdates;

    @JsonProperty(SECRET_TOKEN_FIELD)
    private String secretToken;

    @JsonProperty(DROP_PENDING_UPDATES_FIELD)
    private Boolean dropPendingUpdates;

    public SetWebhook() {
        this.url = "";
    }

    public SetWebhook setUrl(String url) {
        this.url = url;
        return this;
    }

    @Override
    public String getMethod() {
        return PATH;
    }

    @Override
    public void validate() throws TelegramApiValidationException {

    }
}
