
package com.chuang.telegram.bot.model.inlinequery.result;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.model.inlinequery.inputmessagecontent.InputMessageContent;
import com.chuang.telegram.bot.model.replaykeyboard.InlineKeyboardMarkup;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Represents a link to a file. By default, this file will be sent by the user with an
 * optional caption. Alternatively, you can use input_message_content to send a message with the
 * specified content instead of the file.
 * @apiNote  Currently, only .PDF and .ZIP files can be sent using this method.
 * @apiNote  This will only work in Telegram versions released after 9 April, 2016. Older clients will
 * ignore them.
 */
@Data
@Accessors(chain = true)
public class InlineQueryResultDocument implements InlineQueryResult {

    private static final String TYPE_FIELD = "type";
    private static final String ID_FIELD = "id";
    private static final String TITLE_FIELD = "title";
    private static final String DOCUMENT_URL_FIELD = "document_url";
    private static final String MIMETYPE_FIELD = "mime_type";
    private static final String DESCRIPTION_FIELD = "description";
    private static final String CAPTION_FIELD = "caption";
    private static final String REPLY_MARKUP_FIELD = "reply_markup";
    private static final String INPUT_MESSAGE_CONTENT_FIELD = "input_message_content";
    private static final String THUMB_URL_FIELD = "thumb_url";
    private static final String THUMB_WIDTH_FIELD = "thumb_width";
    private static final String THUMB_HEIGHT_FIELD = "thumb_height";
    private static final String PARSE_MODE_FIELD = "parse_mode";

    /** Type of the result, must be "document" */
    @JsonProperty(TYPE_FIELD)
    private final String type = "document";
    /** Unique identifier of this result, 1-64 bytes */
    @JsonProperty(ID_FIELD)
    private String id;
    /** Optional. Title for the result */
    @JsonProperty(TITLE_FIELD)
    private String title;
    /** A valid URL for the file */
    @JsonProperty(DOCUMENT_URL_FIELD)
    private String documentUrl;
    /** Mime type of the content of the file, either “application/pdf” or “application/zip” */
    @JsonProperty(MIMETYPE_FIELD)
    private String mimeType;
    /** Optional. Short description of the result */
    @JsonProperty(DESCRIPTION_FIELD)
    private String description;
    /** Optional. Caption of the document to be sent, 0-200 characters */
    @JsonProperty(CAPTION_FIELD)
    private String caption;
    /** Optional. Inline keyboard attached to the message */
    @JsonProperty(REPLY_MARKUP_FIELD)
    private InlineKeyboardMarkup replyMarkup;
    /** Optional. Content of the message to be sent instead of the file */
    @JsonProperty(INPUT_MESSAGE_CONTENT_FIELD)
    private InputMessageContent inputMessageContent;
    /** Optional. URL of the thumbnail (jpeg only) for the file */
    @JsonProperty(THUMB_URL_FIELD)
    private String thumbUrl;
    /** Optional. Thumbnail width */
    @JsonProperty(THUMB_WIDTH_FIELD)
    private Integer thumbWidth;
    /** Optional. Thumbnail height */
    @JsonProperty(THUMB_HEIGHT_FIELD)
    private Integer thumbHeight;
    /** Send Markdown or HTML, if you want Telegram apps to show bold, italic, fixed-width text or inline URLs in the media caption. */
    @JsonProperty(PARSE_MODE_FIELD)
    private String parseMode;

    @Override
    public void validate() throws TelegramApiValidationException {
        if (id == null || id.isEmpty()) {
            throw new TelegramApiValidationException("ID parameter can't be empty", this);
        }
        if (mimeType == null || mimeType.isEmpty()) {
            throw new TelegramApiValidationException("Mimetype parameter can't be empty", this);
        }
        if (documentUrl == null || documentUrl.isEmpty()) {
            throw new TelegramApiValidationException("DocumentUrl parameter can't be empty", this);
        }
        if (title == null || title.isEmpty()) {
            throw new TelegramApiValidationException("Title parameter can't be empty", this);
        }
        if (inputMessageContent != null) {
            inputMessageContent.validate();
        }
        if (replyMarkup != null) {
            replyMarkup.validate();
        }
    }

}
