
package com.chuang.telegram.bot.model.inlinequery.result;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.model.inlinequery.inputmessagecontent.InputMessageContent;
import com.chuang.telegram.bot.model.replaykeyboard.InlineKeyboardMarkup;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Represents a location on a map. By default, the location will be sent by the user.
 * Alternatively, you can use input_message_content to send a message with the specified content
 * instead of the location.
 * @apiNote  This will only work in Telegram versions released after 9 April, 2016. Older clients will
 * ignore them.
 */
@Data
@Accessors(chain = true)
public class InlineQueryResultLocation implements InlineQueryResult {

    private static final String TYPE_FIELD = "type";
    private static final String ID_FIELD = "id";
    private static final String TITLE_FIELD = "title";
    private static final String LATITUDE_FIELD = "latitude";
    private static final String LONGITUDE_FIELD = "longitude";
    private static final String REPLY_MARKUP_FIELD = "reply_markup";
    private static final String INPUT_MESSAGE_CONTENT_FIELD = "input_message_content";
    private static final String THUMB_URL_FIELD = "thumb_url";
    private static final String THUMB_WIDTH_FIELD = "thumb_width";
    private static final String THUMB_HEIGHT_FIELD = "thumb_height";
    private static final String LIVE_PERIOD_FIELD = "live_period";

    /** Type of the result, must be "location" */
    @JsonProperty(TYPE_FIELD)
    private final String type = "location";
    /** Unique identifier of this result, 1-64 bytes */
    @JsonProperty(ID_FIELD)
    private String id;
    /** Optional. Location title */
    @JsonProperty(TITLE_FIELD)
    private String title;
    /** Location latitude in degrees */
    @JsonProperty(LATITUDE_FIELD)
    private Float latitude;
    /** Location longitude in degrees */
    @JsonProperty(LONGITUDE_FIELD)
    private Float longitude;
    /** Optional. Inline keyboard attached to the message */
    @JsonProperty(REPLY_MARKUP_FIELD)
    private InlineKeyboardMarkup replyMarkup;
    /** Optional. Content of the message to be sent */
    @JsonProperty(INPUT_MESSAGE_CONTENT_FIELD)
    private InputMessageContent inputMessageContent;
    /** Optional. URL of the thumbnail (jpeg only) for the file */
    @JsonProperty(THUMB_URL_FIELD)
    private String thumbUrl;
    /** Optional. Thumbnail width */
    @JsonProperty(THUMB_WIDTH_FIELD)
    private Integer thumbWidth;
    /** Optional. Thumbnail height */
    @JsonProperty(THUMB_HEIGHT_FIELD)
    private Integer thumbHeight;
    /** Optional. Period in seconds for which the location can be updated, should be between 60 and 86400. */
    @JsonProperty(LIVE_PERIOD_FIELD)
    private Long livePeriod;


    @Override
    public void validate() throws TelegramApiValidationException {
        if (id == null || id.isEmpty()) {
            throw new TelegramApiValidationException("ID parameter can't be empty", this);
        }
        if (title == null || title.isEmpty()) {
            throw new TelegramApiValidationException("Title parameter can't be empty", this);
        }
        if (latitude == null) {
            throw new TelegramApiValidationException("Latitude parameter can't be empty", this);
        }
        if (longitude == null) {
            throw new TelegramApiValidationException("Longitude parameter can't be empty", this);
        }
        if (livePeriod != null && (livePeriod < 60 || livePeriod > 86400)) {
            throw new TelegramApiValidationException("Live period parameter must be between 60 and 86400", this);
        }
        if (inputMessageContent != null) {
            inputMessageContent.validate();
        }
        if (replyMarkup != null) {
            replyMarkup.validate();
        }
    }
}
