package com.chuang.tauceti.tools.basic;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.DESKeySpec;
import java.nio.charset.StandardCharsets;
import java.security.Key;
import java.util.Base64;

/***
 * DES 加解密算法
 *
 * 敏感字段加解密工具
 * 规则: 前二后五,  混淆密文
 *
 * @author: Ives.l
 * @since: 2019/4/12 13:56
 */
public class DESKit {

	/**
	 * 密钥算法
	 */
	private static final String KEY_ALGORITHM = "DES";
	private static final String DEFAULT_CIPHER_ALGORITHM = "DES/ECB/PKCS5Padding";

	/**
	 * 初始化密钥
	 *
	 * @return byte[] 密钥
	 */
	public static byte[] initSecretKey() throws Exception {
		//返回生成指定算法的秘密密钥的 KeyGenerator 对象
		KeyGenerator kg = KeyGenerator.getInstance(KEY_ALGORITHM);
		//初始化此密钥生成器，使其具有确定的密钥大小
		kg.init(56);
		//生成一个密钥
		SecretKey secretKey = kg.generateKey();
		return secretKey.getEncoded();
	}

	/**
	 * 转换密钥
	 *
	 * @param key 二进制密钥
	 * @return Key  密钥
	 */
	private static Key toKey(byte[] key) throws Exception {
		//实例化DES密钥规则
		DESKeySpec dks = new DESKeySpec(key);
		//实例化密钥工厂
		SecretKeyFactory skf = SecretKeyFactory.getInstance(KEY_ALGORITHM);
		//生成密钥
		return skf.generateSecret(dks);
	}

	/**
	 * 加密
	 *
	 * @param data 待加密数据
	 * @param key  密钥
	 * @return byte[]   加密数据
	 */
	public static byte[] encrypt(byte[] data, Key key) throws Exception {
		return encrypt(data, key, DEFAULT_CIPHER_ALGORITHM);
	}

	/**
	 * 加密
	 *
	 * @param data 待加密数据
	 * @param key  二进制密钥
	 * @return byte[]   加密数据
	 */
	public static byte[] encrypt(byte[] data, byte[] key) throws Exception {
		return encrypt(data, key, DEFAULT_CIPHER_ALGORITHM);
	}

	/**
	 * 加密
	 *
	 * @param data            待加密数据
	 * @param key             二进制密钥
	 * @param cipherAlgorithm 加密算法/工作模式/填充方式
	 * @return byte[]   加密数据
	 */
	public static byte[] encrypt(byte[] data, byte[] key, String cipherAlgorithm) throws Exception {
		//还原密钥
		Key k = toKey(key);
		return encrypt(data, k, cipherAlgorithm);
	}

	/**
	 * 加密
	 *
	 * @param data            待加密数据
	 * @param key             密钥
	 * @param cipherAlgorithm 加密算法/工作模式/填充方式
	 * @return byte[]   加密数据
	 */
	public static byte[] encrypt(byte[] data, Key key, String cipherAlgorithm) throws Exception {
		//实例化
		Cipher cipher = Cipher.getInstance(cipherAlgorithm);
		//使用密钥初始化，设置为加密模式
		cipher.init(Cipher.ENCRYPT_MODE, key);
		//执行操作
		return cipher.doFinal(data);
	}

	/**
	 * 解密
	 *
	 * @param data 待解密数据
	 * @param key  二进制密钥
	 * @return byte[]   解密数据
	 */
	public static byte[] decrypt(byte[] data, byte[] key) throws Exception {
		return decrypt(data, key, DEFAULT_CIPHER_ALGORITHM);
	}

	/**
	 * 解密
	 *
	 * @param data 待解密数据
	 * @param key  密钥
	 * @return byte[]   解密数据
	 */
	public static byte[] decrypt(byte[] data, Key key) throws Exception {
		return decrypt(data, key, DEFAULT_CIPHER_ALGORITHM);
	}

	/**
	 * 解密
	 *
	 * @param data            待解密数据
	 * @param key             二进制密钥
	 * @param cipherAlgorithm 加密算法/工作模式/填充方式
	 * @return byte[]   解密数据
	 */
	public static byte[] decrypt(byte[] data, byte[] key, String cipherAlgorithm) throws Exception {
		//还原密钥
		Key k = toKey(key);
		return decrypt(data, k, cipherAlgorithm);
	}

	/**
	 * 解密
	 *
	 * @param data            待解密数据
	 * @param key             密钥
	 * @param cipherAlgorithm 加密算法/工作模式/填充方式
	 * @return byte[]   解密数据
	 */
	public static byte[] decrypt(byte[] data, Key key, String cipherAlgorithm) throws Exception {
		//实例化
		Cipher cipher = Cipher.getInstance(cipherAlgorithm);
		//使用密钥初始化，设置为解密模式
		cipher.init(Cipher.DECRYPT_MODE, key);
		//执行操作
		return cipher.doFinal(data);
	}

	/**
	 * 加密数据
	 *
	 * @param input     待加密数据
	 * @param publicKey 秘钥
	 * @return String
	 * @throws Exception 异常
	 */
	public static String encrypt(String input, String publicKey) throws Exception {
		byte[] data = input.getBytes(StandardCharsets.UTF_8);
		byte[] key = publicKey.getBytes();
		return new String(base64Encode(
				DESKit.encrypt(data, key, DEFAULT_CIPHER_ALGORITHM)))
				.replaceAll("\\s*", "");
	}

	/**
	 * 解密数据
	 *
	 * @param input     待解密数据
	 * @param publicKey 秘钥
	 * @return String
	 * @throws Exception 异常
	 */
	public static String decrypt(String input, String publicKey) throws Exception {
		byte[] data = base64Decode(input);
		byte[] key = publicKey.getBytes();
		return new String(DESKit.decrypt(data, key, DEFAULT_CIPHER_ALGORITHM));
	}

	private static byte[] base64Encode(byte[] s) {
		if (s == null) {
			return null;
		}

		return Base64.getEncoder().encode(s);
	}

	private static byte[] base64Decode(String s) {
		if (s == null) {
			return null;
		}
		return Base64.getDecoder().decode(s.getBytes());
	}


}
