package com.chuang.tauceti.tools.basic;


import com.chuang.tauceti.support.BiValue;
import com.chuang.tauceti.support.CompletableFutureWrapper;
import com.chuang.tauceti.support.ThreeValue;
import com.chuang.tauceti.support.exception.ValueException;
import com.chuang.tauceti.support.page.Page;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.*;
import java.util.stream.Collectors;

public class FutureKit {
    public static <T> CompletableFuture<T> error(Throwable throwable) {
        CompletableFuture<T> f = new CompletableFuture<>();
        f.completeExceptionally(throwable);
        return f;
    }

    public static <T1, T2> CompletableFuture<BiValue<T1, T2>> zip(CompletableFuture<T1> t1, CompletableFuture<T2> t2) {
        CompletableFuture<BiValue<T1, T2>> future = new CompletableFuture<>();
        BiValue<T1, T2> bi = new BiValue<>();

        onZip(
                () -> future.complete(bi),
                future::completeExceptionally,
                t1.thenAccept(bi::setOne),
                t2.thenAccept(bi::setTwo)
        );

        return future;
    }

    public static <T1, T2, T3> CompletableFuture<ThreeValue<T1, T2, T3>> zip(CompletableFuture<T1> t1, CompletableFuture<T2> t2, CompletableFuture<T3> t3) {
        CompletableFuture<ThreeValue<T1, T2, T3>> future = new CompletableFuture<>();
        ThreeValue<T1, T2, T3> bi = new ThreeValue<>();

        onZip(
                () -> future.complete(bi),
                future::completeExceptionally,
                t1.thenAccept(bi::setOne),
                t2.thenAccept(bi::setTwo),
                t3.thenAccept(bi::setThree)
        );

        return future;
    }

    private static void onZip(Runnable whenDone, Consumer<Throwable> whenError, CompletableFuture<?>... futures) {
        AtomicInteger count = new AtomicInteger(futures.length);
        for (CompletableFuture<?> future: futures) {
            future.whenComplete((o, throwable) -> {
                if(null != throwable) {
                    whenError.accept(throwable);
                }
                int i = count.decrementAndGet();
                if(i <= 0) {
                    whenDone.run();
                }
            });
        }
    }

    /**
     * 将多个Future<T>合并成一个，结果变成 Future<List<T>>, 如果任意一个Future出现异常，则直接抛出异常。
     * @param futures futures
     */
    public static <T> CompletableFuture<List<T>> allOf(CompletableFuture<T>... futures) {
        return CompletableFuture.allOf(futures)
                .thenApply(v ->
                        Arrays.stream(futures).map(CompletableFuture::join).collect(Collectors.toList())
                );
    }

    /**
     * 将多个Future<T>合并成一个，结果变成 Future<List<T>>, 如果任意一个Future出现异常，则直接抛出异常。
     * @param futures futures
     */
    public static <T> CompletableFuture<List<T>> allOf(List<CompletableFuture<T>> futures) {
        return CompletableFuture.allOf(futures.toArray(new CompletableFuture[0]))
                .thenApply(v ->
                        futures.stream().map(CompletableFuture::join).collect(Collectors.toList())
                );
    }

    /**
     * 将多个Future<T>合并成一个，结果变成 Future<List<T>>, 如果任意一个Future出现异常，将通过 errChg参数将错误转换成 T 对象。
     * @param futures futures
     */
    public static <T> CompletableFuture<List<T>> allOf(BiFunction<CompletableFuture<T>, Throwable, T> errChg, CompletableFuture<T>... futures) {
        return CompletableFuture.allOf(futures)
                .thenApply(v ->
                        Arrays.stream(futures).map(f -> {
                            try {
                                return f.join();
                            } catch (Throwable e) {
                                return errChg.apply(f, e);
                            }
                        }).collect(Collectors.toList())
                );
    }

    /**
     * 将多个Future<T>合并成一个，结果变成 Future<List<T>>, 如果任意一个Future出现异常，将通过 errChg参数将错误转换成 T 对象。
     * @param futures futures
     */
    public static <T> CompletableFuture<List<T>> allOf(BiFunction<CompletableFuture<T>, Throwable, T> errChg, List<CompletableFuture<T>> futures) {
        return CompletableFuture.allOf(futures.toArray(new CompletableFuture[0]))
                .thenApply(v ->
                        futures.stream().map(f -> {
                            try {
                                return f.join();
                            } catch (Throwable e) {
                                return errChg.apply(f, e);
                            }
                        }).collect(Collectors.toList())
                );
    }

    public static <T, R, CT extends Collection<T>, CR extends Collection<R>> CompletableFuture<CR> collectionMap(CompletableFuture<CT> future, Function<T, R> map, Supplier<CR> crg) {
        return future.thenApply(list -> {
            CR cr = crg.get();
            for(T t : list) {
               cr.add(map.apply(t));
            }
            return cr;
        });
    }

    public static <T, R> CompletableFuture<List<R>> listMap(CompletableFuture<List<T>> future, Function<T, R> map) {
        return collectionMap(future, map, ArrayList::new);
    }

    /**
     * 当遇到错误时重试。
     * @param futureGetter 获取future是谓词
     * @param times 重试次数
     */
    public static <T> CompletableFuture<T> retryWhenError(Supplier<CompletableFuture<T>> futureGetter, int times) {
        return retry(futureGetter, times, -1, TimeUnit.MILLISECONDS, (o, t) -> null != t);
    }


    /**
     * 当遇到错误时重试。
     * @param futureGetter 获取future是谓词
     * @param times 重试次数
     * @param delay 延迟时间 毫秒数
     */
    public static <T> CompletableFuture<T> retryWhenError(Supplier<CompletableFuture<T>> futureGetter, int times, int delay) {
        return retry(futureGetter, times, delay, TimeUnit.MILLISECONDS, (o, t) -> null != t);
    }

    /**
     * 当遇到错误时重试。
     * @param futureGetter 获取future是谓词
     * @param times 重试次数
     * @param delay 延迟时间
     * @param unit  延迟时间单位
     */
    public static <T> CompletableFuture<T> retryWhenError(Supplier<CompletableFuture<T>> futureGetter, int times, int delay, TimeUnit unit) {
        return retry(futureGetter, times, delay, unit, (o, t) -> null != t);
    }

    /**
     *
     * @param futureGetter 需要重试的过程
     * @param times 次数
     * @param delay 延迟
     * @param unit 延迟时间单位
     * @param retryMatcher 验证是否需要重试 （会判断错误和结果）
     */
    public static <T> CompletableFuture<T> retry(Supplier<CompletableFuture<T>> futureGetter, int times, int delay, TimeUnit unit, BiPredicate<T, Throwable> retryMatcher) {

        CompletableFutureWrapper<T> future = new CompletableFutureWrapper<>();
        try {
            CompletableFuture<T> f = futureGetter.get();
            future.setCancelHandler(f::cancel);

            f.whenComplete((t, throwable) -> {
                if (times <= 0) {
                    done(future, t, throwable);
                    return;
                }
                boolean needRetry = (!future.isDone()) && retryMatcher.test(t, throwable);
                if (!needRetry) {
                    done(future, t, throwable);
                    return;
                }


                Runnable command = () -> retry(futureGetter, times - 1, delay, unit, retryMatcher)
                        .whenComplete((t1, throwable1) -> done(future, t1, throwable1));

                if (delay < 0) {
                    command.run();
                } else {
                    ScheduleKit.schedule(command, delay, unit);
                }

            });
        } catch (Exception e) {
            done(future, null, e);
        }
        return future;
    }

    public static <T> CompletableFuture<T> retry(Supplier<CompletableFuture<T>> futureGetter, RetryStrategy<T> retryStrategy) {
        return retry(futureGetter, retryStrategy.getTimes(), retryStrategy.getDelay(), retryStrategy.getTimeUnit(), retryStrategy.getTest());
    }

    public static <T> CompletableFuture<T> delay(Supplier<CompletableFuture<T>> futureGetter, int delay, TimeUnit unit) {
        CompletableFutureWrapper<T> future = new CompletableFutureWrapper<>();
        ScheduleKit.schedule(() -> {
            if(future.isDone() || future.isCancelled() || future.isCompletedExceptionally()) {
                return;
            }
            CompletableFuture<T> f = futureGetter.get();
            future.setCancelHandler(f::cancel);
            f.whenComplete((t, throwable) -> done(future, t, throwable));
        }, delay, unit);

        return future;
    }

    private static <T> void done(CompletableFuture<T> future, @Nullable T value, @Nullable Throwable throwable) {
        if(future.isDone() || future.isCancelled() || future.isCompletedExceptionally()) {
            return;
        }
        if(null != throwable) {
            future.completeExceptionally(throwable);
        } else {
            future.complete(value);
        }
    }

    /**
     * 将所有future 一个接一个的执行，前一个得到结果，再执行下一个。
     * @param list 获取future的参数集合
     * @param getter 根据参数获取 future 的谓语
     * @param errConvert 当遇到错误时的转换器
     */
    public static <T, V> CompletableFuture<List<V>> oneByOne(Collection<T> list, Function<T, CompletableFuture<V>> getter, Function<Throwable, V> errConvert) {
        List<Supplier<CompletableFuture<V>>> getters =
                list.stream().map(t -> (Supplier<CompletableFuture<V>>)(() -> getter.apply(t))).collect(Collectors.toList());

        return oneByOne(getters, errConvert);

    }



    /**
     * 将所有future 一个接一个的执行，前一个得到结果，再执行下一个。
     * @param getters 获取future的谓语集合
     * @param errConvert 当遇到错误时的转换器
     */
    public static <T> CompletableFuture<List<T>> oneByOne(Collection<Supplier<CompletableFuture<T>>> getters, Function<Throwable, T> errConvert) {
        if(getters.isEmpty()) {
            return CompletableFuture.completedFuture(new ArrayList<>());
        }
        CompletableFuture<List<T>> result = new CompletableFuture<>();

        List<T> list = new ArrayList<>();

        Function<T, T> successFunc = t -> {
            list.add(t);
            if(list.size() == getters.size()) {
                result.complete(list);
            }
            return t;
        };

        Function<Throwable, T> errorFunc = throwable -> {
            T t = errConvert.apply(throwable);
            list.add(t);
            if(list.size() == getters.size()) {
                result.complete(list);
            }
            return t;
        };

        CompletableFuture<T> temp = null;
        for(Supplier<CompletableFuture<T>> getter: getters) {
            if (null == temp) {
                temp = getter.get().thenApply(successFunc).exceptionally(errorFunc);
            } else {
                temp = temp.thenCompose(t -> getter.get()).thenApply(successFunc).exceptionally(errorFunc);
            }
        }

        return result;
    }


//    /**
//     * 将所有future 一个接一个的执行，前一个得到结果，再执行下一个。
//     * @param list 获取future的参数集合
//     * @param getter 根据参数获取 future 的谓语
//     * @param errConvert 当遇到错误时的转换器
//     */
//    public static <T, V> CompletableFuture<List<V>> oneByOne(Collection<T> list, Function<T, CompletableFuture<V>> getter, BiFunction<T, Throwable, V> errConvert) {
//        List<Supplier<BiValue<T, CompletableFuture<V>>>> getters =
//                list.stream().map(t -> (Supplier<BiValue<T, CompletableFuture<V>>>)(() -> new BiValue<>(t, getter.apply(t)))).collect(Collectors.toList());
//
//        return oneByOne(getters, (t, throwable) -> errConvert.apply(t, throwable));
//
//    }
//
//    /**
//     * 将所有future 一个接一个的执行，前一个得到结果，再执行下一个。
//     * @param getters 获取future的谓语集合
//     * @param errConvert 当遇到错误时的转换器
//     */
//    public static <T> CompletableFuture<List<T>> oneByOne(Collection<Supplier<BiValue<T, CompletableFuture<T>>>> getters, BiFunction<T, Throwable, T> errConvert) {
//        if(getters.isEmpty()) {
//            return CompletableFuture.completedFuture(new ArrayList<>());
//        }
//        CompletableFuture<List<T>> result = new CompletableFuture<>();
//
//        List<T> list = new ArrayList<>();
//
//        Function<T, T> successFunc = t -> {
//            list.add(t);
//            if(list.size() == getters.size()) {
//                result.complete(list);
//            }
//            return t;
//        };
//
//        BiFunction<T, Throwable, T> errorFunc = (r, throwable) -> {
//            T t = errConvert.apply(r, throwable);
//            list.add(t);
//            if(list.size() == getters.size()) {
//                result.complete(list);
//            }
//            return t;
//        };
//
//        CompletableFuture<T> temp = null;
//        for(Supplier<BiValue<T, CompletableFuture<T>>> getter: getters) {
//            BiValue<T, CompletableFuture<T>> bi = getter.get();
//            if (null == temp) {
//                temp = bi.getTwo().thenApply(successFunc)
//                        .exceptionally(throwable -> errorFunc.apply(bi.getOne(), throwable));
//            } else {
//                temp = temp.thenCompose(t -> bi.getTwo()).thenApply(successFunc)
//                        .exceptionally(throwable -> errorFunc.apply(bi.getOne(), throwable));
//            }
//        }
//
//        return result;
//    }

    /**
     * 将Future转换成 Mono
     */
    public static <T> Mono<T> toMono(CompletableFuture<T> future) {
        return Mono.fromFuture(future);
//        return Mono.create(sink -> {
//            sink.onCancel(() -> future.cancel(true));
//            future.whenComplete((t, throwable) -> {
//                if (throwable == null) {
//                    sink.success(t);
//                } else {
//                    sink.error(throwable);
//                }
//            });
//        });
    }


    public static <T> Flux<T> toFlux(CompletableFuture<? extends Collection<T>> future) {
        return Mono.fromFuture(future).flux().flatMapIterable(o -> o);
//        return Flux.create(sink -> {
//            sink.onCancel(() -> future.cancel(true));
//            future.whenComplete((ts, throwable) -> {
//                if (throwable == null) {
//                    ts.forEach(sink::next);
//                    sink.complete();
//                } else {
//                    sink.error(throwable);
//                }
//            });
//        });
    }

    public static <T> CompletableFuture<List<T>> page(BiFunction<Long, Long, CompletableFuture<Page<T>>> query, long page, long pageSize) {
        CompletableFuture<List<T>> future = new CompletableFuture<>();

        pageEach(query, page, pageSize, Page::hasNext, (ts, throwable) -> {
            if(null == throwable) {
                future.complete(ts);
            } else {
                future.completeExceptionally(new ValueException(ts, throwable));
            }
        });

        return future;
    }

    public static <T> void pageEach(BiFunction<Long, Long, CompletableFuture<Page<T>>> query,
                                    long page,
                                    long pageSize,
                                    Predicate<Page<T>> each,
                                    BiConsumer<List<T>, Throwable> done) {
        page(new ArrayList<>(), query, page, pageSize, each, done);
    }

    private static <T> void page(List<T> list,
                                 BiFunction<Long, Long, CompletableFuture<Page<T>>> query,
                                 long page,
                                 long pageSize,
                                 Predicate<Page<T>> test,
                                 BiConsumer<List<T>, Throwable> done) {
        CompletableFuture<Page<T>> future = query.apply(page, pageSize);
        future.thenAccept(p -> {
            list.addAll(p.getRecords());
            if(test.test(p)) {
                page(list, query, page + 1, pageSize, test, done);
            } else {
                done.accept(list, null);
            }
        }).exceptionally(throwable -> {
            done.accept(list, throwable);
            return null;
        });
    }
}
