import { HttpClient } from '@angular/common/http';
import { Inject, Injectable } from '@angular/core';
import { Router } from '@angular/router';
import { ACLService } from '@delon/acl';
import { ALAIN_I18N_TOKEN, MenuService, SettingsService, TitleService, App } from '@delon/theme';
import { NzSafeAny } from 'ng-zorro-antd/core/types';
import { NzIconService } from 'ng-zorro-antd/icon';
import {Observable, of, zip} from 'rxjs';
import { catchError, map } from 'rxjs/operators';
import { AlainConfig, ALAIN_CONFIG } from '@delon/util';

import { ICONS } from '../../../style-icons';
import { ICONS_AUTO } from '../../../style-icons-auto';
import { I18NService } from '../i18n/i18n.service';
import {StompService} from "../ws/stomp.service";

/**
 * Used for application startup
 * Generally used to get the basic data of the application, like: Menu Data, User Data, etc.
 */
@Injectable()
export class StartupService {
  constructor(
    iconSrv: NzIconService,
    private menuService: MenuService,
    @Inject(ALAIN_I18N_TOKEN) private i18n: I18NService,
    private settingService: SettingsService,
    private aclService: ACLService,
    private titleService: TitleService,
    private stomp: StompService,
    private httpClient: HttpClient,
    private router: Router,
    @Inject(ALAIN_CONFIG) private alainCfg: AlainConfig,
  ) {
    iconSrv.addIcon(...ICONS_AUTO, ...ICONS);
  }

  load(): Observable<void> {

    this.i18n.loadAndUse(this.i18n.defaultLang);

    const users$ = this.httpClient.get(`/api/sys/auth/users`).pipe(
      catchError((res) => {
        return of(null);
      }),
    );

    const app$ = this.httpClient.get('/api/sys/app-info?_allow_anonymous=true');

    return zip(app$, users$).pipe(
      // 接收其他拦截器后产生的异常消息
      catchError(res => {
        console.warn(`StartupService.load: Network request failed`, res);
        setTimeout(() => this.router.navigateByUrl(`/exception/500`));
        return [];
      }),
      map(([appResult, usersResult]) => {
        const ar = appResult as NzSafeAny;

        // Setting language data
        const app: App = ar.data as App;

        // Application information: including site name, description, year
        this.settingService.setApp(app);
        // Can be set page suffix title, https://ng-alain.com/theme/title
        if (app.name != null) {
          this.titleService.suffix = app.name;
        }
        if (usersResult) {
          const ur = usersResult as NzSafeAny;
          const users = ur.data;

          const user = users.user;
          const menu = users.menu;


          this.i18n.loadAndUse(user.language.replace('_', '-'));

          // Application data
          // User information: including name, avatar, email address
          this.settingService.setUser(user);
          // ACL: Set the permissions to full, https://ng-alain.com/acl/getting-started
          this.aclService.set({role: users.roles, ability: users.abilities});

          // this.aclService.setFull(true);
          // Menu data, https://ng-alain.com/theme/menu
          this.menuService.add(menu);
          this.stomp.connect("/ws");
        } else {
          setTimeout(() => {
            this.router.navigateByUrl(this.alainCfg.auth?.login_url as string);
          });
        }
      })
    );
  }


}
