import { Component, OnInit } from '@angular/core';
import { SFSchema } from '@delon/form';
import {_HttpClient, Menu} from '@delon/theme';
import { ArrayService } from '@delon/util';
import { NzContextMenuService, NzDropdownMenuComponent } from 'ng-zorro-antd/dropdown';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzFormatBeforeDropEvent, NzFormatEmitEvent, NzTreeNode } from 'ng-zorro-antd/tree';
import {mergeMap, Observable, of} from 'rxjs';
import { map, tap } from 'rxjs/operators';
import {NzUploadXHRArgs} from "ng-zorro-antd/upload";

@Component({
  selector: 'app-sys-ability',
  templateUrl: './ability.component.html',
})
export class SysAbilityComponent implements OnInit {
  private menuEvent!: NzFormatEmitEvent;

  data: NzTreeNode[] = [];
  op!: string;
  item: any;
  delDisabled = false;

  schema: SFSchema = {
    properties: {
      name: { type: 'string', title: '名称' },
      ability: { type: 'string', title: '权限点' },
      sortRank: { type: 'number', title: '排序值' },
      abilityType: {
        type: 'string',
        title: '权限类型',
        default: 'MENU',
        enum: [
          { label: '菜单', title: '菜单', value: 'MENU', checked: true },
          { label: '功能', title: '功能', value: 'FUNCTION' },
          { label: '字段', title: '字段', value: 'FIELD' },
        ],
      },
      enabled: { type: 'boolean', title: '是否启用', default: true },
      description: { type: 'string', title: '描述', ui: { widget: 'textarea', grid: { span: 24 } } },
    },
    required: ['text'],
    ui: { grid: { md: 24, lg: 12 }, spanLabelFixed: 100 },
  };

  objectSelect = {};

  uploading = false;

  constructor(
    private http: _HttpClient,
    private ccSrv: NzContextMenuService,
    private arrSrv: ArrayService,
    private msg: NzMessageService,
  ) {}

  ngOnInit(): void {
    this.getData();
  }

  private getData(): void {
    this.http.get('/api/sys/ability/all').subscribe((res) => {
      this.data = this.arrSrv.arrToTreeNode(res.data, {
        titleMapName: 'name',
        parentIdMapName: 'parentId',
        cb: (item, _parent, deep) => {
          item.expanded = deep <= 1;
        },
      });
    });
  }

  add(item: any): void {
    this.closeContextMenu();
    this.op = 'edit';
    this.item = {
      id: 0,
      text: '',
      parentId: item ? item.id : 0,
    };
  }

  edit(): void {
    this.closeContextMenu();
    this.op = 'edit';
  }

  save(item: any): void {
    this.http.post(`/api/sys/ability/${item.id ? 'update' : 'create'}`, item).subscribe((next) => {
      if (item.id <= 0) {
        this.getData();
        this.op = '';
      } else {
        this.item = item;
        this.menuEvent.node!.title = item.text;
        this.menuEvent.node!.origin = item;
        this.op = 'view';
      }
      this.getData();
    });
  }

  del(): void {
    this.closeContextMenu();
    this.http.delete(`/api/sys/ability/delete/${this.item.id}`).subscribe(() => {
      this.menuEvent.node!.remove();
      this.op = '';
      this.getData();
    });
  }

  get delMsg(): string {
    if (!this.menuEvent) {
      return '';
    }
    const childrenLen = this.menuEvent.node!.children.length;
    if (childrenLen === 0) {
      return `确认删除【${this.menuEvent.node!.title}】吗？`;
    }
    return `确认删除【${this.menuEvent.node!.title}】以及所有子项吗？`;
  }

  move = (e: NzFormatBeforeDropEvent) => {
    // if (e.pos !== 0) {
    //   this.msg.warning(`只支持不同类目的移动，且无法移动至顶层`);
    //   return of(false);
    // }
    if (e.dragNode.origin['parent_id'] === e.node.origin['id']) {
      return of(false);
    }
    const from = e.dragNode.origin['id'];
    const to = e.node.origin['id'];
    return this.http
      .post('/api/sys/ability/move', {
        from,
        to,
        pos: e.pos,
      })
      .pipe(
        tap(() => {
          this.op = '';
        }),
        map((data) => data),
      );
    // tslint:disable-next-line: semicolon
  };

  show(e: NzFormatEmitEvent): void {
    this.op = e.node!.isSelected ? 'view' : '';
    this.item = e.node!.origin;
  }

  showContextMenu(e: NzFormatEmitEvent, menu: NzDropdownMenuComponent): void {
    this.menuEvent = e;
    // 如果不希望直接批量删除子节点，则解除这个注释
    // this.delDisabled = e.node!.children.length !== 0;
    this.ccSrv.create(e.event!, menu);
  }

  closeContextMenu(): void {
    this.ccSrv.close();
  }

  export(item: any): void {
    this.http.get(`/api/sys/ability/export/${item.key}`, {}).subscribe(value => {
      // console.log(JSON.parse(value.data));
      this.download(item.title!, value.data);
    });
  }

  exportAll(): void {
    this.http.get(`/api/sys/ability/export`, {}).subscribe(value => {
      this.download("root", value.data);
    });
  }

  download(filename: string, txt: string): void {
    const file = new File([txt], filename + ".json", {
      type: "text/plain",
    });
    const tmpLink = document.createElement("a");
    const objectUrl = URL.createObjectURL(file);

    tmpLink.href = objectUrl;
    tmpLink.download = file.name;
    document.body.appendChild(tmpLink);
    tmpLink.click();

    document.body.removeChild(tmpLink);
    URL.revokeObjectURL(objectUrl);
  }


  nzCustomRequest(id: string = '') {
    return (item: NzUploadXHRArgs) => {

      this.uploading = true;

      const observable = new Observable(sub => {
        const reader = new FileReader();
        reader.onload = ev => {
          sub.next(reader.result);
          sub.complete();
        }
        reader.readAsText(item.file as any);
      });



      return observable
        .pipe(
          mergeMap(value => this.http.post(item.action!, { id, json: value }))
        )
        .subscribe(value => {
          this.uploading = false;
          if(value.success) {
            this.msg.success('上传成功');
          } else {
            this.msg.error(value.message);
          }
          this.getData();
        }, () => {
          this.uploading = false;
        });


    }
  }

  rebuildPath(item: any) {
    this.http.post(`/api/sys/ability/rebuild-path/${item.id}`, {}).subscribe(value => {
      if(value.success) {
        this.msg.success('重建成功');
        this.getData();
      } else {
        this.msg.error(value.message);
      }
    });

  }
}
