import {GridOptions, GridReadyEvent, ICellRendererParams} from '@ag-grid-community/core';
import {RowNode} from '@ag-grid-community/core/dist/cjs/entities/rowNode';
import {FirstDataRenderedEvent} from '@ag-grid-community/core/dist/cjs/events';
import {Component, OnInit, ViewChild} from '@angular/core';
import {ACLService} from '@delon/acl';
import {SFSchema} from '@delon/form';
import {_HttpClient} from '@delon/theme';
import {
  AclColDef,
  asFilterInputPropertiesUI,
  IGridDataSource,
  NgxGridTableComponent,
  NgxGridTableConstants
} from '@shared';
import {NzMessageService} from 'ng-zorro-antd/message';
import {NzModalService} from 'ng-zorro-antd/modal';

import {DataSourceUtils} from '../../../DataSourceUtils';
import {ACME_ORDER_STATUS} from '../seo.enums';
import {AcmeOrderBatchCreateComponent} from './modal/batch-create.component';
import {AcmeOrderCreateComponent} from './modal/create.component';
import {AcmeOrderEditComponent} from './modal/edit.component';
import {AcmeOrderViewComponent} from './modal/view.component';

@Component({
  selector: 'app-acme-order',
  templateUrl: './acme-order.component.html'
})
export class AcmeOrderComponent implements OnInit {
  // properties 的定义为 filter-input.widget.ts -> FilterInputUISchema 接口
  // id会转换为 { type: 'integer', title: 'id', ui: { widget: 'filter-input', filterType: 'number', options: ['equals'] } }
  schema: SFSchema = asFilterInputPropertiesUI({
    properties: {
      account: { type: 'string', title: 'acme账号', ui: { options: ['contains'] } },
      // 所有未知的复杂类型都当作枚举处理
      status: { type: 'array', title: '状态', ui: { values: ACME_ORDER_STATUS } },
      provider: { type: 'string', title: 'acme提供者', ui: { options: ['contains'] } },
      orderReference: { type: 'string', title: '订单', ui: { options: ['contains'] } },
      domain: { type: 'string', title: '顶级域名', ui: { options: ['contains'] } },
      sec: { type: 'string', title: '二级域名', ui: { options: ['contains'] } },
      createdTime: { type: 'string', title: '创建时间', format: 'date-time', ui: { options: ['inRange'] } },
      creator: { type: 'string', title: '创建人', ui: { options: ['contains'] } },
      updatedTime: { type: 'string', title: '更新时间', format: 'date-time', ui: { options: ['inRange'] } },
      updater: { type: 'string', title: '更新人', ui: { options: ['contains'] } },
      lastRenewTime: { type: 'string', title: 'renew时间', format: 'date-time', ui: { options: ['inRange'] } }
    },
    required: ['text'],
    ui: {
      width: 275,
      spanLabelFixed: 80,
      optionShowType: 'symbol'
      // aclTmpl: 'POST:/{}/OUT',
      // acl: { ability: ['POST:/TEST0'] },
    }
  });

  // 表格配置
  columnDefs: AclColDef[] = [
    // { headerName: 'testACL', field: 'testACL', acl: { ability: ['POST:/TEST'] } }, //加上acl后只有符合权限的才展示出来
    // { headerName: 'group', field: 'typeGroup', enableRowGroup: true }, // 需要分组查询，则将 enableRowGroup设置为true
    { headerName: 'id', field: 'id', sort: 'desc', sortable: true, checkboxSelection: true, headerCheckboxSelection: true },
    { headerName: 'acme账号', field: 'account' },
    { headerName: '状态', field: 'status' },
    { headerName: 'acme提供者', field: 'provider' },
    { headerName: '订单(可以用域名)', field: 'orderReference' },
    { headerName: '顶级域名', field: 'domain' },
    { headerName: '二级域名', field: 'sec' },
    { headerName: '创建时间', field: 'createdTime', sortable: true },
    { headerName: '创建人', field: 'creator' },
    { headerName: '更新时间', field: 'updatedTime', sortable: true },
    { headerName: 'renew时间', field: 'lastRenewTime', sortable: true },
    { headerName: '更新人', field: 'updater' },
    { headerName: '操作', field: NgxGridTableConstants.OPTION_FIELD }
  ];

  gridOptions: GridOptions;

  dataSource: IGridDataSource<any>;

  @ViewChild(NgxGridTableComponent)
  table!: NgxGridTableComponent;

  domains = '';

  constructor(private http: _HttpClient, private aclService: ACLService, private message: NzMessageService, private modal: NzModalService) {
    this.gridOptions = {
      enableCharts: false,
      sideBar: 'default',
      columnDefs: this.columnDefs,
      enableRangeSelection: true,
      // rowSelection: 'multiple',
      getRowNodeId: this.idGetter,
      onFirstDataRendered(event: FirstDataRenderedEvent): void {
        event.columnApi.autoSizeAllColumns();
      }
    };
    this.dataSource = DataSourceUtils.rowQuery(http, '/api/star-ring/acme/order/query', r => r);
  }

  ngOnInit(): void {}

  idGetter(data: any): any {
    return data.id;
  }

  onGridReady(e: { event: GridReadyEvent; gridTable: NgxGridTableComponent }): void {
    // 添加右键菜单
    this.table.addMenu({
      name: '删除',
      show: 'selected',
      acl: { ability: ['acme-order:delete'] },
      callback: selected => {
        this.modal.confirm({
          nzTitle: '是否删除?',
          nzContent: `确认删除ID为 [${this.idGetter(selected)}] 的记录？`,
          nzOnOk: () => {
            this.http.delete(`/api/star-ring/acme/order/delete/${this.idGetter(selected)}`).subscribe(value => {
              this.message.success('删除成功');
              this.table.refresh();
            });
          }
        });
      }
    });
    // 按每列内容重新分配列宽
    this.table.columnApi.autoSizeAllColumns();
  }

  onCreate(): void {
    this.modal
      .create({
        nzContent: AcmeOrderCreateComponent,
        nzFooter: null,
        nzMaskClosable: false
      })
      .afterClose.subscribe(result => {
        this.table.refresh();
      });
  }

  onBatchCreate(): void {
    this.modal
      .create({
        nzContent: AcmeOrderBatchCreateComponent,
        nzFooter: null,
        nzMaskClosable: false
      })
      .afterClose.subscribe(result => {
        this.table.refresh();
      });
  }

  onEdit(cell: ICellRendererParams, row: RowNode): void {
    this.modal
      .create({
        nzContent: AcmeOrderEditComponent,
        nzComponentParams: { record: row.data },
        nzFooter: null,
        nzMaskClosable: false
      })
      .afterClose.subscribe(result => {
        this.table.refresh();
      });
  }

  onView(cell: ICellRendererParams, row: RowNode): void {
    this.modal.create({
      nzContent: AcmeOrderViewComponent,
      nzComponentParams: { record: row.data },
      nzFooter: null,
      nzMaskClosable: true
    });
  }

  reApply(cell: any, row: any): void {
    row.data.loadding = true;
    this.http
      .post(`/api/star-ring/acme/order/re-apply`, { reference: row.data.orderReference })
      .pipe()
      .subscribe(res => {
        row.data.loadding = false;
        if (res) {
          this.message.success('添加成功');
          this.table.refresh();
        }
      });
  }

  onRestart(): void {
    this.http
      .post(`/api/star-ring/acme/order/re-start`)
      .pipe()
      .subscribe(() => {
        this.modal.closeAll();
        this.message.success('任务已提交');
      });
  }

  onPush(): void {
    this.http
      .post(`/api/star-ring/acme/order/push-ssl`, this.domains.split('\n'))
      .pipe()
      .subscribe(() => {
        this.modal.closeAll();
        this.message.success('任务已提交');
      });
  }

  onCheckInfo(cell: ICellRendererParams, row: RowNode): void {
    this.http
      .get(`/api/star-ring/acme/order/check-info/${row.data.id}`)
      .pipe()
      .subscribe(res => {
        if (res) {
          this.modal.success({
            nzTitle: '查证信息',
            nzContent: `
              <p>是否有证书文件: ${res.hasFullChain ? '是' : '否'} </p>
              <p>location: <a href="${res.location}" target="_blank">${res.location}</a> </p>
              <p>服务器: ${res.domain.serverTag} </p>
              <p>ip: ${res.ip} </p>
            `
          });
        }
      });
  }

  onBatchReApply(): void {
    this.http
      .post(`/api/star-ring/acme/order/batch-renew`, this.domains.split('\n'))
      .pipe()
      .subscribe(() => {
        this.modal.closeAll();
        this.message.success('任务已提交');
      });
  }

  onShowQueue() {
    this.http.post('/api/star-ring/acme/order/queue').subscribe(result => {
      if (result.success) {
        this.modal.success({
          nzTitle: '任务信息',
          nzContent: `<pre>${result.data}</pre>`
        });
      }
    });
  }

  onClearTask() {
    this.http.post('/api/star-ring/acme/order/queue/clear').subscribe(result => {
      if (result.success) {
        this.modal.closeAll();
        this.message.success('清理完成');
      }
    });
  }
}
