import { GridOptions, GridReadyEvent, ICellRendererParams } from '@ag-grid-community/core';
import { RowNode } from '@ag-grid-community/core/dist/cjs/entities/rowNode';
import { FirstDataRenderedEvent } from '@ag-grid-community/core/dist/cjs/events';
import { Component, OnInit, ViewChild } from '@angular/core';
import { DownloadTaskService } from '@core';
import { ACLService } from '@delon/acl';
import { SFSchema } from '@delon/form';
import { _HttpClient } from '@delon/theme';
import { AclColDef, asFilterInputPropertiesUI, IFilter, IGridDataSource, NgxGridTableComponent, NgxGridTableConstants } from '@shared';
import { SfQueryFormComponent } from '@shared';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalService } from 'ng-zorro-antd/modal';
import { NzUploadChangeParam } from 'ng-zorro-antd/upload';
import { DataSourceUtils } from '../../../DataSourceUtils';
import { TASK_STATUS } from '../seo.enums';
import { DownloadTaskCreateComponent } from './modal/create.component';
import { DownloadTaskEditComponent } from './modal/edit.component';
import { DownloadTaskViewComponent } from './modal/view.component';

@Component({
  selector: 'app-download-task',
  templateUrl: './download-task.component.html',
})
export class DownloadTaskComponent implements OnInit {
  // properties 的定义为 filter-input.widget.ts -> FilterInputUISchema 接口
  // id会转换为 { type: 'integer', title: 'id', ui: { widget: 'filter-input', filterType: 'number', options: ['equals'] } }
  schema: SFSchema = asFilterInputPropertiesUI({
    properties: {
      website: { type: 'string', title: '资源站点', ui: { options: ['contains'] } },
      url: { type: 'string', title: '资源url', ui: { options: ['contains'] } },
      savePath: { type: 'string', title: '资源保存地址', ui: { options: ['contains'] } },
      // 所有未知的复杂类型都当作枚举处理
      state: { type: 'array', title: '状态', ui: { options: ['in'], values: TASK_STATUS } },
      createdTime: { type: 'string', title: '创建时间', format: 'date-time', ui: { options: ['inRange'] } },
      creator: { type: 'string', title: '创建人', ui: { options: ['contains'] } },
    },
    required: ['text'],
    ui: {
      width: 275,
      spanLabelFixed: 80,
      optionShowType: 'symbol',
      // aclTmpl: 'POST:/{}/OUT',
      // acl: { ability: ['POST:/TEST0'] },
    },
  });

  // 表格配置
  columnDefs: AclColDef[] = [
    // { headerName: 'testACL', field: 'testACL', acl: { ability: ['POST:/TEST'] } }, //加上acl后只有符合权限的才展示出来
    // { headerName: 'group', field: 'typeGroup', enableRowGroup: true }, // 需要分组查询，则将 enableRowGroup设置为true
    { headerName: 'id', field: 'id', sort: 'desc', sortable: true, checkboxSelection: true, headerCheckboxSelection: true },
    { headerName: '操作', field: NgxGridTableConstants.OPTION_FIELD },
    { headerName: '资源站点', field: 'website' },
    { headerName: '资源url', field: 'url' },
    { headerName: '资源保存地址', field: 'savePath' },
    { headerName: '状态', field: 'state' },
    { headerName: '错误', field: 'error' },
    { headerName: '创建时间', field: 'createdTime' },
    { headerName: '更新时间', field: 'updatedTime' },
    { headerName: '创建人', field: 'creator' },
    { headerName: '更新人', field: 'updater' },
    { headerName: '下载错误次数', field: 'failTimes' },
  ];

  gridOptions: GridOptions;

  dataSource: IGridDataSource<any>;

  @ViewChild(NgxGridTableComponent)
  table!: NgxGridTableComponent;

  constructor(
    private http: _HttpClient,
    private aclService: ACLService,
    private downloadService: DownloadTaskService,
    private message: NzMessageService,
    private modal: NzModalService,
  ) {
    this.gridOptions = {
      enableCharts: false,
      sideBar: 'default',
      columnDefs: this.columnDefs,
      enableRangeSelection: true,
      // rowSelection: 'multiple',
      getRowNodeId: this.idGetter,
      onFirstDataRendered(event: FirstDataRenderedEvent): void {
        event.columnApi.autoSizeAllColumns();
      },
    };
    this.dataSource = DataSourceUtils.rowQuery(http, '/api/star-ring/download-task/query', (r) => r);
  }

  ngOnInit(): void {}

  idGetter(data: any): any {
    return data.id;
  }

  onGridReady(e: { event: GridReadyEvent; gridTable: NgxGridTableComponent }): void {
    // 添加右键菜单
    this.table.addMenu({
      name: '删除',
      show: 'selected',
      acl: { ability: ['download-task:delete'] },
      callback: (selected) => {
        this.modal.confirm({
          nzTitle: '是否删除?',
          nzContent: `确认删除ID为 [${this.idGetter(selected)}] 的记录？`,
          nzOnOk: () => {
            this.http.delete(`/api/star-ring/download-task/delete/${this.idGetter(selected)}`).subscribe((value) => {
              this.message.success('删除成功');
              this.table.refresh();
            });
          },
        });
      },
    });
    // 按每列内容重新分配列宽
    this.table.columnApi.autoSizeAllColumns();
  }

  filterHand = (filter: IFilter[], sf: SfQueryFormComponent): IFilter[] => {
    // 不做任何处理，直接返回表格生成的filter对象数组。
    return filter;
  };

  onCreate(): void {
    this.modal
      .create({
        nzContent: DownloadTaskCreateComponent,
        nzFooter: null,
        nzMaskClosable: false,
      })
      .afterClose.subscribe((result) => {
        this.table.refresh();
      });
  }

  onEdit(cell: ICellRendererParams, row: RowNode): void {
    this.modal
      .create({
        nzContent: DownloadTaskEditComponent,
        nzComponentParams: { record: row.data },
        nzFooter: null,
        nzMaskClosable: false,
      })
      .afterClose.subscribe((result) => {
        this.table.refresh();
      });
  }

  onView(cell: ICellRendererParams, row: RowNode): void {
    this.modal.create({
      nzContent: DownloadTaskViewComponent,
      nzComponentParams: { record: row.data },
      nzFooter: null,
      nzMaskClosable: true,
    });
  }

  onDownload(cell: any, row: any): void {
    this.downloadService.download(row.data.id).subscribe(() => {
      this.table.refresh();
    });
  }

  onUploadChange(info: NzUploadChangeParam): void {
    if (info.file.status !== 'uploading') {
      console.log(info.file, info.fileList);
    }
    if (info.file.status === 'done') {
      this.message.success(`${info.file.name} file uploaded successfully`);
    } else if (info.file.status === 'error') {
      this.message.error(`${info.file.name} file upload failed.`);
    }
  }
}
