import { GridOptions, GridReadyEvent, ICellRendererParams } from '@ag-grid-community/core';
import { RowNode } from '@ag-grid-community/core/dist/cjs/entities/rowNode';
import { FirstDataRenderedEvent } from '@ag-grid-community/core/dist/cjs/events';
import { Component, OnInit, ViewChild } from '@angular/core';
import { SpiderPlatformService } from '@core';
import { ACLService } from '@delon/acl';
import { SFSchema } from '@delon/form';
import { _HttpClient } from '@delon/theme';
import { AclColDef, asFilterInputPropertiesUI, IFilter, IGridDataSource, NgxGridTableComponent, NgxGridTableConstants } from '@shared';
import { SfQueryFormComponent } from '@shared';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalService } from 'ng-zorro-antd/modal';
import { DataSourceUtils } from '../../../DataSourceUtils';
import { PLATFORM_TYPE } from '../seo.enums';
import { PlatformCheckContentCreateComponent } from './modal/create.component';
import { PlatformCheckContentEditComponent } from './modal/edit.component';
import { PlatformCheckContentViewComponent } from './modal/view.component';

@Component({
  selector: 'app-platform-check-content',
  templateUrl: './platform-check-content.component.html',
})
export class PlatformCheckContentComponent implements OnInit {
  // properties 的定义为 filter-input.widget.ts -> FilterInputUISchema 接口
  // id会转换为 { type: 'integer', title: 'id', ui: { widget: 'filter-input', filterType: 'number', options: ['equals'] } }
  schema: SFSchema = asFilterInputPropertiesUI({
    properties: {
      // 所有未知的复杂类型都当作枚举处理
      platformType: { type: 'array', title: '平台', ui: { options: ['in'], values: PLATFORM_TYPE } },
      domain: { type: 'string', title: '域名', ui: { options: ['contains'] } },
      username: { type: 'string', title: '账号', ui: { options: ['contains'] } },
      submitted: {
        type: 'array',
        title: '提交成功',
        ui: {
          options: ['in'],
          values: [
            { label: '是', value: true },
            { label: '否', value: false },
          ],
        },
      },
      linkPublished: {
        type: 'array',
        title: '已发布',
        ui: {
          options: ['in'],
          values: [
            { label: '是', value: true },
            { label: '否', value: false },
          ],
        },
      },
    },
    required: ['text'],
    ui: {
      width: 275,
      spanLabelFixed: 80,
      optionShowType: 'symbol',
      // aclTmpl: 'POST:/{}/OUT',
      // acl: { ability: ['POST:/TEST0'] },
    },
  });

  // 表格配置
  columnDefs: AclColDef[] = [
    // { headerName: 'testACL', field: 'testACL', acl: { ability: ['POST:/TEST'] } }, //加上acl后只有符合权限的才展示出来
    // { headerName: 'group', field: 'typeGroup', enableRowGroup: true }, // 需要分组查询，则将 enableRowGroup设置为true
    { headerName: 'id', field: 'id', sort: 'desc', sortable: true, checkboxSelection: true, headerCheckboxSelection: true },
    { headerName: '操作', field: NgxGridTableConstants.OPTION_FIELD },
    { headerName: '平台', field: 'platformType' },
    { headerName: '域名', field: 'domain' },
    { headerName: '检测内容', field: 'checkContent' },
    { headerName: '账号', field: 'username' },
    { headerName: '提交成功', field: 'submitted' },
    { headerName: '返回结果', field: 'result' },
    { headerName: '缓存1', field: 'temp' },
    { headerName: '缓存2', field: 'temp2' },
    { headerName: '已发布', field: 'linkPublished' },
  ];

  gridOptions: GridOptions;

  dataSource: IGridDataSource<any>;

  @ViewChild(NgxGridTableComponent)
  table!: NgxGridTableComponent;

  constructor(
    private http: _HttpClient,
    private aclService: ACLService,
    private message: NzMessageService,
    private spiderService: SpiderPlatformService,
    private modal: NzModalService,
  ) {
    this.gridOptions = {
      enableCharts: false,
      sideBar: 'default',
      columnDefs: this.columnDefs,
      enableRangeSelection: true,
      // rowSelection: 'multiple',
      getRowNodeId: this.idGetter,
      onFirstDataRendered(event: FirstDataRenderedEvent): void {
        event.columnApi.autoSizeAllColumns();
      },
    };
    this.dataSource = DataSourceUtils.rowQuery(http, '/api/star-ring/platform-check-content/query', (r) => r);
  }

  ngOnInit(): void {}

  idGetter(data: any): any {
    return data.id;
  }

  onGridReady(e: { event: GridReadyEvent; gridTable: NgxGridTableComponent }): void {
    // 添加右键菜单
    this.table.addMenu({
      name: '删除',
      show: 'selected',
      acl: { ability: ['platform-check-content:delete'] },
      callback: (selected) => {
        this.modal.confirm({
          nzTitle: '是否删除?',
          nzContent: `确认删除ID为 [${this.idGetter(selected)}] 的记录？`,
          nzOnOk: () => {
            this.http.delete(`/api/star-ring/platform-check-content/delete/${this.idGetter(selected)}`).subscribe((value) => {
              this.message.success('删除成功');
              this.table.refresh();
            });
          },
        });
      },
    });
    // 按每列内容重新分配列宽
    this.table.columnApi.autoSizeAllColumns();
  }

  /**
   * IFilter表示查询条件。
   * 表格内置的 SfQueryFormComponent 将并根 SFSchema 来生成 IFilter 对象数组
   * 可以通过sf.value 将会获得每一个属性UI中填入的值。如果是 {widget: 'filter-input'} 的字段才自动将控件内填入的值转成 IFilter 对象
   * 其他非 filter-input 的控件，需要开发人员手动转换。
   * 如果开发人员需要给表格额外添加条件，也可以在这里加入更多的IFilter
   *
   * 自动生成的代码会将所有 SFSchema 的 UI 转成 filter-input，因此，默认是直接返回所有 filter
   * @param filter 所有filter-input生成的 IFilter对象
   * @param sf 表格的form对象。
   * @result 返回处理后的查询条件数组。
   */
  filterHand = (filter: IFilter[], sf: SfQueryFormComponent): IFilter[] => {
    // 不做任何处理，直接返回表格生成的filter对象数组。
    return filter;
  };

  onCreate(): void {
    this.modal
      .create({
        nzContent: PlatformCheckContentCreateComponent,
        nzFooter: null,
        nzMaskClosable: false,
      })
      .afterClose.subscribe((result) => {
        this.table.refresh();
      });
  }

  onEdit(cell: ICellRendererParams, row: RowNode): void {
    this.modal
      .create({
        nzContent: PlatformCheckContentEditComponent,
        nzComponentParams: { record: row.data },
        nzFooter: null,
        nzMaskClosable: false,
      })
      .afterClose.subscribe((result) => {
        this.table.refresh();
      });
  }

  onView(cell: ICellRendererParams, row: RowNode): void {
    this.modal.create({
      nzContent: PlatformCheckContentViewComponent,
      nzComponentParams: { record: row.data },
      nzFooter: null,
      nzMaskClosable: true,
    });
  }

  onSubmitPlatform(): void {
    const ids = this.table.getCheckedData().map(this.idGetter);
    this.spiderService.submitByContentId(ids).subscribe((value) => {
      this.message.success(value);
      this.table.refresh();
    });
  }

  onPushLink(): void {
    const ids = this.table.getCheckedData().map(this.idGetter);
    this.spiderService.push(ids).subscribe((value) => {
      this.message.success(value);
      this.table.refresh();
    });
  }

  onPushSitemap(): void {
    const ids = this.table.getCheckedData().map(this.idGetter);
    this.spiderService.pushSitemap(ids).subscribe((value) => {
      this.message.success(value);
      this.table.refresh();
    });
  }
}
