import { GridOptions, GridReadyEvent, ICellRendererParams } from '@ag-grid-community/core';
import { RowNode } from '@ag-grid-community/core/dist/cjs/entities/rowNode';
import { FirstDataRenderedEvent } from '@ag-grid-community/core/dist/cjs/events';
import { Component, OnInit, ViewChild } from '@angular/core';
import { ACLService } from '@delon/acl';
import { SFSchema } from '@delon/form';
import { _HttpClient } from '@delon/theme';
import { AclColDef, asFilterInputPropertiesUI, IFilter, IGridDataSource, NgxGridTableComponent, NgxGridTableConstants } from '@shared';
import { SfQueryFormComponent } from '@shared';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalService } from 'ng-zorro-antd/modal';
import { DataSourceUtils } from '../../../DataSourceUtils';
import { PLATFORM_TYPE } from '../seo.enums';
import { PlatformCookieCreateComponent } from './modal/create.component';
import { PlatformCookieEditComponent } from './modal/edit.component';
import { PlatformCookieViewComponent } from './modal/view.component';

@Component({
  selector: 'app-platform-cookie',
  templateUrl: './platform-cookie.component.html',
})
export class PlatformCookieComponent implements OnInit {
  // properties 的定义为 filter-input.widget.ts -> FilterInputUISchema 接口
  // id会转换为 { type: 'integer', title: 'id', ui: { widget: 'filter-input', filterType: 'number', options: ['equals'] } }
  schema: SFSchema = asFilterInputPropertiesUI({
    properties: {
      // 所有未知的复杂类型都当作枚举处理
      platformType: { type: 'array', title: '平台', ui: { options: ['in'], values: PLATFORM_TYPE } },
      username: { type: 'string', title: '账号', ui: { options: ['contains'] } },
      valid: {
        type: 'array',
        title: '是否检验',
        ui: {
          options: ['in'],
          values: [
            { label: '是', value: true },
            { label: '否', value: false },
          ],
        },
      },
      auto: {
        type: 'array',
        title: '是否自动',
        ui: {
          options: ['in'],
          values: [
            { label: '是', value: true },
            { label: '否', value: false },
          ],
        },
      },
    },
    required: ['text'],
    ui: {
      width: 275,
      spanLabelFixed: 80,
      optionShowType: 'symbol',
    },
  });

  // 表格配置
  columnDefs: AclColDef[] = [
    { headerName: 'id', field: 'id', sort: 'desc', sortable: true, checkboxSelection: true, headerCheckboxSelection: true },
    { headerName: '平台', field: 'platformType' },
    { headerName: 'cookie', field: 'cookie', maxWidth: 200 },
    { headerName: '账号', field: 'username' },
    { headerName: '是否检验', field: 'valid' },
    { headerName: '是否自动', field: 'auto' },
    { headerName: '操作', field: NgxGridTableConstants.OPTION_FIELD },
  ];

  gridOptions: GridOptions;

  dataSource: IGridDataSource<any>;

  @ViewChild(NgxGridTableComponent)
  table!: NgxGridTableComponent;

  constructor(private http: _HttpClient, private aclService: ACLService, private message: NzMessageService, private modal: NzModalService) {
    this.gridOptions = {
      enableCharts: false,
      sideBar: 'default',
      columnDefs: this.columnDefs,
      enableRangeSelection: true,
      // rowSelection: 'multiple',
      getRowNodeId: this.idGetter,
      onFirstDataRendered(event: FirstDataRenderedEvent): void {
        event.columnApi.autoSizeAllColumns();
      },
    };
    this.dataSource = DataSourceUtils.rowQuery(http, '/api/star-ring/platform-cookie/query', (r) => r);
  }

  ngOnInit(): void {}

  idGetter(data: any): any {
    return data.id;
  }

  onGridReady(): void {
    // 添加右键菜单
    this.table.addMenu({
      name: '删除',
      show: 'selected',
      acl: { ability: ['platform-cookie:delete'] },
      callback: (selected) => {
        this.modal.confirm({
          nzTitle: '是否删除?',
          nzContent: `确认删除ID为 [${this.idGetter(selected)}] 的记录？`,
          nzOnOk: () => {
            this.http.delete(`/api/star-ring/platform-cookie/delete/${this.idGetter(selected)}`).subscribe((value) => {
              this.message.success('删除成功');
              this.table.refresh();
            });
          },
        });
      },
    });
    // 按每列内容重新分配列宽
    this.table.columnApi.autoSizeAllColumns();
  }

  onCreate(): void {
    this.modal
      .create({
        nzContent: PlatformCookieCreateComponent,
        nzFooter: null,
        nzMaskClosable: false,
      })
      .afterClose.subscribe((result) => {
        this.table.refresh();
      });
  }

  onEdit(cell: ICellRendererParams, row: RowNode): void {
    this.modal
      .create({
        nzContent: PlatformCookieEditComponent,
        nzComponentParams: { record: row.data },
        nzFooter: null,
        nzMaskClosable: false,
      })
      .afterClose.subscribe((result) => {
        this.table.refresh();
      });
  }

  onView(cell: ICellRendererParams, row: RowNode): void {
    this.modal.create({
      nzContent: PlatformCookieViewComponent,
      nzComponentParams: { record: row.data },
      nzFooter: null,
      nzMaskClosable: true,
    });
  }
}
