import { GridOptions, GridReadyEvent, ICellRendererParams } from '@ag-grid-community/core';
import { RowNode } from '@ag-grid-community/core/dist/cjs/entities/rowNode';
import { FirstDataRenderedEvent } from '@ag-grid-community/core/dist/cjs/events';
import { RowDoubleClickedEvent } from '@ag-grid-enterprise/all-modules';
import { Component, OnInit, ViewChild } from '@angular/core';
import { EventManager } from '@angular/platform-browser';
import { DomainProposalService, DownloadTaskService, TemplateGroupService } from '@core';
import { ACLService } from '@delon/acl';
import { SFSchema } from '@delon/form';
import { _HttpClient } from '@delon/theme';
import { AclColDef, asFilterInputPropertiesUI, IFilter, IGridDataSource, NgxGridTableComponent, NgxGridTableConstants } from '@shared';
import { SfQueryFormComponent } from '@shared';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalComponent, NzModalService } from 'ng-zorro-antd/modal';
import { NzUploadChangeParam } from 'ng-zorro-antd/upload';
import { DataSourceUtils } from '../../../DataSourceUtils';
import { DOMAIN_TYPE, PROPOSAL_STATUS } from '../seo.enums';
import { SeoDomainProposalEditComponent } from './modal/edit.component';
import { SeoDomainProposalViewComponent } from './modal/view.component';

@Component({
  selector: 'app-seo-domain-proposal',
  templateUrl: './seo-domain-proposal.component.html',
})
export class SeoDomainProposalComponent implements OnInit {
  // properties 的定义为 filter-input.widget.ts -> FilterInputUISchema 接口
  // id会转换为 { type: 'integer', title: 'id', ui: { widget: 'filter-input', filterType: 'number', options: ['equals'] } }
  schema: SFSchema = asFilterInputPropertiesUI({
    properties: {
      reference: { type: 'string', title: '单号', ui: { options: ['equals'] } },
      domain: { type: 'string', title: '域名', ui: { options: ['contains'] } },
      templateWebsite: { type: 'string', title: '站点模板', ui: { options: ['contains'] } },
      title: { type: 'string', title: 'title', ui: { options: ['contains'] } },
      keywords: { type: 'string', title: '关键词列表', ui: { options: ['contains'] } },
      mainKey: { type: 'string', title: '主关键词', ui: { options: ['contains'] } },
      // 所有未知的复杂类型都当作枚举处理
      type: { type: 'array', title: '类型', ui: { options: ['in'], values: DOMAIN_TYPE } },
      state: { type: 'array', title: '状态', ui: { options: ['in'], values: PROPOSAL_STATUS } },
      createdTime: { type: 'string', title: '创建时间', format: 'date-time', ui: { options: ['inRange'] } },
      creator: { type: 'string', title: '创建人', ui: { options: ['contains'] } },
      auditor: { type: 'string', title: '审核人', ui: { options: ['contains'] } },
      auditedTime: { type: 'string', title: '审核时间', format: 'date-time', ui: { options: ['inRange'] } },
      serverTag: { type: 'string', title: '服务器标记', ui: { options: ['contains'] } },
    },
    required: ['text'],
    ui: {
      width: 275,
      spanLabelFixed: 80,
      optionShowType: 'symbol',
      // aclTmpl: 'POST:/{}/OUT',
      // acl: { ability: ['POST:/TEST0'] },
    },
  });

  // 表格配置
  columnDefs: AclColDef[] = [
    // { headerName: 'testACL', field: 'testACL', acl: { ability: ['POST:/TEST'] } }, //加上acl后只有符合权限的才展示出来
    // { headerName: 'group', field: 'typeGroup', enableRowGroup: true }, // 需要分组查询，则将 enableRowGroup设置为true
    { headerName: 'id', field: 'id', sort: 'desc', sortable: true, checkboxSelection: true, headerCheckboxSelection: true },
    { headerName: '操作', field: NgxGridTableConstants.OPTION_FIELD },
    { headerName: '单号', field: 'reference' },
    { headerName: '域名', field: 'domain' },
    { headerName: '站点模板', field: 'templateWebsite' },
    { headerName: 'title', field: 'title' },
    { headerName: '关键词列表', field: 'keywords' },
    { headerName: 'list页面的title', field: 'listPageTitle' },
    { headerName: '其他菜单名，用逗号隔开', field: 'otherMenus' },
    { headerName: '主关键词', field: 'mainKey' },
    { headerName: '类型', field: 'type' },
    { headerName: '网站描述', field: 'description' },
    { headerName: '状态', field: 'state' },
    { headerName: '创建时间', field: 'createdTime' },
    { headerName: '创建人', field: 'creator' },
    { headerName: '更新时间', field: 'updatedTime' },
    { headerName: '更新人', field: 'updater' },
    { headerName: '审核人', field: 'auditor' },
    { headerName: '审核时间', field: 'auditedTime' },
    { headerName: '审核备注', field: 'remark' },
    { headerName: '服务器标记', field: 'serverTag' },
  ];

  gridOptions: GridOptions;

  dataSource: IGridDataSource<any>;

  @ViewChild(NgxGridTableComponent)
  table!: NgxGridTableComponent;

  submitting = false;
  remark = '';
  selectedData!: any;

  groups!: any;

  selectedGroupId!: number;

  @ViewChild('auditModal') auditModal!: NzModalComponent;
  @ViewChild('groupModal') groupModal!: NzModalComponent;

  constructor(
    private http: _HttpClient,
    private aclService: ACLService,
    private message: NzMessageService,
    private eventManager: EventManager,
    private downloadService: DownloadTaskService,
    private groupService: TemplateGroupService,
    private proposalService: DomainProposalService,
    private modal: NzModalService,
  ) {
    this.gridOptions = {
      enableCharts: false,
      sideBar: 'default',
      columnDefs: this.columnDefs,
      enableRangeSelection: true,
      // rowSelection: 'multiple',
      getRowNodeId: this.idGetter,
      onRowDoubleClicked: (event: RowDoubleClickedEvent): void => {
        let frame: any;
        frame = window.open('about:blank', '_blank');
        frame.location = 'https://' + event.node.data.domain;
      },
      onFirstDataRendered(event: FirstDataRenderedEvent): void {
        event.columnApi.autoSizeAllColumns();
      },
    };
    this.dataSource = DataSourceUtils.rowQuery(http, '/api/star-ring/seo-domain-proposal/query', (r) => r);
  }

  ngOnInit(): void {
    this.eventManager.addGlobalEventListener('window', 'keyup', (e: any) => {
      // debugger
      if (e.ctrlKey && e.key.toLowerCase() === 'capslock') {
        console.log('你输入了 ctrl + CapsLock');
        this.onAudit(this.selectedData);
      }
    });
  }

  idGetter(data: any): any {
    return data.id;
  }

  onGridReady(e: { event: GridReadyEvent; gridTable: NgxGridTableComponent }): void {
    // 添加右键菜单
    this.table.addMenu({
      name: '删除',
      show: 'selected',
      acl: { ability: ['seo-domain-proposal:delete'] },
      callback: (selected) => {
        this.modal.confirm({
          nzTitle: '是否删除?',
          nzContent: `确认删除ID为 [${this.idGetter(selected)}] 的记录？`,
          nzOnOk: () => {
            this.http.delete(`/api/star-ring/seo-domain-proposal/delete/${this.idGetter(selected)}`).subscribe((value) => {
              this.message.success('删除成功');
              this.table.refresh();
            });
          },
        });
      },
    });
    // 按每列内容重新分配列宽
    this.table.columnApi.autoSizeAllColumns();
  }

  filterHand = (filter: IFilter[], sf: SfQueryFormComponent): IFilter[] => {
    // 不做任何处理，直接返回表格生成的filter对象数组。
    return filter;
  };

  onEdit(cell: ICellRendererParams, row: RowNode): void {
    this.modal
      .create({
        nzContent: SeoDomainProposalEditComponent,
        nzComponentParams: { record: row.data },
        nzFooter: null,
        nzMaskClosable: false,
      })
      .afterClose.subscribe((result) => {
        this.table.refresh();
      });
  }

  onView(cell: ICellRendererParams, row: RowNode): void {
    this.modal.create({
      nzContent: SeoDomainProposalViewComponent,
      nzComponentParams: { record: row.data },
      nzFooter: null,
      nzMaskClosable: true,
    });
  }

  onBatchPass(): void {
    const ids = this.table.getCheckedData().map(this.idGetter);
    this.proposalService.batchPass(ids).subscribe(() => {
      this.table.refresh();
    });
  }

  onChangeTemplate(cell: any, row: any): void {
    this.groupService.groups().subscribe((value) => {
      this.selectedData = row.data;
      this.groups = value;
      this.groupModal.open();
    });
  }

  onOffline(cell: any, row: any): void {
    this.proposalService.offline(row.data.id).subscribe(() => {
      this.table.refresh();
    });
  }

  onAudit(data: any): void {
    this.selectedData = data;
    this.proposalService.lock(this.selectedData.id).subscribe((value) => {
      this.table.refresh();
      this.auditModal.open();
    });
  }

  onReDownload(cell: any, row: any): void {
    this.download(row.data, false);
  }

  onDownloadNotDone(cell: any, row: any): void {
    this.download(row.data, true);
  }

  doAudit(pass: boolean): void {
    this.submitting = true;
    this.proposalService.audit(this.selectedData.id, pass, this.remark).subscribe(() => {
      this.submitting = false;
      this.auditModal.close();
      this.table.refresh();
    });
  }

  doChangeGroup(): void {
    this.proposalService.changeTmp([this.selectedData.id], this.selectedGroupId).subscribe(() => {
      this.message.success('修改成功');
      this.groupModal.close();
      this.table.refresh();
    });
  }

  private download(data: any, failOnly: boolean): void {
    this.downloadService.check(data.templateWebsite).subscribe((value) => {
      this.message.success(`共计[${value.total}]条记录，之前下载过：[${value.ends}]条，重新下载后将会再给提示`);
    });
    this.downloadService.downloadFor(data.templateWebsite, failOnly).subscribe((value) => {
      this.message.success('Z成功');
      this.table.refresh();
    });
  }

  onUploadChange(info: NzUploadChangeParam): void {
    if (info.file.status !== 'uploading') {
      console.log(info.file, info.fileList);
    }
    if (info.file.status === 'done') {
      this.message.success(`${info.file.name} file uploaded successfully`);
      this.table.refresh();
    } else if (info.file.status === 'error') {
      this.message.error(`${info.file.name} file upload failed.`);
    }
  }
}
