import { GridOptions, GridReadyEvent, ICellRendererParams } from '@ag-grid-community/core';
import { RowNode } from '@ag-grid-community/core/dist/cjs/entities/rowNode';
import { FirstDataRenderedEvent } from '@ag-grid-community/core/dist/cjs/events';
import { RowDoubleClickedEvent } from '@ag-grid-enterprise/all-modules';
import { Component, OnInit, ViewChild } from '@angular/core';
import { ACLService } from '@delon/acl';
import { SFSchema } from '@delon/form';
import { _HttpClient } from '@delon/theme';
import { environment } from '@env/environment';
import { AclColDef, asFilterInputPropertiesUI, IFilter, IGridDataSource, NgxGridTableComponent, NgxGridTableConstants } from '@shared';
import { SfQueryFormComponent } from '@shared';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalService } from 'ng-zorro-antd/modal';
import { NzUploadChangeParam } from 'ng-zorro-antd/upload';
import { DataSourceUtils } from '../../../DataSourceUtils';
import { DOMAIN_TYPE } from '../seo.enums';
import { SeoDomainEditComponent } from './modal/edit.component';
import { SeoDomainViewComponent } from './modal/view.component';

@Component({
  selector: 'app-seo-domain',
  templateUrl: './seo-domain.component.html',
})
export class SeoDomainComponent implements OnInit {
  // properties 的定义为 filter-input.widget.ts -> FilterInputUISchema 接口
  // id会转换为 { type: 'integer', title: 'id', ui: { widget: 'filter-input', filterType: 'number', options: ['equals'] } }
  schema: SFSchema = asFilterInputPropertiesUI({
    properties: {
      templateWebsite: { type: 'string', title: '模板网址', ui: { options: ['contains'] } },
      domain: { type: 'string', title: '网站域名', ui: { options: ['contains'] } },
      title: { type: 'string', title: '标题', ui: { options: ['contains'] } },
      keywords: { type: 'string', title: '关键词列表', ui: { options: ['contains'] } },
      mainKey: { type: 'string', title: '主关键词', ui: { options: ['contains'] } },
      // 所有未知的复杂类型都当作枚举处理
      type: { type: 'array', title: '类型', ui: { options: ['in'], values: DOMAIN_TYPE } },

      createdTime: { type: 'string', title: '创建时间', format: 'date-time', ui: { options: ['inRange'] } },
      creator: { type: 'string', title: '创建人', ui: { options: ['contains'] } },
      serverTag: { type: 'string', title: '服务器标记', ui: { options: ['contains'] } },
      dir: { type: 'string', title: '目录', ui: { options: ['contains'] } },
      sentenceDir: { type: 'string', title: '句子目录', ui: { options: ['contains'] } },
      // 所有未知的复杂类型都当作枚举处理
      lastArticleTime: { type: 'string', title: '最后一次文章更新时间', format: 'date-time', ui: { options: ['inRange'] } },
      newSite: {
        type: 'array',
        title: '是否为新站',
        ui: {
          options: ['in'],
          values: [
            { label: '是', value: true },
            { label: '否', value: false },
          ],
        },
      },
      jsGroup: { type: 'string', title: 'js分组', ui: { options: ['equals'] } },
      proposalReference: { type: 'string', title: '提案编号', ui: { options: ['contains'] } },
    },
    required: ['text'],
    ui: {
      width: 275,
      spanLabelFixed: 80,
      optionShowType: 'symbol',
      // aclTmpl: 'POST:/{}/OUT',
      // acl: { ability: ['POST:/TEST0'] },
    },
  });

  // 表格配置
  columnDefs: AclColDef[] = [
    // { headerName: 'testACL', field: 'testACL', acl: { ability: ['POST:/TEST'] } }, //加上acl后只有符合权限的才展示出来
    // { headerName: 'group', field: 'typeGroup', enableRowGroup: true }, // 需要分组查询，则将 enableRowGroup设置为true
    { headerName: 'id', field: 'id', sort: 'desc', sortable: true, checkboxSelection: true, headerCheckboxSelection: true },
    { headerName: '操作', field: NgxGridTableConstants.OPTION_FIELD },
    { headerName: '模板网址', field: 'templateWebsite' },
    { headerName: '网站域名', field: 'domain' },
    { headerName: '标题', field: 'title' },
    { headerName: '关键词列表', field: 'keywords' },
    { headerName: '菜单列表', field: 'menus' },
    { headerName: '主关键词', field: 'mainKey' },
    { headerName: '类型', field: 'type' },
    { headerName: '网站说明', field: 'description' },
    { headerName: '创建时间', field: 'createdTime' },
    { headerName: '更新时间', field: 'updatedTime' },
    { headerName: '创建人', field: 'creator' },
    { headerName: '服务器标记', field: 'serverTag' },
    { headerName: '网页后缀', field: 'suffix' },
    { headerName: '目录', field: 'dir' },
    { headerName: '句子目录', field: 'sentenceDir' },
    { headerName: '每天最多更新多少篇', field: 'dailyArticle' },
    { headerName: '文章更新时间', field: 'articleTime' },
    { headerName: '最后一次文章更新时间', field: 'lastArticleTime' },
    { headerName: '是否为新站', field: 'newSite' },
    { headerName: 'js分组', field: 'jsGroup' },
    { headerName: '提案编号', field: 'proposalReference' },
  ];

  gridOptions: GridOptions;

  dataSource: IGridDataSource<any>;

  @ViewChild(NgxGridTableComponent)
  table!: NgxGridTableComponent;

  constructor(private http: _HttpClient, private aclService: ACLService, private message: NzMessageService, private modal: NzModalService) {
    this.gridOptions = {
      enableCharts: false,
      sideBar: 'default',
      columnDefs: this.columnDefs,
      enableRangeSelection: true,
      // rowSelection: 'multiple',
      getRowNodeId: this.idGetter,
      onRowDoubleClicked: (event: RowDoubleClickedEvent): void => {
        let frame: any;
        frame = window.open('about:blank', '_blank');
        frame.location = 'https://' + event.node.data.domain;
      },
      onFirstDataRendered(event: FirstDataRenderedEvent): void {
        event.columnApi.autoSizeAllColumns();
      },
    };
    this.dataSource = DataSourceUtils.rowQuery(http, '/api/star-ring/seo-domain/query', (r) => r);
  }

  ngOnInit(): void {}

  idGetter(data: any): any {
    return data.id;
  }

  onGridReady(e: { event: GridReadyEvent; gridTable: NgxGridTableComponent }): void {
    // 添加右键菜单
    this.table.addMenu({
      name: '删除',
      show: 'selected',
      acl: { ability: ['seo-domain:delete'] },
      callback: (selected) => {
        this.modal.confirm({
          nzTitle: '是否删除?',
          nzContent: `确认删除ID为 [${this.idGetter(selected)}] 的记录？`,
          nzOnOk: () => {
            this.http.delete(`/api/star-ring/seo-domain/delete/${this.idGetter(selected)}`).subscribe((value) => {
              this.message.success('删除成功');
              this.table.refresh();
            });
          },
        });
      },
    });
    // 按每列内容重新分配列宽
    this.table.columnApi.autoSizeAllColumns();
  }

  /**
   * IFilter表示查询条件。
   * 表格内置的 SfQueryFormComponent 将并根 SFSchema 来生成 IFilter 对象数组
   * 可以通过sf.value 将会获得每一个属性UI中填入的值。如果是 {widget: 'filter-input'} 的字段才自动将控件内填入的值转成 IFilter 对象
   * 其他非 filter-input 的控件，需要开发人员手动转换。
   * 如果开发人员需要给表格额外添加条件，也可以在这里加入更多的IFilter
   *
   * 自动生成的代码会将所有 SFSchema 的 UI 转成 filter-input，因此，默认是直接返回所有 filter
   * @param filter 所有filter-input生成的 IFilter对象
   * @param sf 表格的form对象。
   * @result 返回处理后的查询条件数组。
   */
  filterHand = (filter: IFilter[], sf: SfQueryFormComponent): IFilter[] => {
    // 不做任何处理，直接返回表格生成的filter对象数组。
    return filter;
  };

  onPageIndexChange(index: number): void {}

  onEdit(cell: ICellRendererParams, row: RowNode): void {
    this.modal
      .create({
        nzContent: SeoDomainEditComponent,
        nzComponentParams: { record: row.data },
        nzFooter: null,
        nzMaskClosable: false,
      })
      .afterClose.subscribe((result) => {
        this.table.refresh();
      });
  }

  onView(cell: ICellRendererParams, row: RowNode): void {
    this.modal.create({
      nzContent: SeoDomainViewComponent,
      nzComponentParams: { record: row.data },
      nzFooter: null,
      nzMaskClosable: true,
    });
  }

  onUploadChange(info: NzUploadChangeParam): void {
    if (info.file.status !== 'uploading') {
      console.log(info.file, info.fileList);
    }
    if (info.file.status === 'done') {
      this.message.success(`${info.file.name} file uploaded successfully`);
    } else if (info.file.status === 'error') {
      this.message.error(`${info.file.name} file upload failed.`);
    }
  }
}
