import { GridOptions, ICellRendererParams } from '@ag-grid-community/core';
import { RowNode } from '@ag-grid-community/core/dist/cjs/entities/rowNode';
import { FirstDataRenderedEvent } from '@ag-grid-community/core/dist/cjs/events';
import { RowDoubleClickedEvent } from '@ag-grid-enterprise/all-modules';
import { Component, OnInit, ViewChild } from '@angular/core';
import { EventManager } from '@angular/platform-browser';
import { DownloadTaskService, TemplateGroupService, TemplateService } from '@core';
import { ACLService } from '@delon/acl';
import { SFSchema } from '@delon/form';
import { _HttpClient } from '@delon/theme';
import { environment } from '@env/environment';
import {
  AclColDef,
  AclColGroupDef,
  asFilterInputPropertiesUI,
  IGridDataSource,
  NgxGridTableComponent,
  NgxGridTableConstants,
  NUMBER_FILTERS
} from '@shared';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalComponent, NzModalService } from 'ng-zorro-antd/modal';

import { DataSourceUtils } from '../../../DataSourceUtils';
import { CHECKED_STATUS, TASK_STATUS } from '../seo.enums';
import { SeoTemplateEditComponent } from './modal/edit.component';
import { SeoTemplateViewComponent } from './modal/view.component';

@Component({
  selector: 'app-seo-template',
  templateUrl: './seo-template.component.html'
})
export class SeoTemplateComponent implements OnInit {
  // properties 的定义为 filter-input.widget.ts -> FilterInputUISchema 接口
  // id会转换为 { type: 'integer', title: 'id', ui: { widget: 'filter-input', filterType: 'number', options: ['equals'] } }
  schema: SFSchema = asFilterInputPropertiesUI({
    properties: {
      website: { type: 'string', title: '模板域名', ui: { options: ['contains'] } },
      tpType: { type: 'string', title: '模板类型', ui: { options: ['contains'] } },
      interlinkSize: { type: 'integer', title: '内链数', ui: { options: NUMBER_FILTERS } },
      checked: {
        type: 'array',
        title: '审核状态',
        default: ['HIDDEN'],
        defaultOption: 'notIn',
        ui: { values: CHECKED_STATUS },
        optionShowType: 'i18n'
      },
      createdTime: {
        type: 'string',
        title: '创建时间',
        format: 'date-time',
        ui: { options: ['inRange'] }
      },
      creator: { type: 'string', title: '创建人', ui: { options: ['contains'] } },
      indexState: { type: 'array', title: '首页状态', ui: { options: ['in'], values: TASK_STATUS } },
      listState: { type: 'array', title: 'List页状态', ui: { options: ['in'], values: TASK_STATUS } },
      articleState: { type: 'array', title: '文章页状态', ui: { options: ['in'], values: TASK_STATUS } },
      auditor: { type: 'string', title: '审核人', ui: { options: ['contains'] } },
      useTimes: { type: 'integer', title: '使用次数', ui: { options: NUMBER_FILTERS } },
      auditedTime: { type: 'string', title: '审核时间', format: 'date-time', ui: { options: ['equals', 'inRange'] } }
    },
    required: ['text'],
    ui: {
      width: 275,
      spanLabelFixed: 80,
      optionShowType: 'symbol'
      // aclTmpl: 'POST:/{}/OUT',
      // acl: { ability: ['POST:/TEST0'] },
    }
  });

  // 表格配置 (ColDef | ColGroupDef)[]
  columnDefs: Array<AclColDef | AclColGroupDef> = [
    // { headerName: 'testACL', field: 'testACL', acl: { ability: ['POST:/TEST'] } }, //加上acl后只有符合权限的才展示出来
    // { headerName: 'group', field: 'typeGroup', enableRowGroup: true }, // 需要分组查询，则将 enableRowGroup设置为true
    { headerName: 'id', field: 'id', sort: 'desc', sortable: true, checkboxSelection: true, headerCheckboxSelection: true },
    { headerName: '操作', field: NgxGridTableConstants.OPTION_FIELD },
    { headerName: '模板域名', field: 'website' },
    { headerName: '分组', field: 'groupId', valueFormatter: params => this.getGroup(params) },
    { headerName: '使用次数', field: 'useTimes' },
    { headerName: '模板类型', field: 'tpType' },
    {
      headerName: '首页',
      children: [
        {
          headerName: '状态',
          field: 'indexState',
          valueFormatter: params => this.stateMapper[params.data.indexState],
          cellClass: params => this.stateStyle(params.data.indexState)
        },
        { headerName: 'Alt标签', field: 'indexSubKeySize', columnGroupShow: 'open' },
        { headerName: 'A标签', field: 'indexATagSize', columnGroupShow: 'open' },
        { headerName: 'H标签', field: 'indexHSize', columnGroupShow: 'open' },
        { headerName: '句子数', field: 'indexSentenceSize', columnGroupShow: 'open' }
      ]
    },

    {
      headerName: 'List页',
      children: [
        {
          headerName: '状态',
          field: 'listState',
          valueFormatter: params => this.stateMapper[params.data.listState],
          cellClass: params => this.stateStyle(params.data.listState)
        },
        { headerName: 'Alt标签', field: 'listSubKeySize', columnGroupShow: 'open' },
        { headerName: 'A标签', field: 'listATagSize', columnGroupShow: 'open' },
        { headerName: 'H标签', field: 'listHSize', columnGroupShow: 'open' },
        { headerName: '句子数', field: 'listSentenceSize', columnGroupShow: 'open' }
      ]
    },
    {
      headerName: '文章页',
      children: [
        {
          headerName: '状态',
          field: 'articleState',
          valueFormatter: params => this.stateMapper[params.data.articleState],
          cellClass: params => this.stateStyle(params.data.articleState)
        },
        { headerName: 'Alt标签', field: 'articleSubKeySize', columnGroupShow: 'open' },
        { headerName: 'A标签', field: 'articleATagSize', columnGroupShow: 'open' },
        { headerName: 'H标签', field: 'articleHSize', columnGroupShow: 'open' },
        { headerName: '句子数', field: 'articleSentenceSize', columnGroupShow: 'open' },
        { headerName: '内链数', field: 'interlinkSize', columnGroupShow: 'open' }
      ]
    },

    {
      headerName: '审核状态',
      field: 'checked',
      valueFormatter: params => this.checkedMapper[params.data.checked],
      cellClass: params => this.stateStyle(params.data.checked)
    },
    { headerName: '创建时间', field: 'createdTime' },
    { headerName: '创建人', field: 'creator' },
    { headerName: '最后更新时间', field: 'updatedTime' },
    { headerName: '最后更新人', field: 'updater' },
    { headerName: '审核人', field: 'auditor' },
    { headerName: '审核备注', field: 'remark' },
    { headerName: '审核时间', field: 'auditedTime' }
  ];

  gridOptions: GridOptions;

  dataSource: IGridDataSource<any>;

  @ViewChild(NgxGridTableComponent)
  table!: NgxGridTableComponent;

  @ViewChild('groupModal')
  groupModal!: NzModalComponent;

  @ViewChild('reFetchModal')
  reFetchModal!: NzModalComponent;

  @ViewChild('auditModal')
  auditModal!: NzModalComponent;

  groups!: any[];
  selectedData!: any;
  remark!: string;
  selectedGroupId = 1;
  fetchPage = '';

  stateMapper: any = {};
  checkedMapper: any = {};
  constructor(
    private http: _HttpClient,
    private templateService: TemplateService,
    private groupService: TemplateGroupService,
    private downloadTaskService: DownloadTaskService,
    private eventManager: EventManager,
    private aclService: ACLService,
    private message: NzMessageService,
    private modal: NzModalService
  ) {
    this.gridOptions = {
      enableCharts: false,
      sideBar: 'default',
      columnDefs: this.columnDefs,
      enableRangeSelection: true,
      getRowNodeId: this.idGetter,
      // rowSelection: 'multiple',
      onRowDoubleClicked: (event: RowDoubleClickedEvent): void => {
        this.openTemplate(event.node.data);
      },
      onFirstDataRendered(event: FirstDataRenderedEvent): void {
        event.columnApi.autoSizeAllColumns();
      }
    };
    this.dataSource = DataSourceUtils.rowQuery(http, '/api/star-ring/seo-template/query', r => r);

    TASK_STATUS.forEach(value => (this.stateMapper[value.value] = value.label));
    CHECKED_STATUS.forEach(value => (this.checkedMapper[value.value] = value.label));
  }

  ngOnInit(): void {
    this.eventManager.addGlobalEventListener('window', 'keyup.esc', () => {
      console.log('你点击了esc');
      this.table.refresh();
    });
    this.eventManager.addGlobalEventListener('window', 'keyup', (e: any) => {
      // debugger
      if (e.ctrlKey && e.key.toLowerCase() === 'capslock') {
        console.log('你输入了 ctrl + CapsLock');
        this.audit(this.selectedData);
      }
    });

    if (this.aclService.canAbility('template-group:query') || this.aclService.canAbility('seo-template:group')) {
      this.groupService.groups().subscribe(value => {
        this.groups = value;
      });
    }
  }

  idGetter(data: any): any {
    return data.id;
  }

  onGridReady(): void {
    // 添加右键菜单
    this.table.addMenu({
      name: '删除',
      show: 'selected',
      acl: { ability: ['seo-template:delete'] },
      callback: selected => {
        this.modal.confirm({
          nzTitle: '是否删除?',
          nzContent: `确认删除ID为 [${this.idGetter(selected)}] 的记录？`,
          nzOnOk: () => {
            this.http.delete(`/api/star-ring/seo-template/delete/${this.idGetter(selected)}`).subscribe(() => {
              this.message.success('删除成功');
              this.table.refresh();
            });
          }
        });
      }
    });

    this.table.addMenu({
      name: '分组',
      show: 'selected',
      acl: { ability: ['seo-template:group'] },
      callback: selected => {
        this.group(selected);
      }
    });

    this.table.addMenu({
      name: '重抓',
      show: 'selected',
      acl: { ability: ['seo-template:re-fetch'] },
      callback: selected => {
        this.reFetch(selected);
      }
    });

    this.table.addMenu({
      name: '随机下载资源',
      show: 'selected',
      acl: { ability: ['seo-template:re-fetch'] },
      callback: selected => {
        this.randomFetchOne(selected);
      }
    });

    this.table.addMenu({
      name: '下载不存在的文件',
      show: 'selected',
      acl: { ability: ['seo-template:re-fetch'] },
      callback: selected => {
        this.downloadNohave(selected);
      }
    });

    this.table.addMenu({
      name: '重新下载所有资源',
      show: 'selected',
      acl: { ability: ['seo-template:re-fetch'] },
      callback: selected => {
        this.download(selected, false);
      }
    });

    this.table.addMenu({
      name: '下载未完成的资源',
      show: 'selected',
      acl: { ability: ['seo-template:re-fetch'] },
      callback: selected => {
        this.download(selected, true);
      }
    });

    // 按每列内容重新分配列宽
    this.table.columnApi.autoSizeAllColumns();
  }

  onEdit(cell: ICellRendererParams, row: RowNode): void {
    this.modal
      .create({
        nzContent: SeoTemplateEditComponent,
        nzComponentParams: { record: row.data },
        nzFooter: null,
        nzMaskClosable: false
      })
      .afterClose.subscribe(result => {
        this.table.refresh();
      });
  }

  onView(cell: ICellRendererParams, row: RowNode): void {
    this.modal.create({
      nzContent: SeoTemplateViewComponent,
      nzComponentParams: { record: row.data },
      nzFooter: null,
      nzMaskClosable: true
    });
  }

  audit(data: any): void {
    this.selectedData = data;
    this.auditModal.open();
  }

  private group(data: any): void {
    this.selectedData = data;
    this.groupModal.open();
  }

  private reFetch(selected: any): void {
    this.selectedData = selected;
    this.reFetchModal.open();
  }

  private getGroup(params: any): string {
    if (!params.data) {
      return '';
    }
    if (params.data.groupId === 0) {
      return '未分组';
    }

    if (this.groups) {
      const group = this.groups.filter(value => value.groupId !== params.data.groupId)[0];
      return group.group;
    } else {
      return params.data.groupId;
    }
  }

  hidden(data: any): void {
    this.templateService.hidden(data.id).subscribe(params => this.table.refresh());
  }

  openTemplate(data: any): void {
    let frame: any;
    if (data.indexState === 'ENDED') {
      frame = window.open('about:blank', '_blank');
      // @ts-ignore
      frame.location = `${environment.templateViewUrl}/test/${data.website}/index`;
    }
    if (data.listState === 'ENDED') {
      frame = window.open('about:blank', '_blank');
      // @ts-ignore
      frame.location = `${environment.templateViewUrl}/test/${data.website}/list`;
    }
    if (data.articleState === 'ENDED') {
      frame = window.open('about:blank', '_blank');
      // @ts-ignore
      frame.location = `${environment.templateViewUrl}/test/${data.website}/article`;
    }
  }

  doGroup(): void {
    this.templateService.group([this.selectedData.id], this.selectedGroupId).subscribe(() => this.table.refresh());
  }

  doReFetch(): void {
    let obs;
    if ('' === this.fetchPage) {
      obs = this.templateService.reFetch(this.selectedData.id);
    } else {
      obs = this.templateService.reFetchPage(this.selectedData.id, this.fetchPage);
    }

    obs.subscribe(value => {
      let frame: any;
      this.message.success('成功');
      frame = window.open('about:blank', '_blank');
      frame.location = `http://${this.selectedData.website}`;
      this.table.refresh();
    });
  }

  doAudit(pass: boolean): void {
    this.templateService.audit(this.selectedData.id, pass, this.remark).subscribe(value => this.table.refresh());
  }

  private randomFetchOne(selected: any): void {
    this.downloadTaskService.random(selected.website).subscribe(value => {
      this.table.refresh();
    });
  }

  private download(data: any, failOnly: boolean): void {
    this.downloadTaskService.check(data.website).subscribe(value => {
      this.message.success(`共计[${value.data.total}]条记录，之前下载过：[${value.data.ends}]条，重新下载后将会再给提示`);
    });
    this.downloadTaskService.downloadFor(data.website, failOnly).subscribe(value => {
      this.message.success(`提交成功`);
    });
  }

  private downloadNohave(selected: any): void {
    this.downloadTaskService.downloadNohave(selected.website).subscribe(value => {
      this.message.success('提交成功');
    });
  }

  private stateStyle(state: string): string {
    switch (state) {
      case 'CREATED':
        return 'task-created';
      case 'WAITING':
        return 'task-waiting';
      case 'RUNNING':
        return 'task-running';
      case 'INTERRUPTED':
        return 'task-interrupted';
      case 'ENDED':
      case 'PASS':
        return 'task-success';
      case 'ERROR':
      case 'REFUSE':
        return 'task-error';
    }
    return '';
  }
}
