import { GridOptions, GridReadyEvent, ICellRendererParams } from '@ag-grid-community/core';
import { RowNode } from '@ag-grid-community/core/dist/cjs/entities/rowNode';
import { FirstDataRenderedEvent } from '@ag-grid-community/core/dist/cjs/events';
import { Component, OnInit, ViewChild } from '@angular/core';
import { ACLService } from '@delon/acl';
import { SFSchema } from '@delon/form';
import { _HttpClient } from '@delon/theme';
import {
  AclColDef,
  asFilterInputPropertiesUI,
  IGridDataSource,
  NgxGridTableComponent,
  NgxGridTableConstants,
  NUMBER_FILTERS
} from '@shared';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalService } from 'ng-zorro-antd/modal';

import { DataSourceUtils } from '../../../DataSourceUtils';
import { SpiderLogCreateComponent } from './modal/create.component';
import { SpiderLogEditComponent } from './modal/edit.component';
import { SpiderLogViewComponent } from './modal/view.component';

@Component({
  selector: 'app-spider-log',
  templateUrl: './spider-log.component.html'
})
export class SpiderLogComponent implements OnInit {
  // properties 的定义为 filter-input.widget.ts -> FilterInputUISchema 接口
  // id会转换为 { type: 'integer', title: 'id', ui: { widget: 'filter-input', filterType: 'number', options: ['equals'] } }
  schema: SFSchema = asFilterInputPropertiesUI({
    properties: {
      baidu: { type: 'integer', title: '百度蜘蛛个数', ui: { options: NUMBER_FILTERS } },
      haosou: { type: 'integer', title: '360蜘蛛', ui: { options: NUMBER_FILTERS } },
      sogou: { type: 'integer', title: '搜狗', ui: { options: NUMBER_FILTERS } },
      yisou: { type: 'integer', title: '易搜', ui: { options: NUMBER_FILTERS } },
      other: { type: 'integer', title: '其他', ui: { options: NUMBER_FILTERS } },
      logDate: { type: 'string', title: '日期', format: 'date-time', ui: { options: ['inRange'] } },
      updatedTime: { type: 'string', title: '最后更新时间', format: 'date-time', ui: { options: ['inRange'] } }
    },
    required: ['text'],
    ui: {
      width: 275,
      spanLabelFixed: 80,
      optionShowType: 'symbol'
    }
  });

  // 表格配置
  columnDefs: AclColDef[] = [
    { headerName: 'id', field: 'id', sort: 'desc', sortable: true, checkboxSelection: true, headerCheckboxSelection: true },
    { headerName: '百度', field: 'baidu' },
    { headerName: '360', field: 'haosou' },
    { headerName: '搜狗', field: 'sogou' },
    { headerName: '易搜', field: 'yisou' },
    { headerName: '其他', field: 'other' },
    { headerName: '日期', field: 'logDate' },
    { headerName: '更新时间', field: 'updatedTime' },
    { headerName: '操作', field: NgxGridTableConstants.OPTION_FIELD }
  ];

  gridOptions: GridOptions;

  dataSource: IGridDataSource<any>;

  @ViewChild(NgxGridTableComponent)
  table!: NgxGridTableComponent;

  constructor(private http: _HttpClient, private aclService: ACLService, private message: NzMessageService, private modal: NzModalService) {
    this.gridOptions = {
      enableCharts: false,
      sideBar: 'default',
      columnDefs: this.columnDefs,
      enableRangeSelection: true,
      // rowSelection: 'multiple',
      getRowNodeId: this.idGetter,
      onFirstDataRendered(event: FirstDataRenderedEvent): void {
        event.columnApi.autoSizeAllColumns();
      }
    };
    this.dataSource = DataSourceUtils.rowQuery(http, '/api/star-ring/spider-log/query', r => r);
  }

  ngOnInit(): void {}

  idGetter(data: any): any {
    return data.id;
  }

  onGridReady(e: { event: GridReadyEvent; gridTable: NgxGridTableComponent }): void {
    // 添加右键菜单
    this.table.addMenu({
      name: '删除',
      show: 'selected',
      acl: { ability: ['spider-log:delete'] },
      callback: selected => {
        this.modal.confirm({
          nzTitle: '是否删除?',
          nzContent: `确认删除ID为 [${this.idGetter(selected)}] 的记录？`,
          nzOnOk: () => {
            this.http.delete(`/api/star-ring/spider-log/delete/${this.idGetter(selected)}`).subscribe(value => {
              this.message.success('删除成功');
              this.table.refresh();
            });
          }
        });
      }
    });
    // 按每列内容重新分配列宽
    this.table.columnApi.autoSizeAllColumns();
  }

  onCreate(): void {
    this.modal
      .create({
        nzContent: SpiderLogCreateComponent,
        nzFooter: null,
        nzMaskClosable: false
      })
      .afterClose.subscribe(result => {
        this.table.refresh();
      });
  }

  onEdit(cell: ICellRendererParams, row: RowNode): void {
    this.modal
      .create({
        nzContent: SpiderLogEditComponent,
        nzComponentParams: { record: row.data },
        nzFooter: null,
        nzMaskClosable: false
      })
      .afterClose.subscribe(result => {
        this.table.refresh();
      });
  }

  onView(cell: ICellRendererParams, row: RowNode): void {
    this.modal.create({
      nzContent: SpiderLogViewComponent,
      nzComponentParams: { record: row.data },
      nzFooter: null,
      nzMaskClosable: true
    });
  }
}
