import { GridOptions, GridReadyEvent, ICellRendererParams } from '@ag-grid-community/core';
import { RowNode } from '@ag-grid-community/core/dist/cjs/entities/rowNode';
import { FirstDataRenderedEvent } from '@ag-grid-community/core/dist/cjs/events';
import { RowDoubleClickedEvent } from '@ag-grid-enterprise/all-modules';
import { Component, OnInit, ViewChild } from '@angular/core';
import { TargetDomainService } from '@core';
import { ACLService } from '@delon/acl';
import { SFSchema } from '@delon/form';
import { _HttpClient } from '@delon/theme';
import { AclColDef, asFilterInputPropertiesUI, IGridDataSource, NgxGridTableComponent, NgxGridTableConstants } from '@shared';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalComponent, NzModalService } from 'ng-zorro-antd/modal';

import { DataSourceUtils } from '../../../DataSourceUtils';
import { TargetDomainCreateComponent } from './modal/create.component';
import { TargetDomainViewComponent } from './modal/view.component';

@Component({
  selector: 'app-target-domain',
  templateUrl: './target-domain.component.html'
})
export class TargetDomainComponent implements OnInit {
  // properties 的定义为 filter-input.widget.ts -> FilterInputUISchema 接口
  // id会转换为 { type: 'integer', title: 'id', ui: { widget: 'filter-input', filterType: 'number', options: ['equals'] } }
  schema: SFSchema = asFilterInputPropertiesUI({
    properties: {
      targetDomain: { type: 'string', title: '目标站', ui: { options: ['contains'] } },
      xinhuan: {
        type: 'array',
        title: '星环特有',
        ui: {
          options: ['in'],
          values: [
            { label: '是', value: true },
            { label: '否', value: false }
          ]
        }
      },
      used: {
        type: 'array',
        title: '是否使用',
        ui: {
          options: ['in'],
          values: [
            { label: '是', value: true },
            { label: '否', value: false }
          ]
        }
      },
      usedBy: { type: 'string', title: '使用人', ui: { options: ['contains'] } },
      usedTime: { type: 'string', title: '使用时间', format: 'date-time', ui: { options: ['inRange'] } },
      createdTime: { type: 'string', title: '创建时间', format: 'date-time', ui: { options: ['inRange'] } },
      creator: { type: 'string', title: '创建人', ui: { options: ['contains'] } }
    },
    required: ['text'],
    ui: {
      width: 275,
      spanLabelFixed: 80,
      optionShowType: 'symbol'
    }
  });

  // 表格配置
  columnDefs: AclColDef[] = [
    { headerName: 'id', field: 'id', sort: 'desc', sortable: true, checkboxSelection: true, headerCheckboxSelection: true },
    { headerName: '目标站', field: 'targetDomain' },
    { headerName: '星环特有', field: 'xinhuan' },
    { headerName: '是否使用', field: 'used' },
    { headerName: '使用人', field: 'usedBy' },
    { headerName: '使用时间', field: 'usedTime' },
    { headerName: '创建时间', field: 'createdTime' },
    { headerName: '创建人', field: 'creator' },
    { headerName: '操作', field: NgxGridTableConstants.OPTION_FIELD }
  ];

  gridOptions: GridOptions;

  dataSource: IGridDataSource<any>;

  @ViewChild(NgxGridTableComponent)
  table!: NgxGridTableComponent;

  @ViewChild('batchModal')
  batchModal!: NzModalComponent;

  urls!: string;

  constructor(
    private http: _HttpClient,
    private aclService: ACLService,
    private message: NzMessageService,
    private targetDomainService: TargetDomainService,
    private modal: NzModalService
  ) {
    this.gridOptions = {
      enableCharts: false,
      sideBar: 'default',
      columnDefs: this.columnDefs,
      enableRangeSelection: true,
      // rowSelection: 'multiple',
      getRowNodeId: this.idGetter,
      onRowDoubleClicked: (event: RowDoubleClickedEvent): void => {
        this.use(event.node.data);
      },
      onFirstDataRendered(event: FirstDataRenderedEvent): void {
        event.columnApi.autoSizeAllColumns();
      }
    };
    this.dataSource = DataSourceUtils.rowQuery(http, '/api/star-ring/target-domain/query', r => r);
  }

  ngOnInit(): void {}

  idGetter(data: any): any {
    return data.id;
  }

  onGridReady(e: { event: GridReadyEvent; gridTable: NgxGridTableComponent }): void {
    // 添加右键菜单
    this.table.addMenu({
      name: '删除',
      show: 'selected',
      acl: { ability: ['target-domain:delete'] },
      callback: selected => {
        this.modal.confirm({
          nzTitle: '是否删除?',
          nzContent: `确认删除ID为 [${this.idGetter(selected)}] 的记录？`,
          nzOnOk: () => {
            this.http.delete(`/api/star-ring/target-domain/delete/${this.idGetter(selected)}`).subscribe(value => {
              this.message.success('删除成功');
              this.table.refresh();
            });
          }
        });
      }
    });
    // 按每列内容重新分配列宽
    this.table.columnApi.autoSizeAllColumns();
  }

  onCreate(): void {
    this.modal
      .create({
        nzContent: TargetDomainCreateComponent,
        nzFooter: null,
        nzMaskClosable: false
      })
      .afterClose.subscribe(result => {
        this.table.refresh();
      });
  }

  onView(cell: ICellRendererParams, row: RowNode): void {
    this.modal.create({
      nzContent: TargetDomainViewComponent,
      nzComponentParams: { record: row.data },
      nzFooter: null,
      nzMaskClosable: true
    });
  }

  private use(data: any): void {
    this.targetDomainService.useXinhuan(data.id).subscribe(value => {
      this.table.refresh();
      if (value) {
        const frame: any = window.open('about:blank', '_blank');
        frame.location = `http://${data.targetDomain}`;
      } else {
        this.message.warning('似乎已经被使用了');
      }
    });
  }

  batchCreate(): void {
    this.urls = '';
    this.batchModal.open();
  }

  onUrls(xinghuan: boolean): void {
    const strs: string[] = this.urls.replace('\r', '').split('\n');
    this.targetDomainService.add(strs, xinghuan).subscribe(value => {
      this.message.success('成功');
      this.table.refresh();
      this.batchModal.close();
    });
  }

  onSpider(): void {
    const strs: string[] = this.urls.replace('\r', '').split('\n');
    this.targetDomainService.spider(strs).subscribe(value => {
      this.message.success('成功');
      this.table.refresh();
      this.batchModal.close();
    });
  }

  onOrder(): void {
    const strs: string[] = this.urls.replace('\r', '').split('\n');
    this.targetDomainService.order(strs).subscribe(value => {
      this.message.success('成功');
      this.table.refresh();
      this.batchModal.close();
    });
  }
}
