import { GridOptions, GridReadyEvent, ICellRendererParams } from '@ag-grid-community/core';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalService } from 'ng-zorro-antd/modal';
import { RowNode } from '@ag-grid-community/core/dist/cjs/entities/rowNode';
import { FirstDataRenderedEvent } from '@ag-grid-community/core/dist/cjs/events';
import { Component, OnInit, ViewChild } from '@angular/core';
import { ACLService } from '@delon/acl';
import { SFSchema } from '@delon/form';
import { _HttpClient } from '@delon/theme';
import { AclColDef, asFilterInputPropertiesUI, IFilter, IGridDataSource, NgxGridTableComponent, NgxGridTableConstants } from '@shared';
import { SfQueryFormComponent } from '@shared';
import { DataSourceUtils } from '../../../DataSourceUtils';
import { TemplateUploadUserCreateComponent } from './modal/create.component';
import { TemplateUploadUserEditComponent } from './modal/edit.component';
import { TemplateUploadUserViewComponent } from './modal/view.component';

@Component({
  selector: 'app-template-upload-user',
  templateUrl: './template-upload-user.component.html',
})
export class TemplateUploadUserComponent implements OnInit {
  // properties 的定义为 filter-input.widget.ts -> FilterInputUISchema 接口
  // id会转换为 { type: 'integer', title: 'id', ui: { widget: 'filter-input', filterType: 'number', options: ['equals'] } }
  schema: SFSchema = asFilterInputPropertiesUI({
    properties: {
      username: { type: 'string', title: '账号', ui: { options: ['contains'] } },
      password: { type: 'string', title: '密码', ui: { options: ['contains'] } },
      creator: { type: 'string', title: '创建者', ui: { options: ['contains'] } },
      createdTime: { type: 'string', title: '创建时间', format: 'date-time', ui: { options: ['inRange'] } },
    },
    required: ['text'],
    ui: {
      width: 275,
      spanLabelFixed: 80,
      optionShowType: 'symbol',
    },
  });

  // 表格配置
  columnDefs: AclColDef[] = [
    { headerName: 'id', field: 'id', sort: 'desc', sortable: true, checkboxSelection: true, headerCheckboxSelection: true },
    { headerName: '账号', field: 'username' },
    { headerName: '密码', field: 'password' },
    { headerName: '创建者', field: 'creator' },
    { headerName: '创建时间', field: 'createdTime' },
    { headerName: '更新时间', field: 'updatedTime' },
    { headerName: '更新人', field: 'updater' },
    { headerName: '操作', field: NgxGridTableConstants.OPTION_FIELD },
  ];

  gridOptions: GridOptions;

  dataSource: IGridDataSource<any>;

  @ViewChild(NgxGridTableComponent)
  table!: NgxGridTableComponent;

  constructor(private http: _HttpClient, private aclService: ACLService, private message: NzMessageService, private modal: NzModalService) {
    this.gridOptions = {
      enableCharts: false,
      sideBar: 'default',
      columnDefs: this.columnDefs,
      enableRangeSelection: true,
      // rowSelection: 'multiple',
      getRowNodeId: this.idGetter,
      onFirstDataRendered(event: FirstDataRenderedEvent): void {
        event.columnApi.autoSizeAllColumns();
      },
    };
    this.dataSource = DataSourceUtils.rowQuery(http, '/api/star-ring/template-upload-user/query', (r) => r);
  }

  ngOnInit(): void {}

  idGetter(data: any): any {
    return data.id;
  }

  onGridReady(e: { event: GridReadyEvent; gridTable: NgxGridTableComponent }): void {
    // 添加右键菜单
    this.table.addMenu({
      name: '删除',
      show: 'selected',
      acl: { ability: ['template-upload-user:delete'] },
      callback: (selected) => {
        this.modal.confirm({
          nzTitle: '是否删除?',
          nzContent: `确认删除ID为 [${this.idGetter(selected)}] 的记录？`,
          nzOnOk: () => {
            this.http.delete(`/api/star-ring/template-upload-user/delete/${this.idGetter(selected)}`).subscribe((value) => {
              this.message.success('删除成功');
              this.table.refresh();
            });
          },
        });
      },
    });
    // 按每列内容重新分配列宽
    this.table.columnApi.autoSizeAllColumns();
  }

  onCreate(): void {
    this.modal
      .create({
        nzContent: TemplateUploadUserCreateComponent,
        nzFooter: null,
        nzMaskClosable: false,
      })
      .afterClose.subscribe((result) => {
        this.table.refresh();
      });
  }

  onEdit(cell: ICellRendererParams, row: RowNode): void {
    this.modal
      .create({
        nzContent: TemplateUploadUserEditComponent,
        nzComponentParams: { record: row.data },
        nzFooter: null,
        nzMaskClosable: false,
      })
      .afterClose.subscribe((result) => {
        this.table.refresh();
      });
  }

  onView(cell: ICellRendererParams, row: RowNode): void {
    this.modal.create({
      nzContent: TemplateUploadUserViewComponent,
      nzComponentParams: { record: row.data },
      nzFooter: null,
      nzMaskClosable: true,
    });
  }
}
