import {ChangeDetectionStrategy, ChangeDetectorRef, Component, forwardRef, Input, OnInit} from '@angular/core';
import {ControlValueAccessor, NG_VALUE_ACCESSOR} from '@angular/forms';
import {SFSchema} from '@delon/form';
import {getDayRange} from '../../utils/tools';
import {FilterType, Options, Shorthand, Symbols} from './filter.types';
import {NzI18nService} from "ng-zorro-antd/i18n";

@Component({
  selector: 'filter-input',
  templateUrl: './filter-input.component.html',
  providers: [
    {
      provide: NG_VALUE_ACCESSOR,
      useExisting: forwardRef(() => FilterInputComponent),
      multi: true,
    },
  ],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class FilterInputComponent implements ControlValueAccessor, OnInit {
  @Input() options!: Options[];
  @Input() selectWidth = 80;
  @Input() inputPlaceholder = '';
  @Input() selectPlaceholder = '';

  @Input() filterType!: FilterType;
  @Input() optionShowType: 'i18n' | 'shorthand' | 'symbol' = 'i18n';

  @Input() values: Array<{ label: string; value: any }> = [];
  @Input() defaultOption!: Options;

  @Input() defaultValue: any;

  @Input() schema!: SFSchema;

  ranges = {
    今天: getDayRange(1),
    昨天: getDayRange(1, getDayRange(2)[0]),
    七天: getDayRange(7),
  };
  value = {} as { option: Options; value: any; filterType: string; valueTo?: any };

  propagateOnChange: (value: any) => void = (_: any) => {};
  propagateOnTouched: (value: any) => void = (_: any) => {};

  constructor(private cdRef: ChangeDetectorRef, private nzI18nService: NzI18nService) {}

  ngOnInit(): void {
    this.value.option = this.defaultOption ? this.defaultOption : this.options[0];
    this.value.filterType = this.filterType;
    this.value.value = this.defaultValue ? this.defaultValue : undefined;
  }

  registerOnChange(fn: any): void {
    this.propagateOnChange = fn;
  }

  registerOnTouched(fn: any): void {
    this.propagateOnTouched = fn;
  }

  // reset 时都会传入初始化时的值，也就是default的值
  // SFSSchema default是什么，这里就是什么
  writeValue(obj: any): void {
    // console.log('reset->', this.filterType, obj);
    if (Array.isArray(obj) && this.value.option === 'inRange') {
      this.setValue(obj[0]);
      this.setValueTo(obj[1]);
    } else {
      this.setValue(obj);
    }

    this.updateValue();
    this.cdRef.detectChanges();
  }

  onOptionChange(option: Options): void {
    if (option === 'inRange' || this.value.option === 'inRange') {
      delete this.value.value;
      delete this.value.valueTo;
    }
    this.value.option = option;
    this.updateValue();
  }

  /** 修改value */
  onValueChange(value: any): void {
    // console.log('value change ->', this.filterType, value);
    this.setValue(value);
    this.updateValue();
  }
  /** 修改valueTo */
  onValueToChange(valueTo: any): void {
    // console.log('valueTo change ->', this.filterType, valueTo);
    this.setValueTo(valueTo);
    this.updateValue();
  }

  private updateValue(): void {
    if (!this.value.option) {
      this.value.option = this.options[0];
    }
    if (!this.value.filterType) {
      this.value.filterType = this.filterType;
    }

    if (this.value.option !== 'inRange') {
      delete this.value.valueTo;
    }

    if (this.value.value && this.value.value.toString().length) {
      this.propagateOnChange(this.value);
    } else {
      this.propagateOnChange({});
    }
  }

  showLabel(option: string): string {
    switch (this.optionShowType) {
      case 'i18n':
        return this.nzI18nService.translate('filter-input.' + option);
      case 'shorthand':
        // @ts-ignore
        return Shorthand[option];
      case 'symbol':
        // @ts-ignore
        return Symbols[option];
      default:
        return '';
    }
  }

  private setValue(value: any): void {
    if (!value) {
      this.value.value = this.filterType === 'set' || this.filterType === 'date' ? null : '';
      return;
    }
    if (this.filterType === 'date') {
      this.value.value = value ? value : null;
    } else {
      this.value.value = value ? value : '';
    }
  }

  private setValueTo(valueTo: any): void {
    if (!valueTo) {
      this.value.valueTo = this.filterType === 'set' || this.filterType === 'date' ? null : '';
      return;
    }

    if (this.filterType === 'date') {
      this.value.valueTo = valueTo;
    } else {
      this.value.valueTo = valueTo;
    }
  }

  // handDateValue(dates: Date[]): void {
  //   if (dates[0] &&  dates[1]) {
  //     this.value.value = formatDate(dates[0], 'yyyy-MM-dd HH:mm:ss', 'zh_CH');
  //     this.value.valueTo = formatDate(dates[1], 'yyyy-MM-dd HH:mm:ss', 'zh_CH');
  //     this.value.option = 'inRange';
  //   } else if (dates[0]) {
  //     this.value.value = formatDate(dates[0], 'yyyy-MM-dd HH:mm:ss', 'zh_CH');
  //     this.value.option = 'greaterThanOrEqual';
  //   } else if (dates[1]) {
  //     this.value.value = formatDate(dates[1], 'yyyy-MM-dd HH:mm:ss', 'zh_CH');
  //     this.value.option = 'lessThanOrEqual';
  //   }
  // }

  onDateRangeChange(dates: Date[]): void {
    // console.log('date array', dates);

    if (dates.length) {
      this.setValue(dates[0]);
      this.setValueTo(dates[1]);
    } else {
      this.setValue(null);
      this.setValueTo(null);
    }
    this.updateValue();
  }


}
