/*
 *    Copyright (c) 2018-2025, lengleng All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the pig4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: lengleng (wangiegie@gmail.com)
 */
package com.chuang.urras.common.swagger.configuration;

import com.chuang.urras.common.swagger.support.SwaggerProperties;
import io.swagger.models.auth.In;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingClass;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import springfox.documentation.builders.ApiInfoBuilder;
import springfox.documentation.builders.PathSelectors;
import springfox.documentation.builders.RequestHandlerSelectors;
import springfox.documentation.builders.RequestParameterBuilder;
import springfox.documentation.schema.ScalarType;
import springfox.documentation.service.*;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spi.service.contexts.SecurityContext;
import springfox.documentation.spring.web.plugins.ApiSelectorBuilder;
import springfox.documentation.spring.web.plugins.Docket;
import springfox.documentation.swagger2.annotations.EnableSwagger2;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * swagger配置
 *
 * <p>
 * 禁用方法1：使用注解@Profile({"dev","test"})
 *
 * 表示在开发或测试环境开启，而在生产关闭。（推荐使用） 禁用方法2：使用注解@ConditionalOnProperty(name = "swagger.enable",
 *
 * havingValue = "true") 然后在测试配置或者开发配置中添加swagger.enable=true即可开启，生产环境不填则默认关闭Swagger.
 * </p>
 *
 * @author lengleng
 */
@EnableSwagger2
@EnableConfigurationProperties(SwaggerProperties.class)
@ConditionalOnProperty(name = "spring.urras.swagger.enabled", havingValue = "true")
@ConditionalOnMissingClass("org.springframework.cloud.gateway.config.GatewayAutoConfiguration")
public class SwaggerAutoConfiguration {

	/**
	 * 默认的排除路径，排除Spring Boot默认的错误处理路径和端点
	 */
	private static final List<String> DEFAULT_EXCLUDE_PATH = Arrays.asList("/error", "/actuator/**");

	private static final String BASE_PATH = "/**";

	@Bean(value = "urras-api-docs")
	public Docket api(SwaggerProperties swaggerProperties) {
		// base-path处理
		if (swaggerProperties.getBasePath().isEmpty()) {
			swaggerProperties.getBasePath().add(BASE_PATH);
		}

		// exclude-path处理
		if (swaggerProperties.getExcludePath().isEmpty()) {
			swaggerProperties.getExcludePath().addAll(DEFAULT_EXCLUDE_PATH);
		}

		ApiSelectorBuilder builder = new Docket(DocumentationType.SWAGGER_2)
				.apiInfo(apiInfo(swaggerProperties))
				.groupName(swaggerProperties.getGroupName())
				.select()
				.apis(RequestHandlerSelectors.basePackage(swaggerProperties.getBasePackage()));

		swaggerProperties.getBasePath().forEach(p -> builder.paths(PathSelectors.ant(p)));
		swaggerProperties.getExcludePath().forEach(p -> builder.paths(PathSelectors.ant(p).negate()));

		return builder.build()
				.globalRequestParameters(globalRequestParameters(swaggerProperties))
				.securitySchemes(Collections.singletonList(securitySchema(swaggerProperties)))
				.securityContexts(Collections.singletonList(securityContext(swaggerProperties)));
//				.pathMapping(swaggerProperties.getPathMapping());
	}

	private List<RequestParameter> globalRequestParameters(SwaggerProperties properties) {

		List<RequestParameter> pars = new ArrayList<>();

		if(properties.getVersionHeader()) {

			RequestParameterBuilder versionPar = new RequestParameterBuilder()
					.name("VERSION")
					.description("灰度路由版本信息")
					.in(ParameterType.HEADER)
					.required(false)
					.query(param -> param.model(model -> model.scalarModel(ScalarType.STRING)));

			pars.add(versionPar.build());
		}

		if(properties.getAuthorizationHeader()) {
			RequestParameterBuilder versionPar = new RequestParameterBuilder()
					.name("Authorization")
					.description("access token")
					.in(ParameterType.HEADER)
					.required(false)
					.query(param -> param.model(model -> model.scalarModel(ScalarType.STRING)));

			pars.add(versionPar.build());
		}

		return pars;
	}

	/**
	 * 配置默认的全局鉴权策略的开关，通过正则表达式进行匹配；默认匹配所有URL
	 * @return
	 */
	private static SecurityContext securityContext(SwaggerProperties swaggerProperties) {
		return SecurityContext.builder()
				.securityReferences(defaultAuth(swaggerProperties))
				.build();
	}

	/**
	 * 默认的全局鉴权策略
	 * @return
	 */
	private static List<SecurityReference> defaultAuth(SwaggerProperties swaggerProperties) {
		ArrayList<AuthorizationScope> authorizationScopeList = new ArrayList<>();
		swaggerProperties.getAuthorization().getAuthorizationScopeList()
				.forEach(authorizationScope -> authorizationScopeList.add(
						new AuthorizationScope(authorizationScope.getScope(), authorizationScope.getDescription())));
		AuthorizationScope[] authorizationScopes = new AuthorizationScope[authorizationScopeList.size()];
		return Collections
				.singletonList(SecurityReference.builder().reference(swaggerProperties.getAuthorization().getName())
						.scopes(authorizationScopeList.toArray(authorizationScopes)).build());
	}

//	private static OAuth securitySchema(SwaggerProperties swaggerProperties) {
//		ArrayList<AuthorizationScope> authorizationScopeList = new ArrayList<>();
//		swaggerProperties.getAuthorization().getAuthorizationScopeList()
//				.forEach(authorizationScope -> authorizationScopeList.add(
//						new AuthorizationScope(authorizationScope.getScope(), authorizationScope.getDescription())));
//		ArrayList<GrantType> grantTypes = new ArrayList<>();
//		swaggerProperties.getAuthorization().getTokenUrlList()
//				.forEach(tokenUrl -> grantTypes.add(new ResourceOwnerPasswordCredentialsGrant(tokenUrl)));
//		return new OAuth(swaggerProperties.getAuthorization().getName(), authorizationScopeList, grantTypes);
//	}

	private static ApiKey securitySchema(SwaggerProperties swaggerProperties) {
		return new ApiKey(swaggerProperties.getAuthorization().getName(), "Authorization", In.HEADER.name());
	}

	private static ApiInfo apiInfo(SwaggerProperties swaggerProperties) {
		return new ApiInfoBuilder().title(swaggerProperties.getTitle()).description(swaggerProperties.getDescription())
				.license(swaggerProperties.getLicense()).licenseUrl(swaggerProperties.getLicenseUrl())
				.termsOfServiceUrl(swaggerProperties.getTermsOfServiceUrl())
				.contact(new Contact(swaggerProperties.getContact().getName(), swaggerProperties.getContact().getUrl(),
						swaggerProperties.getContact().getEmail()))
				.version(swaggerProperties.getVersion()).build();
	}

}
