package com.chuang.urras.common.task;

import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.support.exception.SystemException;
import com.chuang.tauceti.tools.basic.StringKit;
import com.chuang.urras.common.task.enums.SchedulerStatus;
import com.chuang.urras.common.task.model.co.SchedulerCO;
import com.chuang.urras.common.task.model.uo.SchedulerUO;
import org.quartz.*;
import org.quartz.impl.triggers.CronTriggerImpl;
import org.springframework.util.ClassUtils;

import java.text.ParseException;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.Date;
import java.util.TimeZone;

/**
 * @author Ives.l
 * @since 2021/12/9 13:53
 */
public class SchedulerHelper {

	private final static String JOB_PARAM = "JOB_PARAM";
	private final static ZoneId DEFAULT_ZONE_ID = ZoneId.of("GMT+08:00");
	private final Scheduler scheduler;

	public SchedulerHelper(Scheduler scheduler) {
		this.scheduler = scheduler;
	}

	private static Date toDate(LocalDateTime time) {
		ZonedDateTime zdt = time.atZone(DEFAULT_ZONE_ID);
		return Date.from(zdt.toInstant());
	}

	public void addJob(SchedulerCO schedulerCO) throws SchedulerException,
			ClassNotFoundException {
		if (isExists(schedulerCO.getJobName(), schedulerCO.getJobGroup())) {
			throw new BusinessException("任务已经存在!");
		}
		this.checkExpression(schedulerCO.getCron());

		String jobName = schedulerCO.getJobName();
		String groupName = schedulerCO.getJobGroup();

		@SuppressWarnings("unchecked")
		Class<? extends Job> jobClass = (Class<? extends Job>) Class.forName(schedulerCO.getJobClass());

		CronScheduleBuilder scheduleBuilder = CronScheduleBuilder
				.cronSchedule(schedulerCO.getCron())
				.withMisfireHandlingInstructionDoNothing()
				.inTimeZone(TimeZone.getTimeZone(DEFAULT_ZONE_ID));

		TriggerBuilder<Trigger> trigger = TriggerBuilder.newTrigger();
		trigger.withIdentity(jobName, groupName);
		trigger.withSchedule(scheduleBuilder);
		if (StringKit.isNotBlank(schedulerCO.getDescription())) {
			trigger.withDescription(schedulerCO.getDescription());
		}

		CronTrigger cronTrigger = (CronTrigger) trigger.build();

		JobDetail jobDetail = JobBuilder.newJob(jobClass)
				.withIdentity(jobName, groupName)
				.build();

		jobDetail.getJobDataMap().put(JOB_PARAM, schedulerCO);

		this.scheduler.scheduleJob(jobDetail, cronTrigger);

		if (schedulerCO.getState() == SchedulerStatus.PAUSE) {
			this.pauseJob(jobName, groupName);
		}
	}

	public void pauseJob(String jobName, String groupName) throws SchedulerException {
		if (!isStarted()) {
			return;
		}
		if (!isExists(jobName, groupName)) {
			throw new BusinessException("任务不存在");
		}
		scheduler.pauseJob(JobKey.jobKey(jobName, groupName));
	}

	public void resumeJob(String jobName, String groupName) throws SchedulerException {
		if (!isStarted()) {
			return;
		}
		if (!isExists(jobName, groupName)) {
			throw new BusinessException("任务不存在");
		}

		scheduler.resumeJob(JobKey.jobKey(jobName, groupName));
	}

	public void runOnce(String jobName, String groupName) throws SchedulerException {
		if (!isStarted()) {
			return;
		}
		if (!isExists(jobName, groupName)) {
			throw new BusinessException("任务不存在");
		}

		JobKey jobKey = new JobKey(jobName, groupName);
		this.scheduler.triggerJob(jobKey);
	}

	public void updateJob(SchedulerUO schedulerUO) throws SchedulerException {
		String jobName = schedulerUO.getJobName();
		String groupName = schedulerUO.getJobGroup();
		if (!isExists(jobName, groupName)) {
			throw new BusinessException("任务不存在");
		}
		this.checkExpression(schedulerUO.getCron());

		TriggerKey triggerKey = TriggerKey.triggerKey(schedulerUO.getJobName(), schedulerUO.getJobGroup());
		CronTrigger trigger = (CronTrigger) scheduler.getTrigger(triggerKey);

		CronScheduleBuilder scheduleBuilder = CronScheduleBuilder
				.cronSchedule(schedulerUO.getCron())
				.withMisfireHandlingInstructionDoNothing()
				.inTimeZone(TimeZone.getTimeZone(ZoneId.of("GMT+08:00")));
		trigger = trigger.getTriggerBuilder()
				.withIdentity(triggerKey)
				.withSchedule(scheduleBuilder)
				.build();

		if (schedulerUO.getParams() != null) {
			JobKey jobKey = trigger.getJobKey();
			scheduler.getJobDetail(jobKey).getJobDataMap().put(JOB_PARAM, schedulerUO);
		}
		// 按新的 trigger 重新设置 job 执行
		this.scheduler.rescheduleJob(triggerKey, trigger);

		if (schedulerUO.getState() == SchedulerStatus.PAUSE) {
			this.pauseJob(jobName, groupName);
		}
	}

	public void removeJob(String jobName, String groupName) throws SchedulerException {
		if (!isExists(jobName, groupName)) {
			return;
		}

		TriggerKey triggerKey = TriggerKey.triggerKey(jobName, groupName);
		JobKey jobKey = new JobKey(jobName, groupName);

		this.scheduler.pauseTrigger(triggerKey);
		this.scheduler.unscheduleJob(triggerKey);
		this.scheduler.deleteJob(jobKey);
	}

	public void interruptJob(String jobName, String groupName) throws SchedulerException {
		if (this.isExists(jobName, groupName)) {
			return;
		}
		if (this.isStarted()) {
			JobKey jobKey = new JobKey(jobName, groupName);
			this.scheduler.interrupt(jobKey);
		}
	}

	public void startAllJobs(Integer delayed) throws SchedulerException {
		if (null != delayed && delayed > 0) {
			this.scheduler.startDelayed(delayed);
		} else {
			this.scheduler.start();
		}
	}

	public CronTrigger getCronTrigger(String jobName, String jobGroup) {
		try {
			return (CronTrigger) scheduler.getTrigger(new TriggerKey(jobName, jobGroup));
		} catch (SchedulerException e) {
			throw new BusinessException("Quartz数据异常[{}-{}]", jobGroup, jobName, e);
		}
	}

	public boolean isExists(String jobName, String jobGroup) throws SchedulerException {
		JobKey jobKey = new JobKey(jobName, jobGroup);
		return scheduler.checkExists(jobKey);
	}

	private boolean isStarted() throws SchedulerException {
		if (!scheduler.isStarted()) {
			throw new BusinessException("调度器还没有开启，请先开启调度器!");
		} else {
			return true;
		}
	}

	private void checkJobClass(String jobClass) {
		try {
			ClassUtils.forName(jobClass, null);
		} catch (ClassNotFoundException e) {
			throw new SystemException("任务类名不存在,Not Found(" + jobClass + ")", e);
		}
	}

	private void checkExpression(String cronExpression) {
		if (StringKit.isBlank(cronExpression)) {
			throw new BusinessException("Cron表达式不能为空");
		}
		try {
			CronTriggerImpl trigger = new CronTriggerImpl();
			trigger.setCronExpression(cronExpression);

			Date date = trigger.computeFirstFireTime(null);
			if (date == null) {
				throw new ParseException("解析表达式,时间为空", 0);
			}
		} catch (ParseException e) {
			throw new BusinessException(e.getMessage(), e);
		}
	}
}
