package com.chuang.urras.common.task.service.impl;

import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.basic.StringKit;
import com.chuang.tauceti.tools.basic.reflect.ConvertKit;
import com.chuang.urras.common.task.SchedulerHelper;
import com.chuang.urras.common.task.enums.SchedulerStatus;
import com.chuang.urras.common.task.model.Scheduler;
import com.chuang.urras.common.task.model.co.SchedulerCO;
import com.chuang.urras.common.task.model.ro.QuartzJobRO;
import com.chuang.urras.common.task.model.uo.SchedulerUO;
import com.chuang.urras.common.task.service.IQuartzSchedulerService;
import com.chuang.urras.common.task.service.ISchedulerService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.quartz.*;
import org.quartz.impl.matchers.GroupMatcher;
import org.springframework.boot.context.event.ApplicationStartedEvent;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionSynchronizationManager;

import javax.annotation.Resource;
import javax.transaction.Transactional;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Set;

/**
 * @author Ives.l
 * @since 2021/11/26 15:36
 */
@Slf4j
@Service
public class QuartzSchedulerServiceImpl implements IQuartzSchedulerService {

	private SchedulerHelper schedulerHelper;

	@Resource private org.quartz.Scheduler scheduler;
	@Resource private ISchedulerService schedulerService;

	@EventListener(ApplicationStartedEvent.class)
	public void init() {
		this.schedulerHelper = new SchedulerHelper(scheduler);

		List<Scheduler> schedulers = this.schedulerService.list();
		for (Scheduler schedulers1 : schedulers) {
			String jobName = schedulers1.getJobName();
			String jobGroup = schedulers1.getJobGroup();
			CronTrigger cronTrigger = schedulerHelper.getCronTrigger(jobName, jobGroup);

			//  异常不抛出
			try {
				if (null == cronTrigger) {
					schedulerHelper.addJob(ConvertKit.toBean(schedulers1, SchedulerCO::new));
				} else {
					schedulerHelper.updateJob(ConvertKit.toBean(schedulers1, SchedulerUO::new));
				}
			} catch (Exception e) {
				log.warn("The initialization load scheduling task is abnormal,[{}-{}],{}",
						jobGroup, jobName, e.getMessage(), e);
			}
		}

	}

	@Override
	@SneakyThrows
	public List<QuartzJobRO> findJob(String jobName, String groupName) {
		GroupMatcher<JobKey> matcher = null == groupName
				? GroupMatcher.anyJobGroup()
				: GroupMatcher.groupContains(groupName);

		List<QuartzJobRO> quartzJobs = new ArrayList<>();
		Set<JobKey> jobKeys = this.scheduler.getJobKeys(matcher);
		for (JobKey jobKey : jobKeys) {
			if (StringKit.isNotBlank(jobName)
					&& !Objects.equals(jobKey.getName(), jobName)) {
				continue;
			}
			List<? extends Trigger> triggers = this.scheduler.getTriggersOfJob(jobKey);
			for (Trigger trigger : triggers) {
				QuartzJobRO quartzJobRO = this.obtainJobDetails(trigger);
				quartzJobs.add(quartzJobRO);
			}
		}

		return quartzJobs;
	}

	@Override
	@Transactional(rollbackOn = Exception.class)
	@SneakyThrows
	public void addJob(SchedulerCO schedulerCO) {
		String name = TransactionSynchronizationManager.getCurrentTransactionName();
		log.info("current tx name => " + name);
		this.schedulerHelper.addJob(schedulerCO);
		this.schedulerService.save(ConvertKit.toBean(schedulerCO, Scheduler::new));
	}

	@Override
	@Transactional(rollbackOn = Exception.class)
	@SneakyThrows
	public void pauseJob(Long id) {
		Scheduler schedulers = this.schedulerService.findById(id)
				.orElseThrow(() -> new BusinessException("任务已删除"));
		this.schedulerHelper.pauseJob(schedulers.getJobName(), schedulers.getJobGroup());
		this.schedulerService.lambdaUpdate()
				.ge(Scheduler::getId, id)
				.set(Scheduler::getState, SchedulerStatus.PAUSE)
				.update();
	}

	@Override
	@Transactional(rollbackOn = Exception.class)
	@SneakyThrows
	public void resumeJob(Long id) {
		Scheduler schedulers = this.schedulerService.findById(id)
				.orElseThrow(() -> new BusinessException("任务已删除"));
		this.schedulerHelper.resumeJob(schedulers.getJobName(), schedulers.getJobGroup());
		this.schedulerService.lambdaUpdate()
				.ge(Scheduler::getId, id)
				.set(Scheduler::getState, SchedulerStatus.NORMAL)
				.update();
	}

	@Override
	@Transactional(rollbackOn = Exception.class)
	@SneakyThrows
	public void runOnce(Long id) {
		Scheduler schedulers = this.schedulerService.findById(id)
				.orElseThrow(() -> new BusinessException("任务已删除"));
		this.schedulerHelper.runOnce(schedulers.getJobName(), schedulers.getJobGroup());
	}

	@Override
	@Transactional(rollbackOn = Exception.class)
	public void updateJob(SchedulerUO schedulerUO) {
		this.schedulerService.findById(schedulerUO.getId())
				.ifPresent(v -> {
					try {
						if (schedulerHelper.isExists(schedulerUO.getJobName(), schedulerUO.getJobGroup())) {
							this.schedulerHelper.updateJob(schedulerUO);
						} else {
							this.schedulerHelper.removeJob(v.getJobName(), v.getJobGroup());
							this.schedulerHelper.addJob(ConvertKit.toBean(schedulerUO, SchedulerCO::new));
						}

						this.schedulerService.updateById(ConvertKit.toBean(schedulerUO, Scheduler::new));
					} catch (SchedulerException | ClassNotFoundException e) {
						throw new BusinessException("更新任务异常,{}", e.getMessage(), e);
					}
				});
	}

	@Override
	@Transactional(rollbackOn = Exception.class)
	@SneakyThrows
	public void removeJob(final Long id) {
		this.schedulerService.findById(id)
				.ifPresent(v -> {
					try {
						this.schedulerHelper.removeJob(v.getJobName(), v.getJobGroup());
						this.schedulerService.removeById(id);
					} catch (SchedulerException e) {
						throw new BusinessException("删除任务异常,{}", e.getMessage(), e);
					}
				});
	}

	@Override
	@Transactional(rollbackOn = Exception.class)
	public void interruptJob(Long id) {
		this.schedulerService.findById(id)
				.ifPresent(schedulers -> {
					try {
						this.schedulerHelper.interruptJob(schedulers.getJobName(), schedulers.getJobGroup());
					} catch (SchedulerException e) {
						throw new BusinessException("中止任务异常,{}", e.getMessage(), e);
					}
				});
	}

	@Override
	@Transactional(rollbackOn = Exception.class)
	@SneakyThrows
	public void startAllJobs(Integer delayed) {
		this.schedulerHelper.startAllJobs(delayed);
	}

	@Override
	@Transactional(rollbackOn = Exception.class)
	@SneakyThrows
	public void pauseAllJobs() {
		this.scheduler.pauseAll();
	}

	@Override
	@Transactional(rollbackOn = Exception.class)
	@SneakyThrows
	public void resumeAllJobs() {
		this.scheduler.resumeAll();
	}

	@Override
	@Transactional(rollbackOn = Exception.class)
	@SneakyThrows
	public void shutdownAllJobs(boolean waitForJobsToComplete) {
		if (!scheduler.isShutdown()) {
			// 需谨慎操作关闭 scheduler 容器
			// scheduler 生命周期结束，无法再 start() 启动scheduler
			this.scheduler.shutdown(true);
		}
	}

	private QuartzJobRO obtainJobDetails(Trigger trigger) throws SchedulerException {
		QuartzJobRO job = new QuartzJobRO();
		JobKey jobKey = trigger.getJobKey();
		JobDetail jobDetail = this.scheduler.getJobDetail(jobKey);

		job.setJobClass(jobDetail.getJobClass().getName());
		job.setDescription(jobDetail.getDescription());
		JobDataMap jobDataMap = jobDetail.getJobDataMap();
		if (jobDataMap != null) {
			job.setJobDataMap(jobDataMap);
		}

		job.setJobName(jobKey.getName());
		job.setJobGroup(jobKey.getGroup());
		job.setNextTime(trigger.getNextFireTime());
		job.setPrevTime(trigger.getPreviousFireTime());

		Trigger.TriggerState triggerState = this.scheduler.getTriggerState(trigger.getKey());
		job.setJobStatus(triggerState.name());
		job.setStartAt(trigger.getStartTime());
		job.setEndAt(trigger.getEndTime());
		if (trigger instanceof CronTrigger) {
			CronTrigger cronTrigger = (CronTrigger) trigger;
			String cronExpression = cronTrigger.getCronExpression();
			job.setCronExpression(cronExpression);
		}
		return job;
	}


}
