package com.chuang.urras.common.web.aop;

import com.chuang.tauceti.support.MapResult;
import com.chuang.tauceti.support.Result;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.third.spring.SpringKit;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.context.MessageSource;
import org.springframework.context.NoSuchMessageException;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.validation.BindException;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.Arrays;
import java.util.Objects;
import java.util.UUID;
import java.util.concurrent.CompletionException;
import java.util.stream.Collectors;

@Slf4j
@ControllerAdvice
public class MyControllerAdvice {
    /** 不知道正确还是错误 */
    public static final int DEFAULT_PARAM_ERROR_CODE = -3;

    @ExceptionHandler(value = Throwable.class)
    @ResponseBody
    public Object exception(Throwable exception) {
        if(exception instanceof CompletionException) {
            if(exception != exception.getCause()) {
                return exception(exception.getCause());
            }
        }

        if(isIncorrectCredentialsException(exception)) {
            return Result.fail(getMessage("msg.error.credentials"));
        } if(isShiroException(exception)) {
            return MapResult.fail( "RBAC Error：" + exception.getMessage())
                    .data("refreshInfo", true)
                    .toResult();
        } else if (isConstraintViolationException(exception)) {
            ConstraintViolationException ex = (ConstraintViolationException) exception;
            String msg = ex.getConstraintViolations().stream()
                    .map(ConstraintViolation::getMessageTemplate)
                    .map(String::trim)
                    .map(msg0 -> {
                        try {
                            return getMessage(msg0);
                        } catch (Exception e) {
                            return msg0;
                        }
                    })
                    .collect(Collectors.joining(","));
            return Result.fail(DEFAULT_PARAM_ERROR_CODE, msg);
        } else if (isBindException(exception)) {
            BindingResult bindingResult = ((BindException) exception).getBindingResult();
            if (bindingResult.getFieldError() == null) {
                return Result.fail("param valid error");
            } else {
                return Result.fail(DEFAULT_PARAM_ERROR_CODE, getMessage(bindingResult.getFieldError().getDefaultMessage(), bindingResult.getFieldError().getField()));
            }
        } else if(isBusinessException(exception)) {
            return Result.fail(((BusinessException)exception).getCode(), exception.getMessage());
        } else if(exception instanceof MethodArgumentNotValidException){
            FieldError fe = Objects.requireNonNull(((MethodArgumentNotValidException) exception).getBindingResult().getFieldError());
            return Result.fail(fe.getField() + " " + fe.getDefaultMessage());
        } else if(isHttpRequestMethodNotSupportedException(exception)) {
            return Result.fail(exception.getMessage());
        } else if(isDuplicateKeyException(exception)) {
            return Result.fail("repeat data:" + Objects.requireNonNull(((DuplicateKeyException) exception).getRootCause()).getMessage());
        }

        String err = UUID.randomUUID().toString();
        log.error("controller异常, 错误号:" + err, exception);

        return Result.fail(getMessage("msg.system-error", err));
    }

    private boolean isHttpRequestMethodNotSupportedException(Throwable exception) {
        return exception instanceof HttpRequestMethodNotSupportedException;
    }

    private boolean isIncorrectCredentialsException(Throwable e) {
        return "org.apache.shiro.authc.IncorrectCredentialsException".equalsIgnoreCase(e.getClass().getName());
    }

    private boolean isBusinessException(Throwable exception) {
        return exception instanceof BusinessException;
    }

    private boolean isShiroException(Throwable e) {
        return "org.apache.shiro.ShiroException".equals(e.getClass().getName());
    }
    private boolean isShiroAuthenticationException(Throwable e) {
        return "org.apache.shiro.authc.AuthenticationException".equals(e.getClass().getName());
    }
    private boolean isDuplicateKeyException(Throwable e) {
        return "org.springframework.dao.DuplicateKeyException".equals(e.getClass().getName());
    }

    private boolean isBindException(Throwable e) {
        return "org.springframework.validation.BindException".equals(e.getClass().getName());
    }

    private boolean isConstraintViolationException(Throwable e) {
        return "javax.validation.ConstraintViolationException".equals(e.getClass().getName());
    }

    private String getMessage(String i18n, Object... args) {
        MessageSource messageSource = SpringKit.getBean(MessageSource.class);

        if(null == messageSource) {
            return i18n;
        }
        try {
            return messageSource.getMessage(i18n, args, LocaleContextHolder.getLocale());
        } catch (NoSuchMessageException e) {
            return i18n + Arrays.toString(args);
        }
    }
}
