package com.chuang.urras.sr.admin.controller;

import com.chuang.tauceti.acme.v2.store.AcmeStore;
import com.chuang.tauceti.acme.v2.store.FileStore;
import com.chuang.tauceti.rowquery.controller.basic.IRetrieveController;
import com.chuang.tauceti.rowquery.controller.basic.IUpdateController;
import com.chuang.tauceti.support.MapResult;
import com.chuang.tauceti.support.Result;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.basic.FileKit;
import com.chuang.tauceti.tools.basic.FutureKit;
import com.chuang.tauceti.tools.basic.NetKit;
import com.chuang.tauceti.tools.basic.StringKit;
import com.chuang.tauceti.tools.basic.reflect.ConvertKit;
import com.chuang.urras.common.events.RedisEventManager;
import com.chuang.urras.sr.admin.model.ReApply;
import com.chuang.urras.sr.admin.model.co.AcmeOrderCO;
import com.chuang.urras.sr.admin.model.co.BatchAcmeCO;
import com.chuang.urras.sr.admin.model.ro.AcmeOrderRO;
import com.chuang.urras.sr.admin.model.ro.SeoDomainRO;
import com.chuang.urras.sr.admin.model.uo.AcmeOrderUO;
import com.chuang.urras.sr.core.acme.AcmeManager;
import com.chuang.urras.sr.core.cache.CacheManager;
import com.chuang.urras.sr.core.entity.AcmeOrder;
import com.chuang.urras.sr.core.entity.SeoDomain;
import com.chuang.urras.sr.core.enums.AcmeOrderStatus;
import com.chuang.urras.sr.core.service.single.IAcmeOrderService;
import com.chuang.urras.sr.core.service.single.ISeoDomainService;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Mono;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.io.IOException;
import java.util.Collection;
import java.util.List;
import java.util.Set;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Collectors;


/**
 * <p>
 * Acme订单 前端控制器
 * </p>
 *
 * @author chuang
 * @since 2021-05-31
 */
@Slf4j
@RestController
@RequestMapping("/star-ring/acme/order")
public class AcmeOrderController implements
        IUpdateController<AcmeOrderUO, AcmeOrder, IAcmeOrderService>,
        IRetrieveController<AcmeOrderRO, AcmeOrder, IAcmeOrderService> {

    @Resource private IAcmeOrderService service;
    @Resource private AcmeManager acme;
    @Resource private AcmeStore store;
    @Resource private ISeoDomainService seoDomainService;
    @Resource private CacheManager cacheManager;


    @PostMapping(value = "/create")
    @ApiOperation("创建一条数据")
    @RequiresPermissions("acme-order:create")
    public Mono<Result<AcmeOrder>> create(@RequestBody @ApiParam @Valid AcmeOrderCO co) {
        CompletableFuture<Result<AcmeOrder>> c = acme.apply(co.getAccount(), co.getOrderReference(), co.getDomain(), co.getSec().split(","))
                .thenApply(Result::success);
        return FutureKit.toMono(c);
    }

    @GetMapping(value = "/check-info/{id}")
    @ApiOperation("查证信息")
    @RequiresPermissions("acme-order:query")
    public Result<?> checkInfo(@PathVariable Integer id) {
        AcmeOrder order = service.findById(id)
                .orElseThrow(() -> new BusinessException("记录不存在"));
        SeoDomain domain = seoDomainService.findByDomain(order.getDomain())
                .orElseThrow(() -> new BusinessException("对应的网站记录不存在"));

        String locationFile = ((FileStore)store).getRootPath() + "generic/" + order.getDomain() + "/location.txt";
        boolean hasFullChain = FileKit.exist(((FileStore)store).getRootPath() + "generic/" + order.getDomain() + "/fullchain.pem");
        try {
            String location = FileKit.readString(locationFile);
            return MapResult.success()
                    .data("hasFullChain", hasFullChain)
                    .data("location", location)
                    .data("domain", ConvertKit.toBean(domain, SeoDomainRO::new))
                    .data("ip", NetKit.getDomainIp(domain.getDomain()).orElse("un-known"))
                    .toResult();
        } catch (IOException e) {
            throw new BusinessException("无法读取 >> " + locationFile);
        }
    }

    @PostMapping(value = "/re-apply")
    @ApiOperation("立即重新申请")
    @RequiresPermissions("acme-order:create")
    public Mono<Result<AcmeOrder>> reApply(@RequestBody @ApiParam @Valid ReApply co) {
        CompletableFuture<Result<AcmeOrder>> c = acme.reApply(co.getReference(), co.getSecs()).thenApply(Result::success);
        return FutureKit.toMono(c);
    }

    @PostMapping(value = "/batch-create")
    @ApiOperation("批量创建数据")
    @RequiresPermissions("acme-order:create")
    public Result<Void> batchCreate(@RequestBody @ApiParam @Valid BatchAcmeCO batch) {
        List<AcmeOrder> list = batch.getDomains().stream().map(s -> new AcmeOrder()
                .setStatus(AcmeOrderStatus.PENDING)
                .setSec(StringKit.join(batch.getSecs(), ","))
                .setProvider("")
                .setAccount(batch.getAccount())
                .setDomain(s)
                .setOrderReference(s))
                .collect(Collectors.toList());

        Collection<String> contains = service.saveNotContains(AcmeOrder::getDomain, list);

        Set<String> notContains = list.stream()
                .map(AcmeOrder::getDomain)
                .filter(domain -> !contains.contains(domain))
                .collect(Collectors.toSet());
        cacheManager.getAcmeApplyDomains().addAll(notContains);

        return Result.success();

    }

    @PostMapping(value = "/re-start")
    @ApiOperation("批量重新开始")
    @RequiresPermissions("acme-order:create")
    public Result<Void> reStart() {

        Set<String> notDownload = service.lambdaQuery()
                .select(AcmeOrder::getDomain)
                .ne(AcmeOrder::getStatus, AcmeOrderStatus.DOWNLOADED).list()
                .stream()
                .map(AcmeOrder::getDomain)
                .collect(Collectors.toSet());
        cacheManager.getAcmeRenewDomains().addAll(notDownload);

        return Result.success();
    }

    @PostMapping(value = "/batch-renew")
    @ApiOperation("批量更新证书")
    public Result<?> batchRenew(@RequestBody @ApiParam @Valid List<String> domains) {
        cacheManager.getAcmeRenewDomains().addAll(domains);
        return Result.success().message("已经提交");
    }


    @DeleteMapping(value = "/delete/{id}")
    @ApiOperation("删除以条记录")
    @RequiresPermissions("acme-order:delete")
    public Result<Void> delete(@PathVariable("id") String id) {
        AcmeOrder order = service.findById(id).orElseThrow(() -> new BusinessException("不存在这个记录"));
        store.setProvider(acme.newConnection().getSession().getProvider());
        store.deleteOrder(order.getOrderReference());
        return Result.whether(service().removeById(id));
    }

    /**
     * 推送证书
     */
    @PostMapping("/push-ssl")
    public Result<Void> pushSsl(@RequestBody @Valid List<String> domains) {
        int i = cacheManager.getPublishDomains().addAllCounted(domains);
        return Result.whether(true, "新增" + i + "个push任务");
    }

    @RequestMapping("/queue")
    public Result<String> downloadQueue() {

        String apply = cacheManager.getAcmeApplyDomains().readAll().stream().map(s -> "  " + s)
                .collect(Collectors.joining(",\r\n", "Apply: {\r\n", "\r\n}\r\n"));
        String renew = cacheManager.getAcmeRenewDomains().readAll().stream().map(s -> "  " + s)
                .collect(Collectors.joining(",\r\n", "Renew: {\r\n", "\r\n}\r\n"));
        String publish = cacheManager.getPublishDomains().readAll().stream().map(s -> "  " + s)
                .collect(Collectors.joining(",\r\n", "Publish: {\r\n", "\r\n}\r\n"));

        log.info("redisCommonManager.getAcmeApplyDomains() -> " + cacheManager.getAcmeApplyDomains().size());
        log.info("redisCommonManager.getAcmeRenewDomains() -> " + cacheManager.getAcmeRenewDomains().size());
        log.info("redisCommonManager.getPublishDomains() -> " + cacheManager.getPublishDomains().size());

        return Result.success(apply + renew + publish);
    }

    @RequestMapping("/queue/clear")
    public Result<Void> clearQueue() {
        cacheManager.getAcmeApplyDomains().clear();
        cacheManager.getAcmeRenewDomains().clear();
        cacheManager.getPublishDomains().clear();
        return Result.success();
    }

    @Override
    public IAcmeOrderService service() {
        return service;
    }

    @Override
    public String basePermission() {
        return "acme-order:";
    }

}

