package com.chuang.urras.sr.admin.controller;

import com.chuang.tauceti.rowquery.controller.basic.IDeleteController;
import com.chuang.tauceti.rowquery.controller.basic.IRetrieveController;
import com.chuang.tauceti.support.BiValue;
import com.chuang.tauceti.support.Result;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.basic.HashKit;
import com.chuang.tauceti.tools.basic.IOKit;
import com.chuang.tauceti.tools.basic.StringKit;
import com.chuang.tauceti.tools.basic.collection.CollectionKit;
import com.chuang.tauceti.tools.basic.reflect.ConvertKit;
import com.chuang.urras.sr.admin.model.PublishParam;
import com.chuang.urras.sr.admin.model.co.ArticleCO;
import com.chuang.urras.sr.admin.model.ro.ArticleRO;
import com.chuang.urras.sr.admin.model.uo.ArticleUO;
import com.chuang.urras.sr.core.entity.Article;
import com.chuang.urras.sr.core.entity.SeoDomain;
import com.chuang.urras.sr.core.enums.DomainType;
import com.chuang.urras.sr.core.service.single.IArticleService;
import com.chuang.urras.sr.core.service.single.ISeoDomainService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.archivers.zip.ZipArchiveInputStream;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author chuang
 * @since 2021-05-26
 */
@RestController
@RequestMapping("/star-ring/article")
@Slf4j
@Api("文章接口")
public class ArticleController implements
        IDeleteController<Article, IArticleService>,
        IRetrieveController<ArticleRO, Article, IArticleService> {

    @Resource private IArticleService service;
    @Resource private ISeoDomainService domainService;

    @RequiresPermissions("article:create")
    @PostMapping(value = "/create")
    @ApiOperation("创建一条数据")
    public Result<?> create(@RequestBody @ApiParam @Valid ArticleCO co) {

        Article article = ConvertKit.toBean(co, Article::new);
        article.setMd5(HashKit.md5(article.getArticle()));

        return Result.whether(service().save(article))
                .data(ConvertKit.toBean(article, ArticleRO::new));
    }

    @RequiresPermissions("article:update")
    @PostMapping(value = "/update")
    @ApiOperation("修改资料")
    @ResponseBody
    public Result<Void> update(@Valid @RequestBody @ApiParam ArticleUO uo) {
        Article article = ConvertKit.toBean(uo, Article::new);
        article.setMd5(HashKit.md5(uo.getArticle()));
        return Result.whether(service().updateById(article));
    }


    @RequiresPermissions("article:batch-publish")
    @PostMapping(value = "/batch-publish")
    @ApiOperation("批量发布")
    @ResponseBody
    public Result<Boolean> batchPublish(@Valid @RequestBody @ApiParam PublishParam param) {
        List<SeoDomain> domainList = domainService.lambdaQuery()
                .in(SeoDomain::getDomain, param.getDomains())
                .list();


        Map<DomainType, List<SeoDomain>> map = domainList.stream()
                .collect(Collectors.groupingBy(SeoDomain::getType));

        map.forEach((domainType, vos) -> {
            List<Article> articles = service.findNotUsed(domainType, domainList.size() * param.getNumber());
            int idx = 0;
            for (SeoDomain vo : vos) {
                for (int i = 0; i < param.getNumber(); i++, idx++) {
                    Article article = articles.get(idx);
                    article.setUsed(true)
                            .setMenu(false)
                            .setOwner(vo.getDomain());
                }
                vo.setLastArticleTime(LocalDateTime.now());
            }
            domainService.updateBatchById(ConvertKit.toBeans(vos, SeoDomain::new));
            service.updateBatchById(articles);
        });

        return Result.success(true);
    }

    @PostMapping("/upload/{type}")
    @ApiOperation("上传文章zip文件，每个文档的名字为标题，内容为文章正文。")
    @RequiresPermissions("article:create")
    public Result<Void> upload(@RequestParam("file") MultipartFile file, @PathVariable DomainType type) {

        Map<String, Article> articles = new HashMap<>();
        boolean hasError = false;
        try {

            ZipArchiveInputStream stream =
                    new ZipArchiveInputStream(file.getInputStream(), "gbk");

            while ((stream.getNextZipEntry()) != null) {
                List<String> lines = IOKit.readLines(stream, "gbk", new ArrayList<>());
                String name = lines.remove(0);
                String content = lines.stream().filter(StringUtils::isNoneBlank).map(s -> {
                    if(!s.contains("<p>")) {
                        s = "<p>" + s;
                    }
                    if(!s.contains("</p>")) {
                        s += "</p>";
                    }
                    return s;
                }).reduce((s, s2) -> s + "\n" + s2).orElse("").trim();

                if(StringKit.isBlank(content)) {
                    continue;
                }

                String md5 = HashKit.md5(content);

                articles.put(md5, new Article().setTitle(name).setArticle(content).setMd5(md5).setUsed(false).setType(type));
            }

        } catch (Exception e) {
            log.error("读取异常", e);
            hasError = true;
        }

        if(articles.isEmpty()) {
            throw new BusinessException(-1, "一篇文章都没成功");
        }
        CollectionKit.split(articles.keySet(), 1000, HashSet::new).forEach(md5s -> {

            Set<String> hasSet =
                    service.lambdaQuery().select(Article::getMd5)
                            .in(Article::getMd5, md5s)
                            .list()
                            .stream()
                            .map(Article::getMd5)
                            .collect(Collectors.toSet());

            hasSet.forEach(articles::remove);
        });

        if(articles.isEmpty()) {
            throw new BusinessException(-1, "检测到所有文章都是重复的");
        }
        service.saveBatchSafe(articles.values());
        if(hasError) {
            return Result.fail("读取发生异常,目前只能上传" + articles.size() + "个文章");
        } else {
            return Result.success();
        }
    }

    @GetMapping("/statistics")
    @ApiOperation("统计")
    @RequiresPermissions("article:query")
    public Result<List<BiValue<DomainType, Integer>>> statistics() {
        return Result.success(service.statisticsNotUsed());
    }

    @Override
    public IArticleService service() {
        return service;
    }

    @Override
    public String basePermission() {
        return "article:";
    }

}

