package com.chuang.urras.sr.admin.controller;

import com.chuang.tauceti.rowquery.controller.basic.ICrudController;
import com.chuang.tauceti.support.MapResult;
import com.chuang.tauceti.support.Result;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.basic.FileKit;
import com.chuang.tauceti.tools.basic.IOKit;
import com.chuang.tauceti.tools.basic.collection.CollectionKit;
import com.chuang.urras.sr.admin.model.co.DownloadTaskCO;
import com.chuang.urras.sr.admin.model.ro.DownloadTaskRO;
import com.chuang.urras.sr.admin.model.uo.DownloadTaskUO;
import com.chuang.urras.sr.core.entity.DownloadTask;
import com.chuang.urras.sr.core.enums.TaskStatus;
import com.chuang.urras.sr.core.service.single.IDownloadTaskService;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import reactor.core.publisher.Mono;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author chuang
 * @since 2021-05-26
 */
@RestController
@RequestMapping("/star-ring/download-task")
@Slf4j
public class DownloadTaskController implements ICrudController<DownloadTaskCO, DownloadTaskRO, DownloadTaskUO, DownloadTask, IDownloadTaskService> {

    @Resource private IDownloadTaskService service;
    @Value("${eden.fetch.save-path}") private String savePath;

    @PostMapping("/download/{id}")
    @RequiresPermissions("download-task:download")
    public Mono<Result<Void>> download(@PathVariable Integer id) {
        DownloadTask task = service.findById(id).orElseThrow(() -> new BusinessException(-1, "记录不存在"));
        return Mono.create(sink -> service.download(task).whenComplete((response, throwable) -> {
            if(throwable == null) {
                sink.success(Result.success());
            } else {
                sink.success(Result.fail(throwable.getMessage()));
            }
        }));

    }

    @PostMapping("/upload")
    @RequiresPermissions("download-task:upload")
    public Result<?> upload(Integer id, MultipartFile file) {
        DownloadTask task = service.findById(id).orElseThrow(() -> new BusinessException(-1, "记录不存在"));
        service.updateById(task.setState(TaskStatus.ENDED)
                .setError("已经手动上传")
        );

        try {
            File f = new File(savePath + task.getSavePath());
            FileKit.touch(f);
            IOKit.copy(file.getInputStream(), new FileOutputStream(f));
        } catch (IOException e) {
            log.error("", e);
        }
        return Result.success();
    }

    @RequestMapping("/random")
    public Mono<Result<?>> random(String website) {
        List<DownloadTask> tasks = service.findByWebSite(website);
        return Mono.create(sink -> service.download(CollectionKit.randomOne(tasks)).whenComplete((response, throwable) -> {
            if(throwable == null) {
                sink.success(Result.success());
            } else {
                sink.success(Result.fail(throwable.getMessage()));
            }
        }));
    }

    @RequestMapping("/check")
    public Result<?> check(String website) {
        List<DownloadTask> tasks = service.findByWebSite(website);
        long ends = tasks.stream().filter(task -> task.getState() == TaskStatus.ENDED).count();

        return MapResult.success().data("total", tasks.size()).data("ends", ends).toResult();
    }

    @PostMapping("/download/suc/{website}")
    public Result<?> download(@PathVariable @Valid @NotEmpty String website) {
        List<DownloadTask> tasks = service.findByWebSite(website);

        download(tasks);

        return Result.success();
    }

    @PostMapping("/download/nohave/{website}")
    public Result<?> downloadNoHave(@PathVariable @Valid @NotEmpty String website) {
        List<DownloadTask> tasks = service.findByWebSite(website);

        tasks.forEach(downloadTask -> {
            try {
                service.download(downloadTask, true).join();
            } catch (Exception e) {
                log.error("", e);
            }

        });

        return Result.success();
    }


    @PostMapping("/download/fail/{website}")
    public Result<?> downloadFail(@PathVariable @Valid @NotEmpty String website) {

        List<DownloadTask> tasks = service.lambdaQuery()
                .eq(DownloadTask::getWebsite, website)
                .notIn(DownloadTask::getState, Arrays.asList(TaskStatus.RUNNING, TaskStatus.ENDED))
                .list();
        download(tasks);

        return Result.success();
    }

    private void download(List<DownloadTask> tasks) {
        tasks.forEach(downloadTask -> {
            try {
                service.download(downloadTask).join();
            } catch (Exception e) {
                log.error("", e);
            }

        });
    }

    @Override
    public IDownloadTaskService service() {
        return service;
    }

    @Override
    public String basePermission() {
        return "download-task:";
    }

}

