package com.chuang.urras.sr.admin.controller;

import com.chuang.tauceti.rowquery.controller.basic.ICrudController;
import com.chuang.tauceti.support.Result;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.urras.sr.admin.model.IdsParam;
import com.chuang.urras.sr.admin.model.co.PlatformCheckContentCO;
import com.chuang.urras.sr.admin.model.ro.PlatformCheckContentRO;
import com.chuang.urras.sr.admin.model.uo.PlatformCheckContentUO;
import com.chuang.urras.sr.core.entity.PlatformCheckContent;
import com.chuang.urras.sr.core.entity.PlatformCookie;
import com.chuang.urras.sr.core.entity.SeoDomain;
import com.chuang.urras.sr.core.service.platform.IPushService;
import com.chuang.urras.sr.core.service.single.IArticleService;
import com.chuang.urras.sr.core.service.single.IPlatformCheckContentService;
import com.chuang.urras.sr.core.service.single.IPlatformCookieService;
import com.chuang.urras.sr.core.service.single.ISeoDomainService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.*;
import java.util.function.BiConsumer;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author chuang
 * @since 2021-05-26
 */
@RestController
@RequestMapping("/star-ring/platform-check-content")
@Slf4j
public class PlatformCheckContentController implements ICrudController<PlatformCheckContentCO, PlatformCheckContentRO, PlatformCheckContentUO, PlatformCheckContent, IPlatformCheckContentService> {

    @Resource private IPlatformCheckContentService service;
    @Resource private IPushService pushService;
    @Resource private IPlatformCookieService platformCookieService;
    @Resource private ISeoDomainService seoDomainService;
    @Resource private IArticleService articleService;

    @PostMapping("/submit/{cookieId}")
    public Result<?> submit(@PathVariable @Valid @NotNull Integer cookieId, @RequestBody List<String> dms) {
        PlatformCookie cookie = platformCookieService.findById(cookieId)
                .orElseThrow(() -> new BusinessException(-1, "找不到相关的cookie"));
        if(!cookie.getValid()) {
            throw new BusinessException(-1, "cookie似乎已经不能用了。");
        }
        Set<String> domains = new HashSet<>(dms);

        StringBuilder msg = new StringBuilder();
        for (String domain : domains) {

            PlatformCheckContent pcc =
                    service.getOrCreate(cookie.getPlatformType(), domain);
            if(pcc.getSubmitted()) {
                msg.append(pcc.getDomain()).append("已经检测过\n");
                continue;
            }
            boolean success = pushService.addSite(cookie, pcc, service::updateById).join();
            if(!success) {
                msg.append(pcc.getDomain()).append("添加失败\n");
            }
        }

        return Result.success().message(msg.toString());
    }

    @PostMapping("/submit/by-id")
    @ResponseBody
    public Result<?> submitById(@RequestBody @Valid IdsParam idsParam) {
        StringBuilder msg = new StringBuilder();
        this.ids(idsParam.getIds(), (pcc, cookie) -> {
            if (null == cookie) {
                log.warn("{}无法找到cookie", pcc.getDomain());
                msg.append(pcc.getDomain()).append("无法找到相关cookie\n");
                return;
            }
            if(!cookie.getValid()) {
                log.warn("{}无法找到cookie 失效了", pcc.getDomain());
                msg.append(pcc.getDomain()).append("相关cookie失效了\n");
                return;
            }
            if(pcc.getSubmitted()) {
                msg.append(pcc.getDomain()).append("已经验证过\n");
                return;
            }

            pushService.addSite(cookie, pcc, service::updateById).join();

        });

        return Result.success(msg.toString());
    }


    @PostMapping("/push/links")
    @ResponseBody
    public Result<?> push(@RequestBody @Valid IdsParam idsParam) {
        StringBuilder msg = new StringBuilder();
        this.ids(idsParam.getIds(), (pcc, cookie) -> {
            if(!checkCookieAndPcc(pcc, cookie, msg)) {
                return;
            }
            SeoDomain domain = seoDomainService.findByDomain(pcc.getDomain()).orElse(null);
            if(null == domain) {
                log.warn("{}无法找到网站记录", pcc.getDomain());
                msg.append(pcc.getDomain()).append("无法找到网站记录\n");
                return;
            }


            boolean success = pushService.linkPush(cookie, pcc, articleService.links(domain), service::updateById).join();
            if(!success) {
                msg.append(pcc.getDomain()).append("链接推送失败\n");
            }
        });

        return Result.success(msg.toString());
    }

    @PostMapping("/push/sitemap")
    @ResponseBody
    public Result<?> pushSitemap(@RequestBody @Valid IdsParam idsParam) {
        StringBuilder msg = new StringBuilder();
        this.ids(idsParam.getIds(), (pcc, cookie) -> {
            if(!checkCookieAndPcc(pcc, cookie, msg)) {
                return;
            }
            boolean success = pushService.sitemapPush(cookie, pcc).join();
            if(!success) {
                msg.append(pcc.getDomain()).append("sitemap推送失败\n");
            }
        });
        return Result.success(msg.toString());
    }


    private boolean checkCookieAndPcc(PlatformCheckContent pcc, PlatformCookie cookie, StringBuilder msg) {
        if(!pcc.getSubmitted()) {
            msg.append(pcc.getDomain()).append("需要先提交才能push\n");
            return false;
        }
        if (null == cookie) {
            log.warn("{}无法找到cookie", pcc.getDomain());
            msg.append(pcc.getDomain()).append("无法找到相关cookie\n");
            return false;
        }
        if(!cookie.getValid()) {
            log.warn("{}无法找到cookie 失效了", pcc.getDomain());
            msg.append(pcc.getDomain()).append("相关cookie失效了\n");
            return false;
        }
        return true;
    }

    private void ids(Set<Integer> ids, BiConsumer<PlatformCheckContent, PlatformCookie> consumer) {
        List<PlatformCheckContent> pccs = service.lambdaQuery().in(PlatformCheckContent::getId, ids).list();
        Map<String, PlatformCookie> cookieCache = new HashMap<>();

        for(PlatformCheckContent pcc: pccs) {
            PlatformCookie cookie = cookieCache.computeIfAbsent(pcc.getUsername() + ":" + pcc.getPlatformType(), s ->
                    platformCookieService.lambdaQuery()
                            .eq(PlatformCookie::getPlatformType, pcc.getPlatformType())
                            .eq(PlatformCookie::getUsername, pcc.getUsername())
                            .one()
            );

            consumer.accept(pcc, cookie);
        }
    }

    @Override
    public IPlatformCheckContentService service() {
        return service;
    }

    @Override
    public String basePermission() {
        return "platform-check-content:";
    }

}

