package com.chuang.urras.sr.admin.controller;

import com.chuang.tauceti.rowquery.controller.basic.ICrudController;
import com.chuang.tauceti.support.Result;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.basic.HashKit;
import com.chuang.tauceti.tools.basic.IOKit;
import com.chuang.tauceti.tools.basic.StringKit;
import com.chuang.tauceti.tools.basic.collection.CollectionKit;
import com.chuang.tauceti.tools.basic.reflect.ConvertKit;
import com.chuang.urras.sr.admin.model.co.SentenceCO;
import com.chuang.urras.sr.admin.model.ro.SentenceRO;
import com.chuang.urras.sr.admin.model.uo.SentenceUO;
import com.chuang.urras.sr.core.entity.Sentence;
import com.chuang.urras.sr.core.service.single.ISentenceService;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * 句子库 前端控制器
 * </p>
 *
 * @author chuang
 * @since 2021-06-10
 */
@Slf4j
@RestController
@RequestMapping("/star-ring/sentence")
public class SentenceController implements ICrudController<SentenceCO, SentenceRO, SentenceUO, Sentence, ISentenceService> {

    @Resource private ISentenceService service;


    @RequiresPermissions("sentence:create")
    @PostMapping(value = "/create")
    @ApiOperation("创建一条数据")
    @Override
    public Result<SentenceRO> create(@Valid SentenceCO sentenceCO) {
        Sentence sentence = ConvertKit.toBean(sentenceCO, Sentence::new);
        sentence.setMd5(HashKit.md5(sentence.getSentence()));

        Result<SentenceRO> r = Result.whether(service().save(sentence));

        return r.data(ConvertKit.toBean(sentence, SentenceRO::new));
    }

    @RequiresPermissions("sentence:update")
    @PostMapping(value = "/update")
    @ApiOperation("修改资料")
    @Override
    public Result<Void> update(@Valid SentenceUO sentenceUO) {
        Sentence sentence = ConvertKit.toBean(sentenceUO, Sentence::new);
        sentence.setMd5(HashKit.md5(sentence.getSentence()));
        return Result.whether(service.updateById(sentence));
    }

    @PostMapping("/upload")
    @ApiOperation("上传文章zip文件，每个文档的名字为标题，内容为文章正文。")
    @RequiresPermissions("sentence:create")
    public Result<Void> upload(@RequestParam("file") MultipartFile file) {

        Map<String, Sentence> sentences = new HashMap<>();
        boolean hasError = false;
        try {

            String txt = IOKit.read(file.getInputStream(), "gbk");

            String[] lines = txt.replaceAll("[\r\t ]", "")
                    .replaceAll("([!！?？.。])", "$1\n")
                    .split("\n");

            for(String line : lines) {
                if(StringKit.isBlank(line)) {
                    continue;
                }
                line = line.trim();
                line = line.substring(0, Math.min(line.length(), 200));
                String md5 = HashKit.md5(line);
                sentences.put(md5, new Sentence().setSentence(line).setMd5(md5));
            }

        } catch (Exception e) {
            log.error("读取异常", e);
            hasError = true;
        }

        if(sentences.isEmpty()) {
            throw new BusinessException(-1, "一条句子都没成功");
        }
        CollectionKit.split(sentences.values(), 2000, ArrayList::new)
                .forEach(service::saveBatchSafe);

        if(hasError) {
            return Result.fail("读取发生异常,目前只能上传" + sentences.size() + "个句子");
        } else {
            return Result.success();
        }
    }

    @Override
    public ISentenceService service() {
        return service;
    }

    @Override
    public String basePermission() {
        return "sentence:";
    }

}

