package com.chuang.urras.sr.admin.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.chuang.tauceti.rbac2.rest.OfficeUtils;
import com.chuang.tauceti.rowquery.controller.basic.ICrudController;
import com.chuang.tauceti.support.Result;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.support.exception.SystemException;
import com.chuang.tauceti.support.rowquery.RowQuery;
import com.chuang.tauceti.tools.basic.IOKit;
import com.chuang.tauceti.tools.basic.ReferenceGen;
import com.chuang.tauceti.tools.basic.StringKit;
import com.chuang.tauceti.tools.basic.collection.CollectionKit;
import com.chuang.tauceti.tools.basic.reflect.ConvertKit;
import com.chuang.urras.sr.admin.ControllerUtil;
import com.chuang.urras.sr.admin.Excel;
import com.chuang.urras.sr.admin.model.AuditParam;
import com.chuang.urras.sr.admin.model.BatchAuditParam;
import com.chuang.urras.sr.admin.model.IdsParam;
import com.chuang.urras.sr.admin.model.SeoDomainProposalVO;
import com.chuang.urras.sr.admin.model.co.SeoDomainProposalCO;
import com.chuang.urras.sr.admin.model.ro.SeoDomainProposalRO;
import com.chuang.urras.sr.admin.model.uo.SeoDomainProposalUO;
import com.chuang.urras.sr.core.entity.*;
import com.chuang.urras.sr.core.enums.DomainType;
import com.chuang.urras.sr.core.enums.ProposalStatus;
import com.chuang.urras.sr.core.exception.NohaveArticleException;
import com.chuang.urras.sr.core.service.ISprocketService;
import com.chuang.urras.sr.core.service.polymer.IWebBuildService;
import com.chuang.urras.sr.core.service.single.*;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author chuang
 * @since 2021-05-26
 */
@Slf4j
@RestController
@RequestMapping("/star-ring/seo-domain-proposal")
public class SeoDomainProposalController implements ICrudController<SeoDomainProposalCO, SeoDomainProposalRO, SeoDomainProposalUO, SeoDomainProposal, ISeoDomainProposalService> {

    @Resource private ISeoDomainProposalService service;
    @Resource private IWebBuildService webBuildService;
    @Resource private ISeoTemplateService templateService;
    @Resource private ITemplateGroupService templateGroupService;
    @Resource private ISeoDomainService seoDomainService;
    @Resource private IAcmeOrderService acmeOrderService;
    @Resource private ISprocketService sprocketService;

    @PostMapping("/offline/{id}")
    @RequiresPermissions("seo-domain-proposal:offline")
    public Result<Void> offline(@PathVariable @Valid @NotNull Integer id) {
        webBuildService.offline(id);
        return Result.success();
    }


    @Override
    @PostMapping("/query")
    @ApiOperation("根据RowQuery对象进行查询")
    @RequiresPermissions("seo-domain-proposal:query")
    public IPage<SeoDomainProposalRO> query(@RequestBody RowQuery rowQuery) {
        boolean hasAudit = OfficeUtils.hasPermission("seo-domain-proposal:audit");
        if(!hasAudit) {
            ControllerUtil.querySelf(rowQuery, OfficeUtils.shiroUserNotNull().getUsername());
        }
        return service.pageByRowQuery(rowQuery).convert(proposal -> ConvertKit.toBean(proposal, SeoDomainProposalRO::new));
    }

    @PostMapping("/audit")
    @ApiOperation("审核")
    @RequiresPermissions("seo-domain-proposal:audit")
    public Result<Void> audit(@RequestBody @Valid AuditParam param) {
        boolean success = webBuildService.audit(param.getId(), param.getRemark(), param.getPass(),  OfficeUtils.shiroUserNotNull().getUsername());
        return Result.whether(success);
    }

    @PostMapping("/audit/batch")
    @ApiOperation("审核")
    @RequiresPermissions("seo-domain-proposal:audit")
    public Result<?> batchAudit(@RequestBody @Valid BatchAuditParam param) {
        String loggedUsername = OfficeUtils.shiroUserNotNull().getUsername();
        Set<Integer> set = service.lockBatch(param.getIds(), loggedUsername);
        for (Integer id : set) {
            try {
                boolean success = webBuildService.audit(id, param.getRemark(), param.getPass(), loggedUsername);

                log.info("{} -> {}", id, success);

            } catch (Exception e) {
                log.info("异常, {}", e.getMessage());
                if(e instanceof NohaveArticleException) {
                    return Result.success().message("文章已经不够");
                } else {
                    log.error("", e);
                }
            }

        }
        return Result.success(set);
    }


    @PostMapping("/lock/{id}")
    @ApiOperation("锁定")
    @RequiresPermissions("seo-domain-proposal:lock")
    public Result<Void> lock(@PathVariable @Valid @NotNull Integer id) {
        boolean success = service.lock(id, OfficeUtils.shiroUserNotNull().getUsername());
        return Result.whether(success);
    }

    @PostMapping("/upload")
    @ApiOperation("上传excel文档")
    @RequiresPermissions("seo-domain-proposal:create")
    @Transactional(rollbackFor = Exception.class)
    public Result<Void> upload(@RequestParam("file") MultipartFile file) {
        List<SeoDomainProposalVO> list;
        try {
            Excel<SeoDomainProposalVO> excel = new Excel<>(file.getOriginalFilename(), file.getInputStream(), SeoDomainProposalVO.class);
            list = excel.read();
        } catch (Exception e) {
            throw new BusinessException(-1, "读取错误", e);
        }

        List<SeoTemplate> templates = templateService.random(list.size());
        List<SeoDomainProposal> proposals = new ArrayList<>();
        for (int i = 0; i < list.size(); i++) {
            SeoDomainProposalVO vo = list.get(i);
            if(StringKit.isBlank(vo.getTitle())) {
                continue;
            }
            SeoTemplate template = templates.get(i);
            SeoDomainProposal proposal = ConvertKit.toBean(vo, SeoDomainProposal::new);
            proposal.setReference(ReferenceGen.gen("SDP"));
            proposal.setState(ProposalStatus.CREATED)
                    .setType(valueOf(vo.getType()))
                    .setTemplateWebsite(template.getWebsite());
            proposals.add(proposal);
            template.setUseTimes(template.getUseTimes() + 1);
        }

        service.saveNotContains(SeoDomainProposal::getDomain, proposals);
        templateService.updateBatchById(new HashSet<>(templates));
        return Result.success();
    }

    @PostMapping("/upload/edit")
    @ApiOperation("上传excel文档")
    @RequiresPermissions("seo-domain-proposal:update")
    @Transactional(rollbackFor = Exception.class)
    public Result<Void> uploadEdit(@RequestParam("file") MultipartFile file) {
        List<SeoDomainProposalVO> list;
        try {
            Excel<SeoDomainProposalVO> excel = new Excel<>(file.getOriginalFilename(), file.getInputStream(), SeoDomainProposalVO.class);
            list = excel.read();
        } catch (Exception e) {
            throw new BusinessException(-1, "读取错误", e);
        }

        for (SeoDomainProposalVO vo : list) {
            service.findByDomain(vo.getDomain()).ifPresent(proposal -> {
                proposal.setTitle(vo.getTitle())
                        .setKeywords(vo.getKeywords())
                        .setListPageTitle(vo.getListPageTitle())
                        .setMainKey(vo.getMainKey())
                        .setDescription(vo.getDescription());
                service.updateById(proposal);
            });
            seoDomainService.findByDomain(vo.getDomain()).ifPresent(domain -> {
                domain.setTitle(vo.getTitle())
                        .setKeywords(vo.getKeywords())
                        .setMainKey(vo.getMainKey())
                        .setDescription(vo.getDescription());
                updateListTitle(domain, vo.getListPageTitle());
                seoDomainService.updateById(domain);
            });
        }
        return Result.success();
    }

    @PostMapping("/upload/delete")
    @ApiOperation("上传excel文档")
    @RequiresPermissions("seo-domain-proposal:delete")
    @Transactional(rollbackFor = Exception.class)
    public Result<Void> uploadDelete(@RequestParam("file") MultipartFile file) {
        try {
            Set<String> domains = IOKit.readLines(file.getInputStream(), "utf-8", new HashSet<>())
                    .stream().filter(StringKit::isNotEmpty).collect(Collectors.toSet());

            deleteDomains(domains);
            return Result.success();
        } catch (Exception e) {
            log.error("", e);
            throw new SystemException("", e);
        }
    }

    @PostMapping("/upload/deleteOther")
    @ApiOperation("上传excel文档")
    @RequiresPermissions("seo-domain-proposal:delete")
    @Transactional(rollbackFor = Exception.class)
    public Result<Void> uploadDeleteOther(@RequestParam("file") MultipartFile file) {
        try {
            Set<String> domains = IOKit.readLines(file.getInputStream(), "utf-8", new HashSet<>())
                    .stream().filter(StringKit::isNotEmpty).collect(Collectors.toSet());

            domains = service.lambdaQuery().select(SeoDomainProposal::getDomain)
                    .ne(SeoDomainProposal::getDomain, domains)
                    .list()
                    .stream()
                    .map(SeoDomainProposal::getDomain)
                    .collect(Collectors.toSet());
            deleteDomains(domains);
            return Result.success();
        } catch (Exception e) {
            log.error("", e);
            throw new SystemException("", e);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void deleteDomains(Set<String> domains) {
        CollectionKit.split(domains, 1000, HashSet::new).forEach(ds -> {
            service.lambdaUpdate().in(SeoDomainProposal::getDomain, ds).remove();
            seoDomainService.lambdaUpdate().in(SeoDomain::getDomain, ds).remove();
            acmeOrderService.lambdaUpdate().in(AcmeOrder::getDomain, ds).remove();
        });

        this.sprocketService.lambdaUpdate().eq(Sprocket::getRing, true).remove();
    }

    private void updateListTitle(SeoDomain domain, String title) {
        JSONArray json = JSONArray.parseArray(domain.getMenus());
        String listPageDir = domain.getDir();
        for(int i = 0; i < json.size(); i++) {
            JSONObject obj = json.getJSONObject(i);
            if(obj.getString("pinyin").equalsIgnoreCase(listPageDir)) {
                obj.put("title", title);
            }
        }

        domain.setMenus(json.toJSONString());
    }

    @PutMapping({"/create"})
    @ApiOperation("创建一条数据")
    @RequiresPermissions("seo-domain-proposal:create")
    @Override
    public Result<SeoDomainProposalRO> create(SeoDomainProposalCO co) {
        SeoDomainProposal proposal = ConvertKit.toBean(co, SeoDomainProposal::new);
        proposal.setReference(ReferenceGen.gen("SDP"));
        return Result.whether(service.save(proposal));
    }

    @ApiOperation("修改模板")
    @Transactional
    @PostMapping("/change-template/{groupId}")
    @RequiresPermissions("seo-domain-proposal:update")
    public Result<Void> changeTemplate(@PathVariable @Valid @NotNull Integer groupId,
                                       @RequestBody @Valid IdsParam param) {
        List<SeoDomainProposal> proposals = service.lambdaQuery().in(SeoDomainProposal::getId, param.getIds()).list();
        if(null != groupId && groupId > 0) {
            templateGroupService.findById(groupId).orElseThrow(() -> new BusinessException(-1, "分组不存在"));
        }

        List<SeoTemplate> templates = templateService.random(groupId, proposals.size());

        Map<String, Integer> old = new HashMap<>();
        for (int i = 0; i < proposals.size(); i++) {
            SeoDomainProposal proposal = proposals.get(i);

            //记录被移除的 模板，并记录移除次数
            old.put(proposal.getTemplateWebsite(), old.getOrDefault(proposal.getTemplateWebsite(), 0) + 1);

            SeoTemplate template = templates.get(i);
            proposal.setTemplateWebsite(template.getWebsite());

            // 给使用的模板次数 + 1
            template.setUseTimes(template.getUseTimes() + 1);
        }

        service.updateBatchById(proposals);
        templateService.updateBatchById(new HashSet<>(templates));

        //找到被移除的模板，并减去移除次数
        List<SeoTemplate> olds = templateService.findByWebsites(old.keySet());
        olds.forEach(seoTemplate -> {
            seoTemplate.setUseTimes(seoTemplate.getUseTimes() - old.get(seoTemplate.getWebsite()));
        });
        templateService.updateBatchById(olds);

        return Result.success();
    }



//  @PostMapping("/upload")
//  @ApiOperation("上传excel文档")
//  @RequiresPermissions("seo-domain-proposal:create")
//  @Transactional
//  public Result upload(@RequestParam("file") MultipartFile file) {
//    List<SeoDomainProposalVO> list;
//    try {
//      Excel<SeoDomainProposalVO> excel = new Excel<>(file.getName(), file.getInputStream(), SeoDomainProposalVO.class);
//      list = excel.read();
//    } catch (Exception e) {
//      throw new SystemWarnException(-1, "读取错误", e);
//    }
//
//    // 获取group name -> id 的map
//    List<String> groupNames = list.stream().map(SeoDomainProposalVO::getGroupName).distinct().collect(Collectors.toList());
//    Map<String, Integer> groupNameIdMap = templateGroupService.checkAndFind(groupNames).stream()
//            .collect(Collectors.toMap(TemplateGroup::getGroup, TemplateGroup::getId));
//
//    // 转成 groupId -> 数量 map
//    Map<Integer, Integer> groupCountMap = list.stream()
//            .collect(Collectors.toMap(o -> groupNameIdMap.get(o.getGroupName()), seoDomainProposalVO -> 1, Integer::sum));
//
//    // 传承 groupId -> 模板 map
//    Map<Integer, List<SeoTemplate>> groupTemplateMap = templateService.findByGroupCountMap(groupCountMap);
//
//
//    List<SeoDomainProposal> proposals = list.stream().map(vo -> {
//      SeoDomainProposal proposal = BeanKit.convertObj(vo, SeoDomainProposal::new);
//
//      SeoTemplate template = groupTemplateMap.get(groupNameIdMap.get(vo.getGroupName())).remove(0);
//      proposal.setState(ProposalStatus.CREATED)
//              .setType(valueOf(vo.getType()))
//              .setTemplateWebsite(template.getWebsite());
//      return proposal;
//    }).collect(Collectors.toList());
//
//    service.saveBatch(proposals);
//
//
//    List<SeoTemplate> needSave = groupTemplateMap.values().stream()
//            .flatMap(Collection::stream)
//            .map(seoTemplate -> seoTemplate.setUseTimes(seoTemplate.getUseTimes() + 1))
//            .distinct()
//            .collect(Collectors.toList());
//    templateService.saveBatch(needSave);
//
//    return Result.success();
//  }

    private DomainType valueOf(String type) {
        if("体育".equals(type)) {
            return DomainType.PHYSICAL;
        } else if("新闻".equals(type)){
            return DomainType.NEWS;
        } else if("电竞".equals(type)){
            return DomainType.GAMING;
        } else if("其他".equals(type)){
            return DomainType.OTHERS;
        } else {
            return DomainType.NEWS;
        }
    }




//  @PostMapping("/query")
//  @ResponseBody
//  @ApiOperation("根据RowQuery对象进行查询")
//  @RequiresPermissions("seo-domain-proposal:view")
//  @Override
//  public IPage<SeoDomainProposal> query(@RequestBody RowQuery rowQuery) {
//    boolean hasAudit = super.hasPermission(":audit");
//    if(!hasAudit) {
//      TextFilter filter = new TextFilter();
//      filter.setField("creator");
//      filter.setOption("equals");
//      filter.setText(getLoginUser().getUsername());
//      RowQuery.Filter[] fs = CollectionKit.append(rowQuery.getFilters(), filter);
//      rowQuery.setFilters(fs);
//    }
//    return service.pageByRowQuery(rowQuery);
//
//  }

    @Override
    public ISeoDomainProposalService service() {
        return service;
    }

    @Override
    public String basePermission() {
        return "seo-domain-proposal:";
    }

}

