package com.chuang.urras.sr.admin.task.acme;

import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.support.exception.SystemException;
import com.chuang.tauceti.support.vg.OperatorManager;
import com.chuang.urras.common.events.OfficeMsgEvent;
import com.chuang.urras.common.events.RedisEventManager;
import com.chuang.urras.sr.core.cache.CacheManager;
import com.chuang.urras.sr.core.entity.SeoDomain;
import com.chuang.urras.sr.core.entity.Server;
import com.chuang.urras.sr.core.service.single.ISeoDomainService;
import com.chuang.urras.sr.core.service.single.IServerService;
import com.chuang.urras.sr.core.ssh.Ssh2;
import lombok.extern.slf4j.Slf4j;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.scheduling.quartz.QuartzJobBean;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.Collection;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import static java.util.stream.Collectors.*;

@Slf4j
@Component
public class PublishSSLTask extends QuartzJobBean {

    private static final String OPERATOR = "PublishSslTask";
    @Resource private OperatorManager operatorManager;

    @Resource private RedisEventManager redisEventManager;
    @Resource private CacheManager cacheManager;
    @Resource private ISeoDomainService domainService;
    @Resource private IServerService serverService;

    private final Map<String, Ssh2> serverSshMap = new ConcurrentHashMap<>();
    private volatile boolean pushing = false;

    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {
        operatorManager.temp(OPERATOR, this::push);
    }

//    @Scheduled(cron = "0 */1 * * * ?")
    public void push() {
        try {
            int size = cacheManager.getPublishDomains().size();
            log.info("开始推送证书,  剩下: " + size);
            if (pushing || 0 == size) {
                return;
            }

            pushing = true;

            Set<String> domains = cacheManager.getPublishDomains().removeRandom(1000);
            this.doPushSsl(domains);

            pushing = false;
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }


    private void doPushSsl(Collection<String> domains) {
        log.info("domains: {}", domains.size());
        Map<String, Set<String>> group = domainService.lambdaQuery().select(SeoDomain::getDomain, SeoDomain::getServerTag)
                .in(SeoDomain::getDomain, domains)
                .list().stream()
                .collect(groupingBy(SeoDomain::getServerTag, mapping(SeoDomain::getDomain, toSet())));

        group.forEach((s, dms) -> {
            Ssh2 ssh2 = getOrCreate(s);
            dms.forEach(domain -> {
                try {
                    File[] files = new File("/opt/acme/generic/" + domain).listFiles();
                    ssh2.upload("/opt/acme/" + domain + "/", files);
                    log.info("{}传输完成", domain);
                    redisEventManager.fireAdminMsgEvent(new OfficeMsgEvent(domain, OfficeMsgEvent.MsgType.SUCCESS, "[" + s + "]->[" + domain + "]传输成功"));
                } catch (Exception e) {
                    log.error("推送失败", e);
                    redisEventManager.fireAdminMsgEvent(new OfficeMsgEvent(domain, OfficeMsgEvent.MsgType.ERROR, "[" + s + "]->[" + domain + "]传输失败"));
                }
            });

        });
    }


    private Ssh2 getOrCreate(String serverTag) {
        return this.serverSshMap.computeIfAbsent(serverTag, tag -> {
            Server server = serverService.findByServerName(tag)
                    .orElseThrow(() -> new BusinessException("无法找到创建{}服务器", tag));

            return new Ssh2(server, StandardCharsets.UTF_8, (ssh2, throwable) -> {
                log.warn("连接断开，准备重连", throwable);
                try {
                    ssh2.reConnect();
                } catch (IOException e) {
                    log.error("重连失败");
                    throw new SystemException("重连失败", e);
                }
            });
        });
    }


}
