package com.chuang.urras.sr.admin.task.article;

import com.chuang.tauceti.support.SystemConfigPolicy;
import com.chuang.tauceti.support.vg.OperatorManager;
import com.chuang.tauceti.tools.basic.reflect.ConvertKit;
import com.chuang.urras.common.events.OfficeMsgEvent;
import com.chuang.urras.common.events.RedisEventManager;
import com.chuang.urras.sr.core.SystemKeys;
import com.chuang.urras.sr.core.entity.Article;
import com.chuang.urras.sr.core.entity.SeoDomain;
import com.chuang.urras.sr.core.enums.DomainType;
import com.chuang.urras.sr.core.service.single.IArticleService;
import com.chuang.urras.sr.core.service.single.ISeoDomainService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.RandomUtils;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.scheduling.quartz.QuartzJobBean;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Component
@Slf4j
public class ArticlePublishTasks extends QuartzJobBean {

    private static final String OPERATOR = "Auto-Add-Liquidity";
    @Resource private OperatorManager operatorManager;

    @Resource private SystemConfigPolicy systemConfigPolicy;
    @Resource private IArticleService articleService;
    @Resource private ISeoDomainService domainService;
    @Resource private RedisEventManager redisEventManager;

    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {

    }

//    @Scheduled(cron = "0 */10 * * * ?")
    public void article() {
        log.info("开始自动发布任务");
        boolean enable = systemConfigPolicy.getBoolean(SystemKeys.ENABLE_AUTO_UPDATE_ARTICLE).orElse(SystemKeys.ENABLE_AUTO_UPDATE_ARTICLE_DEFAULT);

        if(!enable) {
            log.info("文章更新任务启动，但更新行为关闭，不做任何事情");
            return;
        }

        try {
            int newSitePublishDays = systemConfigPolicy.getInteger(SystemKeys.NEW_SITE_PUBLISH_DAYS).orElse(SystemKeys.NEW_SITE_PUBLISH_DAYS_DEFAULT);
            int oldSitePublishDays = systemConfigPolicy.getInteger(SystemKeys.OLD_SITE_PUBLISH_DAYS).orElse(SystemKeys.OLD_SITE_PUBLISH_DAYS_DEFAULT);
            log.info("分配新站文章");
            redisEventManager.fireAdminMsgEvent(new OfficeMsgEvent("star-ring-task", OfficeMsgEvent.MsgType.INFO, "开始分配新站文章"));
            updateArticle(newSitePublishDays, true);
            log.info("分配旧站文章");
            redisEventManager.fireAdminMsgEvent(new OfficeMsgEvent("star-ring-task", OfficeMsgEvent.MsgType.INFO, "开始分配旧站文章"));
            updateArticle(oldSitePublishDays, false);
        } catch (Exception e) {
            log.error("", e);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void updateArticle(int updateDays, boolean isNew){
        updateDays--;
        if (updateDays < 0) {
            log.warn("更新天数小于1，不更新");
            return;
        }

        boolean global = systemConfigPolicy.getBoolean(SystemKeys.ENABLE_GLOBAL_ARTICLE).orElse(SystemKeys.ENABLE_GLOBAL_ARTICLE_DEFAULT);
        int globalNum = systemConfigPolicy.getInteger(SystemKeys.GLOBAL_ARTICLE_NUM).orElse(SystemKeys.GLOBAL_ARTICLE_NUM_DEFAULT);

        LocalDateTime subTimes = LocalDate.now().atTime(0, 0, 0).plusDays(-updateDays);
        LocalTime now = LocalTime.now();

        List<SeoDomain> domainList = domainService.lambdaQuery()
                .eq(SeoDomain::getNewSite, isNew)
                .ge(SeoDomain::getDailyArticle, 0)
                //小于天数的0点
                .lt(SeoDomain::getLastArticleTime, subTimes)
                // 小于当前时间，说明还没执行
                .lt(SeoDomain::getArticleTime, now)
                .list();

        Map<DomainType, List<SeoDomainTaskVO>> map = domainList.stream().map(seoDomain ->
                ConvertKit.toBean(seoDomain, SeoDomainTaskVO::new).setNum(RandomUtils.nextInt(1, (global ? globalNum : seoDomain.getDailyArticle()) + 1))
        ).collect(Collectors.groupingBy(SeoDomainTaskVO::getType));

        map.forEach((domainType, vos) -> {
            List<Article> articles = articleService.findNotUsed(domainType, count(vos));
            int idx = 0;
            for (SeoDomainTaskVO vo : vos) {
                for (int i = 0; i < vo.getNum(); i++, idx++) {
                    Article article = articles.get(idx);
                    article.setUsed(true)
                            .setMenu(false)
                            .setOwner(vo.getDomain());
                }
                vo.setLastArticleTime(LocalDateTime.now());
                redisEventManager.fireAdminMsgEvent(new OfficeMsgEvent("star-ring-task", OfficeMsgEvent.MsgType.INFO, vo.getDomain() + "发布了" + vo.getNum() + "篇文章"));
            }
            domainService.updateBatchById(ConvertKit.toBeans(vos, SeoDomain::new));
            articleService.updateBatchById(articles);

        });
    }

    @Scheduled(cron = "0 0 */1 * * ?")
    public void updateNewSite() {
        // 查找所有建站时间超过 配置时间的 站，将其改为旧站
        int days = systemConfigPolicy.getInteger(SystemKeys.NEW_SITE_DAY).orElse(SystemKeys.NEW_SITE_DAY_DEFAULT);

        LocalDateTime time = LocalDateTime.now().plusDays(-days);

        List<SeoDomain> domainList =
                domainService.lambdaQuery().eq(SeoDomain::getNewSite, true).lt(SeoDomain::getCreatedTime, time).list();
        domainList.forEach(seoDomain -> seoDomain.setNewSite(false));

        domainService.updateBatchById(domainList);
    }

    private int count(List<SeoDomainTaskVO> vos) {
        return vos.stream().mapToInt(SeoDomainTaskVO::getNum).sum();
    }
}
