package com.chuang.urras.sr.core.acme;

import com.chuang.urras.common.events.OfficeMsgEvent;
import com.chuang.urras.common.events.RedisEventManager;
import com.chuang.urras.sr.core.cache.CacheManager;
import com.chuang.urras.sr.core.entity.AcmeAccount;
import com.chuang.urras.sr.core.entity.AcmeOrder;
import com.chuang.urras.sr.core.enums.AcmeOrderStatus;
import com.chuang.urras.sr.core.service.single.IAcmeAccountService;
import com.chuang.urras.sr.core.service.single.IAcmeOrderService;
import com.chuang.tauceti.acme.v2.*;
import com.chuang.tauceti.acme.v2.connection.Connection;
import com.chuang.tauceti.acme.v2.exception.AcmeException;
import com.chuang.tauceti.acme.v2.store.AcmeStore;
import com.chuang.tauceti.acme.v2.store.FileStore;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.support.exception.SystemException;
import com.chuang.tauceti.tools.basic.FutureKit;
import com.chuang.tauceti.tools.basic.StringKit;
import com.chuang.tauceti.tools.basic.collection.CollectionKit;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.CompletionException;
import java.util.concurrent.TimeUnit;
import java.util.function.Consumer;
import java.util.stream.Collectors;

@Slf4j
@Component
public class AcmeManager {


    @Resource private AcmeProperties properties;
    @Resource private AcmeStore store;
    @Resource private IAcmeAccountService accountService;
    @Resource private IAcmeOrderService orderService;
    @Resource private RedisEventManager redisEventManager;
    @Resource private CacheManager cacheManager;

    /**
     * 创建账号
     */
    public CompletableFuture<AcmeAccount> createAccount(String account, String email, String remark) {
        return createAccount(account, email)
                .exceptionally(this::acmeError)
                .thenApply(acc -> {
                    AcmeAccount acmeAccount = new AcmeAccount()
                            .setAccount(account)
                            .setContactEmail(email)
                            .setInitialIp(acc.getInitialIp())
                            .setLocation(acc.getLocation())
                            .setRemark(remark)
                            .setProvider(acc.getSession().getProvider().name())

                            .setStatus(acc.getStatus());
                    accountService.save(acmeAccount);
                    return acmeAccount;
                });
    }

    /**
     * 申请证书
     */
    public CompletableFuture<AcmeOrder> apply(String account, String orderReference, String domain, String... secs) {
        Optional<AcmeOrder> orderOpt = orderService.findByReference(orderReference);
        if(orderOpt.isPresent()) {
            String msg = "[" +domain + "]这个域名的订单已经存在, 是否为更新证书:[否], 是否成功:[否]";
            redisEventManager.fireAdminMsgEvent(new OfficeMsgEvent("acme-client", OfficeMsgEvent.MsgType.ERROR, msg));
            return FutureKit.error(new BusinessException("这个域名的订单已经存在"));
        }

        AcmeOrder order = new AcmeOrder()
                .setAccount(account)
                .setOrderReference(orderReference)
                .setDomain(domain)
                .setProvider("")
                .setStatus(AcmeOrderStatus.PENDING)
                .setSec(StringKit.join(secs, ","));
        orderService.save(order);

        Connection connection = newConnection();
        return connection.directory()
                .thenCompose(unused -> connection.nonce())
                .thenCompose(nonce -> apply(connection, order, false));
    }

    public CompletableFuture<List<AcmeOrder>> batchApply(Collection<AcmeOrder> orders, boolean renew) {
        Connection connection = newConnection();
        return connection.directory()
                .thenCompose(unused -> connection.nonce())
                .thenCompose(nonce -> FutureKit.oneByOne(orders, order -> apply(connection, order, renew), throwable -> null));
    }


    public CompletableFuture<AcmeOrder> reApply(String reference, List<String> secs) {
        Optional<AcmeOrder> orderOpt = orderService.findByReference(reference);
        if(!orderOpt.isPresent()) {
            String msg = "[" + reference + "]这个域名的订单不存在, 是否为更新证书:[是], 是否成功:[否]";
            redisEventManager.fireAdminMsgEvent(new OfficeMsgEvent("acme-client", OfficeMsgEvent.MsgType.ERROR, msg));
            return FutureKit.error(new BusinessException("这个域名的订单不存在"));
        }

        AcmeOrder order = orderOpt.get();

        if(CollectionKit.isNotEmpty(secs)) {
            order.setSec(StringKit.join(secs, ","));
        }
        Connection connection = newConnection();

        return connection.directory()
                .thenCompose(unused -> connection.nonce())
                .thenCompose(nonce -> apply(connection, order, true));
    }

    private CompletableFuture<AcmeOrder> apply(Connection conn, AcmeOrder order, boolean renew) {

        waiting();

        order.setStatus(renew ? AcmeOrderStatus.RE_PENDING : AcmeOrderStatus.PENDING);
        String[] secArr = order.getSec().split(",");
        order.setProvider(conn.getSession().getProvider().name());
        orderService.updateById(order);

        return apply(conn, order.getAccount(), order.getOrderReference(), order.getDomain(), renew, orderStatus -> {
            order.setStatus(AcmeOrderStatus.parse(orderStatus));
            orderService.updateById(order);
        }, Arrays.asList(secArr))
                .thenApply(certificate -> {
                    order.setLastRenewTime(LocalDateTime.now());
                    orderService.updateById(order);
                    return order;
                })
                .exceptionally(throwable -> {
                    log.error(throwable.getMessage(), throwable);
                    order.setApplyMsg(throwable.getMessage());
                    orderService.updateById(order);
                    return acmeError(throwable);
                });
    }

    /**
     * 最终
     */
    private CompletableFuture<Certificate> apply(Connection conn, String account, String orderId, String domain, boolean renew, Consumer<String> updateStatus, Collection<String> sec) {
        Optional<Account> accOpt = conn.loadAccount(account);
        if (!accOpt.isPresent()) {
            AcmeStore store = conn.getSession().getStore();
            if(store instanceof FileStore) {
                String path = ((FileStore) store).basePath() + "account/" + account + "_info.json";
                String msg = "[" + domain + "][" + account + "]找不到acme缓存，请联系管理员修复，地址为:[" + path + "]，是否为更新证书:[" + renew + "]";
                redisEventManager.fireAdminMsgEvent(new OfficeMsgEvent("acme-client", OfficeMsgEvent.MsgType.ERROR, msg));
            } else {
                String msg = "[" + domain + "][" + account + "]找不到acme缓存，请联系管理员修复，是否为更新证书:[" + renew + "]";
                redisEventManager.fireAdminMsgEvent(new OfficeMsgEvent("acme-client", OfficeMsgEvent.MsgType.ERROR, msg));
            }
            return FutureKit.error(new BusinessException(account + "找不到acme缓存，请联系管理员修复"));
        }
        Account acc = accOpt.get();
        try {
            return acc.newDnsOrder(orderId, domain, null, null, sec)
                    .thenCompose(Order::updateAuthOneByOne)
                    .thenApply(order -> {
                        updateStatus.accept(order.getStatus().name());
                        return order;
                    })
                    .thenCompose(this::updateChallenge)
                    .thenApply(order -> {
                        updateStatus.accept(order.getStatus().name());
                        return order;
                    })
                    .thenCompose(this::doFinalize)
                    .thenApply(certificate -> {
                        updateStatus.accept("downloaded");
                        String msg = "[" + domain + "][" + account + "]下载成功！，是否为更新证书:[" + renew + "]";
                        redisEventManager.fireAdminMsgEvent(new OfficeMsgEvent("acme-client", OfficeMsgEvent.MsgType.SUCCESS, msg));
                        return certificate;
                    }).exceptionally(throwable -> {
                        String msg = "[" + domain + "][" + account + "]下载失败！，是否为更新证书:[" + renew + "], msg -> " + throwable.getMessage();
                        redisEventManager.fireAdminMsgEvent(new OfficeMsgEvent("acme-client", OfficeMsgEvent.MsgType.ERROR, msg));
                        throw (CompletionException) throwable;
                    });
        } catch (Exception e) {
            String msg = "[" + domain + "][" + account + "]下载失败！，是否为更新证书:[" + renew + "], msg -> " + e.getMessage();
            redisEventManager.fireAdminMsgEvent(new OfficeMsgEvent("acme-client", OfficeMsgEvent.MsgType.ERROR, msg));
            return FutureKit.error(e);
        }
    }


    /**
     * 创建账号
     */
    private CompletableFuture<Account> createAccount(String account, String email) {
        Connection connection = newConnection();
        return connection.directory()
                .thenCompose(unused -> {
//                    cacheDirectory(connection.getSession());
                    return connection.loadOrNewAccount(account, false, "mailto: " + email);
                });
    }


    private CompletableFuture<Certificate> doFinalize(Order order) {
        try {
            return order.doFinalize().thenCompose(Certificate::download).thenApply(certificate -> {
                certificate.save();// 保存证书
                return certificate;
            });
        } catch (Exception e) {
            return FutureKit.error(e);
        }
    }

    private List<Challenge> http(Order order) {
        return order.getAuthorizations().stream()
                .map(Authorization::http)
                .filter(Optional::isPresent)
                .map(Optional::get)
//                .filter(challenge -> !challenge.getStatus().isDone())
                .collect(Collectors.toList());
    }

    private CompletableFuture<Order> updateChallenge(Order order) {
        List<Challenge> challenges = this.http(order);

        CompletableFuture<List<Challenge>> futures = FutureKit.oneByOne(challenges, this::updateChallenge, throwable -> null);

        return futures.thenApply(c -> order);
    }

    private CompletableFuture<Challenge> updateChallenge(Challenge challenge) {
        String thumbprint = challenge.getAuthorization().getOrder().getAccount().thumbprint();

        this.cacheManager.getThumbprintCache().put(challenge.getToken(), thumbprint, 10, TimeUnit.MINUTES);

        return challenge.update(properties.getChallengeWaiting(), properties.getChallengeRetry());
    }

    public Connection newConnection() {
        Session session = new Session(properties.getProviderUri(), store);
//        loadDirectory(session);
        return session.connect();
    }

    public Optional<String> getToken(String challengeToken) {
        String thumbprint = this.cacheManager.getThumbprintCache().getOrDefault(challengeToken, null);
        return Optional.ofNullable(thumbprint);
    }

    private <T> T acmeError(Throwable throwable) {
        if(throwable instanceof CompletionException) {
            if(throwable != throwable.getCause()) {
                return acmeError(throwable.getCause());
            } else {
                throw new SystemException("acme 错误：", throwable);
            }
        } else if(throwable instanceof AcmeException) {
            throw new BusinessException(throwable.getMessage());
        } else {
            throw new SystemException("acme 发送异常: ", throwable);
        }
    }

    private void waiting() {
        try {
            Thread.sleep(300);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }
}
