package com.chuang.urras.sr.core.service.polymer.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.chuang.urras.sr.core.Utils;
import com.chuang.urras.sr.core.entity.*;
import com.chuang.urras.sr.core.enums.PageType;
import com.chuang.urras.sr.core.enums.ProposalStatus;
import com.chuang.urras.sr.core.enums.TaskStatus;
import com.chuang.urras.sr.core.service.ISentenceArticleContentService;
import com.chuang.urras.sr.core.service.platform.IPushService;
import com.chuang.urras.sr.core.service.platform.PlatformManager;
import com.chuang.urras.sr.core.service.polymer.IWebBuildService;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.basic.StringKit;
import com.chuang.tauceti.tools.basic.collection.CollectionKit;
import com.chuang.urras.sr.core.service.single.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.RandomUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Service
@Slf4j
public class WebBuildServiceImpl implements IWebBuildService {

    private static final String[] SUFFIX = new String[] {
            "", ".html", ".php", ".asp", ".aspx", ".shtml", ".htm", ".shtm"
    };

    @Resource private ISeoDomainProposalService proposalService;
    @Resource private ISeoDomainService domainService;
    @Resource private ISeoTemplateService templateService;
    @Resource private IArticleService articleService;
    @Resource private IPushService pushService;
    @Resource private IPlatformCheckContentService platformCheckContentService;
    @Resource private IDownloadTaskService downloadTaskService;
    @Resource private ISentenceService sentenceService;
    @Resource private ITitleService titleService;
    @Resource private ISentenceArticleService sentenceArticleService;
    @Resource private ISentenceArticleContentService sentenceArticleContentService;

    private final List<String> sentenceCaches = new ArrayList<>();
    private static final int SENTENCE_CACHE_SIZE = 2000;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public synchronized boolean audit(Integer id, String remark, Boolean pass, String username) {
        SeoDomainProposal proposal = proposalService.findById(id)
                .orElseThrow(() -> new BusinessException(-1, "找不到这条记录"));
        if(proposal.getState() != ProposalStatus.RECEIPT) {
            throw new BusinessException(-1, "该记录没有锁定，无法操作");
        }
        if(!username.equals(proposal.getAuditor())) {
            throw new BusinessException(-1, "这个记录已经被别人锁定，你不能操作");
        }

        proposal.setAuditedTime(LocalDateTime.now())
                .setState(pass ? ProposalStatus.PASS : ProposalStatus.REJECT)
                .setRemark(remark);
        proposalService.updateById(proposal);

        if(pass) {
            // 建站
            domainService.findByDomain(proposal.getDomain()).ifPresent(seoDomain -> {
                throw new BusinessException(-1, "已经有一个相同域名的站被创建，本次创建中断");
            });

            //分配文章
            SeoTemplate template = templateService.findByWebsite(proposal.getTemplateWebsite())
                    .orElseThrow(() -> new BusinessException(-1, "模板不存在"));

            int max = Stream.of(
                    template.getIndexATagSize(),
                    template.getListATagSize(),
                    template.getArticleATagSize(),
                    template.getInterlinkSize()
            ).max(Integer::compareTo).orElse(0);


            String dir = Utils.converterToFirstSpell(proposal.getListPageTitle());
            String[] others = proposal.getOtherMenus().split(","); //其他菜单
            max = max + others.length;


            List<Article> articleList =  articleService.findNotUsed(proposal.getType(), max);

            articleList.forEach(article -> article.setUsed(true).setOwner(proposal.getDomain()));

            log.info("网站[{}] -> 模板[{}] -> 文章[{}] -> 查询 -> [{}]", proposal.getDomain(), template.getWebsite(), max, articleList.size());
            JSONArray menus = new JSONArray();

            // 添加list菜单
            JSONObject listJson = new JSONObject();
            listJson.put("title", proposal.getListPageTitle());
            listJson.put("pinyin", dir);
            menus.add(listJson);

            List<String> menuStrs = new ArrayList<>();
            //添加其他菜单
            for (int i = 0; i < others.length; i++) {
                Article article = articleList.get(i);
                article.setUsed(true)
                        .setMenu(true)
                        .setOwner(proposal.getDomain());

                String py = Utils.converterToFirstSpell(others[i]);
                JSONObject json = new JSONObject();
                json.put("title", others[i]);
                json.put("pinyin", py);
                json.put("id", article.getId());

                menuStrs.add(py);
                menus.add(json);
            }
            // 分配文章到 Atag
            for (int i = others.length; i < articleList.size(); i++) {
                Article article = articleList.get(i);
                article.setUsed(true)
                        .setMenu(false)
                        .setOwner(proposal.getDomain());
            }
            articleService.updateBatchById(articleList);

            SeoDomain domain = new SeoDomain()
                    .setDomain(proposal.getDomain())
                    .setDescription(proposal.getDescription())
                    .setKeywords(proposal.getKeywords())
                    .setMainKey(proposal.getMainKey())
                    .setSuffix(CollectionKit.randomOne(SUFFIX))
                    .setServerTag(proposal.getServerTag())
                    .setDir(dir)
                    .setDailyArticle(3)
                    .setNewSite(true)
                    .setProposalReference(proposal.getReference())
                    .setJsGroup("default")
                    .setSentenceDir(CollectionKit.randomOne(menuStrs))
                    .setArticleTime(LocalTime.of(RandomUtils.nextInt(8,13), RandomUtils.nextInt(0, 59), RandomUtils.nextInt(0, 59)))
                    .setLastArticleTime(LocalDateTime.now())
                    .setMenus(menus.toJSONString())
                    .setType(proposal.getType())
                    .setTemplateWebsite(proposal.getTemplateWebsite())
                    .setTitle(proposal.getTitle());
            domainService.save(domain);

            // 给domain添加句子文章
            this.genSentenceArticle(PageType.INDEX, domain.getDomain(), template.getIndexSentenceSize());
            this.genSentenceArticle(PageType.LIST, domain.getDomain(), template.getListSentenceSize());
            this.genSentenceArticle(PageType.ARTICLE, domain.getDomain(), template.getArticleSentenceSize());


            (((PlatformManager)pushService)).supports().forEach(platformType -> {
                platformCheckContentService.getOrCreate(platformType, domain.getDomain());
            });


        } else {
            // 如果审核拒绝，要减去模板使用次数
            String temp = proposal.getTemplateWebsite();
            templateService.findByWebsite(temp).ifPresent(seoTemplate -> {
                seoTemplate.setUseTimes(seoTemplate.getUseTimes() - 1);
                templateService.updateById(seoTemplate);
            });
            downloadTaskService.lambdaUpdate()
                    .set(DownloadTask::getState, TaskStatus.ENDED)
                    .set(DownloadTask::getFailTimes, 2000)
                    .eq(DownloadTask::getWebsite, proposal.getTemplateWebsite())
                    .eq(DownloadTask::getState, TaskStatus.ERROR)
                    .update();

        }
        return true;
    }

    private void genSentenceArticle(PageType type, String domain, int num) {
        if(num == 0) {
            return;
        }

        // 随机 num 个标题
        List<String> titles = titleService.random(num);

        // 创建句子文章
        List<SentenceArticle> sas = new ArrayList<>();
        for(int i = 0; i < num; i++) {
            sas.add(new SentenceArticle()
                    .setPage(type)
                    .setDomain(domain)
                    .setTitle(titles.get(i))
            );
        }
        // 批量保存获得id
        sentenceArticleService.saveBatch(sas);

        // 生成内容，批量保存
        List<SentenceArticleContent> contents = sas.stream()
                .map(sentenceArticle -> new SentenceArticleContent()
                        .setSentenceArticleId(sentenceArticle.getId())
                        .setContent(sentenceArticle(5, 7, 4, 8)))
                .collect(Collectors.toList());

        sentenceArticleContentService.saveBatch(contents);
    }

    /**
     * 随机获取句子
     * @param min 最小
     * @param max 最大
     * @return 句子集合
     */
    private Collection<String> randomSentences(int min, int max) {
        if(this.sentenceCaches.size() < max) {
            sentenceCaches.addAll(sentenceService.random(SENTENCE_CACHE_SIZE));
        }
        Collection<String> c = CollectionKit.random(sentenceCaches, RandomUtils.nextInt(min, max));
        sentenceCaches.removeAll(c);

        return c;
    }

    /**
     * 随机生成一篇文章
     * @param pMin 一个段落最少要多少个句子
     * @param pMax 一个段落最多要多少个句子
     * @param aMin 一个文章最少要多少个句子
     * @param aMax 一个文章最多要多少个句子
     * @return 文章
     */
    private String sentenceArticle(int pMin, int pMax, int aMin, int aMax) {
        StringBuilder builder = new StringBuilder();
        int random = RandomUtils.nextInt(aMin, aMax);

        for(int i = 0; i < random; i++) {
            builder.append("<p>").append(StringKit.join(randomSentences(pMin, pMax), "")).append("</p>\n");
        }

        return builder.toString();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void offline(Integer id) {
        SeoDomainProposal proposal = proposalService.findById(id).orElseThrow(() -> new BusinessException(-1, "记录不存在"));

        String reference = proposal.getReference();
        SeoDomain domain = domainService.findByReference(reference).orElseThrow(() -> new BusinessException(-1, "网站不存在"));

        // 删掉站点
        domainService.removeById(domain.getId());

        // 删掉网站使用的文章
        List<Article> articles = articleService.findByOwner(domain.getDomain());
        articles.forEach(article -> article.setUsed(false).setOwner(domain.getDomain()));
        articleService.updateBatchById(articles);

        Set<Integer> ids = sentenceArticleService.findIdByDomain(domain.getDomain());
        sentenceArticleService.deleteByIds(ids);

        sentenceArticleContentService.deleteBySentenceIds(ids);

        // 审核状态变更
        proposal.setState(ProposalStatus.CREATED);
        proposalService.updateById(proposal);



    }

}
