package com.chuang.urras.sr.core.ssh;

import ch.ethz.ssh2.*;
import com.chuang.tauceti.support.Result;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.support.exception.SystemException;
import com.chuang.tauceti.tools.basic.FileKit;
import com.chuang.tauceti.tools.basic.IOKit;
import com.chuang.tauceti.tools.basic.StringKit;
import com.chuang.tauceti.tools.basic.collection.CollectionKit;
import com.chuang.urras.sr.core.entity.Server;
import lombok.extern.slf4j.Slf4j;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.charset.Charset;
import java.util.Arrays;
import java.util.Collection;
import java.util.function.BiConsumer;
import java.util.stream.Collectors;

@Slf4j
public class Ssh2 {

    private final Server server;
    private final Charset charset;
    private final BiConsumer<Ssh2, Throwable> connectionMonitor;
    private Connection connection;

    public Ssh2(Server server, Charset charset, BiConsumer<Ssh2, Throwable> connectionMonitor) {
        this.server = server;
        this.charset = charset;
        this.connectionMonitor = connectionMonitor;
        init();
    }

    private void init() {
        this.connection = new Connection(server.getIntranetIp(), server.getSshPort());
        connection.addConnectionMonitor(reason -> connectionMonitor.accept(this, reason));
        try {
            connection.connect();
            boolean auth = connection.authenticateWithPassword(server.getServerUser(), server.getServerPass());
            log.info(auth ? "认证成功":"认证失败");
        } catch (IOException e) {
            log.error("认证异常", e);
        }
    }

    public Result<String> command(String cmd) {
        try {
            Session session = this.connection.openSession();

            session.execCommand(cmd);

            String r = IOKit.read(new StreamGobbler(session.getStdout()), charset);
            boolean success = true;
            if(StringKit.isBlank(r)) {
                r = IOKit.read(new StreamGobbler(session.getStderr()), charset);
                success = false;
            }

            closeSession(session);

            Result<String> result = Result.whether(success);
            return result.data(r);
        } catch (Exception e) {
            log.error("cmd: {}执行失败", cmd, e);
            throw new BusinessException("执行远程命令失败", e);
        }
    }

    public void download(String localDir, String... remoteFilePathAbs) {
        // 如果传参为空, 则返回
        if (remoteFilePathAbs == null) {
            return;
        }

        try {
            SCPClient client = connection.createSCPClient();
            for(String file: remoteFilePathAbs) {
                SCPInputStream input = client.get(file);
                FileKit.writeFromStream(input, localDir);
                IOKit.close(input);
            }
        } catch (IOException e) {
            throw new SystemException("Scp远程下载失败", e);
        }
    }

    public void downloadByPattern(String localDir, String remoteDir, String fileNamePattern) {
        Result<String> result = command("ls " + remoteDir + "/" + fileNamePattern);
        if(result.isSuccess() && StringKit.isNotBlank(result.getData())) {
            String[] files = result.getData().split("\n");
            download(localDir, files);
        }
    }

    public void upload(String remoteDir, String... localFile) {
        upload(remoteDir, Arrays.stream(localFile).map(File::new).collect(Collectors.toList()));
    }

    public void upload(String remoteDir, File... localFile) {
        upload(remoteDir, Arrays.asList(localFile));
    }

    public void upload(String remoteDir, Collection<File> localFile) {
        // 如果传参为空, 则返回
        if (CollectionKit.isEmpty(localFile)) return;

        try {
            command("mkdir -p " + remoteDir);

            SCPClient client = this.connection.createSCPClient();
            for(File file: localFile) {
                FileInputStream input = new FileInputStream(file);
                SCPOutputStream output = client.put(file.getName(), file.length(), remoteDir, null);
                IOKit.copy(input, output);
                IOKit.close(input);
                IOKit.close(output);
            }
        } catch (IOException e) {
            log.error("Scp上传失败", e);
            throw new SystemException("Scp远程上传失败", e);
        }
    }

    public void reConnect() throws IOException {
        this.connection.connect();
    }

    private void closeSession(Session session) {
        IOKit.close(session.getStdout());
        IOKit.close(session.getStderr());
        IOKit.close(session.getStderr());
        session.close();
    }

}
