package com.chuang.urras.sr.restapp.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.chuang.eden.template.fetch.WebsiteMapping;
import com.chuang.eden.template.fetch.properties.FetchProperties;
import com.chuang.tauceti.support.SystemConfigPolicy;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.basic.*;
import com.chuang.tauceti.tools.basic.collection.CollectionKit;
import com.chuang.tauceti.tools.third.servlet.HttpKit;
import com.chuang.urras.sr.core.ProcessUtils;
import com.chuang.urras.sr.core.Utils;
import com.chuang.urras.sr.core.entity.SeoDomain;
import com.chuang.urras.sr.core.entity.SeoTemplate;
import com.chuang.urras.sr.core.enums.PageType;
import com.chuang.urras.sr.core.service.single.IArticleService;
import com.chuang.urras.sr.core.service.single.ISentenceArticleService;
import com.chuang.urras.sr.restapp.model.Item;
import com.chuang.urras.sr.restapp.model.MenuItem;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
public class BaseController {
    @Resource protected HttpServletRequest request;
    @Resource protected HttpServletResponse response;
    @Resource protected IArticleService articleService;
    @Resource protected SystemConfigPolicy systemSettingPolicy;
    @Resource private FetchProperties fetchProperties;
    @Resource private WebsiteMapping websiteMapping;
    @Resource private ISentenceArticleService sentenceArticleService;

    @Value("${master.ip}") private String masterIp;
    @Value("${master.port}") private String masterPort;
    @Value("${master.key}") private String masterKey;
    @Value("${server.master}") private Boolean master;

    private final Set<String> downloading = new HashSet<>();

    protected String getServer() {
        try {
            String domain = URLKit.getDomainName("http://" + request.getServerName());
            if(RegexKit.isTrue(RegexKit.regExp_ip, domain)) {
                throw new BusinessException(404, "没有绑定域名");
            }
            return domain;
        } catch (Exception e) {
            throw new BusinessException(-1, "系统无法解析域名");
        }
    }

    /**
     * 生成菜单
     */
    protected List<MenuItem> menus(SeoDomain domain) {
        JSONArray menus = JSONArray.parseArray(domain.getMenus());

        List<MenuItem> list = new ArrayList<>();
        list.add(new MenuItem().setName("首页").setHref("/"));

        for (int i = 0; i < menus.size(); i++) {
            JSONObject json = menus.getJSONObject(i);
            MenuItem item = new MenuItem();
            item.setName(json.getString("title"))
                    .setHref("/" + json.getString("pinyin"));
            list.add(item);
        }
        return list;
    }

    /**
     * 菜单标题， 一般菜单页面都需要在title上加上他们的名称
     * @param domain 网站信息
     * @param key key
     * @return 标题，
     */
    protected String getMenuTitle(SeoDomain domain, String key) {
        JSONArray ary = JSONArray.parseArray(domain.getMenus());
        for (int i = 0; i < ary.size(); i++) {
            JSONObject json = ary.getJSONObject(i);
            if(key.equalsIgnoreCase(json.getString("pinyin"))) {
                return json.getString("title");
            }
        }
        return "";
    }

    /**
     * 获取菜单id
     * @param domain 网站信息
     * @param menu 菜单名
     * @return -1 表示list页面，其他表示其他菜单。其他菜单项本质上也是文章页的一种。
     */
    protected int getMenuId(SeoDomain domain, String menu) {
        JSONArray array  =JSONArray.parseArray(domain.getMenus());
        for (int i = 0; i < array.size(); i++) {
            JSONObject obj = array.getJSONObject(i);
            if (menu.equalsIgnoreCase(obj.getString("pinyin"))) {
                if(obj.containsKey("id")) {
                    return obj.getIntValue("id");
                } else {
                    return -1;
                }
            }
        }

        log.info("导航页面不存在:{}", HttpKit.requestDebugText().orElse(""));
        throw new BusinessException(404, "导航页面不存在");
    }

    /**
     * 根据网站信息和 h标签的个数，随机生成N个关键词
     * @param domain 网站信息
     * @param hTagSize h标签数量
     */
    protected List<String> randomKeys(SeoDomain domain, int hTagSize) {
        String[] keys = domain.getKeywords().split(",");
        List<String> keySet = Arrays.stream(keys).map(String::trim).distinct().collect(Collectors.toList());
        Utils.full(keySet, hTagSize);
        return keySet;
    }

    /**
     * 计算出文章，有多少个连接就加载多少个文章
     * @param linkTagSize 连接数
     */
    protected List<Item> articles(SeoDomain domain, Integer linkTagSize) {

        List<Item> items =  articleService.findIdAndTitleByDomain(domain.getDomain(), false, linkTagSize).stream().map(article ->
                new Item()
                        .setHref("/" + domain.getDir() + "/" + article.getId() + domain.getSuffix())
                        .setTitle(article.getTitle())
        ).collect(Collectors.toList());
        if(items.size() < linkTagSize) {
            log.warn("{} 文章不足 {}，自动补全", domain.getDomain(), linkTagSize);
            Utils.full(items, linkTagSize);
        }

        return items;
    }

    protected List<Item> sentences(SeoDomain domain, PageType type, Integer linkTagSize) {
        if(linkTagSize == 0) {
            return Collections.emptyList();
        }
        boolean sentenceToJs = systemSettingPolicy.getBoolean("SENTENCE_TO_JS").orElse(false);

        List<Item> items = sentenceArticleService.findSentences(domain.getDomain(), type, linkTagSize)
                .stream().map(sentenceArticle -> {
                    Item item = new Item()
                            .setHref("/" + domain.getSentenceDir() + "/" + sentenceArticle.getId() + domain.getSuffix())
                            .setTitle(sentenceArticle.getTitle());

                    if(sentenceToJs) {
                        item.setHref("javascript:" + HashKit.md5(sentenceArticle.getId() + sentenceArticle.getTitle()).substring(8, 16) + "();");
                    }

                    return item;
                }).collect(Collectors.toList());

        if(items.size() < linkTagSize) {
            log.warn("{} 句子文章不足 {}，自动补全", domain.getDomain(), linkTagSize);
            Utils.full(items, linkTagSize);
        }

        return items;
    }

    /**
     * 计算出关键词
     */
    protected List<String> subKeys(SeoDomain domain, int keySize) {
        String[] keys = domain.getKeywords().split(",");
        List<String> keySet = Arrays.stream(keys).map(String::trim).distinct().collect(Collectors.toList());
        Utils.full(keySet, keySize);
        return keySet;
    }

    /**
     * 计算内链对象
     */
    protected List<Item> interlinks(SeoDomain domain, int beginId, SeoTemplate template) {
        int size = template.getInterlinkSize();
        if(size == 0) {
            return Collections.emptyList();
        }
        return articleService.findInterlink(domain.getDomain(), beginId, size).stream().map(article ->
                new Item()
                        .setHref("/" + domain.getDir() + "/" + article.getId() + domain.getSuffix())
                        .setTitle(article.getTitle())
        ).collect(Collectors.toList());
    }

    /**
     * 检查并下载模板，如果模板存在则什么也不做，如果模板不存在，则通过 命令行 向主机下载
     * @param domain 网站信息
     */
    protected void checkAndGetTemplate(SeoDomain domain) {
        String dir = fetchProperties.getSavePath() + "/" + websiteMapping.mapping(domain.getTemplateWebsite());
        if(checkListFileNames(dir)) {
            return;
        }
        if(master) {
            log.warn("【警告】无法找到{}路径,开始下载,但因为是master，不进行更新", dir);
            return;
        }

        log.info("无法找到{}路径,开始下载", dir);
        if(downloading.contains(domain.getDomain())) {
            log.info("{}正在更新中，不再更新", domain);
            throw new BusinessException(404, "模板正在下载中");
        }

        downloading.add(domain.getDomain());

        FileKit.mkdir(dir);
        long time = System.currentTimeMillis();
        String cmd = "sshpass -p " + masterKey + " scp -P " + masterPort + " -r root@" + masterIp + ":" + dir + " " + fetchProperties.getSavePath();
        int status = ProcessUtils.run(cmd, throwable -> log.error("下载模板失败", throwable));
        log.info("{}下载模板{},耗时[{}]毫秒", domain.getDomain(), status==0 ? "成功": "失败", System.currentTimeMillis() - time);

        downloading.remove(domain.getDomain());

    }

    private boolean checkListFileNames(String dir) {
        try {
            return FileKit.exist(dir) && CollectionKit.isNotEmpty(FileKit.listFileNames(dir));
        } catch (IOException e) {
            return false;
        }

    }


    /**
     * 是否是从搜索引擎跳转过来的
     */
    protected boolean fromSearchPlatform() {
        String referer = request.getHeader(HttpHeaders.REFERER);
        if(StringKit.isBlank(referer)) {
            return false;
        }
        if (RegexKit.isExist(".*(sogou|soso|baidu|bsb|youdao|lanjie|bing|118114|biso|sm|qq|so|safe|toutiao|biso|360).*", referer.toLowerCase())) {
            return true;
        }
        return false;
    }


}
