package com.chuang.urras.sr.restapp.controller;

import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.basic.StringKit;
import com.chuang.tauceti.tools.third.servlet.HttpKit;
import com.chuang.urras.sr.core.cache.CacheManager;
import com.chuang.urras.sr.core.enums.PlatformType;
import com.chuang.urras.sr.restapp.monitor.RestMetrics;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

@Slf4j
@Controller
public class ErrorController extends BaseController implements org.springframework.boot.web.servlet.error.ErrorController {


    @Resource private RestMetrics restMetrics;
    @Resource private CacheManager cacheManager;
    /**
     * 错误页面
     * 1、如果不是404，直接抛出异常
     * 2、如果是404，并且地址开头已经是星环地址，表示在星环中也找不到，直接跳到星环的404
     * 3、判断是否进入落地页
     * 4、记录蜘蛛，并进入星环页
     * 5、如果记录蜘蛛时发现蜘蛛不被支持，进入错误页面
     */
    @RequestMapping("/error")
    public String handleError(HttpServletRequest request){

        Integer statusCode = (Integer) request.getAttribute("javax.servlet.error.status_code");
        String path = (String) request.getAttribute("javax.servlet.forward.request_uri");

//        log.info("/error 中遇到 " + path + " > " + statusCode);

        if(null == statusCode) {
            throw new BusinessException(-1, "未知错误号");
        }
        if(404 != statusCode) { // 如果不是404 则直接跳转到星环错误页面
            restMetrics.addPathErrorCounter();
            log.warn("request 系统内部异常：" + HttpKit.requestDebugText());
            throw new BusinessException(-1, "系统内部异常!");
        }

        // 如果已经是星环页了，说明在星环页上也找不到页面，这时候直接跳真正的404
        if (path.startsWith(ApiController.STAR_RING_API_PATH)) {
            restMetrics.addPathErrorCounter();
//            log.info("============= ep > /404");
            throw new BusinessException(404, "找不到这个页面呢！！！！");
        }

        // 是否允许所有人查看页面
        boolean open = systemSettingPolicy.getBoolean("TMP_OPEN").orElse(true);
        // 如果不是蜘蛛,转到落地页
        if(!open && !HttpKit.isSpider()) {
//            log.info("============= ep > /landing");
            return forwardStarRing("/landing");
        }

        // 进入星环页面，进入前，先统计蜘蛛
        boolean isSupportSpider = addSpider();
        //如果系统支持这个蜘蛛，进入星环。否则进入错误页面
        if(isSupportSpider) {
            log.info("============= ep > /星环 > " + path);
            return forwardStarRing(path);
        } else {
            if(HttpKit.isSpider()) {
                restMetrics.addBlockSpiderCounter();
                throw new BusinessException(404, "不支持蜘蛛");
            } else {
                restMetrics.addPathErrorCounter();
                throw new BusinessException(404, "不支持路径");
            }
        }

    }

    private String forwardStarRing(String view) {
        // 这里是为了防止有后缀，现在使用的springboot版本已经部支持匹配后缀
        view = view.split("\\.")[0];

        response.setStatus(HttpStatus.OK.value());
        response.setCharacterEncoding("utf-8");
        response.setContentType("text/html;charset=utf-8");
        return "forward:" + ApiController.STAR_RING_API_PATH + view;
    }


    private boolean addSpider() {
        String ua = HttpKit.getRequest().map(request -> request.getHeader("User-Agent")).orElse("");
        if(StringKit.isBlank(ua)) {
            return false;
        }
        String lowcase = ua.toLowerCase();
        if(lowcase.contains("ignore")) {
            return true;
        }

        if(lowcase.contains("baidu")) {
            restMetrics.addSpider(PlatformType.BAIDU);
            cacheManager.addSpider(PlatformType.BAIDU.name());
            return true;
        } else if(lowcase.contains("google")) {
            restMetrics.addSpider(PlatformType.GOOGLE);
            cacheManager.addSpider(PlatformType.GOOGLE.name());
            return true;
        } else if(lowcase.contains("360")) {
            restMetrics.addSpider(PlatformType.M360);
            cacheManager.addSpider(PlatformType.M360.name());
            return true;
        } else if(lowcase.contains("sogou")) {
            restMetrics.addSpider(PlatformType.SOGOU);
            cacheManager.addSpider(PlatformType.SOGOU.name());
            return true;
        } else if(lowcase.contains("yisou")) {
            restMetrics.addSpider(PlatformType.SHENMA);
            cacheManager.addSpider(PlatformType.SHENMA.name());
            return true;
        } else if(lowcase.contains("byte")) {
            restMetrics.addSpider(PlatformType.TOU_TIAO);
            cacheManager.addSpider(PlatformType.TOU_TIAO.name());
            return true;
        } else {
            restMetrics.addSpider(PlatformType.OTHERS);
            cacheManager.addSpider(PlatformType.OTHERS.name());
        }
        return false;
    }


    @Override
    public String getErrorPath() {
        return "/error";
    }
}
